@@ -27,7 +27,7 @@ shift=0;
 num=100;
 num_test=5000;
 cache=10;
-normalize=1;
+normalize=true;
 mkl_stepsize=1;
 block=0;
 single_degree=-1;
