@@ -26,6 +26,15 @@ class CIO;
 #define SG_REF(x)
 #define SG_UNREF(x)
 #endif // HAVE_SWIG
+
+/** Class SGObject is the base class of all shogun objects. Apart from dealing
+ * with reference counting that is used to manage shogung objects in memory
+ * (erase unused object, avoid cleaning objects when they are still in use), it
+ * provides interfaces for:
+ * -# parallel - to determine the number of used CPUs for a method (cf. CParallel)
+ * -# io - to output messages and general i/o (cf. CIO)
+ * -# version - to provide version information of the shogun version used (cf. CVersion)
+ */
 class CSGObject
 {
 public:
@@ -44,20 +53,20 @@ public:
 	{
 	}
 
-	inline INT ref()
+	inline int32_t ref()
 	{
 		++refcount;
 		SG_DEBUG("ref():%ld obj:%p\n", refcount, this);
 		return refcount;
 	}
 
-	inline INT ref_count() const
+	inline int32_t ref_count() const
 	{
 		SG_DEBUG("ref_count(): refcount is: %d\n", refcount);
 		return refcount;
 	}
 
-	inline INT unref()
+	inline int32_t unref()
 	{
 		if (refcount==0 || --refcount==0)
 		{
@@ -75,7 +84,7 @@ public:
 	}
 
 private:
-	INT refcount;
+	int32_t refcount;
 #else //HAVE_R
 	inline CSGObject()
 	{
