@@ -23,13 +23,13 @@ CLabels* CClassifier::classify(CLabels* output)
 {
 	if (labels)
 	{
-		INT num=labels->get_num_labels();
+		int32_t num=labels->get_num_labels();
 		ASSERT(num>0);
 
 		if (!output)
 			output=new CLabels(num);
 
-		for (INT i=0; i<num; i++)
+		for (int32_t i=0; i<num; i++)
 			output->set_label(i, classify_example(i));
 
 		return output;
