@@ -32,7 +32,7 @@ class CKernelPerceptron : public CKernelMachine
 		 * @param num which example to classify
 		 * @return classified value
 		 */
-		virtual DREAL classify_example(INT num);
+		virtual float64_t classify_example(int32_t num);
 
 		/** load KernelPerceptron from file
 		 *
