@@ -32,22 +32,22 @@ class CLPM : public CSparseLinearClassifier
 			return CT_LPM;
 		}
 
-		inline void set_C(DREAL c1, DREAL c2) { C1=c1; C2=c2; }
+		inline void set_C(float64_t c1, float64_t c2) { C1=c1; C2=c2; }
 
-		inline DREAL get_C1() { return C1; }
-		inline DREAL get_C2() { return C2; }
+		inline float64_t get_C1() { return C1; }
+		inline float64_t get_C2() { return C2; }
 
 		inline void set_bias_enabled(bool enable_bias) { use_bias=enable_bias; }
 		inline bool get_bias_enabled() { return use_bias; }
 
-		inline void set_epsilon(DREAL eps) { epsilon=eps; }
-		inline DREAL get_epsilon() { return epsilon; }
+		inline void set_epsilon(float64_t eps) { epsilon=eps; }
+		inline float64_t get_epsilon() { return epsilon; }
 
 	protected:
-		DREAL C1;
-		DREAL C2;
+		float64_t C1;
+		float64_t C2;
 		bool use_bias;
-		DREAL epsilon;
+		float64_t epsilon;
 };
 #endif //USE_CPLEX
 #endif //_LPM_H___
