@@ -44,21 +44,21 @@ class CPerceptron : public CLinearClassifier
 		virtual bool train();
 
 		/// set learn rate of gradient descent training algorithm
-		inline void set_learn_rate(DREAL r)
+		inline void set_learn_rate(float64_t r)
 		{
 			learn_rate=r;
 		}
 
 		/// set maximum number of iterations
-		inline void set_max_iter(INT i)
+		inline void set_max_iter(int32_t i)
 		{
 			max_iter=i;
 		}
 
 	protected:
 		/** learning rate */
-		DREAL learn_rate;
+		float64_t learn_rate;
 		/** maximum number of iterations */
-		INT max_iter;
+		int32_t max_iter;
 };
 #endif
