@@ -24,7 +24,7 @@ class CPluginEstimate: public CClassifier
 		 * @param pos_pseudo pseudo for positive model
 		 * @param neg_pseudo pseudo for negative model
 		 */
-		CPluginEstimate(DREAL pos_pseudo=1e-10, DREAL neg_pseudo=1e-10);
+		CPluginEstimate(float64_t pos_pseudo=1e-10, float64_t neg_pseudo=1e-10);
 		virtual ~CPluginEstimate();
 
 		/** train the estimate
@@ -40,7 +40,7 @@ class CPluginEstimate: public CClassifier
 		 *
 		 * @param feat features to set
 		 */
-		virtual inline void set_features(CStringFeatures<WORD>* feat)
+		virtual inline void set_features(CStringFeatures<uint16_t>* feat)
 		{
 			SG_UNREF(features);
 			SG_REF(feat);
@@ -51,10 +51,10 @@ class CPluginEstimate: public CClassifier
 		 *
 		 * @return features
 		 */
-		virtual CStringFeatures<WORD>* get_features() { SG_REF(features); return features; }
+		virtual CStringFeatures<uint16_t>* get_features() { SG_REF(features); return features; }
 
 		/// classify the test feature vector indexed by vec_idx
-		DREAL classify_example(INT vec_idx);
+		float64_t classify_example(int32_t vec_idx);
 
 		/** obsolete posterior log odds
 		 *
@@ -62,7 +62,8 @@ class CPluginEstimate: public CClassifier
 		 * @param len len
 		 * @return something floaty
 		 */
-		inline DREAL posterior_log_odds_obsolete(WORD* vector, INT len)
+		inline float64_t posterior_log_odds_obsolete(
+			uint16_t* vector, int32_t len)
 		{
 			return pos_model->get_log_likelihood_example(vector, len) - neg_model->get_log_likelihood_example(vector, len);
 		}
@@ -73,7 +74,8 @@ class CPluginEstimate: public CClassifier
 		 * @param position position
 		 * @return log odd at position
 		 */
-		inline DREAL get_parameterwise_log_odds(WORD obs, INT position)
+		inline float64_t get_parameterwise_log_odds(
+			uint16_t obs, int32_t position)
 		{
 			return pos_model->get_positional_log_parameter(obs, position) - neg_model->get_positional_log_parameter(obs, position);
 		}
@@ -84,7 +86,7 @@ class CPluginEstimate: public CClassifier
 		 * @param pos position
 		 * @return positive log derivative
 		 */
-		inline DREAL log_derivative_pos_obsolete(WORD obs, INT pos)
+		inline float64_t log_derivative_pos_obsolete(uint16_t obs, int32_t pos)
 		{
 			return pos_model->get_log_derivative_obsolete(obs, pos);
 		}
@@ -95,7 +97,7 @@ class CPluginEstimate: public CClassifier
 		 * @param pos position
 		 * @return negative log derivative
 		 */
-		inline DREAL log_derivative_neg_obsolete(WORD obs, INT pos)
+		inline float64_t log_derivative_neg_obsolete(uint16_t obs, int32_t pos)
 		{
 			return neg_model->get_log_derivative_obsolete(obs, pos);
 		}
@@ -108,9 +110,11 @@ class CPluginEstimate: public CClassifier
 		 * @param num_symbols numbe of symbols
 		 * @return if operation was successful
 		 */
-		inline bool get_model_params(DREAL*& pos_params, DREAL*& neg_params, INT &seq_length, INT &num_symbols)
+		inline bool get_model_params(
+			float64_t*& pos_params, float64_t*& neg_params,
+			int32_t &seq_length, int32_t &num_symbols)
 		{
-			INT num;
+			int32_t num;
 
 			if ((!pos_model) || (!neg_model))
 			{
@@ -134,9 +138,11 @@ class CPluginEstimate: public CClassifier
 		 * @param seq_length sequence length
 		 * @param num_symbols numbe of symbols
 		 */
-		inline void set_model_params(const DREAL* pos_params, const DREAL* neg_params, INT seq_length, INT num_symbols)
+		inline void set_model_params(
+			const float64_t* pos_params, const float64_t* neg_params,
+			int32_t seq_length, int32_t num_symbols)
 		{
-			INT num_params;
+			int32_t num_params;
 
 			delete pos_model;
 			pos_model=new CLinearHMM(seq_length, num_symbols);
@@ -155,7 +161,7 @@ class CPluginEstimate: public CClassifier
 		 *
 		 * @return number of parameters
 		 */
-		inline INT get_num_params()
+		inline int32_t get_num_params()
 		{
 			return pos_model->get_num_model_parameters()+neg_model->get_num_model_parameters();
 		}
@@ -171,9 +177,9 @@ class CPluginEstimate: public CClassifier
 
 	protected:
 		/** pseudo count for positive class */
-		DREAL m_pos_pseudo;
+		float64_t m_pos_pseudo;
 		/** pseudo count for negative class */
-		DREAL m_neg_pseudo;
+		float64_t m_neg_pseudo;
 
 		/** positive model */
 		CLinearHMM* pos_model;
@@ -181,6 +187,6 @@ class CPluginEstimate: public CClassifier
 		CLinearHMM* neg_model;
 
 		/** features */
-		CStringFeatures<WORD>* features;
+		CStringFeatures<uint16_t>* features;
 };
 #endif
