@@ -23,7 +23,7 @@ CGMNPSVM::CGMNPSVM()
 {
 }
 
-CGMNPSVM::CGMNPSVM(DREAL C, CKernel* k, CLabels* lab)
+CGMNPSVM::CGMNPSVM(float64_t C, CKernel* k, CLabels* lab)
 : CMultiClassSVM(ONE_VS_REST, C, k, lab)
 {
 }
@@ -37,34 +37,34 @@ bool CGMNPSVM::train()
 	ASSERT(kernel);
 	ASSERT(labels && labels->get_num_labels());
 
-	INT num_data = labels->get_num_labels();
-	INT num_classes = labels->get_num_classes();
-	INT num_virtual_data= num_data*(num_classes-1);
+	int32_t num_data = labels->get_num_labels();
+	int32_t num_classes = labels->get_num_classes();
+	int32_t num_virtual_data= num_data*(num_classes-1);
 
 	SG_INFO( "%d trainlabels, %d classes\n", num_data, num_classes);
 
-	DREAL* vector_y = new double[num_data];
-	for (int i=0; i<num_data; i++)
+	float64_t* vector_y = new float64_t[num_data];
+	for (int32_t i=0; i<num_data; i++)
 		vector_y[i]= labels->get_label(i)+1;
 
-	DREAL C = get_C1();
-	INT tmax = 1000000000;
-	DREAL tolabs = 0;
-	DREAL tolrel = epsilon;
+	float64_t C = get_C1();
+	int32_t tmax = 1000000000;
+	float64_t tolabs = 0;
+	float64_t tolrel = epsilon;
 
-	DREAL reg_const=0;
+	float64_t reg_const=0;
 	if( C!=0 )
 		reg_const = 1/(2*C);
 
 
-	DREAL* alpha = new DREAL[num_virtual_data];
-	DREAL* vector_c = new DREAL[num_virtual_data];
-	memset(vector_c, 0, num_virtual_data*sizeof(DREAL));
+	float64_t* alpha = new float64_t[num_virtual_data];
+	float64_t* vector_c = new float64_t[num_virtual_data];
+	memset(vector_c, 0, num_virtual_data*sizeof(float64_t));
 
-	DREAL thlb = 10000000000.0;
-	INT t = 0;
-	DREAL* History = NULL;
-	INT verb = 0;
+	float64_t thlb = 10000000000.0;
+	int32_t t = 0;
+	float64_t* History = NULL;
+	int32_t verb = 0;
 
 	CGMNPLib mnp(vector_y,kernel,num_data, num_virtual_data, num_classes, reg_const);
 
@@ -72,35 +72,35 @@ bool CGMNPSVM::train()
 			tolabs, tolrel, thlb, alpha, &t, &History, verb );
 
 	/* matrix alpha [num_classes x num_data] */
-	DREAL* all_alphas= new DREAL[num_classes*num_data];
-	memset(all_alphas,0,num_classes*num_data*sizeof(DREAL));
+	float64_t* all_alphas= new float64_t[num_classes*num_data];
+	memset(all_alphas,0,num_classes*num_data*sizeof(float64_t));
 
 	/* bias vector b [num_classes x 1] */
-	DREAL* all_bs=new DREAL[num_classes];
-	memset(all_bs,0,num_classes*sizeof(DREAL));
+	float64_t* all_bs=new float64_t[num_classes];
+	memset(all_bs,0,num_classes*sizeof(float64_t));
 
 	/* compute alpha/b from virt_data */
-	for(INT i=0; i < num_classes; i++ )
+	for(int32_t i=0; i < num_classes; i++ )
 	{
-		for(INT j=0; j < num_virtual_data; j++ )
+		for(int32_t j=0; j < num_virtual_data; j++ )
 		{
-			INT inx1=0;
-			INT inx2=0;
+			int32_t inx1=0;
+			int32_t inx2=0;
 
 			mnp.get_indices2( &inx1, &inx2, j );
 
 			all_alphas[(inx1*num_classes)+i] += 
-				alpha[j]*(KDELTA(vector_y[inx1],i+1)+KDELTA(i+1,inx2));
+				alpha[j]*(KDELTA(vector_y[inx1],i+1)-KDELTA(i+1,inx2));
 			all_bs[i] += alpha[j]*(KDELTA(vector_y[inx1],i+1)-KDELTA(i+1,inx2));
 		}
 	}
 
 	create_multiclass_svm(num_classes);
 
-	for (INT i=0; i<num_classes; i++)
+	for (int32_t i=0; i<num_classes; i++)
 	{
-		INT num_sv=0;
-		for (INT j=0; j<num_data; j++)
+		int32_t num_sv=0;
+		for (int32_t j=0; j<num_data; j++)
 		{
 			if (all_alphas[j*num_classes+i] != 0)
 				num_sv++;
@@ -110,16 +110,12 @@ bool CGMNPSVM::train()
 
 		CSVM* svm=new CSVM(num_sv);
 
-		INT k=0;
-		for (INT j=0; j<num_data; j++)
+		int32_t k=0;
+		for (int32_t j=0; j<num_data; j++)
 		{
 			if (all_alphas[j*num_classes+i] != 0)
 			{
-				if (i==vector_y[j]-1)
-					svm->set_alpha(k, all_alphas[j*num_classes+i]);
-				else
-					svm->set_alpha(k, -all_alphas[j*num_classes+i]);
-
+				svm->set_alpha(k, all_alphas[j*num_classes+i]);
 				svm->set_support_vector(k, j);
 				k++;
 			}
