@@ -25,7 +25,8 @@ CLibLinear::CLibLinear(LIBLINEAR_LOSS l)
 	C2=1;
 }
 
-CLibLinear::CLibLinear(DREAL C, CSparseFeatures<DREAL>* traindat, CLabels* trainlab)
+CLibLinear::CLibLinear(
+	float64_t C, CSparseFeatures<float64_t>* traindat, CLabels* trainlab)
 : CSparseLinearClassifier(), C1(C), C2(C), use_bias(true), epsilon(1e-5)
 {
 	set_features(traindat);
@@ -44,37 +45,37 @@ bool CLibLinear::train()
 	ASSERT(get_features());
 	ASSERT(labels->is_two_class_labeling());
 
-	CSparseFeatures<DREAL>* sfeat=(CSparseFeatures<DREAL>*) features;
+	CSparseFeatures<float64_t>* sfeat=(CSparseFeatures<float64_t>*) features;
 
-	INT num_train_labels=labels->get_num_labels();
-	INT num_feat=features->get_num_features();
-	INT num_vec=features->get_num_vectors();
+	int32_t num_train_labels=labels->get_num_labels();
+	int32_t num_feat=features->get_num_features();
+	int32_t num_vec=features->get_num_vectors();
 
 	ASSERT(num_vec==num_train_labels);
 	delete[] w;
 	if (use_bias)
-		w=new DREAL[num_feat+1];
+		w=new float64_t[num_feat+1];
 	else
-		w=new DREAL[num_feat+0];
+		w=new float64_t[num_feat+0];
 	w_dim=num_feat;
 
 	problem prob;
 	if (use_bias)
 	{
 		prob.n=w_dim+1;
-		memset(w, 0, sizeof(DREAL)*(w_dim+1));
+		memset(w, 0, sizeof(float64_t)*(w_dim+1));
 	}
 	else
 	{
 		prob.n=w_dim;
-		memset(w, 0, sizeof(DREAL)*(w_dim+0));
+		memset(w, 0, sizeof(float64_t)*(w_dim+0));
 	}
 	prob.l=num_vec;
 	prob.x=sfeat;
 	prob.y=new int[prob.l];
 	prob.use_bias=use_bias;
 
-	for (int i=0; i<prob.l; i++)
+	for (int32_t i=0; i<prob.l; i++)
 		prob.y[i]=labels->get_int_label(i);
 
 	SG_INFO( "%d training points %d dims\n", prob.l, prob.n);
@@ -98,9 +99,9 @@ bool CLibLinear::train()
 	{
 		CTron tron_obj(fun_obj, epsilon);
 		tron_obj.tron(w);
-		DREAL sgn=prob.y[0];
+		float64_t sgn=prob.y[0];
 
-		for (INT i=0; i<w_dim; i++)
+		for (int32_t i=0; i<w_dim; i++)
 			w[i]*=sgn;
 
 		if (use_bias)
