@@ -21,7 +21,8 @@ CSVMLin::CSVMLin()
 {
 }
 
-CSVMLin::CSVMLin(DREAL C, CSparseFeatures<DREAL>* traindat, CLabels* trainlab)
+CSVMLin::CSVMLin(
+	float64_t C, CSparseFeatures<float64_t>* traindat, CLabels* trainlab)
 : CSparseLinearClassifier(), C1(C), C2(C), epsilon(1e-5), use_bias(true)
 {
 	set_features(traindat);
@@ -38,10 +39,10 @@ bool CSVMLin::train()
 	ASSERT(labels);
 	ASSERT(get_features());
 
-	INT num_train_labels=0;
-	DREAL* train_labels=labels->get_labels(num_train_labels);
-	INT num_feat=features->get_num_features();
-	INT num_vec=features->get_num_vectors();
+	int32_t num_train_labels=0;
+	float64_t* train_labels=labels->get_labels(num_train_labels);
+	int32_t num_feat=features->get_num_features();
+	int32_t num_vec=features->get_num_vectors();
 
 	ASSERT(num_vec==num_train_labels);
 	delete[] w;
@@ -58,7 +59,7 @@ bool CSVMLin::train()
 	Data.nz=num_feat+1;
 	Data.Y=train_labels;
 	Data.features=get_features();
-	Data.C = new double[Data.l];
+	Data.C = new float64_t[Data.l];
 
 	Options.algo = SVM;
 	Options.lambda=1/(2*get_C1());
@@ -76,7 +77,7 @@ bool CSVMLin::train()
 	else
 		Options.bias=0.0;
 
-	for(int i=0;i<num_vec;i++)
+	for (int32_t i=0;i<num_vec;i++)
 	{
 		if(train_labels[i]>0) 
 			Data.C[i]=Options.Cp;
@@ -86,8 +87,8 @@ bool CSVMLin::train()
 	ssl_train(&Data, &Options, &Weights, &Outputs);
 	ASSERT(Weights.vec && Weights.d==num_feat+1);
 
-	DREAL sgn=train_labels[0];
-	for (INT i=0; i<num_feat+1; i++)
+	float64_t sgn=train_labels[0];
+	for (int32_t i=0; i<num_feat+1; i++)
 		Weights.vec[i]*=sgn;
 
 	CSparseLinearClassifier::set_w(Weights.vec, num_feat);
