@@ -27,7 +27,8 @@ CSVMOcas::CSVMOcas(E_SVM_TYPE type)
 	old_w=NULL;
 }
 
-CSVMOcas::CSVMOcas(DREAL C, CSparseFeatures<DREAL>* traindat, CLabels* trainlab)
+CSVMOcas::CSVMOcas(
+	float64_t C, CSparseFeatures<float64_t>* traindat, CLabels* trainlab)
 : CSparseLinearClassifier(), use_bias(false), bufsize(3000), C1(C), C2(C),
 	epsilon(1e-3)
 {
@@ -51,31 +52,31 @@ bool CSVMOcas::train()
 	ASSERT(get_features());
 	ASSERT(labels->is_two_class_labeling());
 
-	INT num_train_labels=0;
+	int32_t num_train_labels=0;
 	lab=labels->get_labels(num_train_labels);
 	w_dim=features->get_num_features();
-	INT num_vec=features->get_num_vectors();
+	int32_t num_vec=features->get_num_vectors();
 
 	ASSERT(num_vec==num_train_labels);
 	ASSERT(num_vec>0);
 
 	delete[] w;
-	w=new DREAL[w_dim];
-	memset(w, 0, w_dim*sizeof(DREAL));
+	w=new float64_t[w_dim];
+	memset(w, 0, w_dim*sizeof(float64_t));
 
 	delete[] old_w;
-	old_w=new DREAL[w_dim];
-	memset(old_w, 0, w_dim*sizeof(DREAL));
+	old_w=new float64_t[w_dim];
+	memset(old_w, 0, w_dim*sizeof(float64_t));
 	bias=0;
 
-	tmp_a_buf=new DREAL[w_dim];
-	cp_value=new DREAL*[bufsize];
+	tmp_a_buf=new float64_t[w_dim];
+	cp_value=new float64_t*[bufsize];
 	cp_index=new uint32_t*[bufsize];
 	cp_nz_dims=new uint32_t[bufsize];
 
-	double TolAbs=0;
-	double QPBound=0;
-	int Method=0;
+	float64_t TolAbs=0;
+	float64_t QPBound=0;
+	int32_t Method=0;
 	if (method == SVM_OCAS)
 		Method = 1;
 	ocas_return_value_T result = svm_ocas_solver( get_C1(), num_vec, get_epsilon(),
@@ -131,13 +132,13 @@ bool CSVMOcas::train()
   sq_norm_W = W'*W;
 
   ---------------------------------------------------------------------------------*/
-double CSVMOcas::update_W( double t, void* ptr )
+float64_t CSVMOcas::update_W( float64_t t, void* ptr )
 {
-  double sq_norm_W = 0;         
+  float64_t sq_norm_W = 0;         
   CSVMOcas* o = (CSVMOcas*) ptr;
   uint32_t nDim = (uint32_t) o->w_dim;
-  double* W=o->w;
-  double* oldW=o->old_w;
+  float64_t* W=o->w;
+  float64_t* oldW=o->old_w;
 
   for(uint32_t j=0; j <nDim; j++)
   {
@@ -156,27 +157,25 @@ double CSVMOcas::update_W( double t, void* ptr )
     sparse_A(:,nSel+1) = new_a;
 
   ---------------------------------------------------------------------------------*/
-void CSVMOcas::add_new_cut( double *new_col_H, 
-                  uint32_t *new_cut, 
-                  uint32_t cut_length, 
-                  uint32_t nSel,
-				  void* ptr)
+void CSVMOcas::add_new_cut(
+	float64_t *new_col_H, uint32_t *new_cut, uint32_t cut_length,
+	uint32_t nSel, void* ptr)
 {
 	CSVMOcas* o = (CSVMOcas*) ptr;
-	CSparseFeatures<DREAL>* f = o->get_features();
+	CSparseFeatures<float64_t>* f = o->get_features();
 	uint32_t nDim=(uint32_t) o->w_dim;
-	DREAL* y = o->lab;
+	float64_t* y = o->lab;
 
-	DREAL** c_val = o->cp_value;
+	float64_t** c_val = o->cp_value;
 	uint32_t** c_idx = o->cp_index;
 	uint32_t* c_nzd = o->cp_nz_dims;
 
-	double sq_norm_a;
+	float64_t sq_norm_a;
 	uint32_t i, j, nz_dims;
 
 	/* temporary vector */
-	double* new_a = o->tmp_a_buf;
-	memset(new_a, 0, sizeof(double)*nDim);
+	float64_t* new_a = o->tmp_a_buf;
+	memset(new_a, 0, sizeof(float64_t)*nDim);
 
 	for(i=0; i < cut_length; i++) 
 		f->add_to_dense_vec(y[new_cut[i]], new_cut[i], new_a, nDim);
@@ -196,7 +195,7 @@ void CSVMOcas::add_new_cut( double *new_col_H,
 	if(nz_dims > 0)
 	{
 		c_idx[nSel]=new uint32_t[nz_dims];
-		c_val[nSel]=new double[nz_dims];
+		c_val[nSel]=new float64_t[nz_dims];
 
 		uint32_t idx=0;
 		for(j=0; j < nDim; j++ )
@@ -213,18 +212,18 @@ void CSVMOcas::add_new_cut( double *new_col_H,
 
 	for(i=0; i < nSel; i++)
 	{
-		double tmp = 0;
+		float64_t tmp = 0;
 		for(j=0; j < c_nzd[i]; j++)
 			tmp += new_a[c_idx[i][j]]*c_val[i][j];
 
 		new_col_H[i] = tmp;
 	}
 	//CMath::display_vector(new_col_H, nSel+1, "new_col_H");
-	//CMath::display_vector((INT*) c_idx[nSel], (INT) nz_dims, "c_idx");
-	//CMath::display_vector((DREAL*) c_val[nSel], nz_dims, "c_val");
+	//CMath::display_vector((int32_t*) c_idx[nSel], (int32_t) nz_dims, "c_idx");
+	//CMath::display_vector((float64_t*) c_val[nSel], nz_dims, "c_val");
 }
 
-void CSVMOcas::sort( double* vals, uint32_t* idx, uint32_t size)
+void CSVMOcas::sort(float64_t* vals, uint32_t* idx, uint32_t size)
 {
 	CMath::qsort_index(vals, idx, size);
 }
@@ -234,13 +233,13 @@ void CSVMOcas::sort( double* vals, uint32_t* idx, uint32_t size)
 
   output = data_X'*W;
   ----------------------------------------------------------------------*/
-void CSVMOcas::compute_output( double *output, void* ptr )
+void CSVMOcas::compute_output(float64_t *output, void* ptr)
 {
 	CSVMOcas* o = (CSVMOcas*) ptr;
-	CSparseFeatures<DREAL>* f=o->get_features();
-	INT nData=f->get_num_vectors();
+	CSparseFeatures<float64_t>* f=o->get_features();
+	int32_t nData=f->get_num_vectors();
 
-	DREAL* y = o->lab;
+	float64_t* y = o->lab;
 	f->dense_dot_range(output, 0, nData, y, o->w, o->w_dim, 0.0);
 	//CMath::display_vector(o->w, o->w_dim, "w");
 	//CMath::display_vector(output, nData, "out");
@@ -255,16 +254,18 @@ void CSVMOcas::compute_output( double *output, void* ptr )
   dp_WoldW = W'*oldW';
 
   ----------------------------------------------------------------------*/
-void CSVMOcas::compute_W( double *sq_norm_W, double *dp_WoldW, double *alpha, uint32_t nSel, void* ptr )
+void CSVMOcas::compute_W(
+	float64_t *sq_norm_W, float64_t *dp_WoldW, float64_t *alpha, uint32_t nSel,
+	void* ptr )
 {
 	CSVMOcas* o = (CSVMOcas*) ptr;
 	uint32_t nDim= (uint32_t) o->w_dim;
 	CMath::swap(o->w, o->old_w);
-	double* W=o->w;
-	double* oldW=o->old_w;
-	memset(W, 0, sizeof(double)*nDim);
+	float64_t* W=o->w;
+	float64_t* oldW=o->old_w;
+	memset(W, 0, sizeof(float64_t)*nDim);
 
-	DREAL** c_val = o->cp_value;
+	float64_t** c_val = o->cp_value;
 	uint32_t** c_idx = o->cp_index;
 	uint32_t* c_nzd = o->cp_nz_dims;
 
