@@ -39,7 +39,9 @@ class CSVMOcas : public CSparseLinearClassifier
 		 * @param traindat training features
 		 * @param trainlab labels for training features
 		 */
-		CSVMOcas(DREAL C, CSparseFeatures<DREAL>* traindat, CLabels* trainlab);
+		CSVMOcas(
+			float64_t C, CSparseFeatures<float64_t>* traindat,
+			CLabels* trainlab);
 		virtual ~CSVMOcas();
 
 		/** get classifier type
@@ -59,31 +61,31 @@ class CSVMOcas : public CSparseLinearClassifier
 		 * @param c1 new C1
 		 * @param c2 new C2
 		 */
-		inline void set_C(DREAL c1, DREAL c2) { C1=c1; C2=c2; }
+		inline void set_C(float64_t c1, float64_t c2) { C1=c1; C2=c2; }
 
 		/** get C1
 		 *
 		 * @return C1
 		 */
-		inline DREAL get_C1() { return C1; }
+		inline float64_t get_C1() { return C1; }
 
 		/** get C2
 		 *
 		 * @return C2
 		 */
-		inline DREAL get_C2() { return C2; }
+		inline float64_t get_C2() { return C2; }
 
 		/** set epsilon
 		 *
 		 * @param eps new epsilon
 		 */
-		inline void set_epsilon(DREAL eps) { epsilon=eps; }
+		inline void set_epsilon(float64_t eps) { epsilon=eps; }
 
 		/** get epsilon
 		 *
 		 * @return epsilon
 		 */
-		inline DREAL get_epsilon() { return epsilon; }
+		inline float64_t get_epsilon() { return epsilon; }
 
 		/** set if bias shall be enabled
 		 *
@@ -101,13 +103,13 @@ class CSVMOcas : public CSparseLinearClassifier
 		 *
 		 * @param sz buffer size
 		 */
-		inline void set_bufsize(INT sz) { bufsize=sz; }
+		inline void set_bufsize(int32_t sz) { bufsize=sz; }
 
 		/** get buffer size
 		 *
 		 * @return buffer size
 		 */
-		inline INT get_bufsize() { return bufsize; }
+		inline int32_t get_bufsize() { return bufsize; }
 
 	protected:
 		/** compute W
@@ -118,7 +120,9 @@ class CSVMOcas : public CSparseLinearClassifier
 		 * @param nSel nSel
 		 * @param ptr ptr
 		 */
-		static void compute_W( double *sq_norm_W, double *dp_WoldW, double *alpha, uint32_t nSel, void* ptr );
+		static void compute_W(
+			float64_t *sq_norm_W, float64_t *dp_WoldW, float64_t *alpha,
+			uint32_t nSel, void* ptr);
 
 		/** update W
 		 *
@@ -126,7 +130,7 @@ class CSVMOcas : public CSparseLinearClassifier
 		 * @param ptr ptr
 		 * @return something floaty
 		 */
-		static double update_W(double t, void* ptr );
+		static float64_t update_W(float64_t t, void* ptr );
 
 		/** add new cut
 		 *
@@ -136,14 +140,16 @@ class CSVMOcas : public CSparseLinearClassifier
 		 * @param nSel nSel
 		 * @param ptr ptr
 		 */
-		static void add_new_cut( double *new_col_H, uint32_t *new_cut, uint32_t cut_length, uint32_t nSel, void* ptr );
+		static void add_new_cut(
+			float64_t *new_col_H, uint32_t *new_cut, uint32_t cut_length,
+			uint32_t nSel, void* ptr );
 
 		/** compute output
 		 *
 		 * @param output output
 		 * @param ptr ptr
 		 */
-		static void compute_output( double *output, void* ptr );
+		static void compute_output( float64_t *output, void* ptr );
 
 		/** sort
 		 *
@@ -151,33 +157,33 @@ class CSVMOcas : public CSparseLinearClassifier
 		 * @param idx idx
 		 * @param size size
 		 */
-		static void sort( double* vals, uint32_t* idx, uint32_t size);
+		static void sort( float64_t* vals, uint32_t* idx, uint32_t size);
 
 
 	protected:
 		/** if bias is used */
 		bool use_bias;
 		/** buffer size */
-		INT bufsize;
+		int32_t bufsize;
 		/** C1 */
-		DREAL C1;
+		float64_t C1;
 		/** C2 */
-		DREAL C2;
+		float64_t C2;
 		/** epsilon */
-		DREAL epsilon;
+		float64_t epsilon;
 		/** method */
 		E_SVM_TYPE method;
 
 		/** old W */
-		DREAL* old_w;
+		float64_t* old_w;
 		/** nDim big */
-		DREAL* tmp_a_buf;
+		float64_t* tmp_a_buf;
 		/** labels */
-		DREAL* lab;
+		float64_t* lab;
 		
 		/** sparse representation of
 		 * cutting planes */
-		DREAL** cp_value;
+		float64_t** cp_value;
 		/** cutting plane index */
 		uint32_t** cp_index;
 		/** cutting plane dimensions */
