@@ -47,7 +47,7 @@
 #include <string.h>
 #include <stdarg.h>
 typedef KERNELCACHE_ELEM Qfloat;
-typedef DREAL schar;
+typedef float64_t schar;
 #ifndef min
 template <class T> inline T min(T x,T y) { return (x<y)?x:y; }
 #endif
@@ -55,7 +55,7 @@ template <class T> inline T min(T x,T y) { return (x<y)?x:y; }
 template <class T> inline T max(T x,T y) { return (x>y)?x:y; }
 #endif
 template <class T> inline void swap(T& x, T& y) { T t=x; x=y; y=t; }
-template <class S, class T> inline void clone(T*& dst, S* src, int n)
+template <class S, class T> inline void clone(T*& dst, S* src, int32_t n)
 {
 	dst = new T[n];
 	memcpy((void *)dst,(void *)src,sizeof(T)*n);
@@ -73,22 +73,23 @@ template <class S, class T> inline void clone(T*& dst, S* src, int n)
 class Cache
 {
 public:
-	Cache(int l,long int size);
+	Cache(int32_t l, int64_t size);
 	~Cache();
 
 	// request data [0,len)
 	// return some position p where [p,len) need to be filled
 	// (p >= len if nothing needs to be filled)
-	int get_data(const int index, Qfloat **data, int len);
-	void swap_index(int i, int j);	// future_option
+	int32_t get_data(const int32_t index, Qfloat **data, int32_t len);
+	void swap_index(int32_t i, int32_t j);	// future_option
+
 private:
-	int l;
-	long int size;
+	int32_t l;
+	int64_t size;
 	struct head_t
 	{
 		head_t *prev, *next;	// a cicular list
 		Qfloat *data;
-		int len;		// data[0,len) is cached in this entry
+		int32_t len;		// data[0,len) is cached in this entry
 	};
 
 	head_t *head;
@@ -97,12 +98,12 @@ private:
 	void lru_insert(head_t *h);
 };
 
-Cache::Cache(int l_,long int size_):l(l_),size(size_)
+Cache::Cache(int32_t l_, int64_t size_):l(l_),size(size_)
 {
 	head = (head_t *)calloc(l,sizeof(head_t));	// initialized to 0
 	size /= sizeof(Qfloat);
 	size -= l * sizeof(head_t) / sizeof(Qfloat);
-	size = max(size, (long int) 2*l);	// cache must be large enough for two columns
+	size = max(size, (int64_t) 2*l);	// cache must be large enough for two columns
 	lru_head.next = lru_head.prev = &lru_head;
 }
 
@@ -129,11 +130,11 @@ void Cache::lru_insert(head_t *h)
 	h->next->prev = h;
 }
 
-int Cache::get_data(const int index, Qfloat **data, int len)
+int32_t Cache::get_data(const int32_t index, Qfloat **data, int32_t len)
 {
 	head_t *h = &head[index];
 	if(h->len) lru_delete(h);
-	int more = len - h->len;
+	int32_t more = len - h->len;
 
 	if(more > 0)
 	{
@@ -159,7 +160,7 @@ int Cache::get_data(const int index, Qfloat **data, int len)
 	return len;
 }
 
-void Cache::swap_index(int i, int j)
+void Cache::swap_index(int32_t i, int32_t j)
 {
 	if(i==j) return;
 
@@ -199,26 +200,26 @@ void Cache::swap_index(int i, int j)
 //
 class QMatrix {
 public:
-	virtual Qfloat *get_Q(int column, int len) const = 0;
+	virtual Qfloat *get_Q(int32_t column, int32_t len) const = 0;
 	virtual Qfloat *get_QD() const = 0;
-	virtual void swap_index(int i, int j) const = 0;
+	virtual void swap_index(int32_t i, int32_t j) const = 0;
 	virtual ~QMatrix() {}
 };
 
 class Kernel: public QMatrix {
 public:
-	Kernel(int l, svm_node * const * x, const svm_parameter& param);
+	Kernel(int32_t l, svm_node * const * x, const svm_parameter& param);
 	virtual ~Kernel();
 
-	virtual Qfloat *get_Q(int column, int len) const = 0;
+	virtual Qfloat *get_Q(int32_t column, int32_t len) const = 0;
 	virtual Qfloat *get_QD() const = 0;
-	virtual void swap_index(int i, int j) const	// no so const...
+	virtual void swap_index(int32_t i, int32_t j) const	// no so const...
 	{
 		swap(x[i],x[j]);
 		if(x_square) swap(x_square[i],x_square[j]);
 	}
 
-	inline double kernel_function(int i, int j) const
+	inline float64_t kernel_function(int32_t i, int32_t j) const
 	{
 		return kernel->kernel(x[i]->index,x[j]->index);
 	}
@@ -226,17 +227,17 @@ public:
 private:
 	CKernel* kernel;
 	const svm_node **x;
-	double *x_square;
+	float64_t *x_square;
 
 	// svm_parameter
-	const int kernel_type;
-	const int degree;
-	const double gamma;
-	const double coef0;
+	const int32_t kernel_type;
+	const int32_t degree;
+	const float64_t gamma;
+	const float64_t coef0;
 };
 
-Kernel::Kernel(int l, svm_node * const * x_, const svm_parameter& param)
-:kernel_type(param.kernel_type), degree(param.degree),
+Kernel::Kernel(int32_t l, svm_node * const * x_, const svm_parameter& param)
+: kernel_type(param.kernel_type), degree(param.degree),
  gamma(param.gamma), coef0(param.coef0)
 {
 	clone(x,x_,l);
@@ -274,38 +275,40 @@ public:
 	virtual ~Solver() {};
 
 	struct SolutionInfo {
-		double obj;
-		double rho;
-		double upper_bound_p;
-		double upper_bound_n;
-		double r;	// for Solver_NU
+		float64_t obj;
+		float64_t rho;
+		float64_t upper_bound_p;
+		float64_t upper_bound_n;
+		float64_t r;	// for Solver_NU
 	};
 
-	void Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
-		   double *alpha_, double Cp, double Cn, double eps,
-		   SolutionInfo* si, int shrinking);
+	void Solve(
+		int32_t l, const QMatrix& Q, const float64_t *p_, const schar *y_,
+		float64_t *alpha_, float64_t Cp, float64_t Cn, float64_t eps,
+		SolutionInfo* si, int32_t shrinking);
+
 protected:
-	int active_size;
+	int32_t active_size;
 	schar *y;
-	double *G;		// gradient of objective function
+	float64_t *G;		// gradient of objective function
 	enum { LOWER_BOUND, UPPER_BOUND, FREE };
 	char *alpha_status;	// LOWER_BOUND, UPPER_BOUND, FREE
-	double *alpha;
+	float64_t *alpha;
 	const QMatrix *Q;
 	const Qfloat *QD;
-	double eps;
-	double Cp,Cn;
-	double *p;
-	int *active_set;
-	double *G_bar;		// gradient, if we treat free variables as 0
-	int l;
+	float64_t eps;
+	float64_t Cp,Cn;
+	float64_t *p;
+	int32_t *active_set;
+	float64_t *G_bar;		// gradient, if we treat free variables as 0
+	int32_t l;
 	bool unshrinked;	// XXX
 
-	double get_C(int i)
+	float64_t get_C(int32_t i)
 	{
 		return (y[i] > 0)? Cp : Cn;
 	}
-	void update_alpha_status(int i)
+	void update_alpha_status(int32_t i)
 	{
 		if(alpha[i] >= get_C(i))
 			alpha_status[i] = UPPER_BOUND;
@@ -313,19 +316,19 @@ protected:
 			alpha_status[i] = LOWER_BOUND;
 		else alpha_status[i] = FREE;
 	}
-	bool is_upper_bound(int i) { return alpha_status[i] == UPPER_BOUND; }
-	bool is_lower_bound(int i) { return alpha_status[i] == LOWER_BOUND; }
-	bool is_free(int i) { return alpha_status[i] == FREE; }
-	void swap_index(int i, int j);
+	bool is_upper_bound(int32_t i) { return alpha_status[i] == UPPER_BOUND; }
+	bool is_lower_bound(int32_t i) { return alpha_status[i] == LOWER_BOUND; }
+	bool is_free(int32_t i) { return alpha_status[i] == FREE; }
+	void swap_index(int32_t i, int32_t j);
 	void reconstruct_gradient();
-	virtual int select_working_set(int &i, int &j, double &gap);
-	virtual double calculate_rho();
+	virtual int32_t select_working_set(int32_t &i, int32_t &j, float64_t &gap);
+	virtual float64_t calculate_rho();
 	virtual void do_shrinking();
 private:
-	bool be_shrunken(int i, double Gmax1, double Gmax2);	
+	bool be_shrunken(int32_t i, float64_t Gmax1, float64_t Gmax2);
 };
 
-void Solver::swap_index(int i, int j)
+void Solver::swap_index(int32_t i, int32_t j)
 {
 	Q->swap_index(i,j);
 	swap(y[i],y[j]);
@@ -343,7 +346,7 @@ void Solver::reconstruct_gradient()
 
 	if(active_size == l) return;
 
-	int i;
+	int32_t i;
 	for(i=active_size;i<l;i++)
 		G[i] = G_bar[i] + p[i];
 	
@@ -351,15 +354,16 @@ void Solver::reconstruct_gradient()
 		if(is_free(i))
 		{
 			const Qfloat *Q_i = Q->get_Q(i,l);
-			double alpha_i = alpha[i];
-			for(int j=active_size;j<l;j++)
+			float64_t alpha_i = alpha[i];
+			for(int32_t j=active_size;j<l;j++)
 				G[j] += alpha_i * Q_i[j];
 		}
 }
 
-void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *p_y,
-		   double *p_alpha, double p_Cp, double p_Cn, double p_eps,
-		   SolutionInfo* p_si, int shrinking)
+void Solver::Solve(
+	int32_t p_l, const QMatrix& p_Q, const float64_t *p_p,
+	const schar *p_y, float64_t *p_alpha, float64_t p_Cp, float64_t p_Cn,
+	float64_t p_eps, SolutionInfo* p_si, int32_t shrinking)
 {
 	this->l = p_l;
 	this->Q = &p_Q;
@@ -375,23 +379,23 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 	// initialize alpha_status
 	{
 		alpha_status = new char[l];
-		for(int i=0;i<l;i++)
+		for(int32_t i=0;i<l;i++)
 			update_alpha_status(i);
 	}
 
 	// initialize active set (for shrinking)
 	{
-		active_set = new int[l];
-		for(int i=0;i<l;i++)
+		active_set = new int32_t[l];
+		for(int32_t i=0;i<l;i++)
 			active_set[i] = i;
 		active_size = l;
 	}
 
 	// initialize gradient
 	{
-		G = new double[l];
-		G_bar = new double[l];
-		int i;
+		G = new float64_t[l];
+		G_bar = new float64_t[l];
+		int32_t i;
 		for(i=0;i<l;i++)
 		{
 			G[i] = p_p[i];
@@ -401,8 +405,8 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 			if(!is_lower_bound(i))
 			{
 				const Qfloat *Q_i = Q->get_Q(i,l);
-				double alpha_i = alpha[i];
-				int j;
+				float64_t alpha_i = alpha[i];
+				int32_t j;
 				for(j=0;j<l;j++)
 					G[j] += alpha_i*Q_i[j];
 				if(is_upper_bound(i))
@@ -413,8 +417,8 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 
 	// optimization step
 
-	int iter = 0;
-	int counter = min(l,1000)+1;
+	int32_t iter = 0;
+	int32_t counter = min(l,1000)+1;
 
 	while(1)
 	{
@@ -427,8 +431,8 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 			//SG_SINFO(".");
 		}
 
-		int i,j;
-		double gap;
+		int32_t i,j;
+		float64_t gap;
 		if(select_working_set(i,j, gap)!=0)
 		{
 			// reconstruct the whole gradient
@@ -451,19 +455,19 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 		const Qfloat *Q_i = Q->get_Q(i,active_size);
 		const Qfloat *Q_j = Q->get_Q(j,active_size);
 
-		double C_i = get_C(i);
-		double C_j = get_C(j);
+		float64_t C_i = get_C(i);
+		float64_t C_j = get_C(j);
 
-		double old_alpha_i = alpha[i];
-		double old_alpha_j = alpha[j];
+		float64_t old_alpha_i = alpha[i];
+		float64_t old_alpha_j = alpha[j];
 
 		if(y[i]!=y[j])
 		{
-			double quad_coef = Q_i[i]+Q_j[j]+2*Q_i[j];
+			float64_t quad_coef = Q_i[i]+Q_j[j]+2*Q_i[j];
 			if (quad_coef <= 0)
 				quad_coef = TAU;
-			double delta = (-G[i]-G[j])/quad_coef;
-			double diff = alpha[i] - alpha[j];
+			float64_t delta = (-G[i]-G[j])/quad_coef;
+			float64_t diff = alpha[i] - alpha[j];
 			alpha[i] += delta;
 			alpha[j] += delta;
 			
@@ -502,11 +506,11 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 		}
 		else
 		{
-			double quad_coef = Q_i[i]+Q_j[j]-2*Q_i[j];
+			float64_t quad_coef = Q_i[i]+Q_j[j]-2*Q_i[j];
 			if (quad_coef <= 0)
 				quad_coef = TAU;
-			double delta = (G[i]-G[j])/quad_coef;
-			double sum = alpha[i] + alpha[j];
+			float64_t delta = (G[i]-G[j])/quad_coef;
+			float64_t sum = alpha[i] + alpha[j];
 			alpha[i] -= delta;
 			alpha[j] += delta;
 
@@ -546,10 +550,10 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 
 		// update G
 
-		double delta_alpha_i = alpha[i] - old_alpha_i;
-		double delta_alpha_j = alpha[j] - old_alpha_j;
+		float64_t delta_alpha_i = alpha[i] - old_alpha_i;
+		float64_t delta_alpha_j = alpha[j] - old_alpha_j;
 		
-		for(int k=0;k<active_size;k++)
+		for(int32_t k=0;k<active_size;k++)
 		{
 			G[k] += Q_i[k]*delta_alpha_i + Q_j[k]*delta_alpha_j;
 		}
@@ -561,7 +565,7 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 			bool uj = is_upper_bound(j);
 			update_alpha_status(i);
 			update_alpha_status(j);
-			int k;
+			int32_t k;
 			if(ui != is_upper_bound(i))
 			{
 				Q_i = Q->get_Q(i,l);
@@ -592,8 +596,8 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 
 	// calculate objective value
 	{
-		double v = 0;
-		int i;
+		float64_t v = 0;
+		int32_t i;
 		for(i=0;i<l;i++)
 			v += alpha[i] * (G[i] + p[i]);
 
@@ -602,7 +606,7 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 
 	// put back the solution
 	{
-		for(int i=0;i<l;i++)
+		for(int32_t i=0;i<l;i++)
 			p_alpha[active_set[i]] = alpha[i];
 	}
 
@@ -621,22 +625,23 @@ void Solver::Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *
 }
 
 // return 1 if already optimal, return 0 otherwise
-int Solver::select_working_set(int &out_i, int &out_j, double &gap)
+int32_t Solver::select_working_set(
+	int32_t &out_i, int32_t &out_j, float64_t &gap)
 {
 	// return i,j such that
 	// i: maximizes -y_i * grad(f)_i, i in I_up(\alpha)
 	// j: minimizes the decrease of obj value
-	//    (if quadratic coefficeint <= 0, replace it with tau)
+	//    (if quadratic coefficient <= 0, replace it with tau)
 	//    -y_j*grad(f)_j < -y_i*grad(f)_i, j in I_low(\alpha)
 	
-	double Gmax = -INF;
-	double Gmax2 = -INF;
-	int Gmax_idx = -1;
-	int Gmin_idx = -1;
-	double obj_diff_min = INF;
+	float64_t Gmax = -INF;
+	float64_t Gmax2 = -INF;
+	int32_t Gmax_idx = -1;
+	int32_t Gmin_idx = -1;
+	float64_t obj_diff_min = INF;
 
-	for(int t=0;t<active_size;t++)
-		if(y[t]==+1)	
+	for(int32_t t=0;t<active_size;t++)
+		if(y[t]==+1)
 		{
 			if(!is_upper_bound(t))
 				if(-G[t] >= Gmax)
@@ -655,24 +660,24 @@ int Solver::select_working_set(int &out_i, int &out_j, double &gap)
 				}
 		}
 
-	int i = Gmax_idx;
+	int32_t i = Gmax_idx;
 	const Qfloat *Q_i = NULL;
 	if(i != -1) // NULL Q_i not accessed: Gmax=-INF if i=-1
 		Q_i = Q->get_Q(i,active_size);
 
-	for(int j=0;j<active_size;j++)
+	for(int32_t j=0;j<active_size;j++)
 	{
 		if(y[j]==+1)
 		{
 			if (!is_lower_bound(j))
 			{
-				double grad_diff=Gmax+G[j];
+				float64_t grad_diff=Gmax+G[j];
 				if (G[j] >= Gmax2)
 					Gmax2 = G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff; 
-					double quad_coef=Q_i[i]+QD[j]-2*y[i]*Q_i[j];
+					float64_t obj_diff; 
+					float64_t quad_coef=Q_i[i]+QD[j]-2*y[i]*Q_i[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
 					else
@@ -690,13 +695,13 @@ int Solver::select_working_set(int &out_i, int &out_j, double &gap)
 		{
 			if (!is_upper_bound(j))
 			{
-				double grad_diff= Gmax-G[j];
+				float64_t grad_diff= Gmax-G[j];
 				if (-G[j] >= Gmax2)
 					Gmax2 = -G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff; 
-					double quad_coef=Q_i[i]+QD[j]+2*y[i]*Q_i[j];
+					float64_t obj_diff; 
+					float64_t quad_coef=Q_i[i]+QD[j]+2*y[i]*Q_i[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
 					else
@@ -721,7 +726,7 @@ int Solver::select_working_set(int &out_i, int &out_j, double &gap)
 	return 0;
 }
 
-bool Solver::be_shrunken(int i, double Gmax1, double Gmax2)
+bool Solver::be_shrunken(int32_t i, float64_t Gmax1, float64_t Gmax2)
 {
 	if(is_upper_bound(i))
 	{
@@ -743,9 +748,9 @@ bool Solver::be_shrunken(int i, double Gmax1, double Gmax2)
 
 void Solver::do_shrinking()
 {
-	int i;
-	double Gmax1 = -INF;		// max { -y_i * grad(f)_i | i in I_up(\alpha) }
-	double Gmax2 = -INF;		// max { y_i * grad(f)_i | i in I_low(\alpha) }
+	int32_t i;
+	float64_t Gmax1 = -INF;		// max { -y_i * grad(f)_i | i in I_up(\alpha) }
+	float64_t Gmax2 = -INF;		// max { y_i * grad(f)_i | i in I_low(\alpha) }
 
 	// find maximal violating pair first
 	for(i=0;i<active_size;i++)
@@ -818,14 +823,14 @@ void Solver::do_shrinking()
 		}
 }
 
-double Solver::calculate_rho()
+float64_t Solver::calculate_rho()
 {
-	double r;
-	int nr_free = 0;
-	double ub = INF, lb = -INF, sum_free = 0;
-	for(int i=0;i<active_size;i++)
+	float64_t r;
+	int32_t nr_free = 0;
+	float64_t ub = INF, lb = -INF, sum_free = 0;
+	for(int32_t i=0;i<active_size;i++)
 	{
-		double yG = y[i]*G[i];
+		float64_t yG = y[i]*G[i];
 
 		if(is_upper_bound(i))
 		{
@@ -865,42 +870,46 @@ class Solver_NU : public Solver
 {
 public:
 	Solver_NU() {}
-	void Solve(int p_l, const QMatrix& p_Q, const double *p_p, const schar *p_y,
-		   double* p_alpha, double p_Cp, double p_Cn, double p_eps,
-		   SolutionInfo* p_si, int shrinking)
+	void Solve(
+		int32_t p_l, const QMatrix& p_Q, const float64_t *p_p,
+		const schar *p_y, float64_t* p_alpha, float64_t p_Cp, float64_t p_Cn,
+		float64_t p_eps, SolutionInfo* p_si, int32_t shrinking)
 	{
 		this->si = p_si;
 		Solver::Solve(p_l,p_Q,p,p_y,p_alpha,p_Cp,p_Cn,p_eps,p_si,shrinking);
 	}
 private:
 	SolutionInfo *si;
-	int select_working_set(int &i, int &j, double &gap);
-	double calculate_rho();
-	bool be_shrunken(int i, double Gmax1, double Gmax2, double Gmax3, double Gmax4);
+	int32_t select_working_set(int32_t &i, int32_t &j, float64_t &gap);
+	float64_t calculate_rho();
+	bool be_shrunken(
+		int32_t i, float64_t Gmax1, float64_t Gmax2, float64_t Gmax3,
+		float64_t Gmax4);
 	void do_shrinking();
 };
 
 // return 1 if already optimal, return 0 otherwise
-int Solver_NU::select_working_set(int &out_i, int &out_j, double &gap)
+int32_t Solver_NU::select_working_set(
+	int32_t &out_i, int32_t &out_j, float64_t &gap)
 {
 	// return i,j such that y_i = y_j and
 	// i: maximizes -y_i * grad(f)_i, i in I_up(\alpha)
 	// j: minimizes the decrease of obj value
-	//    (if quadratic coefficeint <= 0, replace it with tau)
+	//    (if quadratic coefficient <= 0, replace it with tau)
 	//    -y_j*grad(f)_j < -y_i*grad(f)_i, j in I_low(\alpha)
 
-	double Gmaxp = -INF;
-	double Gmaxp2 = -INF;
-	int Gmaxp_idx = -1;
+	float64_t Gmaxp = -INF;
+	float64_t Gmaxp2 = -INF;
+	int32_t Gmaxp_idx = -1;
 
-	double Gmaxn = -INF;
-	double Gmaxn2 = -INF;
-	int Gmaxn_idx = -1;
+	float64_t Gmaxn = -INF;
+	float64_t Gmaxn2 = -INF;
+	int32_t Gmaxn_idx = -1;
 
-	int Gmin_idx = -1;
-	double obj_diff_min = INF;
+	int32_t Gmin_idx = -1;
+	float64_t obj_diff_min = INF;
 
-	for(int t=0;t<active_size;t++)
+	for(int32_t t=0;t<active_size;t++)
 		if(y[t]==+1)
 		{
 			if(!is_upper_bound(t))
@@ -920,8 +929,8 @@ int Solver_NU::select_working_set(int &out_i, int &out_j, double &gap)
 				}
 		}
 
-	int ip = Gmaxp_idx;
-	int in = Gmaxn_idx;
+	int32_t ip = Gmaxp_idx;
+	int32_t in = Gmaxn_idx;
 	const Qfloat *Q_ip = NULL;
 	const Qfloat *Q_in = NULL;
 	if(ip != -1) // NULL Q_ip not accessed: Gmaxp=-INF if ip=-1
@@ -929,19 +938,19 @@ int Solver_NU::select_working_set(int &out_i, int &out_j, double &gap)
 	if(in != -1)
 		Q_in = Q->get_Q(in,active_size);
 
-	for(int j=0;j<active_size;j++)
+	for(int32_t j=0;j<active_size;j++)
 	{
 		if(y[j]==+1)
 		{
 			if (!is_lower_bound(j))	
 			{
-				double grad_diff=Gmaxp+G[j];
+				float64_t grad_diff=Gmaxp+G[j];
 				if (G[j] >= Gmaxp2)
 					Gmaxp2 = G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff; 
-					double quad_coef = Q_ip[ip]+QD[j]-2*Q_ip[j];
+					float64_t obj_diff; 
+					float64_t quad_coef = Q_ip[ip]+QD[j]-2*Q_ip[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
 					else
@@ -959,13 +968,13 @@ int Solver_NU::select_working_set(int &out_i, int &out_j, double &gap)
 		{
 			if (!is_upper_bound(j))
 			{
-				double grad_diff=Gmaxn-G[j];
+				float64_t grad_diff=Gmaxn-G[j];
 				if (-G[j] >= Gmaxn2)
 					Gmaxn2 = -G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff; 
-					double quad_coef = Q_in[in]+QD[j]-2*Q_in[j];
+					float64_t obj_diff; 
+					float64_t quad_coef = Q_in[in]+QD[j]-2*Q_in[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
 					else
@@ -994,7 +1003,9 @@ int Solver_NU::select_working_set(int &out_i, int &out_j, double &gap)
 	return 0;
 }
 
-bool Solver_NU::be_shrunken(int i, double Gmax1, double Gmax2, double Gmax3, double Gmax4)
+bool Solver_NU::be_shrunken(
+	int32_t i, float64_t Gmax1, float64_t Gmax2, float64_t Gmax3,
+	float64_t Gmax4)
 {
 	if(is_upper_bound(i))
 	{
@@ -1016,13 +1027,13 @@ bool Solver_NU::be_shrunken(int i, double Gmax1, double Gmax2, double Gmax3, dou
 
 void Solver_NU::do_shrinking()
 {
-	double Gmax1 = -INF;	// max { -y_i * grad(f)_i | y_i = +1, i in I_up(\alpha) }
-	double Gmax2 = -INF;	// max { y_i * grad(f)_i | y_i = +1, i in I_low(\alpha) }
-	double Gmax3 = -INF;	// max { -y_i * grad(f)_i | y_i = -1, i in I_up(\alpha) }
-	double Gmax4 = -INF;	// max { y_i * grad(f)_i | y_i = -1, i in I_low(\alpha) }
+	float64_t Gmax1 = -INF;	// max { -y_i * grad(f)_i | y_i = +1, i in I_up(\alpha) }
+	float64_t Gmax2 = -INF;	// max { y_i * grad(f)_i | y_i = +1, i in I_low(\alpha) }
+	float64_t Gmax3 = -INF;	// max { -y_i * grad(f)_i | y_i = -1, i in I_up(\alpha) }
+	float64_t Gmax4 = -INF;	// max { y_i * grad(f)_i | y_i = -1, i in I_low(\alpha) }
 
 	// find maximal violating pair first
-	int i;
+	int32_t i;
 	for(i=0;i<active_size;i++)
 	{
 		if(!is_upper_bound(i))
@@ -1083,14 +1094,14 @@ void Solver_NU::do_shrinking()
 		}
 }
 
-double Solver_NU::calculate_rho()
+float64_t Solver_NU::calculate_rho()
 {
-	int nr_free1 = 0,nr_free2 = 0;
-	double ub1 = INF, ub2 = INF;
-	double lb1 = -INF, lb2 = -INF;
-	double sum_free1 = 0, sum_free2 = 0;
+	int32_t nr_free1 = 0,nr_free2 = 0;
+	float64_t ub1 = INF, ub2 = INF;
+	float64_t lb1 = -INF, lb2 = -INF;
+	float64_t sum_free1 = 0, sum_free2 = 0;
 
-	for(int i=0;i<active_size;i++)
+	for(int32_t i=0; i<active_size; i++)
 	{
 		if(y[i]==+1)
 		{
@@ -1118,7 +1129,7 @@ double Solver_NU::calculate_rho()
 		}
 	}
 
-	double r1,r2;
+	float64_t r1,r2;
 	if(nr_free1 > 0)
 		r1 = sum_free1/nr_free1;
 	else
@@ -1143,19 +1154,19 @@ public:
 	:Kernel(prob.l, prob.x, param)
 	{
 		clone(y,y_,prob.l);
-		cache = new Cache(prob.l,(long int)(param.cache_size*(1l<<20)));
+		cache = new Cache(prob.l,(int64_t)(param.cache_size*(1l<<20)));
 		QD = new Qfloat[prob.l];
-		for(int i=0;i<prob.l;i++)
+		for(int32_t i=0;i<prob.l;i++)
 			QD[i]= (Qfloat)kernel_function(i,i);
 	}
 	
-	Qfloat *get_Q(int i, int len) const
+	Qfloat *get_Q(int32_t i, int32_t len) const
 	{
 		Qfloat *data;
-		int start;
+		int32_t start;
 		if((start = cache->get_data(i,&data,len)) < len)
 		{
-			for(int j=start;j<len;j++)
+			for(int32_t j=start;j<len;j++)
 				data[j] = (Qfloat) y[i]*y[j]*kernel_function(i,j);
 		}
 		return data;
@@ -1166,7 +1177,7 @@ public:
 		return QD;
 	}
 
-	void swap_index(int i, int j) const
+	void swap_index(int32_t i, int32_t j) const
 	{
 		cache->swap_index(i,j);
 		Kernel::swap_index(i,j);
@@ -1192,19 +1203,19 @@ public:
 	ONE_CLASS_Q(const svm_problem& prob, const svm_parameter& param)
 	:Kernel(prob.l, prob.x, param)
 	{
-		cache = new Cache(prob.l,(long int)(param.cache_size*(1l<<20)));
+		cache = new Cache(prob.l,(int64_t)(param.cache_size*(1l<<20)));
 		QD = new Qfloat[prob.l];
-		for(int i=0;i<prob.l;i++)
+		for(int32_t i=0;i<prob.l;i++)
 			QD[i]= (Qfloat)kernel_function(i,i);
 	}
 	
-	Qfloat *get_Q(int i, int len) const
+	Qfloat *get_Q(int32_t i, int32_t len) const
 	{
 		Qfloat *data;
-		int start;
+		int32_t start;
 		if((start = cache->get_data(i,&data,len)) < len)
 		{
-			for(int j=start;j<len;j++)
+			for(int32_t j=start;j<len;j++)
 				data[j] = (Qfloat) kernel_function(i,j);
 		}
 		return data;
@@ -1215,7 +1226,7 @@ public:
 		return QD;
 	}
 
-	void swap_index(int i, int j) const
+	void swap_index(int32_t i, int32_t j) const
 	{
 		cache->swap_index(i,j);
 		Kernel::swap_index(i,j);
@@ -1239,11 +1250,11 @@ public:
 	:Kernel(prob.l, prob.x, param)
 	{
 		l = prob.l;
-		cache = new Cache(l,(long int)(param.cache_size*(1l<<20)));
+		cache = new Cache(l,(int64_t)(param.cache_size*(1l<<20)));
 		QD = new Qfloat[2*l];
 		sign = new schar[2*l];
-		index = new int[2*l];
-		for(int k=0;k<l;k++)
+		index = new int32_t[2*l];
+		for(int32_t k=0;k<l;k++)
 		{
 			sign[k] = 1;
 			sign[k+l] = -1;
@@ -1257,20 +1268,20 @@ public:
 		next_buffer = 0;
 	}
 
-	void swap_index(int i, int j) const
+	void swap_index(int32_t i, int32_t j) const
 	{
 		swap(sign[i],sign[j]);
 		swap(index[i],index[j]);
 		swap(QD[i],QD[j]);
 	}
 	
-	Qfloat *get_Q(int i, int len) const
+	Qfloat *get_Q(int32_t i, int32_t len) const
 	{
 		Qfloat *data;
-		int real_i = index[i];
+		int32_t real_i = index[i];
 		if(cache->get_data(real_i,&data,l) < l)
 		{
-			for(int j=0;j<l;j++)
+			for(int32_t j=0;j<l;j++)
 				data[j] = (Qfloat)kernel_function(real_i,j);
 		}
 
@@ -1278,7 +1289,7 @@ public:
 		Qfloat *buf = buffer[next_buffer];
 		next_buffer = 1 - next_buffer;
 		schar si = sign[i];
-		for(int j=0;j<len;j++)
+		for(int32_t j=0;j<len;j++)
 			buf[j] = si * sign[j] * data[index[j]];
 		return buf;
 	}
@@ -1297,12 +1308,13 @@ public:
 		delete[] buffer[1];
 		delete[] QD;
 	}
+
 private:
-	int l;
+	int32_t l;
 	Cache *cache;
 	schar *sign;
-	int *index;
-	mutable int next_buffer;
+	int32_t *index;
+	mutable int32_t next_buffer;
 	Qfloat *buffer[2];
 	Qfloat *QD;
 };
@@ -1312,13 +1324,13 @@ private:
 //
 static void solve_c_svc(
 	const svm_problem *prob, const svm_parameter* param,
-	double *alpha, Solver::SolutionInfo* si, double Cp, double Cn)
+	float64_t *alpha, Solver::SolutionInfo* si, float64_t Cp, float64_t Cn)
 {
-	int l = prob->l;
-	double *minus_ones = new double[l];
+	int32_t l = prob->l;
+	float64_t *minus_ones = new float64_t[l];
 	schar *y = new schar[l];
 
-	int i;
+	int32_t i;
 
 	for(i=0;i<l;i++)
 	{
@@ -1331,7 +1343,7 @@ static void solve_c_svc(
 	s.Solve(l, SVC_Q(*prob,*param,y), minus_ones, y,
 		alpha, Cp, Cn, param->eps, si, param->shrinking);
 
-	double sum_alpha=0;
+	float64_t sum_alpha=0;
 	for(i=0;i<l;i++)
 		sum_alpha += alpha[i];
 
@@ -1347,11 +1359,11 @@ static void solve_c_svc(
 
 static void solve_nu_svc(
 	const svm_problem *prob, const svm_parameter *param,
-	double *alpha, Solver::SolutionInfo* si)
+	float64_t *alpha, Solver::SolutionInfo* si)
 {
-	int i;
-	int l = prob->l;
-	double nu = param->nu;
+	int32_t i;
+	int32_t l = prob->l;
+	float64_t nu = param->nu;
 
 	schar *y = new schar[l];
 
@@ -1361,8 +1373,8 @@ static void solve_nu_svc(
 		else
 			y[i] = -1;
 
-	double sum_pos = nu*l/2;
-	double sum_neg = nu*l/2;
+	float64_t sum_pos = nu*l/2;
+	float64_t sum_neg = nu*l/2;
 
 	for(i=0;i<l;i++)
 		if(y[i] == +1)
@@ -1376,7 +1388,7 @@ static void solve_nu_svc(
 			sum_neg -= alpha[i];
 		}
 
-	double *zeros = new double[l];
+	float64_t *zeros = new float64_t[l];
 
 	for(i=0;i<l;i++)
 		zeros[i] = 0;
@@ -1384,7 +1396,7 @@ static void solve_nu_svc(
 	Solver_NU s;
 	s.Solve(l, SVC_Q(*prob,*param,y), zeros, y,
 		alpha, 1.0, 1.0, param->eps, si,  param->shrinking);
-	double r = si->r;
+	float64_t r = si->r;
 
 	SG_SINFO("C = %f\n",1/r);
 
@@ -1402,14 +1414,14 @@ static void solve_nu_svc(
 
 static void solve_one_class(
 	const svm_problem *prob, const svm_parameter *param,
-	double *alpha, Solver::SolutionInfo* si)
+	float64_t *alpha, Solver::SolutionInfo* si)
 {
-	int l = prob->l;
-	double *zeros = new double[l];
+	int32_t l = prob->l;
+	float64_t *zeros = new float64_t[l];
 	schar *ones = new schar[l];
-	int i;
+	int32_t i;
 
-	int n = (int)(param->nu*prob->l);	// # of alpha's at upper bound
+	int32_t n = (int32_t)(param->nu*prob->l);	// # of alpha's at upper bound
 
 	for(i=0;i<n;i++)
 		alpha[i] = 1;
@@ -1434,13 +1446,13 @@ static void solve_one_class(
 
 static void solve_epsilon_svr(
 	const svm_problem *prob, const svm_parameter *param,
-	double *alpha, Solver::SolutionInfo* si)
+	float64_t *alpha, Solver::SolutionInfo* si)
 {
-	int l = prob->l;
-	double *alpha2 = new double[2*l];
-	double *linear_term = new double[2*l];
+	int32_t l = prob->l;
+	float64_t *alpha2 = new float64_t[2*l];
+	float64_t *linear_term = new float64_t[2*l];
 	schar *y = new schar[2*l];
-	int i;
+	int32_t i;
 
 	for(i=0;i<l;i++)
 	{
@@ -1457,7 +1469,7 @@ static void solve_epsilon_svr(
 	s.Solve(2*l, SVR_Q(*prob,*param), linear_term, y,
 		alpha2, param->C, param->C, param->eps, si, param->shrinking);
 
-	double sum_alpha = 0;
+	float64_t sum_alpha = 0;
 	for(i=0;i<l;i++)
 	{
 		alpha[i] = alpha2[i] - alpha2[i+l];
@@ -1472,16 +1484,16 @@ static void solve_epsilon_svr(
 
 static void solve_nu_svr(
 	const svm_problem *prob, const svm_parameter *param,
-	double *alpha, Solver::SolutionInfo* si)
+	float64_t *alpha, Solver::SolutionInfo* si)
 {
-	int l = prob->l;
-	double C = param->C;
-	double *alpha2 = new double[2*l];
-	double *linear_term = new double[2*l];
+	int32_t l = prob->l;
+	float64_t C = param->C;
+	float64_t *alpha2 = new float64_t[2*l];
+	float64_t *linear_term = new float64_t[2*l];
 	schar *y = new schar[2*l];
-	int i;
+	int32_t i;
 
-	double sum = C * param->nu * l / 2;
+	float64_t sum = C * param->nu * l / 2;
 	for(i=0;i<l;i++)
 	{
 		alpha2[i] = alpha2[i+l] = min(sum,C);
@@ -1513,16 +1525,16 @@ static void solve_nu_svr(
 //
 struct decision_function
 {
-	double *alpha;
-	double rho;	
-	double objective;
+	float64_t *alpha;
+	float64_t rho;	
+	float64_t objective;
 };
 
 decision_function svm_train_one(
 	const svm_problem *prob, const svm_parameter *param,
-	double Cp, double Cn)
+	float64_t Cp, float64_t Cn)
 {
-	double *alpha = Malloc(double,prob->l);
+	float64_t *alpha = Malloc(float64_t, prob->l);
 	Solver::SolutionInfo si;
 	switch(param->svm_type)
 	{
@@ -1548,9 +1560,9 @@ decision_function svm_train_one(
 	// output SVs
 	if (param->svm_type != ONE_CLASS)
 	{
-		int nSV = 0;
-		int nBSV = 0;
-		for(int i=0;i<prob->l;i++)
+		int32_t nSV = 0;
+		int32_t nBSV = 0;
+		for(int32_t i=0;i<prob->l;i++)
 		{
 			if(fabs(alpha[i]) > 0)
 			{
@@ -1580,20 +1592,22 @@ decision_function svm_train_one(
 //
 // label: label name, start: begin of each class, count: #data of classes, perm: indices to the original data
 // perm, length l, must be allocated before calling this subroutine
-void svm_group_classes(const svm_problem *prob, int *nr_class_ret, int **label_ret, int **start_ret, int **count_ret, int *perm)
+void svm_group_classes(
+	const svm_problem *prob, int32_t *nr_class_ret, int32_t **label_ret,
+	int32_t **start_ret, int32_t **count_ret, int32_t *perm)
 {
-	int l = prob->l;
-	int max_nr_class = 16;
-	int nr_class = 0;
-	int *label = Malloc(int,max_nr_class);
-	int *count = Malloc(int,max_nr_class);
-	int *data_label = Malloc(int,l);	
-	int i;
+	int32_t l = prob->l;
+	int32_t max_nr_class = 16;
+	int32_t nr_class = 0;
+	int32_t *label = Malloc(int32_t, max_nr_class);
+	int32_t *count = Malloc(int32_t, max_nr_class);
+	int32_t *data_label = Malloc(int32_t, l);
+	int32_t i;
 
 	for(i=0;i<l;i++)
 	{
-		int this_label = (int)prob->y[i];
-		int j;
+		int32_t this_label=(int32_t) prob->y[i];
+		int32_t j;
 		for(j=0;j<nr_class;j++)
 		{
 			if(this_label == label[j])
@@ -1608,8 +1622,8 @@ void svm_group_classes(const svm_problem *prob, int *nr_class_ret, int **label_r
 			if(nr_class == max_nr_class)
 			{
 				max_nr_class *= 2;
-				label = (int *)realloc(label,max_nr_class*sizeof(int));
-				count = (int *)realloc(count,max_nr_class*sizeof(int));
+				label=(int32_t *) realloc(label,max_nr_class*sizeof(int32_t));
+				count=(int32_t *) realloc(count,max_nr_class*sizeof(int32_t));
 			}
 			label[nr_class] = this_label;
 			count[nr_class] = 1;
@@ -1617,7 +1631,7 @@ void svm_group_classes(const svm_problem *prob, int *nr_class_ret, int **label_r
 		}
 	}
 
-	int *start = Malloc(int,nr_class);
+	int32_t *start = Malloc(int32_t, nr_class);
 	start[0] = 0;
 	for(i=1;i<nr_class;i++)
 		start[i] = start[i-1]+count[i-1];
@@ -1656,20 +1670,20 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		model->nr_class = 2;
 		model->label = NULL;
 		model->nSV = NULL;
-		model->sv_coef = Malloc(double *,1);
+		model->sv_coef = Malloc(float64_t *,1);
 		decision_function f = svm_train_one(prob,param,0,0);
-		model->rho = Malloc(double,1);
+		model->rho = Malloc(float64_t, 1);
 		model->rho[0] = f.rho;
 		model->objective = f.objective;
 
-		int nSV = 0;
-		int i;
+		int32_t nSV = 0;
+		int32_t i;
 		for(i=0;i<prob->l;i++)
 			if(fabs(f.alpha[i]) > 0) ++nSV;
 		model->l = nSV;
 		model->SV = Malloc(svm_node *,nSV);
-		model->sv_coef[0] = Malloc(double,nSV);
-		int j = 0;
+		model->sv_coef[0] = Malloc(float64_t, nSV);
+		int32_t j = 0;
 		for(i=0;i<prob->l;i++)
 			if(fabs(f.alpha[i]) > 0)
 			{
@@ -1683,28 +1697,28 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 	else
 	{
 		// classification
-		int l = prob->l;
-		int nr_class;
-		int *label = NULL;
-		int *start = NULL;
-		int *count = NULL;
-		int *perm = Malloc(int,l);
+		int32_t l = prob->l;
+		int32_t nr_class;
+		int32_t *label = NULL;
+		int32_t *start = NULL;
+		int32_t *count = NULL;
+		int32_t *perm = Malloc(int32_t, l);
 
 		// group training data of the same class
-		svm_group_classes(prob,&nr_class,&label,&start,&count,perm);		
+		svm_group_classes(prob,&nr_class,&label,&start,&count,perm);
 		svm_node **x = Malloc(svm_node *,l);
-		int i;
+		int32_t i;
 		for(i=0;i<l;i++)
 			x[i] = prob->x[perm[i]];
 
 		// calculate weighted C
 
-		double *weighted_C = Malloc(double, nr_class);
+		float64_t *weighted_C = Malloc(float64_t,  nr_class);
 		for(i=0;i<nr_class;i++)
 			weighted_C[i] = param->C;
 		for(i=0;i<param->nr_weight;i++)
-		{	
-			int j;
+		{
+			int32_t j;
 			for(j=0;j<nr_class;j++)
 				if(param->weight_label[i] == label[j])
 					break;
@@ -1721,18 +1735,18 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 			nonzero[i] = false;
 		decision_function *f = Malloc(decision_function,nr_class*(nr_class-1)/2);
 
-		int p = 0;
+		int32_t p = 0;
 		for(i=0;i<nr_class;i++)
-			for(int j=i+1;j<nr_class;j++)
+			for(int32_t j=i+1;j<nr_class;j++)
 			{
 				svm_problem sub_prob;
-				int si = start[i], sj = start[j];
-				int ci = count[i], cj = count[j];
+				int32_t si = start[i], sj = start[j];
+				int32_t ci = count[i], cj = count[j];
 				sub_prob.l = ci+cj;
 				sub_prob.x = Malloc(svm_node *,sub_prob.l);
-				sub_prob.y = Malloc(double,sub_prob.l+1); //dirty hack to surpress valgrind err
+				sub_prob.y = Malloc(float64_t, sub_prob.l+1); //dirty hack to surpress valgrind err
 
-				int k;
+				int32_t k;
 				for(k=0;k<ci;k++)
 				{
 					sub_prob.x[k] = x[si+k];
@@ -1762,21 +1776,21 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		model->objective = f[0].objective;
 		model->nr_class = nr_class;
 		
-		model->label = Malloc(int,nr_class);
+		model->label = Malloc(int32_t, nr_class);
 		for(i=0;i<nr_class;i++)
 			model->label[i] = label[i];
 		
-		model->rho = Malloc(double,nr_class*(nr_class-1)/2);
+		model->rho = Malloc(float64_t, nr_class*(nr_class-1)/2);
 		for(i=0;i<nr_class*(nr_class-1)/2;i++)
 			model->rho[i] = f[i].rho;
 
-		int total_sv = 0;
-		int *nz_count = Malloc(int,nr_class);
-		model->nSV = Malloc(int,nr_class);
+		int32_t total_sv = 0;
+		int32_t *nz_count = Malloc(int32_t, nr_class);
+		model->nSV = Malloc(int32_t, nr_class);
 		for(i=0;i<nr_class;i++)
 		{
-			int nSV = 0;
-			for(int j=0;j<count[i];j++)
+			int32_t nSV = 0;
+			for(int32_t j=0;j<count[i];j++)
 				if(nonzero[start[i]+j])
 				{	
 					++nSV;
@@ -1794,30 +1808,30 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		for(i=0;i<l;i++)
 			if(nonzero[i]) model->SV[p++] = x[i];
 
-		int *nz_start = Malloc(int,nr_class);
+		int32_t *nz_start = Malloc(int32_t, nr_class);
 		nz_start[0] = 0;
 		for(i=1;i<nr_class;i++)
 			nz_start[i] = nz_start[i-1]+nz_count[i-1];
 
-		model->sv_coef = Malloc(double *,nr_class-1);
+		model->sv_coef = Malloc(float64_t *,nr_class-1);
 		for(i=0;i<nr_class-1;i++)
-			model->sv_coef[i] = Malloc(double,total_sv);
+			model->sv_coef[i] = Malloc(float64_t, total_sv);
 
 		p = 0;
 		for(i=0;i<nr_class;i++)
-			for(int j=i+1;j<nr_class;j++)
+			for(int32_t j=i+1;j<nr_class;j++)
 			{
 				// classifier (i,j): coefficients with
 				// i are in sv_coef[j-1][nz_start[i]...],
 				// j are in sv_coef[i][nz_start[j]...]
 
-				int si = start[i];
-				int sj = start[j];
-				int ci = count[i];
-				int cj = count[j];
-				
-				int q = nz_start[i];
-				int k;
+				int32_t si = start[i];
+				int32_t sj = start[j];
+				int32_t ci = count[i];
+				int32_t cj = count[j];
+
+				int32_t q = nz_start[i];
+				int32_t k;
 				for(k=0;k<ci;k++)
 					if(nonzero[si+k])
 						model->sv_coef[j-1][q++] = f[p].alpha[k];
@@ -1858,7 +1872,7 @@ void svm_destroy_model(svm_model* model)
 {
 	if(model->free_sv && model->l > 0)
 		free((void *)(model->SV[0]));
-	for(int i=0;i<model->nr_class-1;i++)
+	for(int32_t i=0;i<model->nr_class-1;i++)
 		free(model->sv_coef[i]);
 	free(model->SV);
 	free(model->sv_coef);
@@ -1874,21 +1888,22 @@ void svm_destroy_param(svm_parameter* param)
 	free(param->weight);
 }
 
-const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *param)
+const char *svm_check_parameter(
+	const svm_problem *prob, const svm_parameter *param)
 {
 	// svm_type
 
-	int svm_type = param->svm_type;
+	int32_t svm_type = param->svm_type;
 	if(svm_type != C_SVC &&
 	   svm_type != NU_SVC &&
 	   svm_type != ONE_CLASS &&
 	   svm_type != EPSILON_SVR &&
 	   svm_type != NU_SVR)
 		return "unknown svm type";
-	
+
 	// kernel_type, degree
-	
-	int kernel_type = param->kernel_type;
+
+	int32_t kernel_type = param->kernel_type;
 	if(kernel_type != LINEAR &&
 	   kernel_type != POLY &&
 	   kernel_type != RBF &&
@@ -1932,17 +1947,17 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 	
 	if(svm_type == NU_SVC)
 	{
-		int l = prob->l;
-		int max_nr_class = 16;
-		int nr_class = 0;
-		int *label = Malloc(int,max_nr_class);
-		int *count = Malloc(int,max_nr_class);
+		int32_t l = prob->l;
+		int32_t max_nr_class = 16;
+		int32_t nr_class = 0;
+		int32_t *label = Malloc(int32_t, max_nr_class);
+		int32_t *count = Malloc(int32_t, max_nr_class);
 
-		int i;
+		int32_t i;
 		for(i=0;i<l;i++)
 		{
-			int this_label = (int)prob->y[i];
-			int j;
+			int32_t this_label = (int32_t) prob->y[i];
+			int32_t j;
 			for(j=0;j<nr_class;j++)
 				if(this_label == label[j])
 				{
@@ -1954,8 +1969,10 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 				if(nr_class == max_nr_class)
 				{
 					max_nr_class *= 2;
-					label = (int *)realloc(label,max_nr_class*sizeof(int));
-					count = (int *)realloc(count,max_nr_class*sizeof(int));
+					label=(int32_t *) realloc(label,
+						max_nr_class*sizeof(int32_t));
+					count=(int32_t *) realloc(count,
+						max_nr_class*sizeof(int32_t));
 				}
 				label[nr_class] = this_label;
 				count[nr_class] = 1;
@@ -1965,10 +1982,10 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 	
 		for(i=0;i<nr_class;i++)
 		{
-			int n1 = count[i];
-			for(int j=i+1;j<nr_class;j++)
+			int32_t n1 = count[i];
+			for(int32_t j=i+1;j<nr_class;j++)
 			{
-				int n2 = count[j];
+				int32_t n2 = count[j];
 				if(param->nu*(n1+n2)/2 > min(n1,n2))
 				{
 					free(label);
