@@ -10,18 +10,18 @@
 #include "classifier/svm/SVM_linear.h"
 #include "classifier/svm/Tron.h"
 
-l2_lr_fun::l2_lr_fun(const problem *p, double Cp, double Cn)
+l2_lr_fun::l2_lr_fun(const problem *p, float64_t Cp, float64_t Cn)
 : function()
 {
-	int i;
-	int l=p->l;
-	int *y=p->y;
+	int32_t i;
+	int32_t l=p->l;
+	int32_t *y=p->y;
 
 	this->prob = p;
 
-	z = new double[l];
-	D = new double[l];
-	C = new double[l];
+	z = new float64_t[l];
+	D = new float64_t[l];
+	C = new float64_t[l];
 
 	for (i=0; i<l; i++)
 	{
@@ -40,18 +40,18 @@ l2_lr_fun::~l2_lr_fun()
 }
 
 
-double l2_lr_fun::fun(double *w)
+float64_t l2_lr_fun::fun(float64_t *w)
 {
-	int i;
-	double f=0;
-	int *y=prob->y;
-	int l=prob->l;
-	int n=prob->n;
+	int32_t i;
+	float64_t f=0;
+	int32_t *y=prob->y;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
 
 	Xv(w, z);
 	for(i=0;i<l;i++)
 	{
-	        double yz = y[i]*z[i];
+	        float64_t yz = y[i]*z[i];
 		if (yz >= 0)
 		        f += C[i]*log(1 + exp(-yz));
 		else
@@ -65,12 +65,12 @@ double l2_lr_fun::fun(double *w)
 	return(f);
 }
 
-void l2_lr_fun::grad(double *w, double *g)
+void l2_lr_fun::grad(float64_t *w, float64_t *g)
 {
-	int i;
-	int *y=prob->y;
-	int l=prob->l;
-	int n=prob->n;
+	int32_t i;
+	int32_t *y=prob->y;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
 
 	for(i=0;i<l;i++)
 	{
@@ -84,17 +84,17 @@ void l2_lr_fun::grad(double *w, double *g)
 		g[i] = w[i] + g[i];
 }
 
-int l2_lr_fun::get_nr_variable(void)
+int32_t l2_lr_fun::get_nr_variable(void)
 {
 	return prob->n;
 }
 
-void l2_lr_fun::Hv(double *s, double *Hs)
+void l2_lr_fun::Hv(float64_t *s, float64_t *Hs)
 {
-	int i;
-	int l=prob->l;
-	int n=prob->n;
-	double *wa = new double[l];
+	int32_t i;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
+	float64_t *wa = new float64_t[l];
 
 	Xv(s, wa);
 	for(i=0;i<l;i++)
@@ -106,15 +106,15 @@ void l2_lr_fun::Hv(double *s, double *Hs)
 	delete[] wa;
 }
 
-void l2_lr_fun::Xv(double *v, double *res_Xv)
+void l2_lr_fun::Xv(float64_t *v, float64_t *res_Xv)
 {
-	int l=prob->l;
-	int n=prob->n;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
 
 	if (prob->use_bias)
 		n--;
 
-	for (int i=0;i<l;i++)
+	for (int32_t i=0;i<l;i++)
 	{
 		res_Xv[i]=prob->x->dense_dot(1.0, i, v, n, 0);
 
@@ -123,17 +123,17 @@ void l2_lr_fun::Xv(double *v, double *res_Xv)
 	}
 }
 
-void l2_lr_fun::XTv(double *v, double *res_XTv)
+void l2_lr_fun::XTv(float64_t *v, float64_t *res_XTv)
 {
-	int l=prob->l;
-	int n=prob->n;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
 
 	if (prob->use_bias)
 		n--;
 
-	memset(res_XTv, 0, sizeof(double)*prob->n);
+	memset(res_XTv, 0, sizeof(float64_t)*prob->n);
 
-	for (int i=0;i<l;i++)
+	for (int32_t i=0;i<l;i++)
 	{
 		prob->x->add_to_dense_vec(v[i], i, res_XTv, n);
 
@@ -142,19 +142,19 @@ void l2_lr_fun::XTv(double *v, double *res_XTv)
 	}
 }
 
-l2loss_svm_fun::l2loss_svm_fun(const problem *p, double Cp, double Cn)
+l2loss_svm_fun::l2loss_svm_fun(const problem *p, float64_t Cp, float64_t Cn)
 : function()
 {
-	int i;
-	int l=p->l;
-	int *y=p->y;
+	int32_t i;
+	int32_t l=p->l;
+	int32_t *y=p->y;
 
 	this->prob = p;
 
-	z = new double[l];
-	D = new double[l];
-	C = new double[l];
-	I = new int[l];
+	z = new float64_t[l];
+	D = new float64_t[l];
+	C = new float64_t[l];
+	I = new int32_t[l];
 
 	for (i=0; i<l; i++)
 	{
@@ -173,19 +173,19 @@ l2loss_svm_fun::~l2loss_svm_fun()
 	delete[] I;
 }
 
-double l2loss_svm_fun::fun(double *w)
+float64_t l2loss_svm_fun::fun(float64_t *w)
 {
-	int i;
-	double f=0;
-	int *y=prob->y;
-	int l=prob->l;
-	int n=prob->n;
+	int32_t i;
+	float64_t f=0;
+	int32_t *y=prob->y;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
 
 	Xv(w, z);
 	for(i=0;i<l;i++)
 	{
 	        z[i] = y[i]*z[i];
-		double d = z[i]-1;
+		float64_t d = z[i]-1;
 		if (d < 0)
 			f += C[i]*d*d;
 	}
@@ -197,12 +197,12 @@ double l2loss_svm_fun::fun(double *w)
 	return(f);
 }
 
-void l2loss_svm_fun::grad(double *w, double *g)
+void l2loss_svm_fun::grad(float64_t *w, float64_t *g)
 {
-	int i;
-	int *y=prob->y;
-	int l=prob->l;
-	int n=prob->n;
+	int32_t i;
+	int32_t *y=prob->y;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
 
 	sizeI = 0;
 	for (i=0;i<l;i++)
@@ -218,17 +218,17 @@ void l2loss_svm_fun::grad(double *w, double *g)
 		g[i] = w[i] + 2*g[i];
 }
 
-int l2loss_svm_fun::get_nr_variable(void)
+int32_t l2loss_svm_fun::get_nr_variable(void)
 {
 	return prob->n;
 }
 
-void l2loss_svm_fun::Hv(double *s, double *Hs)
+void l2loss_svm_fun::Hv(float64_t *s, float64_t *Hs)
 {
-	int i;
-	int l=prob->l;
-	int n=prob->n;
-	double *wa = new double[l];
+	int32_t i;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
+	float64_t *wa = new float64_t[l];
 
 	subXv(s, wa);
 	for(i=0;i<sizeI;i++)
@@ -240,15 +240,15 @@ void l2loss_svm_fun::Hv(double *s, double *Hs)
 	delete[] wa;
 }
 
-void l2loss_svm_fun::Xv(double *v, double *res_Xv)
+void l2loss_svm_fun::Xv(float64_t *v, float64_t *res_Xv)
 {
-	int l=prob->l;
-	int n=prob->n;
+	int32_t l=prob->l;
+	int32_t n=prob->n;
 
 	if (prob->use_bias)
 		n--;
 
-	for(int i=0;i<l;i++)
+	for (int32_t i=0;i<l;i++)
 	{
 		res_Xv[i]=prob->x->dense_dot(1.0, i, v, n, 0);
 
@@ -257,14 +257,14 @@ void l2loss_svm_fun::Xv(double *v, double *res_Xv)
 	}
 }
 
-void l2loss_svm_fun::subXv(double *v, double *res_Xv)
+void l2loss_svm_fun::subXv(float64_t *v, float64_t *res_Xv)
 {
-	int n=prob->n;
+	int32_t n=prob->n;
 
 	if (prob->use_bias)
 		n--;
 
-	for(int i=0;i<sizeI;i++)
+	for (int32_t i=0;i<sizeI;i++)
 	{
 		res_Xv[i]=prob->x->dense_dot(1.0, I[i], v, n, 0);
 
@@ -273,15 +273,15 @@ void l2loss_svm_fun::subXv(double *v, double *res_Xv)
 	}
 }
 
-void l2loss_svm_fun::subXTv(double *v, double *XTv)
+void l2loss_svm_fun::subXTv(float64_t *v, float64_t *XTv)
 {
-	int n=prob->n;
+	int32_t n=prob->n;
 
 	if (prob->use_bias)
 		n--;
 
-	memset(XTv, 0, sizeof(double)*prob->n);
-	for(int i=0;i<sizeI;i++)
+	memset(XTv, 0, sizeof(float64_t)*prob->n);
+	for (int32_t i=0;i<sizeI;i++)
 	{
 		prob->x->add_to_dense_vec(v[i], I[i], XTv, n);
 		
