@@ -15,13 +15,13 @@ extern "C" {
 struct problem
 {
 	/** l */
-	INT l;
+	int32_t l;
 	/** n */
-	INT n;
+	int32_t n;
 	/** y */
-	INT *y;
+	int32_t *y;
 	/** sparse features x */
-	CSparseFeatures<DREAL>* x;
+	CSparseFeatures<float64_t>* x;
 	/** if bias shall be used */
 	bool use_bias;
 };
@@ -30,19 +30,19 @@ struct problem
 struct parameter
 {
 	/** solver type */
-	int solver_type;
+	int32_t solver_type;
 
 	/* these are for training only */
 	/** stopping criteria */
-	double eps;
+	float64_t eps;
 	/** C */
-	double C;
+	float64_t C;
 	/** number of weights */
-	int nr_weight;
+	int32_t nr_weight;
 	/** weight label */
-	int *weight_label;
+	int32_t *weight_label;
 	/** weight */
-	double* weight;
+	float64_t* weight;
 };
 
 /** model */
@@ -51,34 +51,41 @@ struct model
 	/** parameter */
 	struct parameter param;
 	/** number of classes */
-	int nr_class;
+	int32_t nr_class;
 	/** number of features */
-	int nr_feature;
+	int32_t nr_feature;
 	/** w */
-	double *w;
+	float64_t *w;
 	/** label of each class (label[n]) */
-	int *label;
+	int32_t *label;
 	/** bias */
-	double bias;
+	float64_t bias;
 };
 
 struct model* train(const struct problem *prob, const struct parameter *param);
-void cross_validation(const struct problem *prob, const struct parameter *param, int nr_fold, int *target);
+void cross_validation(
+	const struct problem *prob, const struct parameter *param, int32_t nr_fold,
+	int32_t *target);
 
-int predict_values(const struct model *model_, const struct feature_node *x, double* dec_values);
-int predict(const struct model *model_, const struct feature_node *x);
-int predict_probability(const struct model *model_, const struct feature_node *x, double* prob_estimates);
+int32_t predict_values(
+	const struct model *model_, const struct feature_node *x,
+	float64_t* dec_values);
+int32_t predict(const struct model *model_, const struct feature_node *x);
+int32_t predict_probability(
+	const struct model *model_, const struct feature_node *x,
+	float64_t* prob_estimates);
 
-int save_model(const char *model_file_name, const struct model *model_);
+int32_t save_model(const char *model_file_name, const struct model *model_);
 struct model *load_model(const char *model_file_name);
 
-int get_nr_feature(const struct model *model_);
-int get_nr_class(const struct model *model_);
-void get_labels(const struct model *model_, int* label);
+int32_t get_nr_feature(const struct model *model_);
+int32_t get_nr_class(const struct model *model_);
+void get_labels(const struct model *model_, int32_t* label);
 
 void destroy_model(struct model *model_);
 void destroy_param(struct parameter *param);
-const char *check_parameter(const struct problem *prob, const struct parameter *param);
+const char *check_parameter(
+	const struct problem *prob, const struct parameter *param);
 
 #ifdef __cplusplus
 }
@@ -94,7 +101,7 @@ public:
 	 * @param Cp Cp
 	 * @param Cn Cn
 	 */
-	l2loss_svm_fun(const problem *prob, double Cp, double Cn);
+	l2loss_svm_fun(const problem *prob, float64_t Cp, float64_t Cn);
 	~l2loss_svm_fun();
 	
 	/** fun
@@ -102,38 +109,38 @@ public:
 	 * @param w w
 	 * @return something floaty
 	 */
-	double fun(double *w);
+	float64_t fun(float64_t *w);
 	
 	/** grad
 	 *
 	 * @param w w
 	 * @param g g
 	 */
-	void grad(double *w, double *g);
+	void grad(float64_t *w, float64_t *g);
 
 	/** Hv
 	 *
 	 * @param s s
 	 * @param Hs Hs
 	 */
-	void Hv(double *s, double *Hs);
+	void Hv(float64_t *s, float64_t *Hs);
 
 	/** get number of variables
 	 *
 	 * @return number of variables
 	 */
-	int get_nr_variable(void);
+	int32_t get_nr_variable(void);
 
 private:
-	void Xv(double *v, double *Xv);
-	void subXv(double *v, double *Xv);
-	void subXTv(double *v, double *XTv);
+	void Xv(float64_t *v, float64_t *Xv);
+	void subXv(float64_t *v, float64_t *Xv);
+	void subXTv(float64_t *v, float64_t *XTv);
 
-	double *C;
-	double *z;
-	double *D;
-	int *I;
-	int sizeI;
+	float64_t *C;
+	float64_t *z;
+	float64_t *D;
+	int32_t *I;
+	int32_t sizeI;
 	const problem *prob;
 };
 
@@ -147,7 +154,7 @@ public:
 	 * @param Cp Cp
 	 * @param Cn Cn
 	 */
-	l2_lr_fun(const problem *prob, double Cp, double Cn);
+	l2_lr_fun(const problem *prob, float64_t Cp, float64_t Cn);
 	~l2_lr_fun();
 
 	/** fun
@@ -155,31 +162,31 @@ public:
 	 * @param w w
 	 * @return something floaty
 	 */
-	double fun(double *w);
+	float64_t fun(float64_t *w);
 	
 	/** grad
 	 *
 	 * @param w w
 	 * @param g g
 	 */
-	void grad(double *w, double *g);
+	void grad(float64_t *w, float64_t *g);
 
 	/** Hv
 	 *
 	 * @param s s
 	 * @param Hs Hs
 	 */
-	void Hv(double *s, double *Hs);
+	void Hv(float64_t *s, float64_t *Hs);
 
-	int get_nr_variable(void);
+	int32_t get_nr_variable(void);
 
 private:
-	void Xv(double *v, double *Xv);
-	void XTv(double *v, double *XTv);
+	void Xv(float64_t *v, float64_t *Xv);
+	void XTv(float64_t *v, float64_t *XTv);
 
-	double *C;
-	double *z;
-	double *D;
+	float64_t *C;
+	float64_t *z;
+	float64_t *D;
 	const problem *prob;
 };
 #endif //HAVE_LAPACK
