@@ -21,8 +21,8 @@
 
 #undef DEBUG_SUBGRADIENTSVM
 
-extern double sparsity;
-double tim;
+extern float64_t sparsity;
+float64_t tim;
 
 CSubGradientSVM::CSubGradientSVM()
 : CSparseLinearClassifier(), C1(1), C2(1), epsilon(1e-5), qpsize(42),
@@ -30,7 +30,8 @@ CSubGradientSVM::CSubGradientSVM()
 {
 }
 
-CSubGradientSVM::CSubGradientSVM(DREAL C, CSparseFeatures<DREAL>* traindat, CLabels* trainlab)
+CSubGradientSVM::CSubGradientSVM(
+	float64_t C, CSparseFeatures<float64_t>* traindat, CLabels* trainlab)
 : CSparseLinearClassifier(), C1(C), C2(C), epsilon(1e-5), qpsize(42),
 	qpsize_max(2000), use_bias(false), delta_active(0), delta_bound(0)
 {
@@ -44,13 +45,13 @@ CSubGradientSVM::~CSubGradientSVM()
 }
 
 /*
-INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT& num_bound)
+int32_t CSubGradientSVM::find_active(int32_t num_feat, int32_t num_vec, int32_t& num_active, int32_t& num_bound)
 {
-	INT delta_active=0;
+	int32_t delta_active=0;
 	num_active=0;
 	num_bound=0;
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 	{
 		active[i]=0;
 
@@ -76,14 +77,15 @@ INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT
 }
 */
 
-INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT& num_bound)
+int32_t CSubGradientSVM::find_active(
+	int32_t num_feat, int32_t num_vec, int32_t& num_active, int32_t& num_bound)
 {
 	delta_bound=0;
 	delta_active=0;
 	num_active=0;
 	num_bound=0;
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 	{
 		active[i]=0;
 
@@ -123,7 +125,7 @@ INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT
 	num_active=0;
 	num_bound=0;
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 	{
 		tmp_proj[i]=CMath::abs(proj[i]-1);
 		tmp_proj_idx[i]=i;
@@ -144,7 +146,7 @@ INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT
 	{
 		autoselected_epsilon=epsilon;
 
-		INT i=0;
+		int32_t i=0;
 		while (i < num_vec && tmp_proj[i] <= autoselected_epsilon)
 			i++;
 
@@ -153,7 +155,7 @@ INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT
 		if (i>=qpsize_max && autoselected_epsilon>epsilon) //qpsize limit
 		{
 			SG_INFO("qpsize limit (%d) reached\n", qpsize_max);
-			INT num_in_qp=i;
+			int32_t num_in_qp=i;
 			while (--i>=0 && num_in_qp>=qpsize_max)
 			{
 				if (tmp_proj[i] < autoselected_epsilon)
@@ -167,7 +169,7 @@ INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT
 		}
 	}
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 	{
 		active[i]=0;
 
@@ -197,10 +199,9 @@ INT CSubGradientSVM::find_active(INT num_feat, INT num_vec, INT& num_active, INT
 }
 
 
-void CSubGradientSVM::update_active(INT num_feat, INT num_vec)
-
+void CSubGradientSVM::update_active(int32_t num_feat, int32_t num_vec)
 {
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 	{
 		if (active[i]==1 && old_active[i]!=1)
 		{
@@ -219,17 +220,17 @@ void CSubGradientSVM::update_active(INT num_feat, INT num_vec)
 	CMath::swap(active,old_active);
 }
 
-DREAL CSubGradientSVM::line_search(INT num_feat, INT num_vec)
+float64_t CSubGradientSVM::line_search(int32_t num_feat, int32_t num_vec)
 {
-	DREAL sum_B = 0;
-	DREAL A_zero = 0.5*CMath::dot(grad_w, grad_w, num_feat);
-	DREAL B_zero = -CMath::dot(w, grad_w, num_feat);
+	float64_t sum_B = 0;
+	float64_t A_zero = 0.5*CMath::dot(grad_w, grad_w, num_feat);
+	float64_t B_zero = -CMath::dot(w, grad_w, num_feat);
 
-	INT num_hinge=0;
+	int32_t num_hinge=0;
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 	{
-		DREAL p=get_label(i)*features->dense_dot(1.0, i, grad_w, num_feat, grad_b);
+		float64_t p=get_label(i)*features->dense_dot(1.0, i, grad_w, num_feat, grad_b);
 		grad_proj[i]=p;
 		if (p!=0)
 		{
@@ -246,8 +247,8 @@ DREAL CSubGradientSVM::line_search(INT num_feat, INT num_vec)
 	CMath::qsort_index(hinge_point, hinge_idx, num_hinge);
 
 
-	DREAL alpha = hinge_point[0];
-	DREAL grad_val = 2*A_zero*alpha + B_zero + sum_B;
+	float64_t alpha = hinge_point[0];
+	float64_t grad_val = 2*A_zero*alpha + B_zero + sum_B;
 
 	//CMath::display_vector(grad_w, num_feat, "grad_w");
 	//CMath::display_vector(grad_proj, num_vec, "grad_proj");
@@ -258,10 +259,10 @@ DREAL CSubGradientSVM::line_search(INT num_feat, INT num_vec)
 	//SG_PRINT("alpha=%f\n", alpha);
 	//SG_PRINT("grad_val=%f\n", grad_val);
 
-	DREAL old_grad_val = grad_val;
-	DREAL old_alpha = alpha;
+	float64_t old_grad_val = grad_val;
+	float64_t old_alpha = alpha;
 
-	for (INT i=1; i < num_hinge && grad_val < 0; i++)
+	for (int32_t i=1; i < num_hinge && grad_val < 0; i++)
 	{
 		alpha = hinge_point[i];
 		grad_val = 2*A_zero*alpha + B_zero + sum_B;
@@ -269,7 +270,7 @@ DREAL CSubGradientSVM::line_search(INT num_feat, INT num_vec)
 		if (grad_val > 0)
 		{
 			ASSERT(old_grad_val-grad_val != 0);
-			DREAL gamma = -grad_val/(old_grad_val-grad_val);
+			float64_t gamma = -grad_val/(old_grad_val-grad_val);
 			alpha = old_alpha*gamma + (1-gamma)*alpha;
 		}
 		else
@@ -285,9 +286,10 @@ DREAL CSubGradientSVM::line_search(INT num_feat, INT num_vec)
 	return alpha;
 }
 
-DREAL CSubGradientSVM::compute_min_subgradient(INT num_feat, INT num_vec, INT num_active, INT num_bound)
+float64_t CSubGradientSVM::compute_min_subgradient(
+	int32_t num_feat, int32_t num_vec, int32_t num_active, int32_t num_bound)
 {
-	DREAL dir_deriv=0;
+	float64_t dir_deriv=0;
 
 	if (num_bound > 0)
 	{
@@ -298,29 +300,29 @@ DREAL CSubGradientSVM::compute_min_subgradient(INT num_feat, INT num_vec, INT nu
 		if (num_bound>=qpsize_max && num_it_noimprovement!=10) // if qp gets to large, lets just choose a random beta
 		{
 			//SG_PRINT("qpsize too large  (%d>=%d) choosing random subgradient/beta\n", num_bound, qpsize_max);
-			for (INT i=0; i<num_bound; i++)
+			for (int32_t i=0; i<num_bound; i++)
 				beta[i]=CMath::random(0.0,1.0);
 		}
 		else
 		{
-			memset(beta, 0, sizeof(DREAL)*num_bound);
+			memset(beta, 0, sizeof(float64_t)*num_bound);
 
-			DREAL bias_const=0;
+			float64_t bias_const=0;
 
 			if (use_bias)
 				bias_const=1;
 
-			for (INT i=0; i<num_bound; i++)
+			for (int32_t i=0; i<num_bound; i++)
 			{
-				for (INT j=i; j<num_bound; j++)
+				for (int32_t j=i; j<num_bound; j++)
 				{
-					INT alen=0;
-					INT blen=0;
+					int32_t alen=0;
+					int32_t blen=0;
 					bool afree=false;
 					bool bfree=false;
 
-					TSparseEntry<DREAL>* avec=features->get_sparse_feature_vector(idx_bound[i], alen, afree);
-					TSparseEntry<DREAL>* bvec=features->get_sparse_feature_vector(idx_bound[j], blen, bfree);
+					TSparseEntry<float64_t>* avec=features->get_sparse_feature_vector(idx_bound[i], alen, afree);
+					TSparseEntry<float64_t>* bvec=features->get_sparse_feature_vector(idx_bound[j], blen, bfree);
 
 					Z[i*num_bound+j]= 2.0*C1*C1*get_label(idx_bound[i])*get_label(idx_bound[j])* 
 						(features->sparse_dot(1.0, avec,alen, bvec,blen) + bias_const);
@@ -367,7 +369,7 @@ DREAL CSubGradientSVM::compute_min_subgradient(INT num_feat, INT num_vec, INT nu
 		CMath::add(grad_w, 1.0, w, -1.0, sum_CXy_active, num_feat);
 		grad_b = -sum_Cy_active;
 
-		for (INT i=0; i<num_bound; i++)
+		for (int32_t i=0; i<num_bound; i++)
 		{
 			features->add_to_dense_vec(-C1*beta[i]*get_label(idx_bound[i]), idx_bound[i], grad_w, num_feat);
 			if (use_bias)
@@ -375,9 +377,9 @@ DREAL CSubGradientSVM::compute_min_subgradient(INT num_feat, INT num_vec, INT nu
 		}
 
 		dir_deriv = CMath::dot(grad_w, v, num_feat) - grad_b*sum_Cy_active;
-		for (INT i=0; i<num_bound; i++)
+		for (int32_t i=0; i<num_bound; i++)
 		{
-			DREAL val= C1*get_label(idx_bound[i])*features->dense_dot(1.0, idx_bound[i], grad_w, num_feat, grad_b);
+			float64_t val= C1*get_label(idx_bound[i])*features->dense_dot(1.0, idx_bound[i], grad_w, num_feat, grad_b);
 			dir_deriv += CMath::max(0.0, val);
 		}
 	}
@@ -392,11 +394,11 @@ DREAL CSubGradientSVM::compute_min_subgradient(INT num_feat, INT num_vec, INT nu
 	return dir_deriv;
 }
 
-DREAL CSubGradientSVM::compute_objective(INT num_feat, INT num_vec)
+float64_t CSubGradientSVM::compute_objective(int32_t num_feat, int32_t num_vec)
 {
-	DREAL result= 0.5 * CMath::dot(w,w, num_feat);
+	float64_t result= 0.5 * CMath::dot(w,w, num_feat);
 	
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 	{
 		if (proj[i]<1.0)
 			result += C1 * (1.0-proj[i]);
@@ -405,23 +407,23 @@ DREAL CSubGradientSVM::compute_objective(INT num_feat, INT num_vec)
 	return result;
 }
 
-void CSubGradientSVM::compute_projection(INT num_feat, INT num_vec)
+void CSubGradientSVM::compute_projection(int32_t num_feat, int32_t num_vec)
 {
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 		proj[i]=get_label(i)*features->dense_dot(1.0, i, w, num_feat, bias);
 }
 
-void CSubGradientSVM::update_projection(DREAL alpha, INT num_vec)
+void CSubGradientSVM::update_projection(float64_t alpha, int32_t num_vec)
 {
 	CMath::vec1_plus_scalar_times_vec2(proj,-alpha, grad_proj, num_vec);
 }
 
-void CSubGradientSVM::init(INT num_vec, INT num_feat)
+void CSubGradientSVM::init(int32_t num_vec, int32_t num_feat)
 {
 	// alloc normal and bias inited with 0
 	delete[] w;
-	w=new DREAL[num_feat];
-	memset(w,0,sizeof(DREAL)*num_feat);
+	w=new float64_t[num_feat];
+	memset(w,0,sizeof(float64_t)*num_feat);
 	//CMath::random_vector(w, num_feat, -1.0, 1.0);
 	bias=0;
 	num_it_noimprovement=0;
@@ -429,67 +431,67 @@ void CSubGradientSVM::init(INT num_vec, INT num_feat)
 	set_w(w, num_feat);
 	qpsize_limit=5000;
 
-	grad_w=new DREAL[num_feat];
-	memset(grad_w,0,sizeof(DREAL)*num_feat);
+	grad_w=new float64_t[num_feat];
+	memset(grad_w,0,sizeof(float64_t)*num_feat);
 
-	sum_CXy_active=new DREAL[num_feat];
-	memset(sum_CXy_active,0,sizeof(DREAL)*num_feat);
+	sum_CXy_active=new float64_t[num_feat];
+	memset(sum_CXy_active,0,sizeof(float64_t)*num_feat);
 
-	v=new DREAL[num_feat];
-	memset(v,0,sizeof(DREAL)*num_feat);
+	v=new float64_t[num_feat];
+	memset(v,0,sizeof(float64_t)*num_feat);
 
-	old_v=new DREAL[num_feat];
-	memset(old_v,0,sizeof(DREAL)*num_feat);
+	old_v=new float64_t[num_feat];
+	memset(old_v,0,sizeof(float64_t)*num_feat);
 
 	sum_Cy_active=0;
 
-	proj= new DREAL[num_vec];
-	memset(proj,0,sizeof(DREAL)*num_vec);
+	proj= new float64_t[num_vec];
+	memset(proj,0,sizeof(float64_t)*num_vec);
 
-	tmp_proj=new DREAL[num_vec];
-	memset(proj,0,sizeof(DREAL)*num_vec);
+	tmp_proj=new float64_t[num_vec];
+	memset(proj,0,sizeof(float64_t)*num_vec);
 
-	tmp_proj_idx= new INT[num_vec];
-	memset(tmp_proj_idx,0,sizeof(INT)*num_vec);
+	tmp_proj_idx= new int32_t[num_vec];
+	memset(tmp_proj_idx,0,sizeof(int32_t)*num_vec);
 
-	grad_proj= new DREAL[num_vec];
-	memset(grad_proj,0,sizeof(DREAL)*num_vec);
+	grad_proj= new float64_t[num_vec];
+	memset(grad_proj,0,sizeof(float64_t)*num_vec);
 
-	hinge_point= new DREAL[num_vec];
-	memset(hinge_point,0,sizeof(DREAL)*num_vec);
+	hinge_point= new float64_t[num_vec];
+	memset(hinge_point,0,sizeof(float64_t)*num_vec);
 
-	hinge_idx= new INT[num_vec];
-	memset(hinge_idx,0,sizeof(INT)*num_vec);
+	hinge_idx= new int32_t[num_vec];
+	memset(hinge_idx,0,sizeof(int32_t)*num_vec);
 
-	active=new BYTE[num_vec];
-	memset(active,0,sizeof(BYTE)*num_vec);
+	active=new uint8_t[num_vec];
+	memset(active,0,sizeof(uint8_t)*num_vec);
 
-	old_active=new BYTE[num_vec];
-	memset(old_active,0,sizeof(BYTE)*num_vec);
+	old_active=new uint8_t[num_vec];
+	memset(old_active,0,sizeof(uint8_t)*num_vec);
 
-	idx_bound=new INT[num_vec];
-	memset(idx_bound,0,sizeof(INT)*num_vec);
+	idx_bound=new int32_t[num_vec];
+	memset(idx_bound,0,sizeof(int32_t)*num_vec);
 
-	idx_active=new INT[num_vec];
-	memset(idx_active,0,sizeof(INT)*num_vec);
+	idx_active=new int32_t[num_vec];
+	memset(idx_active,0,sizeof(int32_t)*num_vec);
 
-	Z=new DREAL[qpsize_limit*qpsize_limit];
-	memset(Z,0,sizeof(DREAL)*qpsize_limit*qpsize_limit);
+	Z=new float64_t[qpsize_limit*qpsize_limit];
+	memset(Z,0,sizeof(float64_t)*qpsize_limit*qpsize_limit);
 
-	Zv=new DREAL[qpsize_limit];
-	memset(Zv,0,sizeof(DREAL)*qpsize_limit);
+	Zv=new float64_t[qpsize_limit];
+	memset(Zv,0,sizeof(float64_t)*qpsize_limit);
 
-	beta=new DREAL[qpsize_limit];
-	memset(beta,0,sizeof(DREAL)*qpsize_limit);
+	beta=new float64_t[qpsize_limit];
+	memset(beta,0,sizeof(float64_t)*qpsize_limit);
 
-	old_Z=new DREAL[qpsize_limit*qpsize_limit];
-	memset(old_Z,0,sizeof(DREAL)*qpsize_limit*qpsize_limit);
+	old_Z=new float64_t[qpsize_limit*qpsize_limit];
+	memset(old_Z,0,sizeof(float64_t)*qpsize_limit*qpsize_limit);
 
-	old_Zv=new DREAL[qpsize_limit];
-	memset(old_Zv,0,sizeof(DREAL)*qpsize_limit);
+	old_Zv=new float64_t[qpsize_limit];
+	memset(old_Zv,0,sizeof(float64_t)*qpsize_limit);
 
-	old_beta=new DREAL[qpsize_limit];
-	memset(old_beta,0,sizeof(DREAL)*qpsize_limit);
+	old_beta=new float64_t[qpsize_limit];
+	memset(old_beta,0,sizeof(float64_t)*qpsize_limit);
 
 }
 
@@ -537,20 +539,20 @@ bool CSubGradientSVM::train()
 	ASSERT(labels);
 	ASSERT(get_features());
 
-	INT num_iterations=0;
-	INT num_train_labels=labels->get_num_labels();
-	INT num_feat=features->get_num_features();
-	INT num_vec=features->get_num_vectors();
+	int32_t num_iterations=0;
+	int32_t num_train_labels=labels->get_num_labels();
+	int32_t num_feat=features->get_num_features();
+	int32_t num_vec=features->get_num_vectors();
 
 	ASSERT(num_vec==num_train_labels);
 
 	init(num_vec, num_feat);
 
-	INT num_active=0;
-	INT num_bound=0;
-	DREAL alpha=0;
-	DREAL dir_deriv=0;
-	DREAL obj=0;
+	int32_t num_active=0;
+	int32_t num_bound=0;
+	float64_t alpha=0;
+	float64_t dir_deriv=0;
+	float64_t obj=0;
 	delta_active=num_vec;
 	last_it_noimprovement=-1;
 
@@ -560,7 +562,7 @@ bool CSubGradientSVM::train()
 	compute_projection(num_feat, num_vec);
 
 	CTime time;
-	double loop_time=0;
+	float64_t loop_time=0;
 	while (!(CSignal::cancel_computations()))
 	{
 		CTime t;
@@ -594,7 +596,7 @@ bool CSubGradientSVM::train()
 
 		if (num_it_noimprovement==10 || num_bound<qpsize_max)
 		{
-			DREAL norm_grad=CMath::dot(grad_w, grad_w, num_feat) +
+			float64_t norm_grad=CMath::dot(grad_w, grad_w, num_feat) +
 				grad_b*grad_b;
 
 #ifdef DEBUG_SUBGRADIENTSVM
