@@ -9,7 +9,7 @@
 #include "lib/Mathematics.h"
 #include "classifier/svm/Tron.h"
 
-CTron::CTron(const function *f, double e, int it)
+CTron::CTron(const function *f, float64_t e, int32_t it)
 : CSGObject()
 {
 	this->fun_obj=const_cast<function *>(f);
@@ -21,18 +21,20 @@ CTron::~CTron()
 {
 }
 
-void CTron::tron(double *w)
+void CTron::tron(float64_t *w)
 {
 	// Parameters for updating the iterates.
-	double eta0 = 1e-4, eta1 = 0.25, eta2 = 0.75;
+	float64_t eta0 = 1e-4, eta1 = 0.25, eta2 = 0.75;
 
 	// Parameters for updating the trust region size delta.
-	double sigma1 = 0.25, sigma2 = 0.5, sigma3 = 4;
+	float64_t sigma1 = 0.25, sigma2 = 0.5, sigma3 = 4.;
 
-	int n = fun_obj->get_nr_variable();
-	int i, cg_iter;
-	double delta, snorm, one=1.0;
-	double alpha, f, fnew, prered, actred, gs;
+	int32_t i, cg_iter;
+	float64_t delta, snorm, one=1.0;
+	float64_t alpha, f, fnew, prered, actred, gs;
+
+	/* calling external lib */
+	int n = (int) fun_obj->get_nr_variable();
 	int search = 1, iter = 1, inc = 1;
 	double *s = new double[n];
 	double *r = new double[n];
@@ -45,8 +47,8 @@ void CTron::tron(double *w)
         f = fun_obj->fun(w);
 	fun_obj->grad(w, g);
 	delta = cblas_dnrm2(n, g, inc);
-	double gnorm1 = delta;
-	double gnorm = gnorm1;
+	float64_t gnorm1 = delta;
+	float64_t gnorm = gnorm1;
 
 	if (gnorm <= eps*gnorm1)
 		search = 0;
@@ -57,7 +59,7 @@ void CTron::tron(double *w)
 	{
 		cg_iter = trcg(delta, g, s, r);
 
-		memcpy(w_new, w, sizeof(double)*n);
+		memcpy(w_new, w, sizeof(float64_t)*n);
 		cblas_daxpy(n, one, s, inc, w_new, inc);
 
 		gs = cblas_ddot(n, g, inc, s, inc);
@@ -93,7 +95,7 @@ void CTron::tron(double *w)
 		if (actred > eta0*prered)
 		{
 			iter++;
-			memcpy(w, w_new, sizeof(double)*n);
+			memcpy(w, w_new, sizeof(float64_t)*n);
 			f = fnew;
 		        fun_obj->grad(w, g);
 
@@ -125,13 +127,15 @@ void CTron::tron(double *w)
 	delete[] s;
 }
 
-int CTron::trcg(double delta, double *g, double *s, double *r)
+int32_t CTron::trcg(float64_t delta, double* g, double* s, double* r)
 {
-	int i, inc = 1;
-	int n = fun_obj->get_nr_variable();
+	/* calling external lib */
+	int i, cg_iter;
+	int n = (int) fun_obj->get_nr_variable();
+	int inc = 1;
 	double one = 1;
-	double *d = new double[n];
 	double *Hd = new double[n];
+	double *d = new double[n];
 	double rTr, rnewTrnew, alpha, beta, cgtol;
 
 	for (i=0; i<n; i++)
@@ -140,9 +144,9 @@ int CTron::trcg(double delta, double *g, double *s, double *r)
 		r[i] = -g[i];
 		d[i] = r[i];
 	}
-	cgtol = 0.1*cblas_dnrm2(n, g, inc);
+	cgtol = 0.1* cblas_dnrm2(n, g, inc);
 
-	int cg_iter = 0;
+	cg_iter = 0;
 	rTr = cblas_ddot(n, r, inc, r, inc);
 	while (1)
 	{
@@ -188,10 +192,10 @@ int CTron::trcg(double delta, double *g, double *s, double *r)
 	return(cg_iter);
 }
 
-double CTron::norm_inf(int n, double *x)
+float64_t CTron::norm_inf(int32_t n, float64_t *x)
 {
-	double dmax = CMath::abs(x[0]);
-	for (int i=1; i<n; i++)
+	float64_t dmax = CMath::abs(x[0]);
+	for (int32_t i=1; i<n; i++)
 		if (CMath::abs(x[i]) >= dmax)
 			dmax = CMath::abs(x[i]);
 	return(dmax);
