@@ -75,7 +75,9 @@ gmnplib.c: Library of solvers for Generalized Minimal Norm Problem (GMNP).
 #define KDELTA(A,B) (A==B)
 #define KDELTA4(A1,A2,A3,A4) ((A1==A2)||(A1==A3)||(A1==A4)||(A2==A3)||(A2==A4)||(A3==A4))
 
-CGMNPLib::CGMNPLib(DREAL* vector_y, CKernel* kernel, INT num_data, INT num_virt_data, INT num_classes, DREAL reg_const)
+CGMNPLib::CGMNPLib(
+	float64_t* vector_y, CKernel* kernel, int32_t num_data,
+	int32_t num_virt_data, int32_t num_classes, float64_t reg_const)
 : CSGObject()
 {
   m_num_classes=num_classes;
@@ -85,43 +87,43 @@ CGMNPLib::CGMNPLib(DREAL* vector_y, CKernel* kernel, INT num_data, INT num_virt_
   m_vector_y = vector_y;
   m_kernel = kernel;
 
-  Cache_Size = ((LONG) kernel->get_cache_size())*1024*1024/(sizeof(DREAL)*num_data);
-  Cache_Size = CMath::min(Cache_Size, (LONG) num_data);
+  Cache_Size = ((int64_t) kernel->get_cache_size())*1024*1024/(sizeof(float64_t)*num_data);
+  Cache_Size = CMath::min(Cache_Size, (int64_t) num_data);
 
   SG_INFO("using %d kernel cache lines\n", Cache_Size);
   ASSERT(Cache_Size>=2);
 
   /* allocates memory for kernel cache */
-  kernel_columns = new DREAL*[Cache_Size];
-  cache_index = new DREAL[Cache_Size];
+  kernel_columns = new float64_t*[Cache_Size];
+  cache_index = new float64_t[Cache_Size];
 
-  for(INT i = 0; i < Cache_Size; i++ ) 
+  for(int32_t i = 0; i < Cache_Size; i++ )
   {
-    kernel_columns[i] = new DREAL[num_data];
+    kernel_columns[i] = new float64_t[num_data];
     cache_index[i] = -2;
   }
   first_kernel_inx = 0;
 
 
 
-  for(INT i = 0; i < 3; i++ )
+  for(int32_t i = 0; i < 3; i++ )
   {
-    virt_columns[i] = new DREAL[num_virt_data];
+    virt_columns[i] = new float64_t[num_virt_data];
   }
   first_virt_inx = 0;
 
-  diag_H = new DREAL[num_virt_data];
+  diag_H = new float64_t[num_virt_data];
 
-  for(INT i = 0; i < num_virt_data; i++ )
+  for(int32_t i = 0; i < num_virt_data; i++ )
 	  diag_H[i] = kernel_fce(i,i);
 }
 
 CGMNPLib::~CGMNPLib()
 {
-	for(INT i = 0; i < Cache_Size; i++ ) 
+	for(int32_t i = 0; i < Cache_Size; i++ ) 
 		delete[] kernel_columns[i];
 
-	for(INT i = 0; i < 3; i++ ) 
+	for(int32_t i = 0; i < 3; i++ ) 
 		delete[] virt_columns[i];
 
 	delete[] cache_index;
@@ -134,11 +136,11 @@ CGMNPLib::~CGMNPLib()
   Returns pointer at a-th column of the kernel matrix.
   This function maintains FIFO cache of kernel columns.
 ------------------------------------------------------------ */
-DREAL* CGMNPLib::get_kernel_col( INT a ) 
+float64_t* CGMNPLib::get_kernel_col( int32_t a )
 {
-  double *col_ptr;
-  INT i;
-  INT inx;
+  float64_t *col_ptr;
+  int32_t i;
+  int32_t inx;
 
   inx = -1;
   for( i=0; i < Cache_Size; i++ ) {
@@ -167,7 +169,7 @@ DREAL* CGMNPLib::get_kernel_col( INT a )
   Computes index of input example and its class label from 
   index of virtual "single-class" example.
 ------------------------------------------------------------ */
-void CGMNPLib::get_indices2( INT *index, INT *c, INT i )
+void CGMNPLib::get_indices2( int32_t *index, int32_t *c, int32_t i )
 {
    *index = i / (m_num_classes-1);
  
@@ -185,19 +187,19 @@ void CGMNPLib::get_indices2( INT *index, INT *c, INT i )
    updating but b is from (a(t-2), a(t-1)) where a=a(t) and
    thus FIFO with three columns does not have to take care od b.)
 ------------------------------------------------------------ */
-DREAL* CGMNPLib::get_col( INT a, INT b )
+float64_t* CGMNPLib::get_col( int32_t a, int32_t b )
 {
-  INT i;
-  double *col_ptr;
-  double *ker_ptr;
-  double value;
-  INT i1,c1,i2,c2;
+  int32_t i;
+  float64_t *col_ptr;
+  float64_t *ker_ptr;
+  float64_t value;
+  int32_t i1,c1,i2,c2;
 
   col_ptr = virt_columns[first_virt_inx++];
   if( first_virt_inx >= 3 ) first_virt_inx = 0;
 
   get_indices2( &i1, &c1, a );
-  ker_ptr = (double*) get_kernel_col( i1 );
+  ker_ptr = (float64_t*) get_kernel_col( i1 );
 
   for( i=0; i < m_num_virt_data; i++ ) {
     get_indices2( &i2, &c2, i );
@@ -233,46 +235,46 @@ DREAL* CGMNPLib::get_col( INT a, INT b )
                   tmax, tolabs, tolrel, th, &alpha, &t, &History );
 -------------------------------------------------------------- */
 
-int CGMNPLib::gmnp_imdm(double *vector_c,
-            INT dim, 
-            INT tmax,
-            double tolabs,
-            double tolrel,
-            double th,
-            double *alpha,
-            INT  *ptr_t,
-            double **ptr_History,
-            INT verb)
+int8_t CGMNPLib::gmnp_imdm(float64_t *vector_c,
+            int32_t dim, 
+            int32_t tmax,
+            float64_t tolabs,
+            float64_t tolrel,
+            float64_t th,
+            float64_t *alpha,
+            int32_t  *ptr_t,
+            float64_t **ptr_History,
+            int32_t verb)
 {
-  double LB;
-  double UB;
-  double aHa, ac;
-  double tmp, tmp1;
-  double Huu, Huv, Hvv;
-  double min_beta, beta;
-  double max_improv, improv;
-  double lambda;
-  double *History;
-  double *Ha;
-  double *tmp_ptr;
-  double *col_u, *col_v;
-  INT u=0;
-  INT v=0;
-  INT new_u=0;
-  INT i;
-  INT t;
-  INT History_size;
-  int exitflag;
+  float64_t LB;
+  float64_t UB;
+  float64_t aHa, ac;
+  float64_t tmp, tmp1;
+  float64_t Huu, Huv, Hvv;
+  float64_t min_beta, beta;
+  float64_t max_improv, improv;
+  float64_t lambda;
+  float64_t *History;
+  float64_t *Ha;
+  float64_t *tmp_ptr;
+  float64_t *col_u, *col_v;
+  int32_t u=0;
+  int32_t v=0;
+  int32_t new_u=0;
+  int32_t i;
+  int32_t t;
+  int32_t History_size;
+  int8_t exitflag;
 
   /* ------------------------------------------------------------ */
   /* Initialization                                               */
   /* ------------------------------------------------------------ */
 
-  Ha = new double[dim];
+  Ha = new float64_t[dim];
   if( Ha == NULL ) SG_ERROR("Not enough memory.");
 
   History_size = (tmax < HISTORY_BUF ) ? tmax+1 : HISTORY_BUF;
-  History = new double[History_size*2];
+  History = new float64_t[History_size*2];
   if( History == NULL ) SG_ERROR("Not enough memory.");
 
   /* inx = argmin(0.5*diag_H + vector_c ); */
@@ -284,7 +286,7 @@ int CGMNPLib::gmnp_imdm(double *vector_c,
     }
   }
 
-  col_v = (double*)get_col(v,-1);
+  col_v = (float64_t*)get_col(v,-1);
 
   for( min_beta = PLUS_INF, i = 0; i < dim; i++ ) 
   {
@@ -324,12 +326,12 @@ int CGMNPLib::gmnp_imdm(double *vector_c,
   /* Main optimization loop                                       */
   /* ------------------------------------------------------------ */
 
-  col_u = (double*)get_col(u,-1);
+  col_u = (float64_t*)get_col(u,-1);
   while( exitflag == -1 ) 
   {
     t++;     
 
-    col_v = (double*)get_col(v,u);
+    col_v = (float64_t*)get_col(v,u);
 
     /* Adaptation rule and update */
     Huu = diag_H[u];
@@ -366,7 +368,7 @@ int CGMNPLib::gmnp_imdm(double *vector_c,
 
     LB = min_beta - 0.5*aHa; 
     u = new_u;    
-    col_u = (double*)get_col(u,-1);
+    col_u = (float64_t*)get_col(u,-1);
 
     /* search for optimal v while u is fixed */
     for( max_improv =  MINUS_INF, i = 0; i < dim; i++ ) {
@@ -407,7 +409,7 @@ int CGMNPLib::gmnp_imdm(double *vector_c,
       History[INDEX(1,t,2)] = UB;
     }
     else {
-      tmp_ptr = new double[(History_size+HISTORY_BUF)*2];
+      tmp_ptr = new float64_t[(History_size+HISTORY_BUF)*2];
       if( tmp_ptr == NULL ) SG_ERROR("Not enough memory.");
       for( i = 0; i < History_size; i++ ) {
         tmp_ptr[INDEX(0,i,2)] = History[INDEX(0,i,2)];
@@ -445,10 +447,10 @@ int CGMNPLib::gmnp_imdm(double *vector_c,
   Retures (a,b)-th element of the virtual kernel matrix 
   of size [num_virt_data x num_virt_data]. 
 ------------------------------------------------------------ */
-double CGMNPLib::kernel_fce( INT a, INT b )
+float64_t CGMNPLib::kernel_fce( int32_t a, int32_t b )
 {
-  double value;
-  INT i1,c1,i2,c2;
+  float64_t value;
+  int32_t i1,c1,i2,c2;
 
   get_indices2( &i1, &c1, a );
   get_indices2( &i2, &c2, b );
