@@ -29,7 +29,8 @@
 #define INDEX(ROW,COL,DIM) ((COL*DIM)+ROW)
 
 
-CGNPPLib::CGNPPLib(DREAL* vector_y, CKernel* kernel, INT num_data, DREAL reg_const)
+CGNPPLib::CGNPPLib(
+	float64_t* vector_y, CKernel* kernel, int32_t num_data, float64_t reg_const)
 : CSGObject()
 {
   m_reg_const = reg_const;
@@ -37,19 +38,19 @@ CGNPPLib::CGNPPLib(DREAL* vector_y, CKernel* kernel, INT num_data, DREAL reg_con
   m_vector_y = vector_y;
   m_kernel = kernel;
 
-  Cache_Size = ((LONG) kernel->get_cache_size())*1024*1024/(sizeof(DREAL)*num_data);
-  Cache_Size = CMath::min(Cache_Size, (LONG) num_data);
+  Cache_Size = ((int64_t) kernel->get_cache_size())*1024*1024/(sizeof(float64_t)*num_data);
+  Cache_Size = CMath::min(Cache_Size, (int64_t) num_data);
 
   SG_INFO("using %d kernel cache lines\n", Cache_Size);
   ASSERT(Cache_Size>=2);
 
   /* allocates memory for kernel cache */
-  kernel_columns = new DREAL*[Cache_Size];
-  cache_index = new DREAL[Cache_Size];
+  kernel_columns = new float64_t*[Cache_Size];
+  cache_index = new float64_t[Cache_Size];
 
-  for(INT i = 0; i < Cache_Size; i++ ) 
+  for(int32_t i = 0; i < Cache_Size; i++ ) 
   {
-    kernel_columns[i] = new DREAL[num_data];
+    kernel_columns[i] = new float64_t[num_data];
     cache_index[i] = -2;
   }
   first_kernel_inx = 0;
@@ -58,7 +59,7 @@ CGNPPLib::CGNPPLib(DREAL* vector_y, CKernel* kernel, INT num_data, DREAL reg_con
 
 CGNPPLib::~CGNPPLib()
 {
-  for(INT i = 0; i < Cache_Size; i++ ) 
+  for(int32_t i = 0; i < Cache_Size; i++ ) 
       delete[] kernel_columns[i];
 
   delete[] cache_index;
@@ -71,53 +72,53 @@ CGNPPLib::~CGNPPLib()
  Usage: exitflag = gnpp_mdm( diag_H, vector_c, vector_y,
        dim, tmax, tolabs, tolrel, th, &alpha, &t, &aHa11, &aHa22, &History );
 -------------------------------------------------------------- */
-int CGNPPLib::gnpp_mdm(double *diag_H,
-                       double *vector_c,
-                       double *vector_y,
-                       INT dim, 
-                       INT tmax,
-                       double tolabs,
-                       double tolrel,
-                       double th,
-                       double *alpha,
-                       INT  *ptr_t, 
-                       double *ptr_aHa11,
-                       double *ptr_aHa22,
-                       double **ptr_History,
-                       INT verb)
+int8_t CGNPPLib::gnpp_mdm(float64_t *diag_H,
+                       float64_t *vector_c,
+                       float64_t *vector_y,
+                       int32_t dim,
+                       int32_t tmax,
+                       float64_t tolabs,
+                       float64_t tolrel,
+                       float64_t th,
+                       float64_t *alpha,
+                       int32_t  *ptr_t,
+                       float64_t *ptr_aHa11,
+                       float64_t *ptr_aHa22,
+                       float64_t **ptr_History,
+                       int32_t verb)
 {
-  double LB;
-  double UB;
-  double aHa11, aHa12, aHa22, ac1, ac2;
-  double tmp;
-  double Huu, Huv, Hvv;
-  double min_beta1, max_beta1, min_beta2, max_beta2, beta;
-  double lambda;
-  double delta1, delta2;
-  double *History;
-  double *Ha1;
-  double *Ha2;
-  double *tmp_ptr;
-  double *col_u, *col_v;
-  double *col_v1, *col_v2;
-  long u1=0, u2=0;
-  long v1, v2;
-  long i;
-  long t;
-  long History_size;
-  int exitflag;
+  float64_t LB;
+  float64_t UB;
+  float64_t aHa11, aHa12, aHa22, ac1, ac2;
+  float64_t tmp;
+  float64_t Huu, Huv, Hvv;
+  float64_t min_beta1, max_beta1, min_beta2, max_beta2, beta;
+  float64_t lambda;
+  float64_t delta1, delta2;
+  float64_t *History;
+  float64_t *Ha1;
+  float64_t *Ha2;
+  float64_t *tmp_ptr;
+  float64_t *col_u, *col_v;
+  float64_t *col_v1, *col_v2;
+  int64_t u1=0, u2=0;
+  int64_t v1, v2;
+  int64_t i;
+  int64_t t;
+  int64_t History_size;
+  int8_t exitflag;
 
   /* ------------------------------------------------------------ */
   /* Initialization                                               */
   /* ------------------------------------------------------------ */
 
-  Ha1 = new DREAL[dim];
+  Ha1 = new float64_t[dim];
   if( Ha1 == NULL ) SG_ERROR("Not enough memory.\n");
-  Ha2 = new DREAL[dim];
+  Ha2 = new float64_t[dim];
   if( Ha2 == NULL ) SG_ERROR("Not enough memory.\n");
 
   History_size = (tmax < HISTORY_BUF ) ? tmax+1 : HISTORY_BUF;
-  History = new DREAL[History_size*2];
+  History = new float64_t[History_size*2];
   if( History == NULL ) SG_ERROR("Not enough memory.\n");
 
   /* inx1 = firts of find( y ==1 ), inx2 = firts of find( y ==2 ) */
@@ -128,8 +129,8 @@ int CGNPPLib::gnpp_mdm(double *diag_H,
     i++;
   }
 
-  col_v1 = (double*)get_col(v1,-1);
-  col_v2 = (double*)get_col(v2,v1);
+  col_v1 = (float64_t*)get_col(v1,-1);
+  col_v2 = (float64_t*)get_col(v2,v1);
   
   aHa12 = col_v1[v2];
   aHa11 = diag_H[v1];
@@ -191,8 +192,8 @@ int CGNPPLib::gnpp_mdm(double *diag_H,
 
     if( delta1 > delta2 ) 
     {
-      col_u = (double*)get_col(u1,-1);
-      col_v = (double*)get_col(v1,u1);
+      col_u = (float64_t*)get_col(u1,-1);
+      col_v = (float64_t*)get_col(v1,u1);
 
       Huu = diag_H[u1];
       Hvv = diag_H[v1];
@@ -231,8 +232,8 @@ int CGNPPLib::gnpp_mdm(double *diag_H,
     }
     else
     {
-      col_u = (double*)get_col(u2,-1);
-      col_v = (double*)get_col(v2,u2);
+      col_u = (float64_t*)get_col(u2,-1);
+      col_v = (float64_t*)get_col(v2,u2);
 
       Huu = diag_H[u2];
       Hvv = diag_H[v2];
@@ -293,7 +294,7 @@ int CGNPPLib::gnpp_mdm(double *diag_H,
       History[INDEX(1,t,2)] = UB;
     }
     else {
-      tmp_ptr = new DREAL[(History_size+HISTORY_BUF)*2];
+      tmp_ptr = new float64_t[(History_size+HISTORY_BUF)*2];
       if( tmp_ptr == NULL ) SG_ERROR("Not enough memory.\n");
       for( i = 0; i < History_size; i++ ) {
         tmp_ptr[INDEX(0,i,2)] = History[INDEX(0,i,2)];
@@ -336,55 +337,55 @@ int CGNPPLib::gnpp_mdm(double *diag_H,
  Usage: exitflag = gnpp_imdm( diag_H, vector_c, vector_y,
        dim, tmax, tolabs, tolrel, th, &alpha, &t, &aHa11, &aHa22, &History );
 -------------------------------------------------------------- */
-int CGNPPLib::gnpp_imdm(double *diag_H,
-            double *vector_c,
-            double *vector_y,
-            INT dim, 
-            INT tmax,
-            double tolabs,
-            double tolrel,
-            double th,
-            double *alpha,
-            INT  *ptr_t, 
-            double *ptr_aHa11,
-            double *ptr_aHa22,
-            double **ptr_History,
-            INT verb)
+int8_t CGNPPLib::gnpp_imdm(float64_t *diag_H,
+            float64_t *vector_c,
+            float64_t *vector_y,
+            int32_t dim,
+            int32_t tmax,
+            float64_t tolabs,
+            float64_t tolrel,
+            float64_t th,
+            float64_t *alpha,
+            int32_t  *ptr_t,
+            float64_t *ptr_aHa11,
+            float64_t *ptr_aHa22,
+            float64_t **ptr_History,
+            int32_t verb)
 {
-  double LB;
-  double UB;
-  double aHa11, aHa12, aHa22, ac1, ac2;
-  double tmp;
-  double Huu, Huv, Hvv;
-  double min_beta1, max_beta1, min_beta2, max_beta2, beta;
-  double lambda;
-  double delta1, delta2;
-  double improv, max_improv;
-  double *History;
-  double *Ha1;
-  double *Ha2;
-  double *tmp_ptr;
-  double *col_u, *col_v;
-  double *col_v1, *col_v2;
-  long u1=0, u2=0;
-  long v1, v2;
-  long i;
-  long t;
-  long History_size;
-  int exitflag;
-  int which_case;
+  float64_t LB;
+  float64_t UB;
+  float64_t aHa11, aHa12, aHa22, ac1, ac2;
+  float64_t tmp;
+  float64_t Huu, Huv, Hvv;
+  float64_t min_beta1, max_beta1, min_beta2, max_beta2, beta;
+  float64_t lambda;
+  float64_t delta1, delta2;
+  float64_t improv, max_improv;
+  float64_t *History;
+  float64_t *Ha1;
+  float64_t *Ha2;
+  float64_t *tmp_ptr;
+  float64_t *col_u, *col_v;
+  float64_t *col_v1, *col_v2;
+  int64_t u1=0, u2=0;
+  int64_t v1, v2;
+  int64_t i;
+  int64_t t;
+  int64_t History_size;
+  int8_t exitflag;
+  int8_t which_case;
 
   /* ------------------------------------------------------------ */
   /* Initialization                                               */
   /* ------------------------------------------------------------ */
 
-  Ha1 = new DREAL[dim];
+  Ha1 = new float64_t[dim];
   if( Ha1 == NULL ) SG_ERROR("Not enough memory.\n");
-  Ha2 = new DREAL[dim];
+  Ha2 = new float64_t[dim];
   if( Ha2 == NULL ) SG_ERROR("Not enough memory.\n");
 
   History_size = (tmax < HISTORY_BUF ) ? tmax+1 : HISTORY_BUF;
-  History = new DREAL[History_size*2];
+  History = new float64_t[History_size*2];
   if( History == NULL ) SG_ERROR("Not enough memory.\n");
 
   /* inx1 = firts of find( y ==1 ), inx2 = firts of find( y ==2 ) */
@@ -395,8 +396,8 @@ int CGNPPLib::gnpp_imdm(double *diag_H,
     i++;
   }
 
-  col_v1 = (double*)get_col(v1,-1);
-  col_v2 = (double*)get_col(v2,v1);
+  col_v1 = (float64_t*)get_col(v1,-1);
+  col_v2 = (float64_t*)get_col(v2,v1);
   
   aHa12 = col_v1[v2];
   aHa11 = diag_H[v1];
@@ -445,13 +446,13 @@ int CGNPPLib::gnpp_imdm(double *diag_H,
   if( delta1 > delta2 ) 
   {
      which_case = 1;
-     col_u = (double*)get_col(u1,v1);
+     col_u = (float64_t*)get_col(u1,v1);
      col_v = col_v1;
   }
   else
   {
      which_case = 2;
-     col_u = (double*)get_col(u2,v2);
+     col_u = (float64_t*)get_col(u2,v2);
      col_v = col_v2;
   }
 
@@ -552,7 +553,7 @@ int CGNPPLib::gnpp_imdm(double *diag_H,
 
     if( delta1 > delta2 ) 
     {
-       col_u = (double*)get_col(u1,-1);
+       col_u = (float64_t*)get_col(u1,-1);
 
       /* search for optimal v while u is fixed */
       for( max_improv =  MINUS_INF, i = 0; i < dim; i++ ) {
@@ -575,14 +576,14 @@ int CGNPPLib::gnpp_imdm(double *diag_H,
           }
         }
       }
-      col_v = (double*)get_col(v1,u1);
+      col_v = (float64_t*)get_col(v1,u1);
       delta1 = Ha1[v1] + Ha2[v1] + vector_c[v1] - min_beta1;
       which_case = 1;
       
     }
     else
     {
-       col_u = (double*)get_col(u2,-1);
+       col_u = (float64_t*)get_col(u2,-1);
 
       /* search for optimal v while u is fixed */
       for( max_improv =  MINUS_INF, i = 0; i < dim; i++ ) {
@@ -606,7 +607,7 @@ int CGNPPLib::gnpp_imdm(double *diag_H,
         }
       }
 
-      col_v = (double*)get_col(v2,u2);
+      col_v = (float64_t*)get_col(v2,u2);
       delta2 = Ha1[v2] + Ha2[v2] + vector_c[v2] - min_beta2;
       which_case = 2;
     }
@@ -629,7 +630,7 @@ int CGNPPLib::gnpp_imdm(double *diag_H,
       History[INDEX(1,t,2)] = UB;
     }
     else {
-      tmp_ptr = new DREAL[(History_size+HISTORY_BUF)*2];
+      tmp_ptr = new float64_t[(History_size+HISTORY_BUF)*2];
       if( tmp_ptr == NULL ) SG_ERROR("Not enough memory.\n");
       for( i = 0; i < History_size; i++ ) {
         tmp_ptr[INDEX(0,i,2)] = History[INDEX(0,i,2)];
@@ -666,12 +667,12 @@ int CGNPPLib::gnpp_imdm(double *diag_H,
 }
 
 
-DREAL* CGNPPLib::get_col( long a, long b ) 
+float64_t* CGNPPLib::get_col(int64_t a, int64_t b)
 {
-  double *col_ptr;
-  double y;
-  long i;
-  long inx;
+  float64_t *col_ptr;
+  float64_t y;
+  int64_t i;
+  int64_t inx;
 
   inx = -1;
   for( i=0; i < Cache_Size; i++ ) {
