@@ -62,9 +62,10 @@
 
 #define INDEX(ROW,COL,DIM) ((COL*DIM)+ROW)
 
-double sparsity=0;
+float64_t sparsity=0;
 
-CQPBSVMLib::CQPBSVMLib(DREAL* H, INT n, DREAL* f, INT m, DREAL UB)
+CQPBSVMLib::CQPBSVMLib(
+	float64_t* H, int32_t n, float64_t* f, int32_t m, float64_t UB)
 : CSGObject()
 {
 	ASSERT(H && n>0);
@@ -86,23 +87,23 @@ CQPBSVMLib::~CQPBSVMLib()
 	delete[] m_diag_H;
 }
 
-INT CQPBSVMLib::solve_qp(DREAL* result, INT len)
+int32_t CQPBSVMLib::solve_qp(float64_t* result, int32_t len)
 {
-	INT status = -1;
+	int32_t status = -1;
 	ASSERT(len==m_dim);
-	DREAL* Nabla=new DREAL[m_dim];
-	for (INT i=0; i<m_dim; i++)
+	float64_t* Nabla=new float64_t[m_dim];
+	for (int32_t i=0; i<m_dim; i++)
 		Nabla[i]=m_f[i];
 
 	delete[] m_diag_H;
-	m_diag_H=new DREAL[m_dim];
+	m_diag_H=new float64_t[m_dim];
 
-	for (INT i=0; i<m_dim; i++)
+	for (int32_t i=0; i<m_dim; i++)
 		m_diag_H[i]=m_H[i*m_dim+i];
 
-	DREAL* History=NULL;
-	INT t;
-	INT verb=0;
+	float64_t* History=NULL;
+	int32_t t;
+	int32_t verb=0;
 
 	switch (m_solver)
 	{
@@ -150,27 +151,27 @@ Usage: exitflag = qpbsvm_sca(m_UB, m_dim, m_tmax,
                m_tolabs, m_tolrel, m_tolKKT, x, Nabla, &t, &History, verb )
 
 -------------------------------------------------------------- */
-INT CQPBSVMLib::qpbsvm_sca(DREAL *x,
-	        DREAL *Nabla,
-            INT   *ptr_t,
-            DREAL **ptr_History,
-            INT   verb)
+int32_t CQPBSVMLib::qpbsvm_sca(float64_t *x,
+	        float64_t *Nabla,
+            int32_t   *ptr_t,
+            float64_t **ptr_History,
+            int32_t   verb)
 {
-  DREAL *History;
-  DREAL *col_H;
-  DREAL *tmp_ptr;
-  DREAL x_old;
-  DREAL delta_x;
-  DREAL xHx;
-  DREAL Q_P;
-  DREAL Q_D;
-  DREAL xf;
-  DREAL xi_sum;
-  INT History_size;
-  INT t;
-  INT i, j;
-  INT exitflag;
-  INT KKTsatisf;
+  float64_t *History;
+  float64_t *col_H;
+  float64_t *tmp_ptr;
+  float64_t x_old;
+  float64_t delta_x;
+  float64_t xHx;
+  float64_t Q_P;
+  float64_t Q_D;
+  float64_t xf;
+  float64_t xi_sum;
+  int32_t History_size;
+  int32_t t;
+  int32_t i, j;
+  int32_t exitflag;
+  int32_t KKTsatisf;
 
   /* ------------------------------------------------------------ */
   /* Initialization                                               */
@@ -179,8 +180,8 @@ INT CQPBSVMLib::qpbsvm_sca(DREAL *x,
   t = 0;
 
   History_size = (m_tmax < HISTORY_BUF ) ? m_tmax+1 : HISTORY_BUF;
-  History=new DREAL[History_size*2];
-  memset(History, 0, sizeof(DREAL)*History_size*2);
+  History=new float64_t[History_size*2];
+  memset(History, 0, sizeof(float64_t)*History_size*2);
 
   /* compute Q_P and Q_D */
   xHx = 0;
@@ -216,7 +217,7 @@ INT CQPBSVMLib::qpbsvm_sca(DREAL *x,
         /* update Nabla */
         delta_x = x[i] - x_old;
         if( delta_x != 0 ) {
-          col_H = (DREAL*)get_col(i);
+          col_H = (float64_t*)get_col(i);
           for(j = 0; j < m_dim; j++ ) {
             Nabla[j] += col_H[j]*delta_x;
           }
@@ -260,8 +261,8 @@ INT CQPBSVMLib::qpbsvm_sca(DREAL *x,
       History[INDEX(1,t,2)] = Q_D;
     }
     else {
-      tmp_ptr=new DREAL[(History_size+HISTORY_BUF)*2];
-      memset(tmp_ptr, 0, sizeof(DREAL)*(History_size+HISTORY_BUF)*2);
+      tmp_ptr=new float64_t[(History_size+HISTORY_BUF)*2];
+      memset(tmp_ptr, 0, sizeof(float64_t)*(History_size+HISTORY_BUF)*2);
 
       for( i = 0; i < History_size; i++ ) {
         tmp_ptr[INDEX(0,i,2)] = History[INDEX(0,i,2)];
@@ -291,32 +292,32 @@ Usage: exitflag = qpbsvm_scas(m_UB, m_dim, m_tmax,
                m_tolabs, m_tolrel, m_tolKKT, x, Nabla, &t, &History, verb )
 
 -------------------------------------------------------------- */
-INT CQPBSVMLib::qpbsvm_scas(DREAL *x,
-	        DREAL *Nabla,
-            INT   *ptr_t,
-            DREAL **ptr_History,
-            INT   verb)
+int32_t CQPBSVMLib::qpbsvm_scas(float64_t *x,
+	        float64_t *Nabla,
+            int32_t   *ptr_t,
+            float64_t **ptr_History,
+            int32_t   verb)
 {
-  DREAL *History;
-  DREAL *col_H;
-  DREAL *tmp_ptr;
-  DREAL x_old;
-  DREAL x_new;
-  DREAL delta_x;
-  DREAL max_x=CMath::INFTY;
-  DREAL xHx;
-  DREAL Q_P;
-  DREAL Q_D;
-  DREAL xf;
-  DREAL xi_sum;
-  DREAL max_update;
-  DREAL curr_update;
-  INT History_size;
-  INT t;
-  INT i, j;
-  INT max_i=-1;
-  INT exitflag;
-  INT KKTsatisf;
+  float64_t *History;
+  float64_t *col_H;
+  float64_t *tmp_ptr;
+  float64_t x_old;
+  float64_t x_new;
+  float64_t delta_x;
+  float64_t max_x=CMath::INFTY;
+  float64_t xHx;
+  float64_t Q_P;
+  float64_t Q_D;
+  float64_t xf;
+  float64_t xi_sum;
+  float64_t max_update;
+  float64_t curr_update;
+  int32_t History_size;
+  int32_t t;
+  int32_t i, j;
+  int32_t max_i=-1;
+  int32_t exitflag;
+  int32_t KKTsatisf;
 
   /* ------------------------------------------------------------ */
   /* Initialization                                               */
@@ -325,8 +326,8 @@ INT CQPBSVMLib::qpbsvm_scas(DREAL *x,
   t = 0;
 
   History_size = (m_tmax < HISTORY_BUF ) ? m_tmax+1 : HISTORY_BUF;
-  History=new DREAL[History_size*2];
-  memset(History, 0, sizeof(DREAL)*History_size*2);
+  History=new float64_t[History_size*2];
+  memset(History, 0, sizeof(float64_t)*History_size*2);
 
   /* compute Q_P and Q_D */
   xHx = 0;
@@ -377,7 +378,7 @@ INT CQPBSVMLib::qpbsvm_scas(DREAL *x,
     /* update Nabla */
     delta_x = max_x - x_old;
     if( delta_x != 0 ) {
-      col_H = (DREAL*)get_col(max_i);
+      col_H = (float64_t*)get_col(max_i);
       for(j = 0; j < m_dim; j++ ) {
         Nabla[j] += col_H[j]*delta_x;
       }
@@ -418,8 +419,8 @@ INT CQPBSVMLib::qpbsvm_scas(DREAL *x,
       History[INDEX(1,t,2)] = Q_D;
     }
     else {
-      tmp_ptr=new DREAL[(History_size+HISTORY_BUF)*2];
-      memset(tmp_ptr, 0, (History_size+HISTORY_BUF)*2*sizeof(DREAL));
+      tmp_ptr=new float64_t[(History_size+HISTORY_BUF)*2];
+      memset(tmp_ptr, 0, (History_size+HISTORY_BUF)*2*sizeof(float64_t));
       for( i = 0; i < History_size; i++ ) {
         tmp_ptr[INDEX(0,i,2)] = History[INDEX(0,i,2)];
         tmp_ptr[INDEX(1,i,2)] = History[INDEX(1,i,2)];
@@ -445,22 +446,22 @@ Usage: exitflag = qpbsvm_scamv(m_UB, m_dim, m_tmax,
                m_tolabs, m_tolrel, m_tolKKT, x, Nabla, &t, &History, verb )
 
 -------------------------------------------------------------- */
-INT CQPBSVMLib::qpbsvm_scamv(DREAL *x,
-	        DREAL *Nabla,
-            INT   *ptr_t,
-            DREAL **ptr_History,
-            INT   verb)
+int32_t CQPBSVMLib::qpbsvm_scamv(float64_t *x,
+	        float64_t *Nabla,
+            int32_t   *ptr_t,
+            float64_t **ptr_History,
+            int32_t   verb)
 {
-  DREAL *History;
-  DREAL *col_H;
-  DREAL delta_x;
-  DREAL x_new;
-  DREAL max_viol;
-  DREAL fval;
-  INT t;
-  INT i;
-  INT u=-1;
-  INT exitflag;
+  float64_t *History;
+  float64_t *col_H;
+  float64_t delta_x;
+  float64_t x_new;
+  float64_t max_viol;
+  float64_t fval;
+  int32_t t;
+  int32_t i;
+  int32_t u=-1;
+  int32_t exitflag;
 
   /* ------------------------------------------------------------ */
   /* Initialization                                               */
@@ -500,15 +501,15 @@ INT CQPBSVMLib::qpbsvm_scamv(DREAL *x,
       delta_x = x_new - x[u];
       x[u] = x_new;
 
-      col_H = (DREAL*)get_col(u);
+      col_H = (float64_t*)get_col(u);
       for(i = 0; i < m_dim; i++ ) {
         Nabla[i] += col_H[i]*delta_x;
       }
     }
   }
 
-  History=new DREAL[(t+1)*2];
-  memset(History, 0, sizeof(DREAL)*(t+1)*2);
+  History=new float64_t[(t+1)*2];
+  memset(History, 0, sizeof(float64_t)*(t+1)*2);
 
   fval = 0;
   for(fval = 0, i = 0; i < m_dim; i++ ) {
@@ -532,29 +533,29 @@ Usage: exitflag = qpbsvm_prloqo(m_UB, m_dim, m_tmax,
                m_tolabs, m_tolrel, m_tolKKT, x, Nabla, &t, &History, verb )
 
 -------------------------------------------------------------- */
-INT CQPBSVMLib::qpbsvm_prloqo(DREAL *x,
-	        DREAL *Nabla,
-            INT   *ptr_t,
-            DREAL **ptr_History,
-            INT   verb)
+int32_t CQPBSVMLib::qpbsvm_prloqo(float64_t *x,
+	        float64_t *Nabla,
+            int32_t   *ptr_t,
+            float64_t **ptr_History,
+            int32_t   verb)
 {
-	DREAL* lb=new DREAL[m_dim];
-	DREAL* ub=new DREAL[m_dim];
-	DREAL* primal=new DREAL[3*m_dim];
-	DREAL* dual=new DREAL[1+2*m_dim];
-	DREAL* a=new DREAL[m_dim];
+	float64_t* lb=new float64_t[m_dim];
+	float64_t* ub=new float64_t[m_dim];
+	float64_t* primal=new float64_t[3*m_dim];
+	float64_t* dual=new float64_t[1+2*m_dim];
+	float64_t* a=new float64_t[m_dim];
 
-	for (INT i=0; i<m_dim; i++)
+	for (int32_t i=0; i<m_dim; i++)
 	{
 		a[i]=0.0;
 		lb[i]=0;
 		ub[i]=m_UB;
 	}
 
-	DREAL b=0;
+	float64_t b=0;
 
 	CMath::display_vector(m_f, m_dim, "m_f");
-	INT result=pr_loqo(m_dim, 1, m_f, m_H, a, &b, lb, ub, primal, dual,
+	int32_t result=pr_loqo(m_dim, 1, m_f, m_H, a, &b, lb, ub, primal, dual,
 			2, 5, 1, -0.95, 10,0);
 
 	delete[] a;
@@ -568,18 +569,18 @@ INT CQPBSVMLib::qpbsvm_prloqo(DREAL *x,
 	return result;
 }
 
-INT CQPBSVMLib::qpbsvm_gauss_seidel(DREAL *x,
-	        DREAL *Nabla,
-            INT   *ptr_t,
-            DREAL **ptr_History,
-            INT   verb)
+int32_t CQPBSVMLib::qpbsvm_gauss_seidel(float64_t *x,
+	        float64_t *Nabla,
+            int32_t   *ptr_t,
+            float64_t **ptr_History,
+            int32_t   verb)
 {
-	for (INT i=0; i<m_dim; i++)
+	for (int32_t i=0; i<m_dim; i++)
 		x[i]=CMath::random(0.0, 1.0);
 
-	for (INT t=0; t<200; t++)
+	for (int32_t t=0; t<200; t++)
 	{
-		for (INT i=0; i<m_dim; i++)
+		for (int32_t i=0; i<m_dim; i++)
 		{
 			x[i]= (-m_f[i]-(CMath::dot(x,&m_H[m_dim*i], m_dim) -
 						m_H[m_dim*i+i]*x[i]))/m_H[m_dim*i+i];
@@ -587,45 +588,45 @@ INT CQPBSVMLib::qpbsvm_gauss_seidel(DREAL *x,
 		}
 	}
 
-	INT atbound=0;
-	for (INT i=0; i<m_dim; i++)
+	int32_t atbound=0;
+	for (int32_t i=0; i<m_dim; i++)
 	{
 		if (x[i]==0.0 || x[i]==1.0)
 			atbound++;
 	}
-	SG_PRINT("atbound:%d of %d (%2.2f%%)\n", atbound, m_dim, ((double) 100.0*atbound)/m_dim);
-	sparsity+=((double) 100.0*atbound)/m_dim;
+	SG_PRINT("atbound:%d of %d (%2.2f%%)\n", atbound, m_dim, ((float64_t) 100.0*atbound)/m_dim);
+	sparsity+=((float64_t) 100.0*atbound)/m_dim;
 	*ptr_t=0;
 	*ptr_History=NULL;
 	return 0;
 }
 
-INT CQPBSVMLib::qpbsvm_gradient_descent(DREAL *x,
-	        DREAL *Nabla,
-            INT   *ptr_t,
-            DREAL **ptr_History,
-            INT   verb)
+int32_t CQPBSVMLib::qpbsvm_gradient_descent(float64_t *x,
+	        float64_t *Nabla,
+            int32_t   *ptr_t,
+            float64_t **ptr_History,
+            int32_t   verb)
 {
-	for (INT i=0; i<m_dim; i++)
+	for (int32_t i=0; i<m_dim; i++)
 		x[i]=CMath::random(0.0, 1.0);
 
-	for (INT t=0; t<2000; t++)
+	for (int32_t t=0; t<2000; t++)
 	{
-		for (INT i=0; i<m_dim; i++)
+		for (int32_t i=0; i<m_dim; i++)
 		{
 			x[i]-=0.001*(CMath::dot(x,&m_H[m_dim*i], m_dim)+m_f[i]);
 			x[i]=CMath::clamp(x[i], 0.0, 1.0);
 		}
 	}
 
-	INT atbound=0;
-	for (INT i=0; i<m_dim; i++)
+	int32_t atbound=0;
+	for (int32_t i=0; i<m_dim; i++)
 	{
 		if (x[i]==0.0 || x[i]==1.0)
 			atbound++;
 	}
-	SG_PRINT("atbound:%d of %d (%2.2f%%)\n", atbound, m_dim, ((double) 100.0*atbound)/m_dim);
-	sparsity+=((double) 100.0*atbound)/m_dim;
+	SG_PRINT("atbound:%d of %d (%2.2f%%)\n", atbound, m_dim, ((float64_t) 100.0*atbound)/m_dim);
+	sparsity+=((float64_t) 100.0*atbound)/m_dim;
 	*ptr_t=0;
 	*ptr_History=NULL;
 	return 0;
@@ -638,16 +639,16 @@ Usage: exitflag = qpbsvm_prloqo(m_UB, m_dim, m_tmax,
                m_tolabs, m_tolrel, m_tolKKT, x, Nabla, &t, &History, verb )
 
 -------------------------------------------------------------- */
-INT CQPBSVMLib::qpbsvm_cplex(DREAL *x,
-	        DREAL *Nabla,
-            INT   *ptr_t,
-            DREAL **ptr_History,
-            INT   verb)
+int32_t CQPBSVMLib::qpbsvm_cplex(float64_t *x,
+	        float64_t *Nabla,
+            int32_t   *ptr_t,
+            float64_t **ptr_History,
+            int32_t   verb)
 {
-	DREAL* lb=new DREAL[m_dim];
-	DREAL* ub=new DREAL[m_dim];
+	float64_t* lb=new float64_t[m_dim];
+	float64_t* ub=new float64_t[m_dim];
 
-	for (INT i=0; i<m_dim; i++)
+	for (int32_t i=0; i<m_dim; i++)
 	{
 		lb[i]=0;
 		ub[i]=m_UB;
