@@ -41,7 +41,7 @@ void ssl_train(struct data *Data,
 	vector_int    *Subset  = new vector_int[1];
 	initialize(Subset,Data->m);
 	// call the right algorithm
-	int optimality = 0;
+	int32_t optimality = 0;
 	switch(Options->algo)
 	{
 		case -1:
@@ -71,82 +71,82 @@ void ssl_train(struct data *Data,
 	delete[] Subset->vec;
     delete[] Subset;
 	return;
-} 
-int CGLS(const struct data *Data, 
-		const struct options *Options, 
-		const struct vector_int *Subset, 
-		struct vector_double *Weights,
-		struct vector_double *Outputs)
+}
+
+int32_t CGLS(
+	const struct data *Data, const struct options *Options,
+	const struct vector_int *Subset, struct vector_double *Weights,
+	struct vector_double *Outputs)
 {
 	SG_SDEBUG("CGLS starting...");
 
 	/* Disassemble the structures */
-	int active = Subset->d;
-	int *J = Subset->vec;
-	CSparseFeatures<DREAL>* features=Data->features;
-	double *Y = Data->Y;
-	double *C = Data->C;
-	int n  = Data->n;
-	double lambda = Options->lambda;
-	int cgitermax = Options->cgitermax;
-	double epsilon = Options->epsilon;
-	double *beta = Weights->vec;
-	double *o  = Outputs->vec; 
+	int32_t active = Subset->d;
+	int32_t *J = Subset->vec;
+	CSparseFeatures<float64_t>* features=Data->features;
+	float64_t *Y = Data->Y;
+	float64_t *C = Data->C;
+	int32_t n  = Data->n;
+	float64_t lambda = Options->lambda;
+	int32_t cgitermax = Options->cgitermax;
+	float64_t epsilon = Options->epsilon;
+	float64_t *beta = Weights->vec;
+	float64_t *o  = Outputs->vec; 
 	// initialize z 
-	double *z = new double[active];
-	double *q = new double[active];
-	int ii=0;
-	for(int i = active ; i-- ;){
+	float64_t *z = new float64_t[active];
+	float64_t *q = new float64_t[active];
+	int32_t ii=0;
+	for (int32_t i = active ; i-- ;){
 		ii=J[i];      
 		z[i]  = C[ii]*(Y[ii] - o[ii]);
 	}
-	double *r = new double[n];
-	for(int i = n ; i-- ;)
+	float64_t *r = new float64_t[n];
+	for (int32_t i = n ; i-- ;)
 		r[i] = 0.0;
-	for(register int j=0; j < active; j++)
+	for (register int32_t j=0; j < active; j++)
 	{
 		ii=J[j];
-		INT num_entries=0;
+		int32_t num_entries=0;
 		bool free_vec=false;
 
-		TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
-		for (int i=0; i<num_entries; i++)
+		TSparseEntry<float64_t>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+		for (int32_t i=0; i<num_entries; i++)
 			r[vec[i].feat_index]+= vec[i].entry*z[j];
 		features->free_sparse_feature_vector(vec, num_entries, free_vec);
 		r[n-1]+=Options->bias*z[j]; //bias (modelled as last dim)
 	}
-	double *p = new double[n];   
-	double omega1 = 0.0;
-	for(int i = n ; i-- ;)
+	float64_t *p = new float64_t[n];   
+	float64_t omega1 = 0.0;
+	for (int32_t i = n ; i-- ;)
 	{
 		r[i] -= lambda*beta[i];
 		p[i] = r[i];
 		omega1 += r[i]*r[i];
 	}   
-	double omega_p = omega1;
-	double omega_q = 0.0;
-	double inv_omega2 = 1/omega1;
-	double scale = 0.0;
-	double omega_z=0.0;
-	double gamma = 0.0;
-	int cgiter = 0;
-	int optimality = 0;
-	double epsilon2 = epsilon*epsilon;   
+	float64_t omega_p = omega1;
+	float64_t omega_q = 0.0;
+	float64_t inv_omega2 = 1/omega1;
+	float64_t scale = 0.0;
+	float64_t omega_z=0.0;
+	float64_t gamma = 0.0;
+	int32_t cgiter = 0;
+	int32_t optimality = 0;
+	float64_t epsilon2 = epsilon*epsilon;   
 	// iterate
 	while(cgiter < cgitermax)
 	{
 		cgiter++;
 		omega_q=0.0;
-		double t=0.0;
-		register int i,j; 
+		float64_t t=0.0;
+		register int32_t i,j;
 		// #pragma omp parallel for private(i,j)
-		for(i=0; i < active; i++)
+		for (i=0; i < active; i++)
 		{
 			ii=J[i];
 			t=0.0;
-			INT num_entries=0;
+			int32_t num_entries=0;
 			bool free_vec=false;
-			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii,
+			TSparseEntry<float64_t>* vec=features->get_sparse_feature_vector(ii,
 					num_entries, free_vec);
 			for (j=0; j<num_entries; j++)
 				t+=vec[j].entry*p[vec[j].feat_index];
@@ -157,34 +157,34 @@ int CGLS(const struct data *Data,
 		}       
 		gamma = omega1/(lambda*omega_p + omega_q);    
 		inv_omega2 = 1/omega1;     
-		for(i = n ; i-- ;)
+		for (i = n ; i-- ;)
 		{
 			r[i] = 0.0;
 			beta[i] += gamma*p[i];
 		} 
 		omega_z=0.0;
-		for(i = active ; i-- ;)
+		for (i = active ; i-- ;)
 		{
 			ii=J[i];
 			o[ii] += gamma*q[i];
 			z[i] -= gamma*C[ii]*q[i];
 			omega_z+=z[i]*z[i];
 		} 
-		for(j=0; j < active; j++)
+		for (j=0; j < active; j++)
 		{
 			ii=J[j];
 			t=z[j];
-			INT num_entries=0;
+			int32_t num_entries=0;
 			bool free_vec=false;
 
-			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+			TSparseEntry<float64_t>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
 			for (i=0; i<num_entries; i++)
 				r[vec[i].feat_index]+= vec[i].entry*t;
 			features->free_sparse_feature_vector(vec, num_entries, free_vec);
 			r[n-1]+=Options->bias*t; //bias (modelled as last dim)
 		}
 		omega1 = 0.0;
-		for(i = n ; i-- ;)
+		for (i = n ; i-- ;)
 		{
 			r[i] -= lambda*beta[i];
 			omega1 += r[i]*r[i];
@@ -211,27 +211,27 @@ int CGLS(const struct data *Data,
 	delete[] p;
 	return optimality;
 }
-int L2_SVM_MFN(const struct data *Data, 
-		struct options *Options, 
-		struct vector_double *Weights,
-		struct vector_double *Outputs,
-		int ini)
-{ 
+
+int32_t L2_SVM_MFN(
+	const struct data *Data, struct options *Options,
+	struct vector_double *Weights, struct vector_double *Outputs,
+	int32_t ini)
+{
 	/* Disassemble the structures */  
-	CSparseFeatures<DREAL>* features=Data->features;
-	double *Y = Data->Y;
-	double *C = Data->C;
-	int n  = Data->n;
-	int m  = Data->m;
-	double lambda = Options->lambda;
-	double epsilon;
-	double *w = Weights->vec;
-	double *o = Outputs->vec; 
-	double F_old = 0.0;
-	double F = 0.0;
-	double diff=0.0;
+	CSparseFeatures<float64_t>* features=Data->features;
+	float64_t *Y = Data->Y;
+	float64_t *C = Data->C;
+	int32_t n  = Data->n;
+	int32_t m  = Data->m;
+	float64_t lambda = Options->lambda;
+	float64_t epsilon;
+	float64_t *w = Weights->vec;
+	float64_t *o = Outputs->vec; 
+	float64_t F_old = 0.0;
+	float64_t F = 0.0;
+	float64_t diff=0.0;
 	vector_int *ActiveSubset = new vector_int[1];
-	ActiveSubset->vec = new int[m];
+	ActiveSubset->vec = new int32_t[m];
 	ActiveSubset->d = m;
 	// initialize
 	if(ini==0) {
@@ -240,11 +240,11 @@ int L2_SVM_MFN(const struct data *Data,
 		Options->epsilon=BIG_EPSILON;
 	}
 	else {epsilon = Options->epsilon;}  
-	for(int i=0;i<n;i++) F+=w[i]*w[i];
+	for (int32_t i=0;i<n;i++) F+=w[i]*w[i];
 	F=0.5*lambda*F;        
-	int active=0;
-	int inactive=m-1; // l-1      
-	for(int i=0; i<m ; i++)
+	int32_t active=0;
+	int32_t inactive=m-1; // l-1      
+	for (int32_t i=0; i<m ; i++)
 	{ 
 		diff=1-Y[i]*o[i];
 		if(diff>0)
@@ -260,40 +260,40 @@ int L2_SVM_MFN(const struct data *Data,
 		}   
 	}
 	ActiveSubset->d=active;        
-	int iter=0;
-	int opt=0;
-	int opt2=0;
+	int32_t iter=0;
+	int32_t opt=0;
+	int32_t opt2=0;
 	vector_double *Weights_bar = new vector_double[1];
 	vector_double *Outputs_bar = new vector_double[1];
-	double *w_bar = new double[n];
-	double *o_bar = new double[m];
+	float64_t *w_bar = new float64_t[n];
+	float64_t *o_bar = new float64_t[m];
 	Weights_bar->vec=w_bar;
 	Outputs_bar->vec=o_bar;
 	Weights_bar->d=n;
 	Outputs_bar->d=m;
-	double delta=0.0;
-	double t=0.0;
-	int ii = 0;
+	float64_t delta=0.0;
+	float64_t t=0.0;
+	int32_t ii = 0;
 	while(iter<MFNITERMAX)
 	{
 		iter++;
 		SG_SDEBUG("L2_SVM_MFN Iteration# %d (%d active examples, objective_value = %f)\n", iter, active, F);
-		for(int i=n; i-- ;) 
+		for (int32_t i=n; i-- ;)
 			w_bar[i]=w[i];
-		for(int i=m; i-- ;)  
+		for (int32_t i=m; i-- ;)
 			o_bar[i]=o[i];
 		
 		opt=CGLS(Data,Options,ActiveSubset,Weights_bar,Outputs_bar);
-		for(register int i=active; i < m; i++) 
+		for(register int32_t i=active; i < m; i++) 
 		{
 			ii=ActiveSubset->vec[i];   
 			t=0.0;
 
-			INT num_entries=0;
+			int32_t num_entries=0;
 			bool free_vec=false;
 
-			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
-			for (int j=0; j<num_entries; j++)
+			TSparseEntry<float64_t>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+			for (int32_t j=0; j<num_entries; j++)
 				t+=vec[j].entry*w_bar[vec[j].feat_index];
 			features->free_sparse_feature_vector(vec, num_entries, free_vec);
 			t+=Options->bias*w_bar[n-1]; //bias (modelled as last dim)
@@ -302,7 +302,7 @@ int L2_SVM_MFN(const struct data *Data,
 		}
 		if(ini==0) {Options->cgitermax=CGITERMAX; ini=1;};
 		opt2=1;
-		for(int i=0;i<m;i++)
+		for (int32_t i=0;i<m;i++)
 		{ 
 			ii=ActiveSubset->vec[i];
 			if(i<active)
@@ -322,9 +322,9 @@ int L2_SVM_MFN(const struct data *Data,
 			}
 			else
 			{
-				for(int i=n; i-- ;) 
+				for (int32_t i=n; i-- ;)
 					w[i]=w_bar[i];      
-				for(int i=m; i-- ;)
+				for (int32_t i=m; i-- ;)
 					o[i]=o_bar[i]; 
 				delete[] ActiveSubset->vec;
 				delete[] ActiveSubset;
@@ -340,14 +340,14 @@ int L2_SVM_MFN(const struct data *Data,
 		SG_SDEBUG("LINE_SEARCH delta = %f\n", delta);
 		F_old=F;
 		F=0.0;
-		for(int i=n; i-- ;){ 
+		for (int32_t i=n; i-- ;) {
 			w[i]+=delta*(w_bar[i]-w[i]);
 			F+=w[i]*w[i];
 		}
 		F=0.5*lambda*F;      
 		active=0;
 		inactive=m-1;  
-		for(int i=0; i<m ; i++)
+		for (int32_t i=0; i<m ; i++)
 		{
 			o[i]+=delta*(o_bar[i]-o[i]);
 			diff=1-Y[i]*o[i];
@@ -380,20 +380,20 @@ int L2_SVM_MFN(const struct data *Data,
 	return 0;
 }
 
-double line_search(double *w, 
-		double *w_bar,
-		double lambda,
-		double *o, 
-		double *o_bar, 
-		double *Y, 
-		double *C,
-		int d, /* data dimensionality -- 'n' */
-		int l) /* number of examples */                  
+float64_t line_search(float64_t *w, 
+		float64_t *w_bar,
+		float64_t lambda,
+		float64_t *o, 
+		float64_t *o_bar, 
+		float64_t *Y, 
+		float64_t *C,
+		int32_t d, /* data dimensionality -- 'n' */
+		int32_t l) /* number of examples */
 {                       
-	double omegaL = 0.0;
-	double omegaR = 0.0;
-	double diff=0.0;   
-	for(int i=d; i--; )
+	float64_t omegaL = 0.0;
+	float64_t omegaR = 0.0;
+	float64_t diff=0.0;   
+	for(int32_t i=d; i--; )
 	{
 		diff=w_bar[i]-w[i];  
 		omegaL+=w[i]*diff;
@@ -401,10 +401,10 @@ double line_search(double *w,
 	}
 	omegaL=lambda*omegaL;
 	omegaR=lambda*omegaR;
-	double L=0.0;
-	double R=0.0;
-	int ii=0;
-	for(int i=0;i<l;i++)
+	float64_t L=0.0;
+	float64_t R=0.0;
+	int32_t ii=0;
+	for (int32_t i=0;i<l;i++)
 	{
 		if(Y[i]*o[i]<1)
 		{
@@ -416,8 +416,8 @@ double line_search(double *w,
 	L+=omegaL;
 	R+=omegaR;
 	Delta* deltas=new Delta[l];    
-	int p=0;
-	for(int i=0;i<l;i++)
+	int32_t p=0;
+	for(int32_t i=0;i<l;i++)
 	{ 
 		diff=Y[i]*(o_bar[i]-o[i]);
 
@@ -443,8 +443,8 @@ double line_search(double *w,
 		}
 	}
 	std::sort(deltas,deltas+p);            
-	double delta_prime=0.0;  
-	for(int i=0;i<p;i++)
+	float64_t delta_prime=0.0;  
+	for (int32_t i=0;i<p;i++)
 	{
 		delta_prime = L + deltas[i].delta*(R-L);       
 		if(delta_prime>=0)
@@ -456,11 +456,11 @@ double line_search(double *w,
 	}   
 	delete [] deltas;
 	return (-L/(R-L));
-} 
-int TSVM_MFN(const struct data *Data, 
-		struct options *Options, 
-		struct vector_double *Weights,
-		struct vector_double *Outputs)
+}
+
+int32_t TSVM_MFN(
+	const struct data *Data, struct options *Options,
+	struct vector_double *Weights, struct vector_double *Outputs)
 {
 	/* Setup labeled-only examples and train L2_SVM_MFN */
 	struct data *Data_Labeled = new data[1];
@@ -471,22 +471,22 @@ int TSVM_MFN(const struct data *Data,
 	L2_SVM_MFN(Data_Labeled, Options, Weights,Outputs_Labeled,0);
 	///FIXME Clear(Data_Labeled);
 	/* Use this weight vector to classify R*u unlabeled examples as
-	   positive*/   
-	int p=0,q=0; 
-	double t=0.0;
-	int *JU = new int[Data->u];
-	double *ou = new double[Data->u];
-	double lambda_0 = TSVM_LAMBDA_SMALL;
-	for(int i=0;i<Data->m;i++)
+	   positive*/
+	int32_t p=0,q=0;
+	float64_t t=0.0;
+	int32_t *JU = new int32_t[Data->u];
+	float64_t *ou = new float64_t[Data->u];
+	float64_t lambda_0 = TSVM_LAMBDA_SMALL;
+	for (int32_t i=0;i<Data->m;i++)
 	{
 		if(Data->Y[i]==0.0)
 		{
 			t=0.0;
-			INT num_entries=0;
+			int32_t num_entries=0;
 			bool free_vec=false;
 
-			TSparseEntry<DREAL>* vec=Data->features->get_sparse_feature_vector(i, num_entries, free_vec);
-			for (int j=0; j<num_entries; j++)
+			TSparseEntry<float64_t>* vec=Data->features->get_sparse_feature_vector(i, num_entries, free_vec);
+			for (int32_t j=0; j<num_entries; j++)
 				t+=vec[j].entry*Weights->vec[vec[j].feat_index];
 			Data->features->free_sparse_feature_vector(vec, num_entries, free_vec);
 			t+=Options->bias*Weights->vec[Data->n-1]; //bias (modelled as last dim)
@@ -504,27 +504,27 @@ int TSVM_MFN(const struct data *Data,
 			p++;   
 		}
 	}
-	std::nth_element(ou,ou+int((1-Options->R)*Data->u-1),ou+Data->u);
-	double thresh=*(ou+int((1-Options->R)*Data->u)-1);
+	std::nth_element(ou,ou+int32_t((1-Options->R)*Data->u-1),ou+Data->u);
+	float64_t thresh=*(ou+int32_t((1-Options->R)*Data->u)-1);
 	delete [] ou;
-	for(int i=0;i<Data->u;i++)
+	for (int32_t i=0;i<Data->u;i++)
 	{  
 		if(Outputs->vec[JU[i]]>thresh)
 			Data->Y[JU[i]]=1.0;
 		else
 			Data->Y[JU[i]]=-1.0;
 	}
-	for(int i=0;i<Data->n;i++)
+	for (int32_t i=0;i<Data->n;i++)
 		Weights->vec[i]=0.0;
-	for(int i=0;i<Data->m;i++)
+	for (int32_t i=0;i<Data->m;i++)
 		Outputs->vec[i]=0.0;
 	L2_SVM_MFN(Data,Options,Weights,Outputs,0); 
-	int num_switches=0;
-	int s=0;
-	int last_round=0;
+	int32_t num_switches=0;
+	int32_t s=0;
+	int32_t last_round=0;
 	while(lambda_0 <= Options->lambda_u)
-	{   
-		int iter2=0;
+	{
+		int32_t iter2=0;
 		while(1){
 			s=switch_labels(Data->Y,Outputs->vec,JU,Data->u,Options->S);
 			if(s==0) break;
@@ -538,7 +538,7 @@ int TSVM_MFN(const struct data *Data,
 		if(last_round==1) break;
 		lambda_0=TSVM_ANNEALING_RATE*lambda_0;
 		if(lambda_0 >= Options->lambda_u) {lambda_0 = Options->lambda_u; last_round=1;} 
-		for(int i=0;i<Data->u;i++)
+		for (int32_t i=0;i<Data->u;i++)
 			Data->C[JU[i]]=lambda_0*1.0/Data->u;       
 		SG_SDEBUG("****** lambda0 increased to %f%% of lambda_u = %f ************************\n", lambda_0*100/Options->lambda_u, Options->lambda_u);
 		SG_SDEBUG("Optimizing weights\n");
@@ -546,27 +546,28 @@ int TSVM_MFN(const struct data *Data,
 	}
 	SG_SDEBUG("Total Number of Switches = %d\n", num_switches);
 	/* reset labels */
-	for(int i=0;i<Data->u;i++) Data->Y[JU[i]] = 0.0;
-	double F = transductive_cost(norm_square(Weights),Data->Y,Outputs->vec,Outputs->d,Options->lambda,Options->lambda_u);
+	for (int32_t i=0;i<Data->u;i++) Data->Y[JU[i]] = 0.0;
+	float64_t F = transductive_cost(norm_square(Weights),Data->Y,Outputs->vec,Outputs->d,Options->lambda,Options->lambda_u);
 	SG_SDEBUG("Objective Value = %f\n",F);
 	delete [] JU;
 	return num_switches;
 }
-int switch_labels(double* Y, double* o, int* JU, int u, int S)
-{     
-	int npos=0;
-	int nneg=0;
-	for(int i=0;i<u;i++)
+
+int32_t switch_labels(float64_t* Y, float64_t* o, int32_t* JU, int32_t u, int32_t S)
+{
+	int32_t npos=0;
+	int32_t nneg=0;
+	for (int32_t i=0;i<u;i++)
 	{
 		if((Y[JU[i]]>0) && (o[JU[i]]<1.0)) npos++;   
 		if((Y[JU[i]]<0) && (-o[JU[i]]<1.0)) nneg++;        
 	}     
 	Delta* positive=new Delta[npos];
 	Delta* negative=new Delta[nneg];  
-	int p=0;
-	int n=0;
-	int ii=0;
-	for(int i=0;i<u;i++)
+	int32_t p=0;
+	int32_t n=0;
+	int32_t ii=0;
+	for (int32_t i=0;i<u;i++)
 	{
 		ii=JU[i];
 		if((Y[ii]>0.0) && (o[ii]<1.0)) {
@@ -583,7 +584,7 @@ int switch_labels(double* Y, double* o, int* JU, int u, int S)
 	}
 	std::sort(positive,positive+npos);
 	std::sort(negative,negative+nneg);  
-	int s=-1;
+	int32_t s=-1;
 	while(1)
 	{
 		s++;    
@@ -597,41 +598,40 @@ int switch_labels(double* Y, double* o, int* JU, int u, int S)
 	return s;
 }
 
-int DA_S3VM(struct data *Data, 
-		struct options *Options, 
-		struct vector_double *Weights,
-		struct vector_double *Outputs)
+int32_t DA_S3VM(
+	struct data *Data, struct options *Options, struct vector_double *Weights,
+	struct vector_double *Outputs)
 {
-	double T = DA_INIT_TEMP*Options->lambda_u;
-	int iter1 = 0, iter2 =0;
-	double *p = new double[Data->u];
-	double *q = new double[Data->u];
-	double *g = new double[Data->u];
-	double F,F_min;
-	double *w_min = new double[Data->n];
-	double *o_min = new double[Data->m];
-	double *w = Weights->vec;
-	double *o = Outputs->vec;
-	double kl_divergence = 1.0;
+	float64_t T = DA_INIT_TEMP*Options->lambda_u;
+	int32_t iter1 = 0, iter2 =0;
+	float64_t *p = new float64_t[Data->u];
+	float64_t *q = new float64_t[Data->u];
+	float64_t *g = new float64_t[Data->u];
+	float64_t F,F_min;
+	float64_t *w_min = new float64_t[Data->n];
+	float64_t *o_min = new float64_t[Data->m];
+	float64_t *w = Weights->vec;
+	float64_t *o = Outputs->vec;
+	float64_t kl_divergence = 1.0;
 	/*initialize */
 	SG_SDEBUG("Initializing weights, p");
-	for(int i=0;i<Data->u; i++)
+	for (int32_t i=0;i<Data->u; i++)
 		p[i] = Options->R;
 	/* record which examples are unlabeled */
-	int *JU = new int[Data->u];
-	int j=0;
-	for(int i=0;i<Data->m;i++)
+	int32_t *JU = new int32_t[Data->u];
+	int32_t j=0;
+	for(int32_t i=0;i<Data->m;i++)
 	{
 		if(Data->Y[i]==0.0)
 		{JU[j]=i;j++;}
 	}  
-	double H = entropy(p,Data->u);
+	float64_t H = entropy(p,Data->u);
 	optimize_w(Data,p,Options,Weights,Outputs,0);  
 	F = transductive_cost(norm_square(Weights),Data->Y,Outputs->vec,Outputs->d,Options->lambda,Options->lambda_u);
 	F_min = F;
-	for(int i=0;i<Weights->d;i++)
+	for (int32_t i=0;i<Weights->d;i++)
 		w_min[i]=w[i];
-	for(int i=0;i<Outputs->d;i++)
+	for (int32_t i=0;i<Outputs->d;i++)
 		o_min[i]=o[i];
 	while((iter1 < DA_OUTER_ITERMAX) && (H > Options->epsilon))
 	{
@@ -641,7 +641,7 @@ int DA_S3VM(struct data *Data,
 		while((iter2 < DA_INNER_ITERMAX) && (kl_divergence > Options->epsilon)) 
 		{
 			iter2++;
-			for(int i=0;i<Data->u;i++)
+			for (int32_t i=0;i<Data->u;i++)
 			{
 				q[i]=p[i];
 				g[i] = Options->lambda_u*((o[JU[i]] > 1 ? 0 : (1 - o[JU[i]])*(1 - o[JU[i]])) - (o[JU[i]]< -1 ? 0 : (1 + o[JU[i]])*(1 + o[JU[i]]))); 
@@ -655,9 +655,9 @@ int DA_S3VM(struct data *Data,
 			if(F < F_min)
 			{
 				F_min = F;
-				for(int i=0;i<Weights->d;i++)
+				for (int32_t i=0;i<Weights->d;i++)
 					w_min[i]=w[i];
-				for(int i=0;i<Outputs->d;i++)
+				for (int32_t i=0;i<Outputs->d;i++)
 					o_min[i]=o[i];
 			}
 			SG_SDEBUG("***** outer_iter = %d  T = %g  inner_iter = %d  kl = %g  cost = %g *****\n",iter1,T,iter2,kl_divergence,F); 
@@ -666,9 +666,9 @@ int DA_S3VM(struct data *Data,
 		SG_SDEBUG("***** Finished outer_iter = %d T = %g  Entropy = %g ***\n", iter1,T,H);
 		T = T/DA_ANNEALING_RATE;
 	}
-	for(int i=0;i<Weights->d;i++)
+	for (int32_t i=0;i<Weights->d;i++)
 		w[i]=w_min[i];
-	for(int i=0;i<Outputs->d;i++)
+	for (int32_t i=0;i<Outputs->d;i++)
 		o[i]=o_min[i];
 	/* may want to reset the original Y */ 
 	delete [] p; 
@@ -680,31 +680,29 @@ int DA_S3VM(struct data *Data,
 	SG_SINFO("(min) Objective Value = %f", F_min);
 	return 1;
 }
-int optimize_w(const struct data *Data, 
-		const double *p,
-		struct options *Options, 
-		struct vector_double *Weights,
-		struct vector_double *Outputs,
-		int ini)
-{ 
-	int i,j;
-	CSparseFeatures<DREAL>* features=Data->features;
-	int n  = Data->n;
-	int m  = Data->m;
-	int u  = Data->u;
-	double lambda = Options->lambda;
-	double epsilon;
-	double *w = Weights->vec;
-	double *o = new double[m+u];
-	double *Y = new double[m+u];
-	double *C = new double[m+u];
-	int *labeled_indices = new int[m];
-	double F_old = 0.0;
-	double F = 0.0;
-	double diff=0.0;
-	double lambda_u_by_u = Options->lambda_u/u;
+
+int32_t optimize_w(
+	const struct data *Data, const float64_t *p, struct options *Options,
+	struct vector_double *Weights, struct vector_double *Outputs, int32_t ini)
+{
+	int32_t i,j;
+	CSparseFeatures<float64_t>* features=Data->features;
+	int32_t n  = Data->n;
+	int32_t m  = Data->m;
+	int32_t u  = Data->u;
+	float64_t lambda = Options->lambda;
+	float64_t epsilon;
+	float64_t *w = Weights->vec;
+	float64_t *o = new float64_t[m+u];
+	float64_t *Y = new float64_t[m+u];
+	float64_t *C = new float64_t[m+u];
+	int32_t *labeled_indices = new int32_t[m];
+	float64_t F_old = 0.0;
+	float64_t F = 0.0;
+	float64_t diff=0.0;
+	float64_t lambda_u_by_u = Options->lambda_u/u;
 	vector_int *ActiveSubset = new vector_int[1];
-	ActiveSubset->vec = new int[m];
+	ActiveSubset->vec = new int32_t[m];
 	ActiveSubset->d = m;
 	// initialize
 	if(ini==0) 
@@ -716,10 +714,10 @@ int optimize_w(const struct data *Data,
 
 	for(i=0;i<n;i++) F+=w[i]*w[i];
 	F=lambda*F;        
-	int active=0;
-	int inactive=m-1; // l-1      
-	double temp1;
-	double temp2;
+	int32_t active=0;
+	int32_t inactive=m-1; // l-1      
+	float64_t temp1;
+	float64_t temp2;
 
 	j = 0;
 	for(i=0; i<m ; i++)
@@ -783,20 +781,20 @@ int optimize_w(const struct data *Data,
 	}
 	F=0.5*F;
 	ActiveSubset->d=active;
-	int iter=0;
-	int opt=0;
-	int opt2=0;
+	int32_t iter=0;
+	int32_t opt=0;
+	int32_t opt2=0;
 	vector_double *Weights_bar = new vector_double[1];
 	vector_double *Outputs_bar = new vector_double[1];
-	double *w_bar = new double[n];
-	double *o_bar = new double[m+u];
+	float64_t *w_bar = new float64_t[n];
+	float64_t *o_bar = new float64_t[m+u];
 	Weights_bar->vec=w_bar;
 	Outputs_bar->vec=o_bar;
 	Weights_bar->d=n;
 	Outputs_bar->d=m; /* read only the top m ; bottom u will be copies */
-	double delta=0.0;
-	double t=0.0;
-	int ii = 0;
+	float64_t delta=0.0;
+	float64_t t=0.0;
+	int32_t ii = 0;
 	while(iter<MFNITERMAX)
 	{
 		iter++;
@@ -811,10 +809,10 @@ int optimize_w(const struct data *Data,
 		{
 			ii=ActiveSubset->vec[i];   
 			t=0.0;
-			INT num_entries=0;
+			int32_t num_entries=0;
 			bool free_vec=false;
 
-			TSparseEntry<DREAL>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
+			TSparseEntry<float64_t>* vec=features->get_sparse_feature_vector(ii, num_entries, free_vec);
 			for (j=0; j<num_entries; j++)
 				t+=vec[j].entry*w_bar[vec[j].feat_index];
 			features->free_sparse_feature_vector(vec, num_entries, free_vec);
@@ -968,28 +966,30 @@ int optimize_w(const struct data *Data,
 	SG_SINFO("L2_SVM_MFN converged in %d iterations", iter);
 	return 0;
 }
-void optimize_p(const double* g, int u, double T, double r, double* p)
+
+void optimize_p(
+	const float64_t* g, int32_t u, float64_t T, float64_t r, float64_t* p)
 {
-	int iter=0;
-	double epsilon=1e-10;
-	int maxiter=500; 
-	double nu_minus=g[0];
-	double nu_plus=g[0];
-	for(int i=0;i<u;i++)
+	int32_t iter=0;
+	float64_t epsilon=1e-10;
+	int32_t maxiter=500; 
+	float64_t nu_minus=g[0];
+	float64_t nu_plus=g[0];
+	for (int32_t i=0;i<u;i++)
 	{
 		if(g[i]<nu_minus) nu_minus=g[i]; 
 		if(g[i]>nu_plus) nu_plus=g[i];
 	};
 
-	double b=T*log((1-r)/r);
+	float64_t b=T*log((1-r)/r);
 	nu_minus-=b;
 	nu_plus-=b;
-	double nu=(nu_plus+nu_minus)/2;
-	double Bnu=0.0;
-	double BnuPrime=0.0;
-	double s=0.0;
-	double tmp=0.0;
-	for(int i=0;i<u;i++)
+	float64_t nu=(nu_plus+nu_minus)/2;
+	float64_t Bnu=0.0;
+	float64_t BnuPrime=0.0;
+	float64_t s=0.0;
+	float64_t tmp=0.0;
+	for (int32_t i=0;i<u;i++)
 	{
 		s=exp((g[i]-nu)/T);
 		if(!(isinf(s)))
@@ -1002,7 +1002,7 @@ void optimize_p(const double* g, int u, double T, double r, double* p)
 	Bnu=Bnu/u;
 	Bnu-=r;
 	BnuPrime=BnuPrime/(T*u);
-	double nuHat=0.0;
+	float64_t nuHat=0.0;
 	while((CMath::abs(Bnu)>epsilon) && (iter < maxiter))
 	{
 		iter++;
@@ -1014,7 +1014,7 @@ void optimize_p(const double* g, int u, double T, double r, double* p)
 			nu=nuHat; 
 		Bnu=0.0;
 		BnuPrime=0.0;
-		for(int i=0;i<u;i++)
+		for(int32_t i=0;i<u;i++)
 		{
 			s=exp((g[i]-nu)/T);
 			if(!(isinf(s)))
@@ -1037,7 +1037,7 @@ void optimize_p(const double* g, int u, double T, double r, double* p)
 	if(CMath::abs(Bnu)>epsilon)
 		SG_SWARNING("Warning (Root): root not found to required precision\n");
 
-	for(int i=0;i<u;i++)
+	for (int32_t i=0;i<u;i++)
 	{
 		s=exp((g[i]-nu)/T);
 		if(isinf(s)) p[i]=0.0;
@@ -1045,11 +1045,14 @@ void optimize_p(const double* g, int u, double T, double r, double* p)
 	}
 	SG_SINFO(" root (nu) = %f B(nu) = %f", nu, Bnu);
 }
-double transductive_cost(double normWeights,double *Y, double *Outputs, int m, double lambda,double lambda_u)
+
+float64_t transductive_cost(
+	float64_t normWeights, float64_t *Y, float64_t *Outputs, int32_t m,
+	float64_t lambda, float64_t lambda_u)
 {
-	double F1=0.0,F2=0.0, o=0.0, y=0.0; 
-	int u=0,l=0;
-	for(int i=0;i<m;i++)
+	float64_t F1=0.0,F2=0.0, o=0.0, y=0.0; 
+	int32_t u=0,l=0;
+	for (int32_t i=0;i<m;i++)
 	{
 		o=Outputs[i];
 		y=Y[i];
@@ -1058,16 +1061,16 @@ double transductive_cost(double normWeights,double *Y, double *Outputs, int m, d
 		else
 		{F2 += y*o > 1 ? 0 : (1-y*o)*(1-y*o); l++;}   
 	}
-	double F;
+	float64_t F;
 	F = 0.5*(lambda*normWeights + lambda_u*F1/u + F2/l);
 	return F;
 }
 
-double entropy(const double *p, int u)
+float64_t entropy(const float64_t *p, int32_t u)
 {
-	double h=0.0;
-	double q=0.0; 
-	for(int i=0;i<u;i++)
+	float64_t h=0.0;
+	float64_t q=0.0;
+	for (int32_t i=0;i<u;i++)
 	{
 		q=p[i];
 		if(q>0 && q<1)
@@ -1075,13 +1078,14 @@ double entropy(const double *p, int u)
 	}
 	return h/u;
 }
-double KL(const double *p, const double *q, int u)
+
+float64_t KL(const float64_t *p, const float64_t *q, int32_t u)
 {
-	double h=0.0;
-	double p1=0.0;
-	double q1=0.0;
-	double g=0.0;
-	for(int i=0;i<u;i++)
+	float64_t h=0.0;
+	float64_t p1=0.0;
+	float64_t q1=0.0;
+	float64_t g=0.0;
+	for (int32_t i=0;i<u;i++)
 	{
 		p1=p[i];
 		q1=q[i];
@@ -1095,45 +1099,49 @@ double KL(const double *p, const double *q, int u)
 	}
 	return h/u;   
 }
+
 /********************** UTILITIES ********************/
-double norm_square(const vector_double *A)
+float64_t norm_square(const vector_double *A)
 {
-	double x=0.0, t=0.0;
-	for(int i=0;i<A->d;i++)
+	float64_t x=0.0, t=0.0;
+	for(int32_t i=0;i<A->d;i++)
 	{
 		t=A->vec[i];
 		x+=t*t;
 	}
 	return x;
-} 
-void initialize(struct vector_double *A, int k, double a)
+}
+
+void initialize(struct vector_double *A, int32_t k, float64_t a)
 {
-	double *vec = new double[k];
-	for(int i=0;i<k;i++)
+	float64_t *vec = new float64_t[k];
+	for (int32_t i=0;i<k;i++)
 		vec[i]=a;
 	A->vec = vec;
 	A->d   = k;
 	return;
 }
-void initialize(struct vector_int *A, int k)
-{  
-	int *vec = new int[k];
-	for(int i=0;i<k;i++)
+
+void initialize(struct vector_int *A, int32_t k)
+{
+	int32_t *vec = new int32_t[k];
+	for(int32_t i=0;i<k;i++)
 		vec[i]=i; 
 	A->vec = vec;
 	A->d   = k;
 	return;
 }
+
 void GetLabeledData(struct data *D, const struct data *Data)
 {
 	/*FIXME
-	int *J = new int[Data->l];
-	D->C   = new double[Data->l];
-	D->Y   = new double[Data->l];
-	int nz=0;
-	int k=0;
-	int rowptrs_=Data->l;
-	for(int i=0;i<Data->m;i++)
+	int32_t *J = new int[Data->l];
+	D->C   = new float64_t[Data->l];
+	D->Y   = new float64_t[Data->l];
+	int32_t nz=0;
+	int32_t k=0;
+	int32_t rowptrs_=Data->l;
+	for(int32_t i=0;i<Data->m;i++)
 	{
 		if(Data->Y[i]!=0.0)
 		{
@@ -1144,14 +1152,14 @@ void GetLabeledData(struct data *D, const struct data *Data)
 			k++;
 		}
 	}  
-	D->val    = new double[nz];
-	D->colind = new int[nz]; 
-	D->rowptr = new int[rowptrs_+1];
+	D->val    = new float64_t[nz];
+	D->colind = new int32_t[nz]; 
+	D->rowptr = new int32_trowptrs_+1];
 	nz=0;
-	for(int i=0;i<Data->l;i++)
+	for(int32_t i=0;i<Data->l;i++)
 	{
 		D->rowptr[i]=nz;
-		for(int j=Data->rowptr[J[i]];j<Data->rowptr[J[i]+1];j++)
+		for(int32_t j=Data->rowptr[J[i]];j<Data->rowptr[J[i]+1];j++)
 		{
 			D->val[nz] = Data->val[j];
 			D->colind[nz] = Data->colind[j];
