@@ -30,7 +30,7 @@ CKMeans::CKMeans()
 {
 }
 
-CKMeans::CKMeans(INT k_, CDistance* d)
+CKMeans::CKMeans(int32_t k_, CDistance* d)
 : CDistanceMachine(), max_iter(10000), k(k_), dimensions(0), R(NULL),
 	mus(NULL), Weights(NULL)
 {
@@ -50,10 +50,10 @@ bool CKMeans::train()
 	ASSERT(distance->get_distance_type()==D_EUCLIDIAN);
 	CRealFeatures* lhs=(CRealFeatures*) distance->get_lhs();
 	ASSERT(lhs);
-	INT num=lhs->get_num_vectors();
+	int32_t num=lhs->get_num_vectors();
 
-	Weights=new DREAL[num];
-	for (INT i=0; i<num; i++)
+	Weights=new float64_t[num];
+	for (int32_t i=0; i<num; i++)
 		Weights[i]=1.0;
 
 	clustknb(false, NULL);
@@ -75,21 +75,21 @@ bool CKMeans::save(FILE* dstfile)
 
 struct thread_data
 {
-	double* x;
+	float64_t* x;
 	CRealFeatures* y;
-	double* z;
-	int n1, n2, m ; 
-	int js, je ; /* defines the matrix stripe */
-	int offs;
+	float64_t* z;
+	int32_t n1, n2, m;
+	int32_t js, je; /* defines the matrix stripe */
+	int32_t offs;
 };
 
-void *sqdist_thread_func(void * P) 
+void *sqdist_thread_func(void * P)
 {
 	struct thread_data *TD=(struct thread_data*) P;
-	double* x=TD->x;
+	float64_t* x=TD->x;
 	CRealFeatures* y=TD->y;
-	double* z=TD->z;
-	int n1=TD->n1, 
+	float64_t* z=TD->z;
+	int32_t n1=TD->n1,
 		m=TD->m,
 		js=TD->js,
 		je=TD->je,
@@ -98,13 +98,13 @@ void *sqdist_thread_func(void * P)
 
 	for (j=js; j<je; j++)
 	{
-		INT vlen=0;
+		int32_t vlen=0;
 		bool vfree=false;
-		double* vec=y->get_feature_vector(j+offs, vlen, vfree);
+		float64_t* vec=y->get_feature_vector(j+offs, vlen, vfree);
 
 		for (i=0; i<n1; i++)
 		{
-			double sum=0;
+			float64_t sum=0;
 			for (k=0; k<m; k++) 
 				sum = sum + CMath::sq(x[i*m + k] - vec[k]);
 			z[j*n1 + i] = sum;
@@ -115,11 +115,12 @@ void *sqdist_thread_func(void * P)
 	return NULL;
 } 
 
-void CKMeans::sqdist(double * x, CRealFeatures* y, double *z,
-		int n1, int offs, int n2, int m)
+void CKMeans::sqdist(
+	float64_t* x, CRealFeatures* y, float64_t* z, int32_t n1, int32_t offs,
+	int32_t n2, int32_t m)
 {
-	const int num_threads=parallel.get_num_threads();
-	int nc, n2_nc = n2/num_threads;
+	const int32_t num_threads=parallel.get_num_threads();
+	int32_t nc, n2_nc = n2/num_threads;
 	struct thread_data TD[num_threads];
 	pthread_t tid[num_threads];
 	void *status;
@@ -159,32 +160,32 @@ void CKMeans::sqdist(double * x, CRealFeatures* y, double *z,
 	}
 }
 
-void CKMeans::clustknb(bool use_old_mus, double *mus_start)
+void CKMeans::clustknb(bool use_old_mus, float64_t *mus_start)
 {
 	ASSERT(distance && distance->get_feature_type()==F_DREAL);
 	CRealFeatures* lhs = (CRealFeatures*) distance->get_lhs();
 	ASSERT(lhs && lhs->get_num_features()>0 && lhs->get_num_vectors()>0);
-	
-	int XSize=lhs->get_num_vectors();
+
+	int32_t XSize=lhs->get_num_vectors();
 	dimensions=lhs->get_num_features();
-	int i, changed=1;
-	const int XDimk=dimensions*k;
-	int iter=0;
+	int32_t i, changed=1;
+	const int32_t XDimk=dimensions*k;
+	int32_t iter=0;
 
 	delete[] R;
-	R=new DREAL[k];
+	R=new float64_t[k];
 
 	delete[] mus;
-	mus=new DREAL[XDimk];
+	mus=new float64_t[XDimk];
 
-	int *ClList = (int*) calloc(XSize, sizeof(int));
-	double *weights_set = (double*) calloc(k, sizeof(double));
-	double *oldmus = (double*) calloc(XDimk, sizeof(double));
-	double *dists = (double*) calloc(k*XSize, sizeof(double));
+	int32_t *ClList = (int32_t*) calloc(XSize, sizeof(int32_t));
+	float64_t *weights_set = (float64_t*) calloc(k, sizeof(float64_t));
+	float64_t *oldmus = (float64_t*) calloc(XDimk, sizeof(float64_t));
+	float64_t *dists = (float64_t*) calloc(k*XSize, sizeof(float64_t));
 
-	INT vlen=0;
+	int32_t vlen=0;
 	bool vfree=false;
-	double* vec=NULL;
+	float64_t* vec=NULL;
 
 	/* ClList=zeros(XSize,1) ; */
 	for (i=0; i<XSize; i++) ClList[i]=0;
@@ -216,9 +217,9 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 
 		for (i=0; i<XSize; i++) 
 		{
-			const int Cl= (int) ( CMath::random(0, k-1) );
-			int j;
-			double weight=Weights[i];
+			const int32_t Cl=CMath::random(0, k-1);
+			int32_t j;
+			float64_t weight=Weights[i];
 
 			weights_set[Cl]+=weight;
 			ClList[i]=Cl;
@@ -232,7 +233,7 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 		}
 		for (i=0; i<k; i++)
 		{
-			int j;
+			int32_t j;
 
 			if (weights_set[i]!=0.0)
 				for (j=0; j<dimensions; j++)
@@ -247,8 +248,8 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 
 		for (i=0; i<XSize; i++)
 		{
-			double mini=dists[i*k];
-			int Cl = 0, j;
+			float64_t mini=dists[i*k];
+			int32_t Cl = 0, j;
 
 			for (j=1; j<k; j++)
 			{
@@ -265,8 +266,8 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 		 * and count the points */
 		for (i=0; i<XSize; i++) 
 		{
-			const int Cl = ClList[i];
-			double weight=Weights[i];
+			const int32_t Cl = ClList[i];
+			float64_t weight=Weights[i];
 			weights_set[Cl]+=weight;
 #ifndef MUSRECALC
 			vec=lhs->get_feature_vector(i, vlen, vfree);
@@ -283,7 +284,7 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 		{
 			if (weights_set[i]!=0.0)
 			{
-				int j;
+				int32_t j;
 				for (j=0; j<dimensions; j++)
 					mus[i*dimensions+j] /= weights_set[i];
 			}
@@ -309,9 +310,9 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 
 		for (i=0; i<XSize; i++) 
 		{
-			int j;
-			int Cl=ClList[i];
-			double weight=Weights[i];
+			int32_t j;
+			int32_t Cl=ClList[i];
+			float64_t weight=Weights[i];
 
 			vec=lhs->get_feature_vector(i, vlen, vfree);
 
@@ -322,7 +323,7 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 		}
 		for (i=0; i<k; i++)
 		{
-			int j;
+			int32_t j;
 
 			if (weights_set[i]!=0.0)
 				for (j=0; j<dimensions; j++)
@@ -333,10 +334,10 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 		for (i=0; i<XSize; i++)
 		{
 			/* ks=ceil(rand(1,XSize)*XSize) ; */
-			const int Pat= CMath::random(0, XSize-1);
-			const int ClList_Pat=ClList[Pat];
-			int imini, j;
-			double mini, weight;
+			const int32_t Pat= CMath::random(0, XSize-1);
+			const int32_t ClList_Pat=ClList[Pat];
+			int32_t imini, j;
+			float64_t mini, weight;
 
 			weight=Weights[Pat];
 
@@ -396,17 +397,17 @@ void CKMeans::clustknb(bool use_old_mus, double *mus_start)
 	/* compute the ,,variances'' of the clusters */
 	for (i=0; i<k; i++)
 	{
-		double rmin1=0;
-		double rmin2=0;
+		float64_t rmin1=0;
+		float64_t rmin2=0;
 
 		bool first_round=true;
 
-		for (INT j=0; j<k; j++) 
+		for (int32_t j=0; j<k; j++)
 		{
 			if (j!=i)
 			{
-				int l;
-				double dist = 0;
+				int32_t l;
+				float64_t dist = 0;
 
 				for (l=0; l<dimensions; l++)
 					dist+=CMath::sq(mus[i*dimensions+l]-mus[j*dimensions+l]);
