@@ -21,7 +21,18 @@
 
 class CDistanceMachine;
 
-/** class KMeans */
+/** KMeans clustering,  partitions the data into k (a-priori specified) clusters.
+ * It minimizes
+ * \f[
+ *  \sum_{i=1}^k\sum_{x_j\in S_i} (x_j-\mu_i)^2
+ * \f]
+ *
+ * where \f$\mu_i\f$ are the cluster centers and \f$S_i,\;i=1,\dots,k\f$ are the index
+ * sets of the clusters.
+ *
+ * Beware that this algorithm obtains only a <em>local</em> optimum.
+ *
+ * cf. http://en.wikipedia.org/wiki/K-means_algorithm */
 class CKMeans : public CDistanceMachine
 {
 	public:
@@ -33,7 +44,7 @@ class CKMeans : public CDistanceMachine
 		 * @param k parameter k
 		 * @param d distance
 		 */
-		CKMeans(INT k, CDistance* d);
+		CKMeans(int32_t k, CDistance* d);
 		virtual ~CKMeans();
 
 		/** get classifier type
@@ -66,7 +77,7 @@ class CKMeans : public CDistanceMachine
 		 *
 		 * @param p_k new k
 		 */
-		inline void set_k(INT p_k)
+		inline void set_k(int32_t p_k)
 		{
 			ASSERT(p_k>0);
 			this->k=p_k;
@@ -76,7 +87,7 @@ class CKMeans : public CDistanceMachine
 		 *
 		 * @return the parameter k
 		 */
-		inline INT get_k()
+		inline int32_t get_k()
 		{
 			return k;
 		}
@@ -85,7 +96,7 @@ class CKMeans : public CDistanceMachine
 		 *
 		 * @param iter the new maximum
 		 */
-		inline void set_max_iter(INT iter)
+		inline void set_max_iter(int32_t iter)
 		{
 			ASSERT(iter>0);
 			max_iter=iter;
@@ -95,7 +106,7 @@ class CKMeans : public CDistanceMachine
 		 *
 		 * @return maximum number of iterations
 		 */
-		inline DREAL get_max_iter()
+		inline float64_t get_max_iter()
 		{
 			return max_iter;
 		}
@@ -105,7 +116,7 @@ class CKMeans : public CDistanceMachine
 		 * @param radi current radi are stored in here
 		 * @param num number of radi is stored in here
 		 */
-		inline void get_radi(DREAL*& radi, INT& num)
+		inline void get_radi(float64_t*& radi, int32_t& num)
 		{
 			radi=R;
 			num=k;
@@ -117,7 +128,7 @@ class CKMeans : public CDistanceMachine
 		 * @param dim dimensions are stored in here
 		 * @param num number of centers is stored in here
 		 */
-		inline void get_centers(DREAL*& centers, INT& dim, INT& num)
+		inline void get_centers(float64_t*& centers, int32_t& dim, int32_t& num)
 		{
 			centers=mus;
 			dim=dimensions;
@@ -129,10 +140,10 @@ class CKMeans : public CDistanceMachine
 		 * @param radii current radiuses are stored in here
 		 * @param num number of radiuses is stored in here
 		 */
-		inline void get_radiuses(DREAL** radii, INT* num)
+		inline void get_radiuses(float64_t** radii, int32_t* num)
 		{
 			size_t sz=sizeof(*R)*k;
-			*radii=(DREAL*) malloc(sz);
+			*radii=(float64_t*) malloc(sz);
 			ASSERT(*radii);
 
 			memcpy(*radii, R, sz);
@@ -145,10 +156,11 @@ class CKMeans : public CDistanceMachine
 		 * @param dim dimensions are stored in here
 		 * @param num number of centers is stored in here
 		 */
-		inline void get_cluster_centers(DREAL** centers, INT* dim, INT* num)
+		inline void get_cluster_centers(
+			float64_t** centers, int32_t* dim, int32_t* num)
 		{
 			size_t sz=sizeof(*mus)*dimensions*k;
-			*centers=(DREAL*) malloc(sz);
+			*centers=(float64_t*) malloc(sz);
 			ASSERT(*centers);
 
 			memcpy(*centers, mus, sz);
@@ -160,7 +172,7 @@ class CKMeans : public CDistanceMachine
 		 *
 		 * @return number of dimensions
 		 */
-		inline INT get_dimensions()
+		inline int32_t get_dimensions()
 		{
 			return dimensions;
 		}
@@ -177,35 +189,36 @@ class CKMeans : public CDistanceMachine
 		 * @param n2 n2
 		 * @param m m
 		 */
-		void sqdist(double * x, CRealFeatures* y, double *z,
-				int n1, int offs, int n2, int m);
+		void sqdist(
+			float64_t* x, CRealFeatures* y, float64_t *z, int32_t n1,
+			int32_t offs, int32_t n2, int32_t m);
 
 		/** clustknb
 		 *
 		 * @param use_old_mus if old mus shall be used
 		 * @param mus_start mus start
 		 */
-		void clustknb(bool use_old_mus, double *mus_start);
+		void clustknb(bool use_old_mus, float64_t *mus_start);
 
 	protected:
 		/// maximum number of iterations
-		INT max_iter;
+		int32_t max_iter;
 
 		/// the k parameter in KMeans
-		INT k;
+		int32_t k;
 
 		/// number of dimensions
-		INT dimensions;
+		int32_t dimensions;
 
 		/// radi of the clusters (size k)
-		DREAL* R;
+		float64_t* R;
 		
 		/// centers of the clusters (size dimensions x k)
-		DREAL* mus;
+		float64_t* mus;
 
 	private:
 		/// weighting over the train data
-		DREAL* Weights;
+		float64_t* Weights;
 };
 #endif
 
