@@ -16,20 +16,21 @@
 #include "lib/io.h"
 
 CCanberraWordDistance::CCanberraWordDistance()
-: CStringDistance<WORD>()
+: CStringDistance<uint16_t>()
 {
 	SG_DEBUG("CCanberraWordDistance created");
-	dictionary_size= 1<<(sizeof(WORD)*8);
-	dictionary_weights = new DREAL[dictionary_size];
+	dictionary_size= 1<<(sizeof(uint16_t)*8);
+	dictionary_weights = new float64_t[dictionary_size];
 	SG_DEBUG( "using dictionary of %d bytes\n", dictionary_size);
 }
 
-CCanberraWordDistance::CCanberraWordDistance(CStringFeatures<WORD>* l, CStringFeatures<WORD>* r)
-: CStringDistance<WORD>()
+CCanberraWordDistance::CCanberraWordDistance(
+	CStringFeatures<uint16_t>* l, CStringFeatures<uint16_t>* r)
+: CStringDistance<uint16_t>()
 {
 	SG_DEBUG("CCanberraWordDistance created");
-	dictionary_size= 1<<(sizeof(WORD)*8);
-	dictionary_weights = new DREAL[dictionary_size];
+	dictionary_size= 1<<(sizeof(uint16_t)*8);
+	dictionary_weights = new float64_t[dictionary_size];
 	SG_DEBUG( "using dictionary of %d bytes\n", dictionary_size);
 
 	init(l, r);
@@ -41,10 +42,10 @@ CCanberraWordDistance::~CCanberraWordDistance()
 
 	delete[] dictionary_weights;
 }
-  
+
 bool CCanberraWordDistance::init(CFeatures* l, CFeatures* r)
 {
-	return CStringDistance<WORD>::init(l,r);
+	return CStringDistance<uint16_t>::init(l,r);
 }
 
 void CCanberraWordDistance::cleanup()
@@ -60,26 +61,28 @@ bool CCanberraWordDistance::save_init(FILE* dest)
 {
 	return false;
 }
-  
-DREAL CCanberraWordDistance::compute(INT idx_a, INT idx_b)
+
+float64_t CCanberraWordDistance::compute(int32_t idx_a, int32_t idx_b)
 {
-	INT alen, blen;
+	int32_t alen, blen;
 
-	WORD* avec=((CStringFeatures<WORD>*) lhs)->get_feature_vector(idx_a, alen);
-	WORD* bvec=((CStringFeatures<WORD>*) rhs)->get_feature_vector(idx_b, blen);
+	uint16_t* avec=((CStringFeatures<uint16_t>*) lhs)->
+		get_feature_vector(idx_a, alen);
+	uint16_t* bvec=((CStringFeatures<uint16_t>*) rhs)->
+		get_feature_vector(idx_b, blen);
 
-	DREAL result=0;
+	float64_t result=0;
 
-	INT left_idx=0;
-	INT right_idx=0;
+	int32_t left_idx=0;
+	int32_t right_idx=0;
 
 	while (left_idx < alen && right_idx < blen)
 	{
-		WORD sym=avec[left_idx];
+		uint16_t sym=avec[left_idx];
 		if (avec[left_idx]==bvec[right_idx])
 		{
-			INT old_left_idx=left_idx;
-			INT old_right_idx=right_idx;
+			int32_t old_left_idx=left_idx;
+			int32_t old_right_idx=right_idx;
 
 			while (left_idx< alen && avec[left_idx]==sym)
 				left_idx++;
@@ -87,8 +90,11 @@ DREAL CCanberraWordDistance::compute(INT idx_a, INT idx_b)
 			while (right_idx< blen && bvec[right_idx]==sym)
 				right_idx++;
 
-			result += CMath::abs( (DREAL) ((left_idx-old_left_idx) - (right_idx-old_right_idx)) )/
-						( (DREAL) ((left_idx-old_left_idx) + (right_idx-old_right_idx)) );
+			result +=
+				CMath::abs((float64_t)
+					((left_idx-old_left_idx)-(right_idx-old_right_idx)))/
+				((float64_t)
+					((left_idx-old_left_idx) + (right_idx-old_right_idx)));
 		}
 		else if (avec[left_idx]<bvec[right_idx])
 		{
@@ -109,7 +115,7 @@ DREAL CCanberraWordDistance::compute(INT idx_a, INT idx_b)
 	
 	while (left_idx < alen)
 	{
-		WORD sym=avec[left_idx];
+		uint16_t sym=avec[left_idx];
 		result++;
 
 		while (left_idx< alen && avec[left_idx]==sym)
@@ -118,7 +124,7 @@ DREAL CCanberraWordDistance::compute(INT idx_a, INT idx_b)
 
 	while (right_idx < blen)
 	{
-		WORD sym=bvec[right_idx];
+		uint16_t sym=bvec[right_idx];
 		result++;
 
 		while (right_idx< blen && bvec[right_idx]==sym)
