@@ -18,7 +18,7 @@
 #include "distance/StringDistance.h"
 
 /** class HammingWordDistance */
-class CHammingWordDistance: public CStringDistance<WORD>
+class CHammingWordDistance: public CStringDistance<uint16_t>
 {
 	public:
 		/** constructor
@@ -33,7 +33,7 @@ class CHammingWordDistance: public CStringDistance<WORD>
 		 * @param r features of right-hand side
 		 * @param use_sign if sign shall be used
 		 */
-		CHammingWordDistance(CStringFeatures<WORD>* l, CStringFeatures<WORD>* r, bool use_sign);
+		CHammingWordDistance(CStringFeatures<uint16_t>* l, CStringFeatures<uint16_t>* r, bool use_sign);
 		virtual ~CHammingWordDistance();
 
 		/** init distance
@@ -71,14 +71,14 @@ class CHammingWordDistance: public CStringDistance<WORD>
 		 *
 		 * @return name HammingWord
 		 */
-		virtual const CHAR* get_name() { return "HammingWord"; }
+		virtual const char* get_name() { return "HammingWord"; }
 
 		/** get dictionary weights
 		 *
 		 * @param dsize size of the dictionary
 		 * @param dweights dictionary weights are stored in here
 		 */
-		void get_dictionary(INT& dsize, DREAL*& dweights)
+		void get_dictionary(int32_t& dsize, float64_t*& dweights)
 		{
 			dsize=dictionary_size;
 			dweights = dictionary_weights;
@@ -88,13 +88,13 @@ class CHammingWordDistance: public CStringDistance<WORD>
 		/// compute kernel function for features a and b
 		/// idx_{a,b} denote the index of the feature vectors
 		/// in the corresponding feature object
-		DREAL compute(INT idx_a, INT idx_b);
+		float64_t compute(int32_t idx_a, int32_t idx_b);
 
 	protected:
 		/** size of the dictionary */
-		INT dictionary_size;
+		int32_t dictionary_size;
 		/** dictionary weights */
-		DREAL* dictionary_weights;
+		float64_t* dictionary_weights;
 		/** if sign shall be used */
 		bool use_sign;
 };
