@@ -16,21 +16,21 @@
 #include "lib/io.h"
 
 CManhattanWordDistance::CManhattanWordDistance()
-: CStringDistance<WORD>()
+: CStringDistance<uint16_t>()
 {
 	SG_DEBUG("CManhattanWordDistance created");
-	dictionary_size= 1<<(sizeof(WORD)*8);
-	dictionary_weights = new DREAL[dictionary_size];
+	dictionary_size= 1<<(sizeof(uint16_t)*8);
+	dictionary_weights = new float64_t[dictionary_size];
 	SG_DEBUG( "using dictionary of %d bytes\n", dictionary_size);
 }
 
 CManhattanWordDistance::CManhattanWordDistance(
-	CStringFeatures<WORD>* l, CStringFeatures<WORD>* r)
-: CStringDistance<WORD>()
+	CStringFeatures<uint16_t>* l, CStringFeatures<uint16_t>* r)
+: CStringDistance<uint16_t>()
 {
 	SG_DEBUG("CManhattanWordDistance created");
-	dictionary_size= 1<<(sizeof(WORD)*8);
-	dictionary_weights = new DREAL[dictionary_size];
+	dictionary_size= 1<<(sizeof(uint16_t)*8);
+	dictionary_weights = new float64_t[dictionary_size];
 	SG_DEBUG( "using dictionary of %d bytes\n", dictionary_size);
 
 	init(l, r);
@@ -45,7 +45,7 @@ CManhattanWordDistance::~CManhattanWordDistance()
 
 bool CManhattanWordDistance::init(CFeatures* l, CFeatures* r)
 {
-	bool result=CStringDistance<WORD>::init(l,r);
+	bool result=CStringDistance<uint16_t>::init(l,r);
 	return result;
 }
 
@@ -63,25 +63,27 @@ bool CManhattanWordDistance::save_init(FILE* dest)
 	return false;
 }
 
-DREAL CManhattanWordDistance::compute(INT idx_a, INT idx_b)
+float64_t CManhattanWordDistance::compute(int32_t idx_a, int32_t idx_b)
 {
-	INT alen, blen;
+	int32_t alen, blen;
 
-	WORD* avec=((CStringFeatures<WORD>*) lhs)->get_feature_vector(idx_a, alen);
-	WORD* bvec=((CStringFeatures<WORD>*) rhs)->get_feature_vector(idx_b, blen);
+	uint16_t* avec=((CStringFeatures<uint16_t>*) lhs)->
+		get_feature_vector(idx_a, alen);
+	uint16_t* bvec=((CStringFeatures<uint16_t>*) rhs)->
+		get_feature_vector(idx_b, blen);
 
-	INT result=0;
+	int32_t result=0;
 
-	INT left_idx=0;
-	INT right_idx=0;
+	int32_t left_idx=0;
+	int32_t right_idx=0;
 
 	while (left_idx < alen && right_idx < blen)
 	{
-		WORD sym=avec[left_idx];
+		uint16_t sym=avec[left_idx];
 		if (avec[left_idx]==bvec[right_idx])
 		{
-			INT old_left_idx=left_idx;
-			INT old_right_idx=right_idx;
+			int32_t old_left_idx=left_idx;
+			int32_t old_right_idx=right_idx;
 
 			while (left_idx< alen && avec[left_idx]==sym)
 				left_idx++;
