@@ -18,7 +18,7 @@
 #include "distance/StringDistance.h"
 
 /** class ManhattanWordDistance */
-class CManhattanWordDistance: public CStringDistance<WORD>
+class CManhattanWordDistance: public CStringDistance<uint16_t>
 {
 	public:
 		/** default constructor */
@@ -29,7 +29,7 @@ class CManhattanWordDistance: public CStringDistance<WORD>
 		 * @param l features of left-hand side
 		 * @param r features of right-hand side
 		 */
-		CManhattanWordDistance(CStringFeatures<WORD>* l, CStringFeatures<WORD>* r);
+		CManhattanWordDistance(CStringFeatures<uint16_t>* l, CStringFeatures<uint16_t>* r);
 		virtual ~CManhattanWordDistance();
 
 		/** init distance
@@ -67,14 +67,14 @@ class CManhattanWordDistance: public CStringDistance<WORD>
 		 *
 		 * @return name ManhattanWord
 		 */
-		virtual const CHAR* get_name() { return "ManhattanWord"; }
+		virtual const char* get_name() { return "ManhattanWord"; }
 
 		/** get dictionary weights
 		 *
 		 * @param dsize size of the dictionary
 		 * @param dweights dictionary weights are stored in here
 		 */
-		void get_dictionary(INT& dsize, DREAL*& dweights)
+		void get_dictionary(int32_t& dsize, float64_t*& dweights)
 		{
 			dsize=dictionary_size;
 			dweights = dictionary_weights;
@@ -84,12 +84,12 @@ class CManhattanWordDistance: public CStringDistance<WORD>
 		/// compute distance function for features a and b
 		/// idx_{a,b} denote the index of the feature vectors
 		/// in the corresponding feature object
-		DREAL compute(INT idx_a, INT idx_b);
+		float64_t compute(int32_t idx_a, int32_t idx_b);
 
 	protected:
 		/** size of the dictionary */
-		INT dictionary_size;
+		int32_t dictionary_size;
 		/** dictionary weights */
-		DREAL* dictionary_weights;
+		float64_t* dictionary_weights;
 };
 #endif /* _MANHATTANWORDDISTANCE_H___ */
