@@ -64,42 +64,42 @@ template <class ST> class CSimpleDistance : public CDistance
  *
  * @return feature type DREAL
  */
-template<> inline EFeatureType CSimpleDistance<DREAL>::get_feature_type() { return F_DREAL; }
+template<> inline EFeatureType CSimpleDistance<float64_t>::get_feature_type() { return F_DREAL; }
 
 /** get feature type the ULONG distance can deal with
  *
  * @return feature type ULONG
  */
-template<> inline EFeatureType CSimpleDistance<ULONG>::get_feature_type() { return F_ULONG; }
+template<> inline EFeatureType CSimpleDistance<uint64_t>::get_feature_type() { return F_ULONG; }
 
 /** get feature type the INT distance can deal with
  *
  * @return feature type INT
  */
-template<> inline EFeatureType CSimpleDistance<INT>::get_feature_type() { return F_INT; }
+template<> inline EFeatureType CSimpleDistance<int32_t>::get_feature_type() { return F_INT; }
 
 /** get feature type the WORD distance can deal with
  *
  * @return feature type WORD
  */
-template<> inline EFeatureType CSimpleDistance<WORD>::get_feature_type() { return F_WORD; }
+template<> inline EFeatureType CSimpleDistance<uint16_t>::get_feature_type() { return F_WORD; }
 
 /** get feature type the SHORT distance can deal with
  *
  * @return feature type SHORT
  */
-template<> inline EFeatureType CSimpleDistance<SHORT>::get_feature_type() { return F_SHORT; }
+template<> inline EFeatureType CSimpleDistance<int16_t>::get_feature_type() { return F_SHORT; }
 
 /** get feature type the BYTE distance can deal with
  *
  * @return feature type BYTE
  */
-template<> inline EFeatureType CSimpleDistance<BYTE>::get_feature_type() { return F_BYTE; }
+template<> inline EFeatureType CSimpleDistance<uint8_t>::get_feature_type() { return F_BYTE; }
 
 /** get feature type the CHAR distance can deal with
  *
  * @return feature type CHAR
  */
-template<> inline EFeatureType CSimpleDistance<CHAR>::get_feature_type() { return F_CHAR; }
+template<> inline EFeatureType CSimpleDistance<char>::get_feature_type() { return F_CHAR; }
 
 #endif
