@@ -61,41 +61,41 @@ template <class ST> class CSparseDistance : public CDistance
  *
  * @return feature type DREAL
  */
-template<> inline EFeatureType CSparseDistance<DREAL>::get_feature_type() { return F_DREAL; }
+template<> inline EFeatureType CSparseDistance<float64_t>::get_feature_type() { return F_DREAL; }
 
 /** get feature type the ULONG distance can deal with
  *
  * @return feature type ULONG
  */
-template<> inline EFeatureType CSparseDistance<ULONG>::get_feature_type() { return F_ULONG; }
+template<> inline EFeatureType CSparseDistance<uint64_t>::get_feature_type() { return F_ULONG; }
 
 /** get feature type the INT distance can deal with
  *
  * @return feature type INT
  */
-template<> inline EFeatureType CSparseDistance<INT>::get_feature_type() { return F_INT; }
+template<> inline EFeatureType CSparseDistance<int32_t>::get_feature_type() { return F_INT; }
 
 /** get feature type the WORD distance can deal with
  *
  * @return feature type WORD
  */
-template<> inline EFeatureType CSparseDistance<WORD>::get_feature_type() { return F_WORD; }
+template<> inline EFeatureType CSparseDistance<uint16_t>::get_feature_type() { return F_WORD; }
 
 /** get feature type the SHORT distance can deal with
  *
  * @return feature type SHORT
  */
-template<> inline EFeatureType CSparseDistance<SHORT>::get_feature_type() { return F_SHORT; }
+template<> inline EFeatureType CSparseDistance<int16_t>::get_feature_type() { return F_SHORT; }
 
 /** get feature type the BYTE distance can deal with
  *
  * @return feature type BYTE
  */
-template<> inline EFeatureType CSparseDistance<BYTE>::get_feature_type() { return F_BYTE; }
+template<> inline EFeatureType CSparseDistance<uint8_t>::get_feature_type() { return F_BYTE; }
 
 /** get feature type the CHAR distance can deal with
  *
  * @return feature type CHAR
  */
-template<> inline EFeatureType CSparseDistance<CHAR>::get_feature_type() { return F_CHAR; }
+template<> inline EFeatureType CSparseDistance<char>::get_feature_type() { return F_CHAR; }
 #endif
