@@ -58,43 +58,43 @@ template <class ST> class CStringDistance : public CDistance
  *
  * @return feature type DREAL
  */
-template<> inline EFeatureType CStringDistance<DREAL>::get_feature_type() { return F_DREAL; }
+template<> inline EFeatureType CStringDistance<float64_t>::get_feature_type() { return F_DREAL; }
 
 /** get feature type the ULONG distance can deal with
  *
  * @return feature type ULONG
  */
-template<> inline EFeatureType CStringDistance<ULONG>::get_feature_type() { return F_ULONG; }
+template<> inline EFeatureType CStringDistance<uint64_t>::get_feature_type() { return F_ULONG; }
 
 /** get feature type the INT distance can deal with
  *
  * @return feature type INT
  */
-template<> inline EFeatureType CStringDistance<INT>::get_feature_type() { return F_INT; }
+template<> inline EFeatureType CStringDistance<int32_t>::get_feature_type() { return F_INT; }
 
 /** get feature type the WORD distance can deal with
  *
  * @return feature type WORD
  */
-template<> inline EFeatureType CStringDistance<WORD>::get_feature_type() { return F_WORD; }
+template<> inline EFeatureType CStringDistance<uint16_t>::get_feature_type() { return F_WORD; }
 
 /** get feature type the SHORT distance can deal with
  *
  * @return feature type SHORT
  */
-template<> inline EFeatureType CStringDistance<SHORT>::get_feature_type() { return F_SHORT; }
+template<> inline EFeatureType CStringDistance<int16_t>::get_feature_type() { return F_SHORT; }
 
 /** get feature type the BYTE distance can deal with
  *
  * @return feature type BYTE
  */
-template<> inline EFeatureType CStringDistance<BYTE>::get_feature_type() { return F_BYTE; }
+template<> inline EFeatureType CStringDistance<uint8_t>::get_feature_type() { return F_BYTE; }
 
 /** get feature type the CHAR distance can deal with
  *
  * @return feature type CHAR
  */
-template<> inline EFeatureType CStringDistance<CHAR>::get_feature_type() { return F_CHAR; }
+template<> inline EFeatureType CStringDistance<char>::get_feature_type() { return F_CHAR; }
 
 #endif
 
