@@ -21,36 +21,36 @@ CDistribution::~CDistribution()
 {
 }
 
-DREAL CDistribution::get_log_likelihood_sample()
+float64_t CDistribution::get_log_likelihood_sample()
 {
 	ASSERT(features);
 
-	DREAL sum=0;
-	for (INT i=0; i<features->get_num_vectors(); i++)
+	float64_t sum=0;
+	for (int32_t i=0; i<features->get_num_vectors(); i++)
 		sum+=get_log_likelihood_example(i);
 
 	return sum/features->get_num_vectors();
 }
 
-void CDistribution::get_log_likelihood(DREAL **dst, INT *num)
+void CDistribution::get_log_likelihood(float64_t **dst, int32_t *num)
 {
 	ASSERT(features);
 
 	*num=features->get_num_vectors();
-	size_t sz=sizeof(DREAL)*(*num);
-	*dst=(DREAL*) malloc(sz);
+	size_t sz=sizeof(float64_t)*(*num);
+	*dst=(float64_t*) malloc(sz);
 	ASSERT(dst);
 
-	for (INT i=0; i<(*num); i++)
+	for (int32_t i=0; i<(*num); i++)
 		*(*dst+i)=get_log_likelihood_example(i);
 }
 
-INT CDistribution::get_num_relevant_model_parameters()
+int32_t CDistribution::get_num_relevant_model_parameters()
 {
-	INT total_num=get_num_model_parameters();
-	INT num=0;
+	int32_t total_num=get_num_model_parameters();
+	int32_t num=0;
 
-	for (INT i=0; i<total_num; i++)
+	for (int32_t i=0; i<total_num; i++)
 	{
 		if (get_log_model_parameter(i)>CMath::ALMOST_NEG_INFTY)
 			num++;
