@@ -24,22 +24,22 @@ bool CGHMM::train()
 	return false;
 }
 
-INT CGHMM::get_num_model_parameters()
+int32_t CGHMM::get_num_model_parameters()
 {
 	return 0;
 }
 
-DREAL CGHMM::get_log_model_parameter(INT param_num)
+float64_t CGHMM::get_log_model_parameter(int32_t param_num)
 {
 	return 0;
 }
 
-DREAL CGHMM::get_log_derivative(INT param_num, INT num_example)
+float64_t CGHMM::get_log_derivative(int32_t param_num, int32_t num_example)
 {
 	return 0;
 }
 
-DREAL CGHMM::get_log_likelihood_example(INT num_example)
+float64_t CGHMM::get_log_likelihood_example(int32_t num_example)
 {
 	return 0;
 }
