@@ -16,7 +16,23 @@
 #include "features/Labels.h"
 #include "distributions/Distribution.h"
 
-/** class LinearHMM */
+/** The class LinearHMM is for learning Higher Order Markov chains, i.e.
+ * the parameters \f${\bf \theta}\f$ in
+ *
+ * \f{eqnarray*}
+ * P({\bf x}|{\bf \theta}^\pm)&=&P(x_1, \ldots, x_N|{\bf \theta}^\pm)\\
+ * &=&P(x_1,\ldots,x_{d}|{\bf \theta}^\pm)\prod_{i=d+1}^N
+ * P(x_i|x_{i-1},\ldots,x_{i-d},{\bf \theta}^\pm)
+ * \f}
+ *
+ * are determined in training. 
+ *
+ * A more detailed description can be found in
+ *
+ * Durbin et.al, Biological Sequence Analysis -Probabilistic Models of Proteins
+ * and Nucleic Acids, 1998
+ *
+ * */
 class CLinearHMM : public CDistribution
 {
 	public:
@@ -24,14 +40,14 @@ class CLinearHMM : public CDistribution
 		 *
 		 * @param f features to use
 		 */
-		CLinearHMM(CStringFeatures<WORD>* f);
+		CLinearHMM(CStringFeatures<uint16_t>* f);
 
 		/** constructor
 		 *
 		 * @param p_num_features number of features
 		 * @param p_num_symbols number of symbols in features
 		 */
-		CLinearHMM(INT p_num_features, INT p_num_symbols);
+		CLinearHMM(int32_t p_num_features, int32_t p_num_symbols);
 		~CLinearHMM();
 
 		/** train distribution
@@ -47,7 +63,9 @@ class CLinearHMM : public CDistribution
 		 * @param pseudo_count pseudo count
 		 * @return if training was successful
 		 */
-		bool train(const INT* indizes, INT num_indizes, DREAL pseudo_count);
+		bool train(
+			const int32_t* indizes, int32_t num_indizes,
+			float64_t pseudo_count);
 
 		/** get logarithm of one example's likelihood
 		 *
@@ -55,7 +73,7 @@ class CLinearHMM : public CDistribution
 		 * @param len length of vector
 		 * @return logarithm of likelihood
 		 */
-		DREAL get_log_likelihood_example(WORD* vector, INT len);
+		float64_t get_log_likelihood_example(uint16_t* vector, int32_t len);
 
 		/** get one example's likelihood
 		 *
@@ -63,14 +81,14 @@ class CLinearHMM : public CDistribution
 		 * @param len length of vector
 		 * @return likelihood
 		 */
-		DREAL get_likelihood_example(WORD* vector, INT len);
+		float64_t get_likelihood_example(uint16_t* vector, int32_t len);
 
 		/** get logarithm of one example's likelihood
 		 *
 		 * @param num_example which example
 		 * @return logarithm of example's likelihood
 		 */
-		virtual DREAL get_log_likelihood_example(INT num_example);
+		virtual float64_t get_log_likelihood_example(int32_t num_example);
 
 		/** get logarithm of one example's derivative's likelihood
 		 *
@@ -78,7 +96,8 @@ class CLinearHMM : public CDistribution
 		 * @param num_example which example
 		 * @return logarithm of example's derivative
 		 */
-		virtual DREAL get_log_derivative(INT num_param, INT num_example);
+		virtual float64_t get_log_derivative(
+			int32_t num_param, int32_t num_example);
 
 		/** obsolete get logarithm of one example's derivative's
 		 *  likelihood
@@ -86,7 +105,8 @@ class CLinearHMM : public CDistribution
 		 * @param obs observation
 		 * @param pos position
 		 */
-		virtual inline DREAL get_log_derivative_obsolete(WORD obs, INT pos)
+		virtual inline float64_t get_log_derivative_obsolete(
+			uint16_t obs, int32_t pos)
 		{
 			return 1.0/transition_probs[pos*num_symbols+obs];
 		}
@@ -97,7 +117,8 @@ class CLinearHMM : public CDistribution
 		 * @param len length
 		 * @param pos position
 		 */
-		virtual inline DREAL get_derivative_obsolete(WORD* vector, INT len, INT pos)
+		virtual inline float64_t get_derivative_obsolete(
+			uint16_t* vector, int32_t len, int32_t pos)
 		{
 			ASSERT(pos<len);
 			return get_likelihood_example(vector, len)/transition_probs[pos*num_symbols+vector[pos]];
@@ -107,19 +128,19 @@ class CLinearHMM : public CDistribution
 		 *
 		 * @return sequence length of each example
 		 */
-		virtual inline INT get_sequence_length() { return sequence_length; }
+		virtual inline int32_t get_sequence_length() { return sequence_length; }
 
 		/** get number of symbols in examples
 		 *
 		 * @return number of symbols in examples
 		 */
-		virtual inline INT get_num_symbols() { return num_symbols; }
+		virtual inline int32_t get_num_symbols() { return num_symbols; }
 
 		/** get number of model parameters
 		 *
 		 * @return number of model parameters
 		 */
-		virtual inline INT get_num_model_parameters() { return num_params; }
+		virtual inline int32_t get_num_model_parameters() { return num_params; }
 
 		/** get positional log parameter
 		 *
@@ -127,7 +148,8 @@ class CLinearHMM : public CDistribution
 		 * @param position position
 		 * @return positional log parameter
 		 */
-		virtual inline DREAL get_positional_log_parameter(WORD obs, INT position)
+		virtual inline float64_t get_positional_log_parameter(
+			uint16_t obs, int32_t position)
 		{
 			return log_transition_probs[position*num_symbols+obs];
 		}
@@ -137,7 +159,7 @@ class CLinearHMM : public CDistribution
 		 * @param num_param which param
 		 * @result logarithm of given model parameter
 		 */
-		virtual inline DREAL get_log_model_parameter(INT num_param)
+		virtual inline float64_t get_log_model_parameter(int32_t num_param)
 		{
 			ASSERT(log_transition_probs);
 			ASSERT(num_param<num_params);
@@ -152,7 +174,7 @@ class CLinearHMM : public CDistribution
 		 * @param num where number of logarithm of transition probs
 		 *        will be stored
 		 */
-		virtual void get_log_transition_probs(DREAL** dst, INT* num);
+		virtual void get_log_transition_probs(float64_t** dst, int32_t* num);
 
 		/** set logarithm of all transition probs
 		 *
@@ -160,14 +182,15 @@ class CLinearHMM : public CDistribution
 		 * @param num number of logarithms of transition probs
 		 * @return if setting was succesful
 		 */
-		virtual bool set_log_transition_probs(const DREAL* src, INT num);
+		virtual bool set_log_transition_probs(
+			const float64_t* src, int32_t num);
 
 		/** get all transition probs
 		 *
 		 * @param dst where transition probs will be stored
 		 * @param num where number of transition probs will be stored
 		 */
-		virtual void get_transition_probs(DREAL** dst, INT* num);
+		virtual void get_transition_probs(float64_t** dst, int32_t* num);
 
 		/** set all transition probs
 		 *
@@ -175,18 +198,18 @@ class CLinearHMM : public CDistribution
 		 * @param num number of transition probs
 		 * @return if setting was succesful
 		 */
-		virtual bool set_transition_probs(const DREAL* src, INT num);
+		virtual bool set_transition_probs(const float64_t* src, int32_t num);
 
 	protected:
 		/** examples' sequence length */
-		INT sequence_length;
+		int32_t sequence_length;
 		/** number of symbols in examples */
-		INT num_symbols;
+		int32_t num_symbols;
 		/** number of parameters */
-		INT num_params;
+		int32_t num_params;
 		/** transition probs */
-		DREAL* transition_probs;
+		float64_t* transition_probs;
 		/** logarithm of transition probs */
-		DREAL* log_transition_probs;
+		float64_t* log_transition_probs;
 };
 #endif
