@@ -12,32 +12,32 @@
 #include "lib/common.h"
 #include "lib/File.h"
 
-CByteFeatures::CByteFeatures(E_ALPHABET a, INT size)
-: CSimpleFeatures<BYTE>(size)
+CByteFeatures::CByteFeatures(EAlphabet a, int32_t size)
+: CSimpleFeatures<uint8_t>(size)
 {
 	alphabet=new CAlphabet(a);
 }
 
-CByteFeatures::CByteFeatures(CAlphabet* a, INT size)
-: CSimpleFeatures<BYTE>(size)
+CByteFeatures::CByteFeatures(CAlphabet* a, int32_t size)
+: CSimpleFeatures<uint8_t>(size)
 {
 	alphabet=a;
 }
 
 CByteFeatures::CByteFeatures(const CByteFeatures & orig)
-: CSimpleFeatures<BYTE>(orig)
+: CSimpleFeatures<uint8_t>(orig)
 {
 	alphabet=orig.alphabet;
 }
 
-CByteFeatures::CByteFeatures(E_ALPHABET a, BYTE* fm, INT num_feat, INT num_vec)
-: CSimpleFeatures<BYTE>(fm, num_feat, num_vec)
+CByteFeatures::CByteFeatures(EAlphabet a, uint8_t* fm, int32_t num_feat, int32_t num_vec)
+: CSimpleFeatures<uint8_t>(fm, num_feat, num_vec)
 {
 	alphabet=new CAlphabet(a);
 }
 
-CByteFeatures::CByteFeatures(E_ALPHABET a, CHAR* fname)
-: CSimpleFeatures<BYTE>(fname)
+CByteFeatures::CByteFeatures(EAlphabet a, char* fname)
+: CSimpleFeatures<uint8_t>(fname)
 {
 	alphabet=new CAlphabet(a);
 	load(fname);
@@ -49,11 +49,11 @@ CByteFeatures::~CByteFeatures()
 	alphabet=NULL;
 }
 
-bool CByteFeatures::load(CHAR* fname)
+bool CByteFeatures::load(char* fname)
 {
 	SG_INFO( "loading...\n");
-    LONG length=0;
-	LONG linelen=0;
+	int64_t length=0;
+	int64_t linelen=0;
 
 	CFile f(fname, 'r', F_BYTE);
 	free_feature_matrix();
@@ -77,9 +77,9 @@ bool CByteFeatures::load(CHAR* fname)
 
 		if (length && (num_vectors*linelen==length))
 		{
-			for (INT lines=0; lines<num_vectors; lines++)
+			for (int32_t lines=0; lines<num_vectors; lines++)
 			{
-				for (INT columns=0; columns<num_features; columns++)
+				for (int32_t columns=0; columns<num_features; columns++)
 					feature_matrix[lines*num_features+columns]=feature_matrix[lines*linelen+columns];
 
 				if (feature_matrix[lines*linelen+num_features]!='\n')
@@ -100,7 +100,7 @@ bool CByteFeatures::load(CHAR* fname)
 	return false;
 }
 
-bool CByteFeatures::save(CHAR* fname)
+bool CByteFeatures::save(char* fname)
 {
 	return false;
 }
