@@ -16,13 +16,57 @@
 #include "lib/common.h"
 #include "base/SGObject.h"
 
-#include "preproc/PreProc.h"
+enum EFeatureType
+{
+	F_UNKNOWN = 0,
+	F_CHAR = 10,
+	F_BYTE = 20,
+	F_SHORT = 30,
+	F_WORD = 40,
+	F_INT = 50,
+	F_UINT = 60,
+	F_LONG = 70,
+	F_ULONG = 80,
+	F_SHORTREAL = 90,
+	F_DREAL = 100,
+	F_LONGREAL = 110,
+	F_ANY = 1000
+};
 
+enum EFeatureClass
+{
+	C_UNKNOWN = 0,
+	C_SIMPLE = 10,
+	C_SPARSE = 20,
+	C_STRING = 30,
+	C_COMBINED = 40,
+	C_MINDYGRAM = 50,
+	C_ANY = 1000
+};
+
+
+#include "preproc/PreProc.h"
 class CPreProc;
 class CFeatures;
 
-/** class Features
- * Features can just be DREALs, SHORT or STRINGs, FILES, or...
+
+/** The class Features is the base class of all feature objects. It can be
+ * understood as a dense real valued feature matrix (with e.g. columns as
+ * single feature vectors), a set of strings, graphs or any other arbitrary
+ * collection of objects. As a result this class is kept very general and
+ * implements only very weak interfaces to
+ *
+ * - duplicate the Feature object
+ * - obtain the feature type (like F_DREAL, F_SHORT ...)
+ * - obtain the feature class (like Simple dense matrices, sparse or strings)
+ * - obtain the number of feature "vectors"
+ *
+ *   In addition it provides helpers to check e.g. for compability of feature objects.
+ *
+ *   Currently there are 3 general feature classes, which are CSimpleFeatures
+ *   (dense matrices), CSparseFeatures (sparse matrices), CStringFeatures (a
+ *   set of strings) from which all the specific features like CRealFeatures
+ *   (dense real valued feature matrices) are derived.
  */
 class CFeatures : public CSGObject
 {
@@ -31,7 +75,7 @@ class CFeatures : public CSGObject
 		 *
 		 * @param size cache size
 		 */
-		CFeatures(INT size);
+		CFeatures(int32_t size);
 
 		/** copy constructor */
 		CFeatures(const CFeatures& orig);
@@ -40,7 +84,7 @@ class CFeatures : public CSGObject
 		 *
 		 * @param fname filename to load features from
 		 */
-		CFeatures(CHAR* fname);
+		CFeatures(char* fname);
 
 		/** duplicate feature object
 		 *
@@ -73,44 +117,44 @@ class CFeatures : public CSGObject
 		 * @param p preprocessor to set
 		 * @return something inty
 		 */
-		virtual INT add_preproc(CPreProc* p);
+		virtual int32_t add_preproc(CPreProc* p);
 
 		/** delete preprocessor from list
 		 * caller has to clean up returned preproc
 		 *
 		 * @param num index of preprocessor in list
 		 */
-		virtual CPreProc* del_preproc(INT num);
+		virtual CPreProc* del_preproc(int32_t num);
 
 		/** get specified preprocessor
 		 *
 		 * @param num index of preprocessor in list
 		 */
-		CPreProc* get_preproc(INT num);
+		CPreProc* get_preproc(int32_t num);
 
 		/** set applied flag for preprocessor
 		 *
 		 * @param num index of preprocessor in list
 		 */
-		inline void set_preprocessed(INT num) { preprocessed[num]=true; }
+		inline void set_preprocessed(int32_t num) { preprocessed[num]=true; }
 
 		/** get whether specified preprocessor was already applied
 		 *
 		 * @param num index of preprocessor in list
 		 */
-		inline bool is_preprocessed(INT num) { return preprocessed[num]; }
+		inline bool is_preprocessed(int32_t num) { return preprocessed[num]; }
 
 		/** get the number of applied preprocs
 		 *
 		 * @return number of applied preprocessors
 		 */
-		INT get_num_preprocessed();
+		int32_t get_num_preprocessed();
 
 		/** get number of preprocessors
 		 *
 		 * @return number of preprocessors
 		 */
-		inline INT get_num_preproc() { return num_preproc; }
+		inline int32_t get_num_preproc() { return num_preproc; }
 
 		/** clears all preprocs */
 		void clean_preprocs();
@@ -119,7 +163,7 @@ class CFeatures : public CSGObject
 		 *
 		 * @return cache size
 		 */
-		inline INT get_cache_size() { return cache_size; };
+		inline int32_t get_cache_size() { return cache_size; };
 
 		/** get number of examples/vectors
 		 *
@@ -127,7 +171,7 @@ class CFeatures : public CSGObject
 		 *
 		 * @return number of examples/vectors
 		 */
-		virtual INT get_num_vectors()=0 ;
+		virtual int32_t get_num_vectors()=0 ;
 
 		/** in case there is a feature matrix allow for reshaping
 		 *
@@ -137,7 +181,7 @@ class CFeatures : public CSGObject
 		 * @param num_vectors new number of vectors
 		 * @return if reshaping was succesful
 		 */
-		virtual bool reshape(INT num_features, INT num_vectors) { return false; }
+		virtual bool reshape(int32_t num_features, int32_t num_vectors) { return false; }
 
 		/** get memory footprint of one feature
 		 *
@@ -145,7 +189,7 @@ class CFeatures : public CSGObject
 		 *
 		 * @return memory footprint of one feature
 		 */
-		virtual INT get_size()=0;
+		virtual int32_t get_size()=0;
 
 		/** list feature object */
 		void list_feature_obj();
@@ -155,14 +199,14 @@ class CFeatures : public CSGObject
 		 * @param fname filename to load from
 		 * @return if loading was successful
 		 */
-		virtual bool load(CHAR* fname);
+		virtual bool load(char* fname);
 
 		/** save features to file
 		 *
 		 * @param fname filename to save to
 		 * @return if saving was successful
 		 */
-		virtual bool save(CHAR* fname);
+		virtual bool save(char* fname);
 
 		/** check feature compatibility
 		 *
@@ -173,13 +217,13 @@ class CFeatures : public CSGObject
 
 	private:
 		/// size of cache in MB
-		INT cache_size;
+		int32_t cache_size;
 
 		/// list of preprocessors
 		CPreProc** preproc;
 
 		/// number of preprocs in list
-		INT num_preproc;
+		int32_t num_preproc;
 
 		/// i'th entry is true if features were already preprocessed with preproc i
 		bool* preprocessed;
