@@ -12,22 +12,23 @@
 #include "features/ShortFeatures.h"
 #include "features/CharFeatures.h"
 
-CShortFeatures::CShortFeatures(INT size)
-: CSimpleFeatures<SHORT>(size)
+CShortFeatures::CShortFeatures(int32_t size)
+: CSimpleFeatures<int16_t>(size)
 {
 }
 
 CShortFeatures::CShortFeatures(const CShortFeatures & orig)
-: CSimpleFeatures<SHORT>(orig)
+: CSimpleFeatures<int16_t>(orig)
 {
 }
 
-CShortFeatures::CShortFeatures(CHAR* fname)
-: CSimpleFeatures<SHORT>(fname)
+CShortFeatures::CShortFeatures(char* fname)
+: CSimpleFeatures<int16_t>(fname)
 {
 }
 
-bool CShortFeatures::obtain_from_char_features(CCharFeatures* cf, INT start, INT order, INT gap)
+bool CShortFeatures::obtain_from_char_features(
+	CCharFeatures* cf, int32_t start, int32_t order, int32_t gap)
 {
 	ASSERT(cf);
 
@@ -37,32 +38,32 @@ bool CShortFeatures::obtain_from_char_features(CCharFeatures* cf, INT start, INT
 	CAlphabet* alpha=cf->get_alphabet();
 	ASSERT(alpha);
 
-	INT len=num_vectors*num_features;
+	int32_t len=num_vectors*num_features;
 	free_feature_matrix();
-	feature_matrix=new SHORT[len];
-	INT num_cf_feat=0;
-	INT num_cf_vec=0;
-	CHAR* fm=cf->get_feature_matrix(num_cf_feat, num_cf_vec);
+	feature_matrix=new int16_t[len];
+	int32_t num_cf_feat=0;
+	int32_t num_cf_vec=0;
+	char* fm=cf->get_feature_matrix(num_cf_feat, num_cf_vec);
 
 	ASSERT(num_cf_vec==num_vectors);
 	ASSERT(num_cf_feat==num_features);
 
-	INT max_val=0;
-	for (INT i=0; i<len; i++)
+	int32_t max_val=0;
+	for (int32_t i=0; i<len; i++)
 	{
-		feature_matrix[i]=(SHORT) alpha->remap_to_bin(fm[i]);
-		max_val=CMath::max((INT) feature_matrix[i],max_val);
+		feature_matrix[i]=(int16_t) alpha->remap_to_bin(fm[i]);
+		max_val=CMath::max((int32_t) feature_matrix[i],max_val);
 	}
 
-	for (INT line=0; line<num_vectors; line++)
+	for (int32_t line=0; line<num_vectors; line++)
 		translate_from_single_order(&feature_matrix[line*num_features], num_features, start+gap, order+gap, max_val, gap);
 
 	if (start+gap!=0)
 	{
 		// condensing feature matrix ...
 		ASSERT(start+gap>=0);
-		for (INT line=0; line<num_vectors; line++)
-			for (INT j=0; j<num_features-start-gap; j++)
+		for (int32_t line=0; line<num_vectors; line++)
+			for (int32_t j=0; j<num_features-start-gap; j++)
 				feature_matrix[line*(num_features-(start+gap))+j]=feature_matrix[line*num_features+j] ;
 		num_features=num_features-(start+gap) ;
 	}
@@ -71,21 +72,22 @@ bool CShortFeatures::obtain_from_char_features(CCharFeatures* cf, INT start, INT
 }
 
 
-void CShortFeatures::translate_from_single_order(SHORT* obs, INT sequence_length, INT start, INT order, INT max_val, INT gap)
+void CShortFeatures::translate_from_single_order(
+	int16_t* obs, int32_t sequence_length, int32_t start, int32_t order,
+	int32_t max_val, int32_t gap)
 {
 	ASSERT(gap>=0);
 
-	const INT start_gap = (order - gap)/2 ;
-	const INT end_gap = start_gap + gap ;
-
-	INT i,j;
-	SHORT value=0;
+	const int32_t start_gap = (order - gap)/2;
+	const int32_t end_gap = start_gap + gap;
+	int32_t i,j;
+	int16_t value=0;
 
 	// almost all positions
-	for (i=sequence_length-1; i>= ((int) order)-1; i--)	//convert interval of size T
+	for (i=sequence_length-1; i>=order-1; i--) //convert interval of size T
 	{
 		value=0;
-		for (j=i; j>=i-((int) order)+1; j--)
+		for (j=i; j>=i-order+1; j--)
 		{
 			if (i-j<start_gap)
 				value= (value >> max_val) | (obs[j] << (max_val * (order-1-gap)));
@@ -121,12 +123,12 @@ void CShortFeatures::translate_from_single_order(SHORT* obs, INT sequence_length
 		obs[i-start]=obs[i];
 }
 
-bool CShortFeatures::load(CHAR* fname)
+bool CShortFeatures::load(char* fname)
 {
 	return false;
 }
 
-bool CShortFeatures::save(CHAR* fname)
+bool CShortFeatures::save(char* fname)
 {
 	return false;
 }
