@@ -11,13 +11,13 @@
 #include "features/ShortRealFeatures.h"
 #include "lib/File.h"
 
-bool CShortRealFeatures::load(CHAR* fname)
+bool CShortRealFeatures::load(char* fname)
 {
 	bool status=false;
 	num_vectors=1;
     num_features=0;
 	CFile f(fname, 'r', F_SHORTREAL);
-	LONG numf=0 ;
+	int64_t numf=0;
 	free_feature_matrix();
 	feature_matrix=f.load_shortreal_data(NULL, numf);
 	num_features=numf;
@@ -32,15 +32,15 @@ bool CShortRealFeatures::load(CHAR* fname)
 	return status;
 }
 
-bool CShortRealFeatures::save(CHAR* fname)
+bool CShortRealFeatures::save(char* fname)
 {
-	INT len;
+	int32_t len;
 	bool free;
-	SHORTREAL* fv;
+	float32_t* fv;
 
 	CFile f(fname, 'w', F_SHORTREAL);
 
-    for (INT i=0; i< (INT) num_vectors && f.is_ok(); i++)
+    for (int32_t i=0; i< (int32_t) num_vectors && f.is_ok(); i++)
 	{
 		if (!(i % (num_vectors/10+1)))
 			SG_PRINT( "%02d%%.", (int) (100.0*i/num_vectors));
@@ -53,7 +53,7 @@ bool CShortRealFeatures::save(CHAR* fname)
 	}
 
 	if (f.is_ok())
-		SG_INFO( "%d vectors with %d features each successfully written (filesize: %ld)\n", num_vectors, num_features, num_vectors*num_features*sizeof(SHORTREAL));
+		SG_INFO( "%d vectors with %d features each successfully written (filesize: %ld)\n", num_vectors, num_features, num_vectors*num_features*sizeof(float32_t));
 
     return true;
 }
