@@ -71,22 +71,20 @@ CGUIClassifier::CGUIClassifier(CSGInterface* ui_)
 	svm_qpsize=41;
 	svm_bufsize=3000;
 	svm_max_qpsize=1000;
+	svm_mkl_norm=1;
 	svm_C1=1;
 	svm_C2=1;
 	svm_C_mkl=0;
 	svm_weight_epsilon=1e-5;
 	svm_epsilon=1e-5;
 	svm_tube_epsilon=1e-2;
-	svm_nu=1e-2;
+	svm_nu=0.5;
 	svm_use_shrinking = true ;
 
 	svm_use_bias = true;
 	svm_use_mkl = false ;
 	svm_use_batch_computation = true ;
 	svm_use_linadd = true ;
-	svm_use_precompute = false ;
-	svm_use_precompute_subkernel = false ;
-	svm_use_precompute_subkernel_light = false ;
 	svm_do_auc_maximization = false ;
 
 	// KRR parameters
@@ -98,7 +96,7 @@ CGUIClassifier::~CGUIClassifier()
 	delete classifier;
 }
 
-bool CGUIClassifier::new_classifier(CHAR* name, INT d, INT from_d)
+bool CGUIClassifier::new_classifier(char* name, int32_t d, int32_t from_d)
 {
 	if (strcmp(name,"LIBSVM_ONECLASS")==0)
 	{
@@ -341,7 +339,7 @@ bool CGUIClassifier::train_svm()
 	if (!ui->ui_kernel->is_initialized() || !kernel->get_lhs())
 		SG_ERROR("Kernel not initialized.\n");
 
-	INT num_vec=kernel->get_lhs()->get_num_vectors();
+	int32_t num_vec=kernel->get_lhs()->get_num_vectors();
 	if (!oneclass && trainlabels->get_num_labels() != num_vec)
 		SG_ERROR("Number of train labels (%d) and training vectors (%d) differs!\n", trainlabels->get_num_labels(), num_vec);
 
@@ -349,6 +347,7 @@ bool CGUIClassifier::train_svm()
 
 	svm->set_bias_enabled(svm_use_bias);
 	svm->set_weight_epsilon(svm_weight_epsilon);
+	svm->set_mkl_norm(svm_mkl_norm);
 	svm->set_epsilon(svm_epsilon);
 	svm->set_max_train_time(max_train_time);
 	svm->set_tube_epsilon(svm_tube_epsilon);
@@ -363,18 +362,15 @@ bool CGUIClassifier::train_svm()
 	if(!oneclass)
 		((CKernelMachine*) svm)->set_labels(trainlabels);
 	((CKernelMachine*) svm)->set_kernel(kernel);
-	((CSVM*) svm)->set_precomputed_subkernels_enabled(svm_use_precompute_subkernel_light);
-	kernel->set_precompute_matrix(svm_use_precompute, svm_use_precompute_subkernel);
 
 
 
 	bool result=svm->train();
-	kernel->set_precompute_matrix(false,false);
 
 	return result;
 }
 
-bool CGUIClassifier::train_clustering(INT k, INT max_iter)
+bool CGUIClassifier::train_clustering(int32_t k, int32_t max_iter)
 {
 	bool result=false;
 	CDistance* distance=ui->ui_distance->get_distance();
@@ -407,7 +403,7 @@ bool CGUIClassifier::train_clustering(INT k, INT max_iter)
 	return result;
 }
 
-bool CGUIClassifier::train_knn(INT k)
+bool CGUIClassifier::train_knn(int32_t k)
 {
 	CLabels* trainlabels=ui->ui_labels->get_train_labels();
 	CDistance* distance=ui->ui_distance->get_distance();
@@ -432,7 +428,7 @@ bool CGUIClassifier::train_knn(INT k)
 	return result;
 }
 
-bool CGUIClassifier::train_linear()
+bool CGUIClassifier::train_linear(float64_t gamma)
 {
 	CFeatures* trainfeatures=ui->ui_features->get_train_features();
 	CLabels* trainlabels=ui->ui_labels->get_train_labels();
@@ -454,6 +450,11 @@ bool CGUIClassifier::train_linear()
 		((CPerceptron*) classifier)->set_max_iter(perceptron_maxiter);
 	}
 
+#ifdef HAVE_LAPACK
+	if (classifier->get_classifier_type()==CT_LDA)
+		((CLDA*) classifier)->set_gamma(gamma);
+#endif
+
 	((CLinearClassifier*) classifier)->set_labels(trainlabels);
 	((CLinearClassifier*) classifier)->set_features((CRealFeatures*) trainfeatures);
 	result=((CLinearClassifier*) classifier)->train();
@@ -479,7 +480,7 @@ bool CGUIClassifier::train_wdocas()
 		SG_ERROR("No labels available.\n");
 
 	((CWDSVMOcas*) classifier)->set_labels(trainlabels);
-	((CWDSVMOcas*) classifier)->set_features((CStringFeatures<BYTE>*) trainfeatures);
+	((CWDSVMOcas*) classifier)->set_features((CStringFeatures<uint8_t>*) trainfeatures);
 	result=((CWDSVMOcas*) classifier)->train();
 
 	return result;
@@ -487,6 +488,7 @@ bool CGUIClassifier::train_wdocas()
 
 bool CGUIClassifier::train_sparse_linear()
 {
+	EClassifierType ctype=classifier->get_classifier_type();
 	CFeatures* trainfeatures=ui->ui_features->get_train_features();
 	CLabels* trainlabels=ui->ui_labels->get_train_labels();
 
@@ -502,14 +504,27 @@ bool CGUIClassifier::train_sparse_linear()
 	if (!trainlabels)
 		SG_ERROR("No labels available.\n");
 
+	if (ctype==CT_SVMOCAS)
+		((CSVMOcas*) classifier)->set_C(svm_C1, svm_C2);
+#ifdef HAVE_LAPACK
+	else if (ctype==CT_LIBLINEAR)
+		((CLibLinear*) classifier)->set_C(svm_C1, svm_C2);
+#endif
+	else if (ctype==CT_SVMLIN)
+		((CSVMLin*) classifier)->set_C(svm_C1, svm_C2);
+	else if (ctype==CT_SVMSGD)
+		((CSVMSGD*) classifier)->set_C(svm_C1, svm_C2);
+	else if (ctype==CT_SUBGRADIENTSVM)
+		((CSubGradientSVM*) classifier)->set_C(svm_C1, svm_C2);
+
 	((CSparseLinearClassifier*) classifier)->set_labels(trainlabels);
-	((CSparseLinearClassifier*) classifier)->set_features((CSparseFeatures<DREAL>*) trainfeatures);
+	((CSparseLinearClassifier*) classifier)->set_features((CSparseFeatures<float64_t>*) trainfeatures);
 	result=((CSparseLinearClassifier*) classifier)->train();
 
 	return result;
 }
 
-bool CGUIClassifier::test(CHAR* filename_out, CHAR* filename_roc)
+bool CGUIClassifier::test(char* filename_out, char* filename_roc)
 {
 	FILE* file_out=stdout;
 	FILE* file_roc=NULL;
@@ -549,15 +564,14 @@ bool CGUIClassifier::test(CHAR* filename_out, CHAR* filename_roc)
 	SG_INFO("Starting svm testing.\n");
 	((CKernelMachine*) classifier)->set_labels(testlabels);
 	((CKernelMachine*) classifier)->set_kernel(ui->ui_kernel->get_kernel());
-	ui->ui_kernel->get_kernel()->set_precompute_matrix(false,false);
 	((CKernelMachine*) classifier)->set_batch_computation_enabled(svm_use_batch_computation);
 
 	CLabels* predictions= classifier->classify();
 
-	INT len=0;
-	DREAL* output= predictions->get_labels(len);
-	INT total=testfeatures->get_num_vectors();
-	INT* label=testlabels->get_int_labels(len);
+	int32_t len=0;
+	float64_t* output= predictions->get_labels(len);
+	int32_t total=testfeatures->get_num_vectors();
+	int32_t* label=testlabels->get_int_labels(len);
 	ASSERT(label);
 
 	SG_DEBUG("len:%d total:%d\n", len, total);
@@ -575,7 +589,7 @@ bool CGUIClassifier::test(CHAR* filename_out, CHAR* filename_roc)
 	return true;
 }
 
-bool CGUIClassifier::load(CHAR* filename, CHAR* type)
+bool CGUIClassifier::load(char* filename, char* type)
 {
 	bool result=false;
 
@@ -606,7 +620,7 @@ bool CGUIClassifier::load(CHAR* filename, CHAR* type)
 	return false;
 }
 
-bool CGUIClassifier::save(CHAR* param)
+bool CGUIClassifier::save(char* param)
 {
 	bool result=false;
 	param=CIO::skip_spaces(param);
@@ -632,7 +646,8 @@ bool CGUIClassifier::save(CHAR* param)
 	return result;
 }
 
-bool CGUIClassifier::set_perceptron_parameters(DREAL learnrate, INT maxiter)
+bool CGUIClassifier::set_perceptron_parameters(
+	float64_t learnrate, int32_t maxiter)
 {
 	if (learnrate<=0)
 		perceptron_learnrate=0.01;
@@ -648,7 +663,7 @@ bool CGUIClassifier::set_perceptron_parameters(DREAL learnrate, INT maxiter)
 	return true;
 }
 
-bool CGUIClassifier::set_svm_epsilon(DREAL epsilon)
+bool CGUIClassifier::set_svm_epsilon(float64_t epsilon)
 {
 	if (epsilon<0)
 		svm_epsilon=1e-4;
@@ -659,7 +674,7 @@ bool CGUIClassifier::set_svm_epsilon(DREAL epsilon)
 	return true;
 }
 
-bool CGUIClassifier::set_max_train_time(DREAL max)
+bool CGUIClassifier::set_max_train_time(float64_t max)
 {
 	if (max>0)
 	{
@@ -672,16 +687,19 @@ bool CGUIClassifier::set_max_train_time(DREAL max)
 	return true;
 }
 
-bool CGUIClassifier::set_svr_tube_epsilon(DREAL tube_epsilon)
+bool CGUIClassifier::set_svr_tube_epsilon(float64_t tube_epsilon)
 {
 	if (tube_epsilon<0)
 		svm_tube_epsilon=1e-2;
+	svm_tube_epsilon=tube_epsilon;
+
+	((CSVM*) classifier)->set_tube_epsilon(svm_tube_epsilon);
 	SG_INFO("Set to svr_tube_epsilon=%f.\n", svm_tube_epsilon);
 
 	return true;
 }
 
-bool CGUIClassifier::set_svm_one_class_nu(DREAL nu)
+bool CGUIClassifier::set_svm_one_class_nu(float64_t nu)
 {
 	if (nu<0 || nu>1)
 		svm_nu=0.5;
@@ -690,19 +708,28 @@ bool CGUIClassifier::set_svm_one_class_nu(DREAL nu)
 	return true;
 }
 
-bool CGUIClassifier::set_svm_mkl_parameters(DREAL weight_epsilon, DREAL C_mkl)
+bool CGUIClassifier::set_svm_mkl_parameters(
+	float64_t weight_epsilon, float64_t C_mkl, int32_t mkl_norm)
 {
 	if (weight_epsilon<0)
 		svm_weight_epsilon=1e-4;
 	if (C_mkl<0)
-		svm_C_mkl=1e-4;
+		svm_C_mkl=0;
+	if (mkl_norm!=1 || mkl_norm!=2)
+		svm_mkl_norm=1;
+
+	svm_weight_epsilon=weight_epsilon;
+	svm_C_mkl=C_mkl;
+	svm_mkl_norm=mkl_norm;
+
 	SG_INFO("Set to weight_epsilon=%f.\n", svm_weight_epsilon);
 	SG_INFO("Set to C_mkl=%f.\n", svm_C_mkl);
+	SG_INFO("Set to mkl_norm=%d.\n", svm_mkl_norm);
 
 	return true;
 }
 
-bool CGUIClassifier::set_svm_C(DREAL C1, DREAL C2)
+bool CGUIClassifier::set_svm_C(float64_t C1, float64_t C2)
 {
 	if (C1<0)
 		svm_C1=1.0;
@@ -719,7 +746,7 @@ bool CGUIClassifier::set_svm_C(DREAL C1, DREAL C2)
 	return true;
 }
 
-bool CGUIClassifier::set_svm_qpsize(INT qpsize)
+bool CGUIClassifier::set_svm_qpsize(int32_t qpsize)
 {
 	if (qpsize<2)
 		svm_qpsize=41;
@@ -730,7 +757,7 @@ bool CGUIClassifier::set_svm_qpsize(INT qpsize)
 	return true;
 }
 
-bool CGUIClassifier::set_svm_max_qpsize(INT max_qpsize)
+bool CGUIClassifier::set_svm_max_qpsize(int32_t max_qpsize)
 {
 	if (max_qpsize<50)
 		svm_max_qpsize=50;
@@ -741,7 +768,7 @@ bool CGUIClassifier::set_svm_max_qpsize(INT max_qpsize)
 	return true;
 }
 
-bool CGUIClassifier::set_svm_bufsize(INT bufsize)
+bool CGUIClassifier::set_svm_bufsize(int32_t bufsize)
 {
 	if (svm_bufsize<0)
 		svm_bufsize=3000;
@@ -785,30 +812,6 @@ bool CGUIClassifier::set_svm_batch_computation_enabled(bool enabled)
 	return true;
 }
 
-bool CGUIClassifier::set_svm_precompute_enabled(INT precompute)
-{
-	svm_use_precompute=(precompute==1);
-	svm_use_precompute_subkernel=(precompute==2);
-	svm_use_precompute_subkernel_light=(precompute==3);
-
-	if (svm_use_precompute)
-		SG_INFO("Enabling Kernel Matrix Precomputation.\n");
-	else
-		SG_INFO("Disabling Kernel Matrix Precomputation.\n");
-
-	if (svm_use_precompute_subkernel)
-		SG_INFO("Enabling Subkernel Matrix Precomputation.\n");
-	else
-		SG_INFO("Disabling Subkernel Matrix Precomputation.\n");
-
-	if (svm_use_precompute_subkernel_light)
-		SG_INFO("Enabling Subkernel Matrix Precomputation by SVM Light.\n");
-	else
-		SG_INFO("Disabling Subkernel Matrix Precomputation by SVM Light.\n");
-
-	return true;
-}
-
 bool CGUIClassifier::set_svm_linadd_enabled(bool enabled)
 {
 	svm_use_linadd=enabled;
@@ -843,6 +846,7 @@ bool CGUIClassifier::set_do_auc_maximization(bool do_auc)
 	return true;
 }
 
+
 CLabels* CGUIClassifier::classify(CLabels* output)
 {
 	ASSERT(classifier);
@@ -892,7 +896,6 @@ CLabels* CGUIClassifier::classify_kernelmachine(CLabels* output)
 {
 	CFeatures* trainfeatures=ui->ui_features->get_train_features();
 	CFeatures* testfeatures=ui->ui_features->get_test_features();
-	ui->ui_kernel->get_kernel()->set_precompute_matrix(false,false);
 
 	if (!classifier)
 		SG_ERROR("No kernelmachine available.\n");
@@ -905,15 +908,16 @@ CLabels* CGUIClassifier::classify_kernelmachine(CLabels* output)
 
 	CKernelMachine* km=(CKernelMachine*) classifier;
 	km->set_kernel(ui->ui_kernel->get_kernel());
-	ui->ui_kernel->get_kernel()->set_precompute_matrix(false,false);
 	km->set_batch_computation_enabled(svm_use_batch_computation);
 
 	SG_INFO("Starting kernel machine testing.\n");
 	return classifier->classify(output);
 }
 
-bool CGUIClassifier::get_trained_classifier(DREAL* &weights, INT &rows, INT &cols,
-		DREAL*& bias, INT& brows, INT& bcols)
+bool CGUIClassifier::get_trained_classifier(
+	float64_t* &weights, int32_t &rows, int32_t &cols, float64_t*& bias,
+	int32_t& brows, int32_t& bcols,
+	int32_t idx) // which SVM for MultiClass
 {
 	ASSERT(classifier);
 
@@ -932,7 +936,7 @@ bool CGUIClassifier::get_trained_classifier(DREAL* &weights, INT &rows, INT &col
 		case CT_LIBSVMONECLASS:
 		case CT_SVRLIGHT:
 		case CT_KRR:
-			return get_svm(weights, rows, cols, bias, brows, bcols);
+			return get_svm(weights, rows, cols, bias, brows, bcols, idx);
 			break;
 		case CT_PERCEPTRON:
 		case CT_LDA:
@@ -963,23 +967,35 @@ bool CGUIClassifier::get_trained_classifier(DREAL* &weights, INT &rows, INT &col
 	return false;
 }
 
-bool CGUIClassifier::get_svm(DREAL* &weights, INT& rows, INT& cols,
-		DREAL*& bias, INT& brows, INT& bcols)
+
+int32_t CGUIClassifier::get_num_svms()
+{
+	ASSERT(classifier);
+	return ((CMultiClassSVM*) classifier)->get_num_svms();
+}
+
+
+bool CGUIClassifier::get_svm(
+	float64_t* &weights, int32_t& rows, int32_t& cols, float64_t*& bias,
+	int32_t& brows, int32_t& bcols, int32_t idx)
 {
 	CSVM* svm=(CSVM*) classifier;
 
+	if (idx>-1) // should be MultiClassSVM
+		svm=((CMultiClassSVM*) svm)->get_svm(idx);
+
 	if (svm)
 	{
 		brows=1;
 		bcols=1;
-		bias=new DREAL[1];
+		bias=new float64_t[1];
 		*bias=svm->get_bias();
 
 		rows=svm->get_num_support_vectors();
 		cols=2;
-		weights=new DREAL[rows*cols];
+		weights=new float64_t[rows*cols];
 
-		for (int i=0; i<rows; i++)
+		for (int32_t i=0; i<rows; i++)
 		{
 			weights[i]=svm->get_alpha(i);
 			weights[i+rows]=svm->get_support_vector(i);
@@ -991,8 +1007,9 @@ bool CGUIClassifier::get_svm(DREAL* &weights, INT& rows, INT& cols,
 	return false;
 }
 
-bool CGUIClassifier::get_clustering(DREAL* &centers, INT& rows, INT& cols,
-		DREAL*& radi, INT& brows, INT& bcols)
+bool CGUIClassifier::get_clustering(
+	float64_t* &centers, int32_t& rows, int32_t& cols, float64_t*& radi,
+	int32_t& brows, int32_t& bcols)
 {
 	if (!classifier)
 		return false;
@@ -1015,26 +1032,16 @@ bool CGUIClassifier::get_clustering(DREAL* &centers, INT& rows, INT& cols,
 		{
 			CHierarchical* clustering=(CHierarchical*) classifier;
 
-			INT* a=NULL;
+			// radi == merge_distances, centers == pairs
 			bcols=1;
-			clustering->get_assignment(a, brows);
-			radi = new DREAL[brows*bcols];
-			for (INT i=0; i<brows*bcols; i++)
-				radi[i]=a[i];
+			clustering->get_merge_distance(radi, brows);
 
-			DREAL* d=NULL;
-			clustering->get_merge_distance(d, cols);
+			int32_t* p=NULL;
+			clustering->get_pairs(p, rows, cols);
+			centers=new float64_t[rows*cols]; // FIXME memleak
+			for (int32_t i=0; i<rows*cols; i++)
+				centers[i]=(float64_t) p[i];
 
-			INT* c=NULL;
-			clustering->get_pairs(c, rows, cols);
-			rows=rows+1;
-			centers=new DREAL[rows*cols];//FIXME memleak
-			for (INT i=0; i<cols; i++)
-			{
-				centers[3*i]=c[2*i];
-				centers[3*i+1]=c[2*i+1];
-				centers[3*i+2]=d[i];
-			}
 			break;
 		}
 
@@ -1045,15 +1052,16 @@ bool CGUIClassifier::get_clustering(DREAL* &centers, INT& rows, INT& cols,
 	return true;
 }
 
-bool CGUIClassifier::get_linear(DREAL* &weights, INT& rows, INT& cols,
-		DREAL*& bias, INT& brows, INT& bcols)
+bool CGUIClassifier::get_linear(
+	float64_t* &weights, int32_t& rows, int32_t& cols, float64_t*& bias,
+	int32_t& brows, int32_t& bcols)
 {
 	CLinearClassifier* linear=(CLinearClassifier*) classifier;
 
 	if (!linear)
 		return false;
 
-	bias=new DREAL[1];
+	bias=new float64_t[1];
 	*bias=linear->get_bias();
 	brows=1;
 	bcols=1;
@@ -1063,15 +1071,16 @@ bool CGUIClassifier::get_linear(DREAL* &weights, INT& rows, INT& cols,
 	return true;
 }
 
-bool CGUIClassifier::get_sparse_linear(DREAL* &weights, INT& rows, INT& cols,
-		DREAL*& bias, INT& brows, INT& bcols)
+bool CGUIClassifier::get_sparse_linear(
+	float64_t* &weights, int32_t& rows, int32_t& cols, float64_t*& bias,
+	int32_t& brows, int32_t& bcols)
 {
 	CSparseLinearClassifier* linear=(CSparseLinearClassifier*) classifier;
 
 	if (!linear)
 		return false;
 
-	bias=new DREAL[1];
+	bias=new float64_t[1];
 	*bias=linear->get_bias();
 	brows=1;
 	bcols=1;
@@ -1086,7 +1095,6 @@ CLabels* CGUIClassifier::classify_distancemachine(CLabels* output)
 {
 	CFeatures* trainfeatures=ui->ui_features->get_train_features();
 	CFeatures* testfeatures=ui->ui_features->get_test_features();
-	ui->ui_distance->get_distance()->set_precompute_matrix(false);
 
 	if (!classifier)
 	{
@@ -1113,7 +1121,6 @@ CLabels* CGUIClassifier::classify_distancemachine(CLabels* output)
 	  
 	((CDistanceMachine*) classifier)->set_distance(
 		ui->ui_distance->get_distance());
-	ui->ui_distance->get_distance()->set_precompute_matrix(false);
 	SG_INFO("starting distance machine testing\n") ;
 	return classifier->classify(output);
 }
@@ -1166,7 +1173,7 @@ CLabels* CGUIClassifier::classify_byte_linear(CLabels* output)
 		return false ;
 	}
 
-	((CWDSVMOcas*) classifier)->set_features((CStringFeatures<BYTE>*) testfeatures);
+	((CWDSVMOcas*) classifier)->set_features((CStringFeatures<uint8_t>*) testfeatures);
 	SG_INFO("starting linear classifier testing\n") ;
 	return classifier->classify(output);
 }
@@ -1192,16 +1199,15 @@ CLabels* CGUIClassifier::classify_sparse_linear(CLabels* output)
 		return false ;
 	}
 
-	((CSparseLinearClassifier*) classifier)->set_features((CSparseFeatures<DREAL>*) testfeatures);
+	((CSparseLinearClassifier*) classifier)->set_features((CSparseFeatures<float64_t>*) testfeatures);
 	SG_INFO("starting linear classifier testing\n") ;
 	return classifier->classify(output);
 }
 
-bool CGUIClassifier::classify_example(INT idx, DREAL &result)
+bool CGUIClassifier::classify_example(int32_t idx, float64_t &result)
 {
 	CFeatures* trainfeatures=ui->ui_features->get_train_features();
 	CFeatures* testfeatures=ui->ui_features->get_test_features();
-	ui->ui_kernel->get_kernel()->set_precompute_matrix(false,false);
 
 	if (!classifier)
 	{
@@ -1232,4 +1238,19 @@ bool CGUIClassifier::classify_example(INT idx, DREAL &result)
 	result=classifier->classify_example(idx);
 	return true ;
 }
+
+
+bool CGUIClassifier::set_krr_tau(float64_t tau)
+{
+#ifdef HAVE_LAPACK
+	krr_tau=tau;
+	((CKRR*) classifier)->set_tau(krr_tau);
+	SG_INFO("Set to krr_tau=%f.\n", krr_tau);
+
+	return true;
+#else
+	return false;
+#endif
+}
+
 #endif
