@@ -20,6 +20,7 @@
 #define N_TRANSLATE_STRING					"translate_string"
 #define N_GET_HMM							"get_hmm"
 #define N_GET_VITERBI_PATH					"get_viterbi_path"
+#define N_GET_NUM_SVMS						"get_num_svms"
 #define N_GET_SVM							"get_svm"
 #define N_GET_CLASSIFIER					"get_classifier"
 #define N_GET_CLUSTERING					"get_clustering"
@@ -51,11 +52,13 @@
 #define N_MODEL_PROB_NO_B_TRANS				"model_prob_no_b_trans"
 #define N_BEST_PATH_NO_B_TRANS				"best_path_no_b_trans"
 #define N_BEST_PATH_TRANS_DERIV				"best_path_trans_deriv"
-#define N_BEST_PATH_TRANS				"best_path_trans"
-#define N_SET_PLIF_STRUCT				"set_plif_struct"
-#define N_GET_PLIF_STRUCT				"get_plif_struct"
+#define N_BEST_PATH_TRANS					"best_path_trans"
+#define N_SET_PLIF_STRUCT					"set_plif_struct"
+#define N_GET_PLIF_STRUCT					"get_plif_struct"
+#define N_PRECOMPUTE_SUBKERNELS				"precompute_subkernels"
 #define N_PRECOMPUTE_CONTENT_SVMS			"precompute_content_svms"
-#define N_SET_MODEL					"set_model"
+#define N_PRECOMPUTE_TILING_FEATURES		"precompute_tiling_features"
+#define N_SET_MODEL							"set_model"
 #define N_SET_FEATURE_MATRIX				"set_feature_matrix"
 #define N_BEST_PATH_2STRUCT					"best_path_2struct"
 #define N_BEST_PATH_TRANS_SIMPLE			"best_path_trans_simple"
@@ -63,6 +66,7 @@
 #define N_SET_SVM							"set_svm"
 #define N_SET_CUSTOM_KERNEL					"set_custom_kernel"
 #define N_SET_KERNEL_INIT					"set_kernel_init"
+#define N_SET_KERNEL_NORMALIZATION			"set_kernel_normalization"
 #define N_SET_FEATURES						"set_features"
 #define N_FROM_POSITION_LIST				"from_position_list"
 #define N_ADD_FEATURES						"add_features"
@@ -101,8 +105,8 @@
 #define N_SAVE_PREPROC					"save_preproc"
 #define N_LOAD_HMM						"load_hmm"
 #define N_SAVE_HMM						"save_hmm"
-#define N_LOAD_SVM						"load_svm"
-#define N_SAVE_SVM						"save_svm"
+#define N_LOAD_CLASSIFIER				"load_classifier"
+#define N_SAVE_CLASSIFIER				"save_classifier"
 #define N_LOAD_KERNEL_INIT				"load_kernel_init"
 #define N_SAVE_KERNEL_INIT				"save_kernel_init"
 #define N_LOAD_LABELS					"load_labels"
@@ -127,6 +131,8 @@
 
 
 #define N_SET_KERNEL_OPTIMIZATION_TYPE	"set_kernel_optimization_type"
+#define N_SET_PRIOR_PROBS				"set_prior_probs"
+#define N_SET_PRIOR_PROBS_FROM_LABELS	"set_prior_probs_from_labels"
 #define N_ATTACH_PREPROC				"attach_preproc"
 #define N_ADD_PREPROC					"add_preproc"
 #define N_DEL_PREPROC					"del_preproc"
@@ -174,7 +180,6 @@
 #define N_USE_MKL						"use_mkl"
 #define N_USE_SHRINKING					"use_shrinking"
 #define N_USE_LINADD					"use_linadd"
-#define N_USE_PRECOMPUTE				"use_precompute"
 #define N_OUTPUT_HMM					"output_hmm"
 #define N_OUTPUT_HMM_DEFINED			"output_hmm_defined"
 #define N_QUIT							"quit"
@@ -184,6 +189,7 @@
 #define N_HELP							"help"
 #define N_SYSTEM						"!"
 #define N_SET_THRESHOLD					"set_threshold"
+#define N_INIT_RANDOM					"init_random"
 #define N_SVM_TRAIN						"svm_train"
 #define N_SVM_TEST						"svm_test"
 #define N_ONE_CLASS_HMM_TEST			"one_class_hmm_test"
