@@ -35,15 +35,15 @@ class CGUIDistance : public CSGObject
 	/** create generic distance given by type */
 	CDistance* create_generic(EDistanceType type);
 	/** create Minkowski Metric */
-	CDistance* create_minkowski(DREAL k=3);
+	CDistance* create_minkowski(float64_t k=3);
 	/** create HammingWord Distance */
 	CDistance* create_hammingword(bool use_sign=false);
 
 	/** initialize distance */
-	bool init_distance(CHAR* target);
-	bool load_distance_init(CHAR* param);
-	bool save_distance_init(CHAR* param);
-	bool save_distance(CHAR* param);
+	bool init_distance(char* target);
+	bool load_distance_init(char* param);
+	bool save_distance_init(char* param);
+	bool save_distance(char* param);
 
 	bool is_initialized() { return initialized; }
 
