@@ -47,8 +47,8 @@ void CGUIFeatures::invalidate_test()
 }
 
 bool CGUIFeatures::load(
-	CHAR* filename, CHAR* fclass, CHAR* type, CHAR* target, INT size,
-	INT comp_features)
+	char* filename, char* fclass, char* type, char* target, int32_t size,
+	int32_t comp_features)
 {
 	bool result=false;
 	CFeatures** f_ptr=NULL;
@@ -108,29 +108,29 @@ bool CGUIFeatures::load(
 	{
 		if (strncmp(type, "REAL", 4)==0)
 		{
-			*f_ptr=new CStringFeatures<DREAL>(filename);
+			*f_ptr=new CStringFeatures<float64_t>(filename);
 		}
 		else if (strncmp(type, "BYTE", 4)==0)
 		{
 			///FIXME make CHAR type configurable... it is DNA by default
-			*f_ptr=new CStringFeatures<BYTE>(filename, DNA);
+			*f_ptr=new CStringFeatures<uint8_t>(filename, DNA);
 		}
 		else if (strncmp(type, "CHAR", 4)==0)
 		{
 			///FIXME make CHAR type configurable... it is DNA by default
-			*f_ptr=new CStringFeatures<CHAR>(filename, DNA);
+			*f_ptr=new CStringFeatures<char>(filename, DNA);
 		}
 		else if (strncmp(type, "SHORT", 5)==0)
 		{
-			*f_ptr=new CStringFeatures<SHORT>(filename);
+			*f_ptr=new CStringFeatures<int16_t>(filename);
 		}
 		else if (strncmp(type, "WORD", 4)==0)
 		{
-			*f_ptr=new CStringFeatures<WORD>(filename);
+			*f_ptr=new CStringFeatures<uint16_t>(filename);
 		}
 		else if (strncmp(type, "ULONG", 5)==0)
 		{
-			*f_ptr=new CStringFeatures<ULONG>(filename);
+			*f_ptr=new CStringFeatures<uint64_t>(filename);
 		}
 		else
 		{
@@ -142,7 +142,7 @@ bool CGUIFeatures::load(
 	return result;
 }
 
-bool CGUIFeatures::save(CHAR* filename, CHAR* type, CHAR* target)
+bool CGUIFeatures::save(char* filename, char* type, char* target)
 {
 	bool result=false;
 
@@ -201,7 +201,7 @@ bool CGUIFeatures::save(CHAR* filename, CHAR* type, CHAR* target)
 	return result;
 }
 
-bool CGUIFeatures::clean(CHAR* target)
+bool CGUIFeatures::clean(char* target)
 {
 	if (strncmp(target, "TRAIN", 5)==0)
 		set_train_features(NULL);
@@ -214,7 +214,7 @@ bool CGUIFeatures::clean(CHAR* target)
 }
 
 bool CGUIFeatures::obtain_by_sliding_window(
-	CHAR* target, INT winsize, INT shift, INT skip)
+	char* target, int32_t winsize, int32_t shift, int32_t skip)
 {
 	ASSERT(winsize>0);
 	ASSERT(shift>0);
@@ -246,13 +246,13 @@ bool CGUIFeatures::obtain_by_sliding_window(
 	switch (features->get_feature_type())
 	{
 		case F_CHAR:
-			return ( ((CStringFeatures<CHAR>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
+			return ( ((CStringFeatures<char>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
 		case F_BYTE:
-			return ( ((CStringFeatures<BYTE>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
+			return ( ((CStringFeatures<uint8_t>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
 		case F_WORD:
-			return ( ((CStringFeatures<WORD>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
+			return ( ((CStringFeatures<uint16_t>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
 		case F_ULONG:
-			return ( ((CStringFeatures<ULONG>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
+			return ( ((CStringFeatures<uint64_t>*) features)->obtain_by_sliding_window(winsize, shift, skip)>0);
 		default:
 			SG_SERROR("Unsupported string features type.\n");
 			return false;
@@ -261,7 +261,7 @@ bool CGUIFeatures::obtain_by_sliding_window(
 	return false;
 }
 
-bool CGUIFeatures::reshape(CHAR* target, INT num_feat, INT num_vec)
+bool CGUIFeatures::reshape(char* target, int32_t num_feat, int32_t num_vec)
 {
 	CFeatures** f_ptr=NULL;
 
@@ -294,7 +294,7 @@ bool CGUIFeatures::reshape(CHAR* target, INT num_feat, INT num_vec)
 	return result;
 }
 
-CFeatures* CGUIFeatures::get_convert_features(CHAR* target)
+CFeatures* CGUIFeatures::get_convert_features(char* target)
 {
 	CFeatures* features;
 
@@ -311,7 +311,7 @@ CFeatures* CGUIFeatures::get_convert_features(CHAR* target)
 	return features;
 }
 
-bool CGUIFeatures::set_convert_features(CFeatures* features, CHAR* target)
+bool CGUIFeatures::set_convert_features(CFeatures* features, char* target)
 {
 	CFeatures* features_prev;
 
@@ -343,7 +343,8 @@ bool CGUIFeatures::set_convert_features(CFeatures* features, CHAR* target)
 	return true;
 }
 
-CSparseFeatures<DREAL>* CGUIFeatures::convert_simple_real_to_sparse_real(CRealFeatures* src)
+CSparseFeatures<float64_t>* CGUIFeatures::convert_simple_real_to_sparse_real(
+	CRealFeatures* src)
 {
 	if (src &&
 		src->get_feature_class()==C_SIMPLE &&
@@ -351,10 +352,10 @@ CSparseFeatures<DREAL>* CGUIFeatures::convert_simple_real_to_sparse_real(CRealFe
 	{
 		//create sparse features with 0 cache
 		SG_INFO("Attempting to convert dense feature matrix to a sparse one.\n");
-		CSparseFeatures<DREAL>* target=new CSparseFeatures<DREAL>(0);
-		INT num_f=0;
-		INT num_v=0;
-		DREAL* feats=src->get_feature_matrix(num_f, num_v);
+		CSparseFeatures<float64_t>* target=new CSparseFeatures<float64_t>(0);
+		int32_t num_f=0;
+		int32_t num_v=0;
+		float64_t* feats=src->get_feature_matrix(num_f, num_v);
 		if (target->set_full_feature_matrix(feats, num_f, num_v))
 			return target;
 
@@ -366,29 +367,30 @@ CSparseFeatures<DREAL>* CGUIFeatures::convert_simple_real_to_sparse_real(CRealFe
 	return NULL;
 }
 
-CStringFeatures<CHAR>* CGUIFeatures::convert_simple_char_to_string_char(CCharFeatures* src)
+CStringFeatures<char>* CGUIFeatures::convert_simple_char_to_string_char(
+	CCharFeatures* src)
 {
 	if (src && src->get_feature_class()==C_SIMPLE)
 	{
-		INT num_vec=src->get_num_vectors();
-		T_STRING<CHAR>* strings=new T_STRING<CHAR>[num_vec];
-		INT max_len=-1;
+		int32_t num_vec=src->get_num_vectors();
+		T_STRING<char>* strings=new T_STRING<char>[num_vec];
+		int32_t max_len=-1;
 
-		for (INT i=0; i<num_vec; i++)
+		for (int32_t i=0; i<num_vec; i++)
 		{
 			bool to_free=false;
-			INT len=0;
-			CHAR* str=src->get_feature_vector(i, len, to_free);
+			int32_t len=0;
+			char* str=src->get_feature_vector(i, len, to_free);
 			strings[i].length=len ;
-			for (int j=0; j<len; j++)
+			for (int32_t j=0; j<len; j++)
 				if (str[j]==0)
 				{
 					strings[i].length=j ;
 					break ;
 				} ;
-			strings[i].string=new CHAR[strings[i].length];
+			strings[i].string=new char[strings[i].length];
 
-			for (int j=0; j<strings[i].length; j++)
+			for (int32_t j=0; j<strings[i].length; j++)
 				strings[i].string[j]=str[j];
 
 			if (strings[i].length> max_len)
@@ -397,7 +399,7 @@ CStringFeatures<CHAR>* CGUIFeatures::convert_simple_char_to_string_char(CCharFea
 			src->free_feature_vector(str, i, to_free);
 		}
 
-		CStringFeatures<CHAR>* target=new CStringFeatures<CHAR>(
+		CStringFeatures<char>* target=new CStringFeatures<char>(
 			new CAlphabet(src->get_alphabet()));
 		target->set_features(strings, num_vec, max_len);
 		return target;
@@ -408,7 +410,8 @@ CStringFeatures<CHAR>* CGUIFeatures::convert_simple_char_to_string_char(CCharFea
 	return NULL;
 }
 
-CRealFeatures* CGUIFeatures::convert_simple_word_to_simple_salzberg(CWordFeatures* src)
+CRealFeatures* CGUIFeatures::convert_simple_word_to_simple_salzberg(
+	CWordFeatures* src)
 {
 	CPluginEstimate* pie=ui->ui_pluginestimate->get_estimator();
 
@@ -418,20 +421,20 @@ CRealFeatures* CGUIFeatures::convert_simple_word_to_simple_salzberg(CWordFeature
 		pie)
 	{
 		CRealFeatures* target=new CRealFeatures(0);
-		INT num_feat=src->get_num_features();
-		INT num_vec=src->get_num_vectors();
-		DREAL* fm=new DREAL[num_vec*num_feat];
+		int32_t num_feat=src->get_num_features();
+		int32_t num_vec=src->get_num_vectors();
+		float64_t* fm=new float64_t[num_vec*num_feat];
 
 		if (fm)
 		{
-			for (INT i=0; i<num_vec; i++)
+			for (int32_t i=0; i<num_vec; i++)
 			{
-				INT len=0;
+				int32_t len=0;
 				bool to_free=false;
-				WORD* vec = src->get_feature_vector(i, len, to_free);
+				uint16_t* vec = src->get_feature_vector(i, len, to_free);
 				ASSERT(num_feat==len);
 
-				for (INT j=0; j<num_feat; j++)
+				for (int32_t j=0; j<num_feat; j++)
 					fm[i*num_feat+j]=
 						pie->get_parameterwise_log_odds(vec[j], j);
 
@@ -449,7 +452,8 @@ CRealFeatures* CGUIFeatures::convert_simple_word_to_simple_salzberg(CWordFeature
 }
 
 
-CTOPFeatures* CGUIFeatures::convert_string_word_to_simple_top(CStringFeatures<WORD>* src)
+CTOPFeatures* CGUIFeatures::convert_string_word_to_simple_top(
+	CStringFeatures<uint16_t>* src)
 {
 	CTOPFeatures* tf=NULL;
 
@@ -481,7 +485,8 @@ CTOPFeatures* CGUIFeatures::convert_string_word_to_simple_top(CStringFeatures<WO
 	return tf;
 }
 
-CFKFeatures* CGUIFeatures::convert_string_word_to_simple_fk(CStringFeatures<WORD>* src)
+CFKFeatures* CGUIFeatures::convert_string_word_to_simple_fk(
+	CStringFeatures<uint16_t>* src)
 {
 	CFKFeatures* fkf=NULL;
 
@@ -489,12 +494,12 @@ CFKFeatures* CGUIFeatures::convert_string_word_to_simple_fk(CStringFeatures<WORD
 
 	if (ui->ui_hmm->get_pos() && ui->ui_hmm->get_neg())
 	{
-		CStringFeatures<WORD>* old_obs_pos=
+		CStringFeatures<uint16_t>* old_obs_pos=
 			ui->ui_hmm->get_pos()->get_observations();
-		CStringFeatures<WORD>* old_obs_neg=
+		CStringFeatures<uint16_t>* old_obs_neg=
 			ui->ui_hmm->get_neg()->get_observations();
 
-		CStringFeatures<WORD>* string_feat=src;
+		CStringFeatures<uint16_t>* string_feat=src;
 		ui->ui_hmm->get_pos()->set_observations(string_feat);
 		ui->ui_hmm->get_neg()->set_observations(string_feat);
 
@@ -518,7 +523,8 @@ CFKFeatures* CGUIFeatures::convert_string_word_to_simple_fk(CStringFeatures<WORD
 }
 
 
-CRealFeatures* CGUIFeatures::convert_sparse_real_to_simple_real(CSparseFeatures<DREAL>* src)
+CRealFeatures* CGUIFeatures::convert_sparse_real_to_simple_real(
+	CSparseFeatures<float64_t>* src)
 {
 	if (src &&
 		src->get_feature_class()==C_SPARSE &&
@@ -529,9 +535,9 @@ CRealFeatures* CGUIFeatures::convert_sparse_real_to_simple_real(CSparseFeatures<
 		CRealFeatures* rf=new CRealFeatures(0);
 		if (rf)
 		{
-			INT num_f=0;
-			INT num_v=0;
-			DREAL* feats=src->get_full_feature_matrix(num_f, num_v);
+			int32_t num_f=0;
+			int32_t num_v=0;
+			float64_t* feats=src->get_full_feature_matrix(num_f, num_v);
 			rf->set_feature_matrix(feats, num_f, num_v);
 			return rf;
 		}
@@ -543,7 +549,7 @@ CRealFeatures* CGUIFeatures::convert_sparse_real_to_simple_real(CSparseFeatures<
 }
 
 CWordFeatures* CGUIFeatures::convert_simple_char_to_simple_word(
-	CCharFeatures* src, INT order, INT start, INT gap)
+	CCharFeatures* src, int32_t order, int32_t start, int32_t gap)
 {
 	if (src &&
 		src->get_feature_class()==C_SIMPLE &&
@@ -571,7 +577,8 @@ CWordFeatures* CGUIFeatures::convert_simple_char_to_simple_word(
 	return NULL;
 }
 
-CShortFeatures* CGUIFeatures::convert_simple_char_to_simple_short(CCharFeatures* src, INT order, INT start, INT gap)
+CShortFeatures* CGUIFeatures::convert_simple_char_to_simple_short(
+	CCharFeatures* src, int32_t order, int32_t start, int32_t gap)
 {
 	if (src &&
 		src->get_feature_class()==C_SIMPLE &&
@@ -599,7 +606,8 @@ CShortFeatures* CGUIFeatures::convert_simple_char_to_simple_short(CCharFeatures*
 	return NULL;
 }
 
-CRealFeatures* CGUIFeatures::convert_simple_char_to_simple_align(CCharFeatures* src, DREAL gap_cost)
+CRealFeatures* CGUIFeatures::convert_simple_char_to_simple_align(
+	CCharFeatures* src, float64_t gap_cost)
 {
 	if (src &&
 		src->get_feature_class()==C_SIMPLE &&
@@ -625,7 +633,7 @@ CRealFeatures* CGUIFeatures::convert_simple_char_to_simple_align(CCharFeatures*
 	return NULL;
 }
 
-bool CGUIFeatures::set_reference_features(CHAR* target)
+bool CGUIFeatures::set_reference_features(char* target)
 {
 	if (strncmp(target, "TRAIN", 5)==0)
 	{
@@ -693,7 +701,7 @@ void CGUIFeatures::add_test_features(CFeatures* f)
 		SG_ERROR("Appending feature object failed.\n");
 }
 
-bool CGUIFeatures::del_last_features(CHAR* target)
+bool CGUIFeatures::del_last_features(char* target)
 {
 	CCombinedFeatures* cf=NULL;
 	if (strncmp(target, "TRAIN", 5)==0)
