@@ -43,7 +43,7 @@ CGUIHMM::~CGUIHMM()
 
 }
 
-bool CGUIHMM::new_hmm(INT n, INT m)
+bool CGUIHMM::new_hmm(int32_t n, int32_t m)
 {
 	delete working;
 	working=new CHMM(n, m, NULL, PSEUDO);
@@ -63,8 +63,8 @@ bool CGUIHMM::baum_welch_train()
 		trainfeatures->get_feature_class()!=C_STRING)
 		SG_ERROR("Features must be STRING of type WORD.\n");
 
-	CStringFeatures<WORD>* sf=(CStringFeatures<WORD>*) trainfeatures;
-	SG_DEBUG("Stringfeatures have %ld orig_symbols %ld symbols %d order %ld max_symbols\n",  (LONG) sf->get_original_num_symbols(), (LONG) sf->get_num_symbols(), sf->get_order(), (LONG) sf->get_max_num_symbols());
+	CStringFeatures<uint16_t>* sf=(CStringFeatures<uint16_t>*) trainfeatures;
+	SG_DEBUG("Stringfeatures have %ld orig_symbols %ld symbols %d order %ld max_symbols\n",  (int64_t) sf->get_original_num_symbols(), (int64_t) sf->get_num_symbols(), sf->get_order(), (int64_t) sf->get_max_num_symbols());
 
 	working->set_observations(sf);
 
@@ -84,7 +84,7 @@ bool CGUIHMM::baum_welch_trans_train()
 		trainfeatures->get_feature_class()!=C_STRING)
 		SG_ERROR("Features must be STRING of type WORD.\n");
 
-	working->set_observations((CStringFeatures<WORD>*) trainfeatures);
+	working->set_observations((CStringFeatures<uint16_t>*) trainfeatures);
 
 	return working->baum_welch_viterbi_train(BW_TRANS);
 }
@@ -120,7 +120,7 @@ bool CGUIHMM::viterbi_train_defined()
 	return working->baum_welch_viterbi_train(VIT_DEFINED);
 }
 
-bool CGUIHMM::linear_train(CHAR align)
+bool CGUIHMM::linear_train(char align)
 {
 	if (!working)
 		SG_ERROR("Create HMM first.\n");
@@ -132,7 +132,7 @@ bool CGUIHMM::linear_train(CHAR align)
 		trainfeatures->get_feature_class()!=C_STRING)
 		SG_ERROR("Features must be STRING of type WORD.\n");
 
-	working->set_observations((CStringFeatures<WORD>*) ui->
+	working->set_observations((CStringFeatures<uint16_t>*) ui->
 		ui_features->get_train_features());
 
 	bool right_align=false;
@@ -149,7 +149,7 @@ bool CGUIHMM::linear_train(CHAR align)
 }
 
 bool CGUIHMM::one_class_test(
-	CHAR* filename_out, CHAR* filename_roc, bool is_linear)
+	char* filename_out, char* filename_roc, bool is_linear)
 {
 	bool result=false;
 	FILE* file_out=stdout;
@@ -177,17 +177,17 @@ bool CGUIHMM::one_class_test(
 	if (ui->ui_features->get_test_features())
 		SG_ERROR("Assign posttest and negtest observations first!\n");
 
-	CStringFeatures<WORD>* obs= (CStringFeatures<WORD>*) ui->
+	CStringFeatures<uint16_t>* obs= (CStringFeatures<uint16_t>*) ui->
 		ui_features->get_test_features();
 	working->set_observations(obs);
-	CStringFeatures<WORD>* old_test=working->get_observations();
+	CStringFeatures<uint16_t>* old_test=working->get_observations();
 	CLabels* lab=ui->ui_labels->get_test_labels();
-	INT total=obs->get_num_vectors();
+	int32_t total=obs->get_num_vectors();
 	ASSERT(lab && total==lab->get_num_labels());
-	DREAL* output=new DREAL[total];
-	INT* label=new INT[total];
+	float64_t* output=new float64_t[total];
+	int32_t* label=new int32_t[total];
 
-	for (INT dim=0; dim<total; dim++)
+	for (int32_t dim=0; dim<total; dim++)
 	{
 		output[dim]= is_linear ? working->linear_model_probability(dim) : working->model_probability(dim);
 		label[dim]= lab->get_int_label(dim);
@@ -209,16 +209,16 @@ bool CGUIHMM::one_class_test(
 	return result;
 }
 
-bool CGUIHMM::hmm_classify(CHAR* param)
+bool CGUIHMM::hmm_classify(char* param)
 {
 	bool result=false;
-	CHAR outputname[1024];
-	CHAR rocfname[1024];
+	char outputname[1024];
+	char rocfname[1024];
 	FILE* outputfile=stdout;
 	FILE* rocfile=NULL;
-	INT numargs=-1;
-	INT poslinear=0;
-	INT neglinear=0;
+	int32_t numargs=-1;
+	int32_t poslinear=0;
+	int32_t neglinear=0;
 
 	param=CIO::skip_spaces(param);
 
@@ -250,24 +250,24 @@ bool CGUIHMM::hmm_classify(CHAR* param)
 	{
 		if (ui->ui_features->get_test_features())
 		{
-			CStringFeatures<WORD>* o= (CStringFeatures<WORD>*) ui->
+			CStringFeatures<uint16_t>* o= (CStringFeatures<uint16_t>*) ui->
 				ui_features->get_test_features();
 			CLabels* lab= ui->ui_labels->get_test_labels();
 
-			//CStringFeatures<WORD>* old_pos=pos->get_observations();
-			//CStringFeatures<WORD>* old_neg=neg->get_observations();
+			//CStringFeatures<uint16_t>* old_pos=pos->get_observations();
+			//CStringFeatures<uint16_t>* old_neg=neg->get_observations();
 
 			pos->set_observations(o);
 			neg->set_observations(o);
 
-			INT total=o->get_num_vectors();
+			int32_t total=o->get_num_vectors();
 
-			DREAL* output = new DREAL[total];	
-			INT* label= new INT[total];	
+			float64_t* output = new float64_t[total];	
+			int32_t* label= new int32_t[total];	
 
 			SG_INFO( "classifying using neg %s hmm vs. pos %s hmm\n", neglinear ? "linear" : "", poslinear ? "linear" : "");
 
-			for (INT dim=0; dim<total; dim++)
+			for (int32_t dim=0; dim<total; dim++)
 			{
 				output[dim]= 
 					(poslinear ? pos->linear_model_probability(dim) : pos->model_probability(dim)) -
@@ -297,7 +297,7 @@ bool CGUIHMM::hmm_classify(CHAR* param)
 }
 
 bool CGUIHMM::hmm_test(
-	CHAR* filename_out, CHAR* filename_roc,
+	char* filename_out, char* filename_roc,
 	bool is_pos_linear, bool is_neg_linear)
 {
 	bool result=false;
@@ -326,22 +326,22 @@ bool CGUIHMM::hmm_test(
 	if (!ui->ui_features->get_test_features())
 		SG_ERROR("Assign test features first!\n");
 
-	CStringFeatures<WORD>* o=(CStringFeatures<WORD>*) ui->
+	CStringFeatures<uint16_t>* o=(CStringFeatures<uint16_t>*) ui->
 		ui_features->get_test_features();
 	ASSERT(o);
 	CLabels* lab=ui->ui_labels->get_test_labels();
-	CStringFeatures<WORD>* old_pos=pos->get_observations();
-	CStringFeatures<WORD>* old_neg=neg->get_observations();
+	CStringFeatures<uint16_t>* old_pos=pos->get_observations();
+	CStringFeatures<uint16_t>* old_neg=neg->get_observations();
 	pos->set_observations(o);
 	neg->set_observations(o);
-	INT total=o->get_num_vectors();
+	int32_t total=o->get_num_vectors();
 	ASSERT(lab && total==lab->get_num_labels());
-	DREAL* output=new DREAL[total];
-	INT* label=new INT[total];
+	float64_t* output=new float64_t[total];
+	int32_t* label=new int32_t[total];
 
 	SG_INFO("Testing using neg %s hmm vs. pos %s hmm\n", is_neg_linear ? "linear" : "", is_pos_linear ? "linear" : "");
 
-	for (INT dim=0; dim<total; dim++)
+	for (int32_t dim=0; dim<total; dim++)
 	{
 		output[dim]=
 			(is_pos_linear ? pos->linear_model_probability(dim) : pos->model_probability(dim)) -
@@ -370,13 +370,13 @@ bool CGUIHMM::hmm_test(
 
 CLabels* CGUIHMM::classify(CLabels* result)
 {
-	CStringFeatures<WORD>* obs= (CStringFeatures<WORD>*) ui->
+	CStringFeatures<uint16_t>* obs= (CStringFeatures<uint16_t>*) ui->
 		ui_features->get_test_features();
 	ASSERT(obs);
-	INT num_vec=obs->get_num_vectors();
+	int32_t num_vec=obs->get_num_vectors();
 
-	//CStringFeatures<WORD>* old_pos=pos->get_observations();
-	//CStringFeatures<WORD>* old_neg=neg->get_observations();
+	//CStringFeatures<uint16_t>* old_pos=pos->get_observations();
+	//CStringFeatures<uint16_t>* old_neg=neg->get_observations();
 
 	pos->set_observations(obs);
 	neg->set_observations(obs);
@@ -384,7 +384,7 @@ CLabels* CGUIHMM::classify(CLabels* result)
 	if (!result)
 		result=new CLabels(num_vec);
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 		result->set_label(i, pos->model_probability(i) - neg->model_probability(i));
 
 	//pos->set_observations(old_pos);
@@ -392,19 +392,19 @@ CLabels* CGUIHMM::classify(CLabels* result)
 	return result;
 }
 
-DREAL CGUIHMM::classify_example(INT idx)
+float64_t CGUIHMM::classify_example(int32_t idx)
 {
-	CStringFeatures<WORD>* obs= (CStringFeatures<WORD>*) ui->
+	CStringFeatures<uint16_t>* obs= (CStringFeatures<uint16_t>*) ui->
 		ui_features->get_test_features();
 	ASSERT(obs);
 
-	//CStringFeatures<WORD>* old_pos=pos->get_observations();
-	//CStringFeatures<WORD>* old_neg=neg->get_observations();
+	//CStringFeatures<uint16_t>* old_pos=pos->get_observations();
+	//CStringFeatures<uint16_t>* old_neg=neg->get_observations();
 
 	pos->set_observations(obs);
 	neg->set_observations(obs);
 
-	DREAL result=pos->model_probability(idx) - neg->model_probability(idx);
+	float64_t result=pos->model_probability(idx) - neg->model_probability(idx);
 	//pos->set_observations(old_pos);
 	//neg->set_observations(old_neg);
 	return result;
@@ -414,18 +414,18 @@ CLabels* CGUIHMM::one_class_classify(CLabels* result)
 {
 	ASSERT(working);
 
-	CStringFeatures<WORD>* obs= (CStringFeatures<WORD>*) ui->
+	CStringFeatures<uint16_t>* obs= (CStringFeatures<uint16_t>*) ui->
 		ui_features->get_test_features();
 	ASSERT(obs);
-	INT num_vec=obs->get_num_vectors();
+	int32_t num_vec=obs->get_num_vectors();
 
-	//CStringFeatures<WORD>* old_pos=working->get_observations();
+	//CStringFeatures<uint16_t>* old_pos=working->get_observations();
 	working->set_observations(obs);
 
 	if (!result)
 		result=new CLabels(num_vec);
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 		result->set_label(i, working->model_probability(i));
 
 	//working->set_observations(old_pos);
@@ -436,18 +436,18 @@ CLabels* CGUIHMM::linear_one_class_classify(CLabels* result)
 {
 	ASSERT(working);
 
-	CStringFeatures<WORD>* obs= (CStringFeatures<WORD>*) ui->
+	CStringFeatures<uint16_t>* obs= (CStringFeatures<uint16_t>*) ui->
 		ui_features->get_test_features();
 	ASSERT(obs);
-	INT num_vec=obs->get_num_vectors();
+	int32_t num_vec=obs->get_num_vectors();
 
-	//CStringFeatures<WORD>* old_pos=working->get_observations();
+	//CStringFeatures<uint16_t>* old_pos=working->get_observations();
 	working->set_observations(obs);
 
 	if (!result)
 		result=new CLabels(num_vec);
 
-	for (INT i=0; i<num_vec; i++)
+	for (int32_t i=0; i<num_vec; i++)
 		result->set_label(i, working->linear_model_probability(i));
 
 	//working->set_observations(old_pos);
@@ -455,25 +455,25 @@ CLabels* CGUIHMM::linear_one_class_classify(CLabels* result)
 }
 
 
-DREAL CGUIHMM::one_class_classify_example(INT idx)
+float64_t CGUIHMM::one_class_classify_example(int32_t idx)
 {
 	ASSERT(working);
 
-	CStringFeatures<WORD>* obs= (CStringFeatures<WORD>*) ui->
+	CStringFeatures<uint16_t>* obs= (CStringFeatures<uint16_t>*) ui->
 		ui_features->get_test_features();
 	ASSERT(obs);
 
-	//CStringFeatures<WORD>* old_pos=pos->get_observations();
+	//CStringFeatures<uint16_t>* old_pos=pos->get_observations();
 
 	pos->set_observations(obs);
 	neg->set_observations(obs);
 
-	DREAL result=working->model_probability(idx);
+	float64_t result=working->model_probability(idx);
 	//working->set_observations(old_pos);
 	return result;
 }
 
-bool CGUIHMM::append_model(CHAR* filename, INT base1, INT base2)
+bool CGUIHMM::append_model(char* filename, int32_t base1, int32_t base2)
 {
 	if (!working)
 		SG_ERROR("Create HMM first.\n");
@@ -498,10 +498,10 @@ bool CGUIHMM::append_model(CHAR* filename, INT base1, INT base2)
 	SG_DEBUG("h %d , M: %d\n", h, h->get_M());
 	if (base1!=-1 && base2!=-1)
 	{
-		DREAL* cur_o=new DREAL[h->get_M()];
-		DREAL* app_o=new DREAL[h->get_M()];
+		float64_t* cur_o=new float64_t[h->get_M()];
+		float64_t* app_o=new float64_t[h->get_M()];
 
-		for (INT i=0; i<h->get_M(); i++)
+		for (int32_t i=0; i<h->get_M(); i++)
 		{
 			if (i==base1)
 				cur_o[i]=0;
@@ -527,7 +527,7 @@ bool CGUIHMM::append_model(CHAR* filename, INT base1, INT base2)
 	return true;
 }
 
-bool CGUIHMM::add_states(INT num_states, DREAL value)
+bool CGUIHMM::add_states(int32_t num_states, float64_t value)
 {
 	if (!working)
 		SG_ERROR("Create HMM first.\n");
@@ -537,14 +537,14 @@ bool CGUIHMM::add_states(INT num_states, DREAL value)
 	return true;
 }
 
-bool CGUIHMM::set_pseudo(DREAL pseudo)
+bool CGUIHMM::set_pseudo(float64_t pseudo)
 {
 	PSEUDO=pseudo;
 	SG_INFO("Current setting: pseudo=%e.\n", PSEUDO);
 	return true;
 }
 
-bool CGUIHMM::convergence_criteria(INT num_iterations, DREAL epsilon)
+bool CGUIHMM::convergence_criteria(int32_t num_iterations, float64_t epsilon)
 {
 	if (!working)
 		SG_ERROR("Create HMM first.\n");
@@ -556,7 +556,7 @@ bool CGUIHMM::convergence_criteria(INT num_iterations, DREAL epsilon)
 	return true;
 }
 
-bool CGUIHMM::set_hmm_as(CHAR* target)
+bool CGUIHMM::set_hmm_as(char* target)
 {
 	if (!working)
 		SG_ERROR("Create HMM first!\n");
@@ -585,7 +585,7 @@ bool CGUIHMM::set_hmm_as(CHAR* target)
 	return true;
 }
 
-bool CGUIHMM::load(CHAR* filename)
+bool CGUIHMM::load(char* filename)
 {
 	bool result=false;
 
@@ -608,7 +608,7 @@ bool CGUIHMM::load(CHAR* filename)
 	return result;
 }
 
-bool CGUIHMM::save(CHAR* filename, bool is_binary)
+bool CGUIHMM::save(char* filename, bool is_binary)
 {
 	bool result=false;
 
@@ -635,7 +635,7 @@ bool CGUIHMM::save(CHAR* filename, bool is_binary)
 	return result;
 }
 
-bool CGUIHMM::load_definitions(CHAR* filename, bool do_init)
+bool CGUIHMM::load_definitions(char* filename, bool do_init)
 {
 	if (!working)
 		SG_ERROR("Create HMM first.\n");
@@ -657,7 +657,7 @@ bool CGUIHMM::load_definitions(CHAR* filename, bool do_init)
 	return result;
 }
 
-bool CGUIHMM::save_likelihood(CHAR* filename, bool is_binary)
+bool CGUIHMM::save_likelihood(char* filename, bool is_binary)
 {
 	bool result=false;
 
@@ -686,7 +686,7 @@ bool CGUIHMM::save_likelihood(CHAR* filename, bool is_binary)
 	return result;
 }
 
-bool CGUIHMM::save_path(CHAR* filename, bool is_binary)
+bool CGUIHMM::save_path(char* filename, bool is_binary)
 {
 	bool result=false;
 	if (!working)
@@ -699,7 +699,7 @@ bool CGUIHMM::save_path(CHAR* filename, bool is_binary)
 		//if (binary)
 		//_train()/	result=working->save_model_bin(file);
 		//else
-		CStringFeatures<WORD>* obs=(CStringFeatures<WORD>*) ui->
+		CStringFeatures<uint16_t>* obs=(CStringFeatures<uint16_t>*) ui->
 			ui_features->get_test_features();
 		ASSERT(obs);
 		working->set_observations(obs);
@@ -718,7 +718,7 @@ bool CGUIHMM::save_path(CHAR* filename, bool is_binary)
 	return result;
 }
 
-bool CGUIHMM::chop(DREAL value)
+bool CGUIHMM::chop(float64_t value)
 {
 	if (!working)
 		SG_ERROR("Create HMM first.\n");
@@ -754,7 +754,7 @@ bool CGUIHMM::output_hmm_defined()
 	return true;
 }
 
-bool CGUIHMM::best_path(INT from, INT to)
+bool CGUIHMM::best_path(int32_t from, int32_t to)
 {
 	// FIXME: from unused???
 
@@ -764,7 +764,7 @@ bool CGUIHMM::best_path(INT from, INT to)
 	//get path
 	working->best_path(0);
 
-	for (INT t=0; t<working->get_observations()->get_vector_length(0)-1 && t<to; t++)
+	for (int32_t t=0; t<working->get_observations()->get_vector_length(0)-1 && t<to; t++)
 		SG_PRINT("%d ", working->get_best_path_state(0, t));
 	SG_PRINT("\n");
 
@@ -783,27 +783,27 @@ bool CGUIHMM::normalize(bool keep_dead_states)
 	return true;
 }
 
-bool CGUIHMM::relative_entropy(DREAL*& values, INT& len)
+bool CGUIHMM::relative_entropy(float64_t*& values, int32_t& len)
 {
 	if (!pos || !neg)
 		SG_ERROR("Set pos and neg HMM first!\n");
 
-	INT pos_N=pos->get_N();
-	INT neg_N=neg->get_N();
-	INT pos_M=pos->get_M();
-	INT neg_M=neg->get_M();
+	int32_t pos_N=pos->get_N();
+	int32_t neg_N=neg->get_N();
+	int32_t pos_M=pos->get_M();
+	int32_t neg_M=neg->get_M();
 	if (pos_M!=neg_M || pos_N!=neg_N)
 		SG_ERROR("Pos and neg HMM's differ in number of emissions or states.\n");
 
-	DREAL* p=new DREAL[pos_M];
-	DREAL* q=new DREAL[neg_M];
+	float64_t* p=new float64_t[pos_M];
+	float64_t* q=new float64_t[neg_M];
 
 	delete[] values;
-	values=new DREAL[pos_N];
+	values=new float64_t[pos_N];
 
-	for (INT i=0; i<pos_N; i++)
+	for (int32_t i=0; i<pos_N; i++)
 	{
-		for (INT j=0; j<pos_M; j++)
+		for (int32_t j=0; j<pos_M; j++)
 		{
 			p[j]=pos->get_b(i, j);
 			q[j]=neg->get_b(i, j);
@@ -818,21 +818,21 @@ bool CGUIHMM::relative_entropy(DREAL*& values, INT& len)
 	return true;
 }
 
-bool CGUIHMM::entropy(DREAL*& values, INT& len)
+bool CGUIHMM::entropy(float64_t*& values, int32_t& len)
 {
 	if (!working)
 		SG_ERROR("Create HMM first!\n");
 
-	INT n=working->get_N();
-	INT m=working->get_M();
-	DREAL* p=new DREAL[m];
+	int32_t n=working->get_N();
+	int32_t m=working->get_M();
+	float64_t* p=new float64_t[m];
 
 	delete[] values;
-	values=new DREAL[n];
+	values=new float64_t[n];
 
-	for (INT i=0; i<n; i++)
+	for (int32_t i=0; i<n; i++)
 	{
-		for (INT j=0; j<m; j++)
+		for (int32_t j=0; j<m; j++)
 			p[j]=working->get_b(i, j);
 
 		values[i]=CMath::entropy(p, m);
@@ -843,7 +843,7 @@ bool CGUIHMM::entropy(DREAL*& values, INT& len)
 	return true;
 }
 
-bool CGUIHMM::permutation_entropy(INT width, INT seq_num)
+bool CGUIHMM::permutation_entropy(int32_t width, int32_t seq_num)
 {
 	if (!working)
 		SG_ERROR("Create hmm first.\n");
