@@ -35,15 +35,15 @@
 #include "kernel/PolyKernel.h"
 #include "kernel/CustomKernel.h"
 #include "kernel/ConstKernel.h"
-#include "kernel/PolyMatchWordKernel.h"
+#include "kernel/PolyMatchWordStringKernel.h"
 #include "kernel/PolyMatchStringKernel.h"
 #include "kernel/LocalAlignmentStringKernel.h"
-#include "kernel/WordMatchKernel.h"
+#include "kernel/MatchWordStringKernel.h"
 #include "kernel/CommWordStringKernel.h"
 #include "kernel/WeightedCommWordStringKernel.h"
 #include "kernel/CommUlongStringKernel.h"
-#include "kernel/HistogramWordKernel.h"
-#include "kernel/SalzbergWordKernel.h"
+#include "kernel/HistogramWordStringKernel.h"
+#include "kernel/SalzbergWordStringKernel.h"
 #include "kernel/GaussianKernel.h"
 #include "kernel/GaussianShiftKernel.h"
 #include "kernel/SigmoidKernel.h"
@@ -52,7 +52,14 @@
 #include "kernel/SparseGaussianKernel.h"
 #include "kernel/DiagKernel.h"
 #include "kernel/MindyGramKernel.h"
+#include "kernel/OligoKernel.h"
 #include "kernel/DistanceKernel.h"
+
+#include "kernel/AvgDiagKernelNormalizer.h"
+#include "kernel/FirstElementKernelNormalizer.h"
+#include "kernel/IdentityKernelNormalizer.h"
+#include "kernel/SqrtDiagKernelNormalizer.h"
+
 #include "classifier/svm/SVM.h"
 
 
@@ -74,7 +81,9 @@ CKernel* CGUIKernel::get_kernel()
 }
 
 #ifdef HAVE_MINDY
-CKernel* CGUIKernel::create_mindygram(INT size, CHAR* meas_str, CHAR* norm_str, DREAL width, CHAR* param_str)
+CKernel* CGUIKernel::create_mindygram(
+	int32_t size, char* meas_str, char* norm_str, float64_t width,
+	char* param_str)
 {
 	CKernel* kern=new CMindyGramKernel(size, meast_str, width);
 	if (!kern)
@@ -90,7 +99,15 @@ CKernel* CGUIKernel::create_mindygram(INT size, CHAR* meas_str, CHAR* norm_str,
 }
 #endif
 
-CKernel* CGUIKernel::create_diag(INT size, DREAL diag)
+CKernel* CGUIKernel::create_oligo(int32_t size, int32_t k, float64_t width)
+{
+	CKernel* kern=new COligoKernel(size, k, width);
+	SG_DEBUG("created OligoKernel (%p) with size %d, k %d, width %f.\n", kern, size, k, width);
+
+	return kern;
+}
+
+CKernel* CGUIKernel::create_diag(int32_t size, float64_t diag)
 {
 	CKernel* kern=new CDiagKernel(size, diag);
 	if (!kern)
@@ -101,7 +118,7 @@ CKernel* CGUIKernel::create_diag(INT size, DREAL diag)
 	return kern;
 }
 
-CKernel* CGUIKernel::create_const(INT size, DREAL c)
+CKernel* CGUIKernel::create_const(int32_t size, float64_t c)
 {
 	CKernel* kern=new CConstKernel(c);
 	if (!kern)
@@ -127,7 +144,7 @@ CKernel* CGUIKernel::create_custom()
 
 
 CKernel* CGUIKernel::create_gaussianshift(
-	INT size, DREAL width, INT max_shift, INT shift_step)
+	int32_t size, float64_t width, int32_t max_shift, int32_t shift_step)
 {
 	CKernel* kern=new CGaussianShiftKernel(size, width, max_shift, shift_step);
 	if (!kern)
@@ -138,7 +155,7 @@ CKernel* CGUIKernel::create_gaussianshift(
 	return kern;
 }
 
-CKernel* CGUIKernel::create_sparsegaussian(INT size, DREAL width)
+CKernel* CGUIKernel::create_sparsegaussian(int32_t size, float64_t width)
 {
 	CKernel* kern=new CSparseGaussianKernel(size, width);
 	if (!kern)
@@ -149,7 +166,7 @@ CKernel* CGUIKernel::create_sparsegaussian(INT size, DREAL width)
 	return kern;
 }
 
-CKernel* CGUIKernel::create_gaussian(INT size, DREAL width)
+CKernel* CGUIKernel::create_gaussian(int32_t size, float64_t width)
 {
 	CKernel* kern=new CGaussianKernel(size, width);
 	if (!kern)
@@ -160,7 +177,8 @@ CKernel* CGUIKernel::create_gaussian(INT size, DREAL width)
 	return kern;
 }
 
-CKernel* CGUIKernel::create_sigmoid(INT size, DREAL gamma, DREAL coef0)
+CKernel* CGUIKernel::create_sigmoid(
+	int32_t size, float64_t gamma, float64_t coef0)
 {
 	CKernel* kern=new CSigmoidKernel(size, gamma, coef0);
 	if (!kern)
@@ -172,31 +190,29 @@ CKernel* CGUIKernel::create_sigmoid(INT size, DREAL gamma, DREAL coef0)
 }
 
 CKernel* CGUIKernel::create_sparsepoly(
-	INT size, INT degree, bool inhomogene, bool normalize)
+	int32_t size, int32_t degree, bool inhomogene, bool normalize)
 {
-	CKernel* kern=new CSparsePolyKernel(size, degree, inhomogene, normalize);
-	if (!kern)
-		SG_ERROR("Couldn't create SparsePolyKernel with size %d, degree %d, inhomoegene %d, normalize %d.\n", size, degree, inhomogene, normalize);
-	else
-		SG_DEBUG("created SparsePolyKernel with size %d, degree %d, inhomoegene %d, normalize %d.\n", kern, size, degree, inhomogene, normalize);
+	CKernel* kern=new CSparsePolyKernel(size, degree, inhomogene);
+	if (!normalize)
+		kern->set_normalizer(new CIdentityKernelNormalizer());
+	SG_DEBUG("created SparsePolyKernel with size %d, degree %d, inhomogene %d normalize %d.\n", kern, size, degree, inhomogene, normalize);
 
 	return kern;
 }
 
 CKernel* CGUIKernel::create_poly(
-	INT size, INT degree, bool inhomogene, bool normalize)
+	int32_t size, int32_t degree, bool inhomogene, bool normalize)
 {
-	CKernel* kern=new CPolyKernel(size, degree, inhomogene, normalize);
-	if (!kern)
-		SG_ERROR("Couldn't create PolyKernel with size %d, degree %d, inhomoegene %d, normalize %d.\n", size, degree, inhomogene, normalize);
-	else
-		SG_DEBUG("created PolyKernel (%p) with size %d, degree %d, inhomoegene %d, normalize %d.\n", kern, size, degree, inhomogene, normalize);
+	CKernel* kern=new CPolyKernel(size, degree, inhomogene);
+	if (!normalize)
+		kern->set_normalizer(new CIdentityKernelNormalizer());
+	SG_DEBUG("created PolyKernel (%p) with size %d, degree %d, inhomogene %d, normalize %d.\n", kern, size, degree, inhomogene, normalize);
 
 	return kern;
 }
 
 CKernel* CGUIKernel::create_localityimprovedstring(
-	INT size, INT length, INT inner_degree, INT outer_degree,
+	int32_t size, int32_t length, int32_t inner_degree, int32_t outer_degree,
 	EKernelType ktype)
 {
 	CKernel* kern=NULL;
@@ -221,12 +237,12 @@ CKernel* CGUIKernel::create_localityimprovedstring(
 }
 
 CKernel* CGUIKernel::create_weighteddegreestring(
-	INT size, INT order, INT max_mismatch, bool use_normalization,
-	INT mkl_stepsize, bool block_computation, INT single_degree)
+	int32_t size, int32_t order, int32_t max_mismatch, bool use_normalization,
+	int32_t mkl_stepsize, bool block_computation, int32_t single_degree)
 {
-	DREAL* weights=get_weights(order, max_mismatch);
+	float64_t* weights=get_weights(order, max_mismatch);
 
-	INT i=0;
+	int32_t i=0;
 	if (single_degree>=0)
 	{
 		ASSERT(single_degree<order);
@@ -240,13 +256,12 @@ CKernel* CGUIKernel::create_weighteddegreestring(
 	}
 
 	CKernel* kern=new CWeightedDegreeStringKernel(weights, order);
-	if (!kern)
-		SG_ERROR("Couldn't create WeightedDegreeStringKernel with size %d, order %d, max_mismatch %d, use_normalization %d, mkl_stepsize %d, block_computation %d, single_degree %f.\n", size, order, max_mismatch, use_normalization, mkl_stepsize, block_computation, single_degree);
-	else
-		SG_DEBUG("created WeightedDegreeStringKernel (%p) with size %d, order %d, max_mismatch %d, use_normalization %d, mkl_stepsize %d, block_computation %d, single_degree %f.\n", kern, size, order, max_mismatch, use_normalization, mkl_stepsize, block_computation, single_degree);
 
-	((CWeightedDegreeStringKernel*) kern)->
-		set_use_normalization(use_normalization);
+	SG_DEBUG("created WeightedDegreeStringKernel (%p) with size %d, order %d, max_mismatch %d, use_normalization %d, mkl_stepsize %d, block_computation %d, single_degree %f.\n", kern, size, order, max_mismatch, mkl_stepsize, block_computation, single_degree);
+
+	if (!use_normalization)
+		kern->set_normalizer(new CIdentityKernelNormalizer());
+		
 	((CWeightedDegreeStringKernel*) kern)->
 		set_use_block_computation(block_computation);
 	((CWeightedDegreeStringKernel*) kern)->set_max_mismatch(max_mismatch);
@@ -258,17 +273,17 @@ CKernel* CGUIKernel::create_weighteddegreestring(
 }
 
 CKernel* CGUIKernel::create_weighteddegreepositionstring(
-	INT size, INT order, INT max_mismatch, INT length, INT center,
-	DREAL step)
+	int32_t size, int32_t order, int32_t max_mismatch, int32_t length,
+	int32_t center, float64_t step)
 {
-	INT i=0;
-	INT* shifts=new INT[length];
+	int32_t i=0;
+	int32_t* shifts=new int32_t[length];
 
 	for (i=center; i<length; i++)
-		shifts[i]=(int) floor(((DREAL) (i-center))/step);
+		shifts[i]=(int32_t) floor(((float64_t) (i-center))/step);
 
 	for (i=center-1; i>=0; i--)
-		shifts[i]=(int) floor(((DREAL) (center-i))/step);
+		shifts[i]=(int32_t) floor(((float64_t) (center-i))/step);
 
 	for (i=0; i<length; i++)
 	{
@@ -279,7 +294,7 @@ CKernel* CGUIKernel::create_weighteddegreepositionstring(
 	for (i=0; i<length; i++)
 		SG_INFO( "shift[%i]=%i\n", i, shifts[i]);
 
-	DREAL* weights=get_weights(order, max_mismatch);
+	float64_t* weights=get_weights(order, max_mismatch);
 
 	CKernel* kern=new CWeightedDegreePositionStringKernel(size, weights, order, max_mismatch, shifts, length);
 	if (!kern)
@@ -293,21 +308,20 @@ CKernel* CGUIKernel::create_weighteddegreepositionstring(
 }
 
 CKernel* CGUIKernel::create_weighteddegreepositionstring3(
-	INT size, INT order, INT max_mismatch, INT* shifts, INT length,
-	INT mkl_stepsize, DREAL* position_weights)
+	int32_t size, int32_t order, int32_t max_mismatch, int32_t* shifts,
+	int32_t length, int32_t mkl_stepsize, float64_t* position_weights)
 {
-	DREAL* weights=get_weights(order, max_mismatch);
+	float64_t* weights=get_weights(order, max_mismatch);
 
-	CKernel* kern=new CWeightedDegreePositionStringKernel(size, weights, order, max_mismatch, shifts, length, false, mkl_stepsize);
-	if (!kern)
-		SG_ERROR("Couldn't create WeightedDegreePositionStringKernel with size %d, order %d, max_mismatch %d, length %d and position_weights (MKL stepsize: %d).\n", size, order, max_mismatch, length, mkl_stepsize);
-	else
-		SG_DEBUG("created WeightedDegreePositionStringKernel (%p) with size %d, order %d, max_mismatch %d, length %d and position_weights (MKL stepsize: %d).\n", kern, size, order, max_mismatch, length, mkl_stepsize);
+	CKernel* kern=new CWeightedDegreePositionStringKernel(size, weights, order, max_mismatch, shifts, length, mkl_stepsize);
+	kern->set_normalizer(new CIdentityKernelNormalizer());
+
+	SG_DEBUG("created WeightedDegreePositionStringKernel (%p) with size %d, order %d, max_mismatch %d, length %d and position_weights (MKL stepsize: %d).\n", kern, size, order, max_mismatch, length, mkl_stepsize);
 
 	if (!position_weights)
 	{
-		position_weights=new DREAL[length];
-		for (INT i=0; i<length; i++)
+		position_weights=new float64_t[length];
+		for (int32_t i=0; i<length; i++)
 			position_weights[i]=1.0/length;
 	}
 	((CWeightedDegreePositionStringKernel*) kern)->
@@ -318,26 +332,27 @@ CKernel* CGUIKernel::create_weighteddegreepositionstring3(
 }
 
 CKernel* CGUIKernel::create_weighteddegreepositionstring2(
-	INT size, INT order, INT max_mismatch, INT* shifts, INT length,
-	bool use_normalization)
+	int32_t size, int32_t order, int32_t max_mismatch, int32_t* shifts,
+	int32_t length, bool use_normalization)
 {
-	DREAL* weights=get_weights(order, max_mismatch);
+	float64_t* weights=get_weights(order, max_mismatch);
 
-	CKernel* kern=new CWeightedDegreePositionStringKernel(size, weights, order, max_mismatch, shifts, length, use_normalization);
-	if (!kern)
-		SG_ERROR("Couldn't create WeightedDegreePositionStringKernel with size %d, order %d, max_mismatch %d, length %d, use_normalization %d.\n", size, order, max_mismatch, length, use_normalization);
-	else
-		SG_DEBUG("created WeightedDegreePositionStringKernel (%p) with size %d, order %d, max_mismatch %d, length %d, use_normalization %d.\n", kern, size, order, max_mismatch, length, use_normalization);
+	CKernel* kern=new CWeightedDegreePositionStringKernel(size, weights, order, max_mismatch, shifts, length);
+	if (!use_normalization)
+		kern->set_normalizer(new CIdentityKernelNormalizer());
+
+
+	SG_DEBUG("created WeightedDegreePositionStringKernel (%p) with size %d, order %d, max_mismatch %d, length %d, use_normalization %d.\n", kern, size, order, max_mismatch, length, use_normalization);
 
 	delete[] weights;
 	return kern;
 }
 
-DREAL* CGUIKernel::get_weights(INT order, INT max_mismatch)
+float64_t* CGUIKernel::get_weights(int32_t order, int32_t max_mismatch)
 {
-	DREAL *weights=new DREAL[order*(1+max_mismatch)];
-	DREAL sum=0;
-	INT i=0;
+	float64_t *weights=new float64_t[order*(1+max_mismatch)];
+	float64_t sum=0;
+	int32_t i=0;
 
 	for (i=0; i<order; i++)
 	{
@@ -349,11 +364,11 @@ DREAL* CGUIKernel::get_weights(INT order, INT max_mismatch)
 	
 	for (i=0; i<order; i++)
 	{
-		for (INT j=1; j<=max_mismatch; j++)
+		for (int32_t j=1; j<=max_mismatch; j++)
 		{
 			if (j<i+1)
 			{
-				INT nk=CMath::nchoosek(i+1, j);
+				int32_t nk=CMath::nchoosek(i+1, j);
 				weights[i+j*order]=weights[i]/(nk*pow(3, j));
 			}
 			else
@@ -365,7 +380,7 @@ DREAL* CGUIKernel::get_weights(INT order, INT max_mismatch)
 }
 
 
-CKernel* CGUIKernel::create_localalignmentstring(INT size)
+CKernel* CGUIKernel::create_localalignmentstring(int32_t size)
 {
 	CKernel* kern=new CLocalAlignmentStringKernel(size);
 	if (!kern)
@@ -376,7 +391,7 @@ CKernel* CGUIKernel::create_localalignmentstring(INT size)
 	return kern;
 }
 
-CKernel* CGUIKernel::create_fixeddegreestring(INT size, INT d)
+CKernel* CGUIKernel::create_fixeddegreestring(int32_t size, int32_t d)
 {
 	CKernel* kern=new CFixedDegreeStringKernel(size, d);
 	if (!kern)
@@ -387,7 +402,7 @@ CKernel* CGUIKernel::create_fixeddegreestring(INT size, INT d)
 	return kern;
 }
 
-CKernel* CGUIKernel::create_chi2(INT size, DREAL width)
+CKernel* CGUIKernel::create_chi2(int32_t size, float64_t width)
 {
 	CKernel* kern=new CChi2Kernel(size, width);
 	if (!kern)
@@ -399,121 +414,68 @@ CKernel* CGUIKernel::create_chi2(INT size, DREAL width)
 }
 
 CKernel* CGUIKernel::create_commstring(
-	INT size, bool use_sign, CHAR* norm_str, EKernelType ktype)
+	int32_t size, bool use_sign, char* norm_str, EKernelType ktype)
 {
-	ENormalizationType normalization=get_normalization_from_str(norm_str);
-
 	CKernel* kern=NULL;
 	if (ktype==K_COMMULONGSTRING)
-		kern=new CCommUlongStringKernel(size, use_sign, normalization);
+		kern=new CCommUlongStringKernel(size, use_sign);
 	else if (ktype==K_COMMWORDSTRING)
-		kern=new CCommWordStringKernel(size, use_sign, normalization);
+		kern=new CCommWordStringKernel(size, use_sign);
 	else if (ktype==K_WEIGHTEDCOMMWORDSTRING)
-		kern=new CWeightedCommWordStringKernel(size, use_sign, normalization);
+		kern=new CWeightedCommWordStringKernel(size, use_sign);
 
-	if (!kern)
-		SG_ERROR("Couldn't create WeightedCommWord/CommWord/CommUlongStringKernel with size %d, use_sign  %d and normalization %d.\n", size, use_sign, normalization);
-	else
-		SG_DEBUG("created WeightedCommWord/CommWord/CommUlongStringKernel (%p) with size %d, use_sign  %d and normalization %d.\n", kern, size, use_sign, normalization);
+	SG_DEBUG("created WeightedCommWord/CommWord/CommUlongStringKernel (%p) with size %d, use_sign  %d.\n", kern, size, use_sign);
 
 	return kern;
 }
 
-ENormalizationType CGUIKernel::get_normalization_from_str(CHAR* str)
+CKernel* CGUIKernel::create_matchwordstring(
+	int32_t size, int32_t d, bool normalize)
 {
-	if (!str)
-		return FULL_NORMALIZATION;
-
-	ENormalizationType norm=FULL_NORMALIZATION;
-
-	if (strncmp(str, "NO", 2)==0)
-	{
-		norm=NO_NORMALIZATION;
-		SG_INFO("Using no normalization.\n");
-	}
-	else if (strncmp(str, "SQRT", 4)==0)
-	{
-		norm=SQRT_NORMALIZATION;
-		SG_INFO("Using sqrt normalization.\n");
-	}
-	else if (strncmp(str, "SQRTLEN", 7)==0)
-	{
-		norm=SQRTLEN_NORMALIZATION;
-		SG_INFO("Using sqrt-len normalization.\n");
-	}
-	else if (strncmp(str, "LEN", 3)==0)
-	{
-		norm=LEN_NORMALIZATION;
-		SG_INFO("Using len normalization.\n");
-	}
-	else if (strncmp(str, "SQLEN", 5)==0)
-	{
-		norm=SQLEN_NORMALIZATION;
-		SG_INFO("Using squared len normalization.\n");
-	}
-	else if (strncmp(str, "FULL", 4)==0)
-	{
-		norm=FULL_NORMALIZATION;
-		SG_INFO("Using full normalization.\n");
-	}
-	else
-	{
-		norm=FULL_NORMALIZATION;
-		SG_INFO("Using default full normalization.\n");
-	}
-
-	return norm;
-}
-
-CKernel* CGUIKernel::create_wordmatch(INT size, INT d)
-{
-	CKernel* kern=new CWordMatchKernel(size, d);
-	if (!kern)
-		SG_ERROR("Couldn't create WordMatchKernel with size %d and d %d.\n", size, d);
-	else
-		SG_DEBUG("created WordMatchKernel (%p) with size %d and d %d.\n", kern, size, d);
+	CKernel* kern=new CMatchWordStringKernel(size, d);
+	SG_DEBUG("created MatchWordStringKernel (%p) with size %d and d %d.\n", kern, size, d);
+	if (!normalize)
+		kern->set_normalizer(new CIdentityKernelNormalizer());
 
 	return kern;
 }
 
 CKernel* CGUIKernel::create_polymatchstring(
-	INT size, INT degree, bool inhomogene, bool normalize)
+	int32_t size, int32_t degree, bool inhomogene, bool normalize)
 {
-	CKernel* kern=new CPolyMatchStringKernel(size, degree, inhomogene, normalize);
-	if (!kern)
-		SG_ERROR("Couldn't create PolyMatchStringKernel with size %d, degree %d, inhomogene %d, normalize %d.\n", size, degree, inhomogene, normalize);
-	else
-		SG_DEBUG("created PolyMatchStringKernel (%p) with size %d, degree %d, inhomogene %d, normalize %d.\n", kern, size, degree, inhomogene, normalize);
+	CKernel* kern=new CPolyMatchStringKernel(size, degree, inhomogene);
+	SG_DEBUG("created PolyMatchStringKernel (%p) with size %d, degree %d, inhomogene %d normalize %d.\n", kern, size, degree, inhomogene, normalize);
+	if (!normalize)
+		kern->set_normalizer(new CIdentityKernelNormalizer());
 
 	return kern;
 }
 
-CKernel* CGUIKernel::create_polymatchword(
-	INT size, INT degree, bool inhomogene, bool normalize)
+CKernel* CGUIKernel::create_polymatchwordstring(
+	int32_t size, int32_t degree, bool inhomogene, bool normalize)
 {
-	CKernel* kern=new CPolyMatchWordKernel(size, degree, inhomogene, normalize);
-	if (!kern)
-		SG_ERROR("Couldn't create PolyMatchWordKernel with size %d, degree %d, inhomogene %d, normalize %d.\n", size, degree, inhomogene, normalize);
-	else
-		SG_DEBUG("created PolyMatchWordKernel (%p) with size %d, degree %d, inhomogene %d, normalize %d.\n", kern, size, degree, inhomogene, normalize);
+	CKernel* kern=new CPolyMatchWordStringKernel(size, degree, inhomogene);
+	SG_DEBUG("created PolyMatchWordStringKernel (%p) with size %d, degree %d, inhomogene %d, normalize %d.\n", kern, size, degree, inhomogene, normalize);
+	if (!normalize)
+		kern->set_normalizer(new CIdentityKernelNormalizer());
 
 	return kern;
 }
 
-CKernel* CGUIKernel::create_salzbergword(INT size)
+CKernel* CGUIKernel::create_salzbergword(int32_t size)
 {
-
 	SG_INFO("Getting estimator.\n");
 	CPluginEstimate* estimator=ui->ui_pluginestimate->get_estimator();
 	if (!estimator)
 		SG_ERROR("No estimator set.\n");
 
-	CKernel* kern=new CSalzbergWordKernel(size, estimator);
+	CKernel* kern=new CSalzbergWordStringKernel(size, estimator);
 	if (!kern)
-		SG_ERROR("Couldn't create HistogramWord with size %d.\n", size);
+		SG_ERROR("Couldn't create SalzbergWordString with size %d.\n", size);
 	else
-		SG_DEBUG("created HistogramWord (%p) with size %d.\n", kern, size);
+		SG_DEBUG("created SalzbergWordString (%p) with size %d.\n", kern, size);
 
+/*
 	// prior stuff
 	SG_INFO("Getting labels.\n");
 	CLabels* train_labels=ui->ui_labels->get_train_labels();
@@ -522,113 +484,83 @@ CKernel* CGUIKernel::create_salzbergword(INT size)
 		SG_INFO("Assign train labels first!\n");
 		return NULL;
 	}
-
-	INT num_pos=0, num_neg=0;
-	for (INT i=0; i<train_labels->get_num_labels(); i++)
-	{
-		if (train_labels->get_int_label(i)==1)
-			num_pos++;
-		if (train_labels->get_int_label(i)==-1)
-			num_neg++;
-	}
-	SG_INFO("priors: pos=%1.3f (%i)  neg=%1.3f (%i)\n",
-		(DREAL) num_pos/(num_pos+num_neg), num_pos,
-		(DREAL) num_neg/(num_pos+num_neg), num_neg);
-
-	((CSalzbergWordKernel*) kern)->set_prior_probs(
-		(DREAL)num_pos/(num_pos+num_neg),
-		(DREAL)num_neg/(num_pos+num_neg));
+	((CSalzbergWordStringKernel *) kern)->set_prior_probs_from_labels(train_labels);
+*/
 
 	return kern;
 }
 
-CKernel* CGUIKernel::create_histogramword(INT size)
+CKernel* CGUIKernel::create_histogramword(int32_t size)
 {
 	SG_INFO("Getting estimator.\n");
 	CPluginEstimate* estimator=ui->ui_pluginestimate->get_estimator();
 	if (!estimator)
 		SG_ERROR("No estimator set.\n");
 
-	CKernel* kern=new CHistogramWordKernel(size, estimator);
+	CKernel* kern=new CHistogramWordStringKernel(size, estimator);
 	if (!kern)
-		SG_ERROR("Couldn't create HistogramWord with size %d.\n", size);
+		SG_ERROR("Couldn't create HistogramWordString with size %d.\n", size);
 	else
-		SG_DEBUG("created HistogramWord (%p) with size %d.\n", kern, size);
+		SG_DEBUG("created HistogramWordString (%p) with size %d.\n", kern, size);
 
 	return kern;
 }
 
-CKernel* CGUIKernel::create_linearbyte(INT size, DREAL scale)
+CKernel* CGUIKernel::create_linearbyte(int32_t size, float64_t scale)
 {
+	size=0;
+	CKernel* kern=new CLinearByteKernel();
+	kern->set_normalizer(new CAvgDiagKernelNormalizer(scale));
+	SG_DEBUG("created LinearByteKernel (%p) with size %d and scale %f.\n", kern, size, scale);
+
+	return kern;
+}
+
+CKernel* CGUIKernel::create_linearword(int32_t size, float64_t scale)
+{
+	size=0;
+	CKernel* kern=new CLinearWordKernel();
+	kern->set_normalizer(new CAvgDiagKernelNormalizer(scale));
+	SG_DEBUG("created LinearWordKernel (%p) with size %d and scale %f.\n", kern, size, scale);
+
+	return kern;
+}
+
+CKernel* CGUIKernel::create_linearstring(int32_t size, float64_t scale)
+{
+	size=0;
 	CKernel* kern=NULL;
-	if (scale==-1)
-		kern=new CLinearByteKernel(size, true);
-	else
-		kern=new CLinearByteKernel(size, false, scale);
+	kern=new CLinearStringKernel();
+	kern->set_normalizer(new CAvgDiagKernelNormalizer(scale));
 
-	if (!kern)
-		SG_ERROR("Couldn't create LinearByteKernel with size %d and scale %f.\n", size, scale);
-	else
-		SG_DEBUG("created LinearByteKernel (%p) with size %d and scale %f.\n", kern, size, scale);
+	SG_DEBUG("created LinearStringKernel (%p) with size %d and scale %f.\n", kern, size, scale);
 
 	return kern;
 }
 
-CKernel* CGUIKernel::create_linearword(INT size, DREAL scale)
+CKernel* CGUIKernel::create_linear(int32_t size, float64_t scale)
 {
-	CKernel* kern=NULL;
-	if (scale==-1)
-		kern=new CLinearWordKernel(size, true);
-	else
-		kern=new CLinearWordKernel(size, false, scale);
+	size=0;
+	CKernel* kern=new CLinearKernel();
+	kern->set_normalizer(new CAvgDiagKernelNormalizer(scale));
 
-	if (!kern)
-		SG_ERROR("Couldn't create LinearWordKernel with size %d and scale %f.\n", size, scale);
-	else
-		SG_DEBUG("created LinearWordKernel (%p) with size %d and scale %f.\n", kern, size, scale);
+	SG_DEBUG("created LinearKernel (%p) with size %d and scale %f.\n", kern, size, scale);
 
 	return kern;
 }
 
-CKernel* CGUIKernel::create_linearstring(INT size, DREAL scale)
+CKernel* CGUIKernel::create_sparselinear(int32_t size, float64_t scale)
 {
-	CKernel* kern=NULL;
-	if (scale==-1)
-		kern=new CLinearStringKernel(size, true);
-	else
-		kern=new CLinearStringKernel(size, false, scale);
+	size=0;
+	CKernel* kern=new CSparseLinearKernel();
+	kern->set_normalizer(new CAvgDiagKernelNormalizer(scale));
 
-	if (!kern)
-		SG_ERROR("Couldn't create LinearStringKernel with size %d and scale %f.\n", size, scale);
-	else
-		SG_DEBUG("created LinearStringKernel (%p) with size %d and scale %f.\n", kern, size, scale);
+	SG_DEBUG("created SparseLinearKernel (%p) with size %d and scale %f.\n", kern, size, scale);
 
 	return kern;
 }
 
-CKernel* CGUIKernel::create_linear(INT size, DREAL scale)
-{
-	CKernel* kern=new CLinearKernel(size, scale);
-	if (!kern)
-		SG_ERROR("Couldn't create LinearKernel with size %d and scale %f.\n", size, scale);
-	else
-		SG_DEBUG("created LinearKernel (%p) with size %d and scale %f.\n", kern, size, scale);
-
-	return kern;
-}
-
-CKernel* CGUIKernel::create_sparselinear(INT size, DREAL scale)
-{
-	CKernel* kern=new CSparseLinearKernel(size, scale);
-	if (!kern)
-		SG_ERROR("Couldn't create SparseLinearKernel with size %d and scale %f.\n", size, scale);
-	else
-		SG_DEBUG("created SparseLinearKernel (%p) with size %d and scale %f.\n", kern, size, scale);
-
-	return kern;
-}
-
-CKernel* CGUIKernel::create_distance(INT size, DREAL width)
+CKernel* CGUIKernel::create_distance(int32_t size, float64_t width)
 {
 	CDistance* dist=ui->ui_distance->get_distance();
 	if (!dist)
@@ -644,7 +576,7 @@ CKernel* CGUIKernel::create_distance(INT size, DREAL width)
 }
 
 CKernel* CGUIKernel::create_combined(
-	INT size, bool append_subkernel_weights)
+	int32_t size, bool append_subkernel_weights)
 {
 	CKernel* kern=new CCombinedKernel(size, append_subkernel_weights);
 	if (!kern)
@@ -655,6 +587,42 @@ CKernel* CGUIKernel::create_combined(
 	return kern;
 }
 
+bool CGUIKernel::set_normalization(char* normalization, float64_t c)
+{
+	CKernel* k=kernel;
+
+	if (k && k->get_kernel_type()==K_COMBINED)
+		k=((CCombinedKernel*) kernel)->get_last_kernel();
+
+	if (!k)
+		SG_ERROR("No kernel available.\n");
+
+	if (strncmp(normalization, "IDENTITY", 8)==0)
+	{
+		SG_INFO("Identity Normalization (==NO NORMALIZATION) selected\n");
+		return k->set_normalizer(new CIdentityKernelNormalizer());
+	}
+	else if (strncmp(normalization,"AVGDIAG", 7)==0)
+	{
+		SG_INFO("Average Kernel Diagonal Normalization selected\n");
+		return k->set_normalizer(new CAvgDiagKernelNormalizer(c));
+	}
+	else if (strncmp(normalization,"SQRTDIAG", 8)==0)
+	{
+		SG_INFO("Sqrt Diagonal Normalization selected\n");
+		return k->set_normalizer(new CSqrtDiagKernelNormalizer());
+	}
+	else if (strncmp(normalization,"FIRSTELEMENT", 12)==0)
+	{
+		SG_INFO("First Element Normalization selected\n");
+		return k->set_normalizer(new CFirstElementKernelNormalizer());
+	}
+	else
+		SG_ERROR("Wrong kernel normalizer name.\n");
+
+	return false;
+}
+
 bool CGUIKernel::set_kernel(CKernel* kern)
 {
 	if (kern)
@@ -670,7 +638,7 @@ bool CGUIKernel::set_kernel(CKernel* kern)
 		return false;
 }
 
-bool CGUIKernel::load_kernel_init(CHAR* filename)
+bool CGUIKernel::load_kernel_init(char* filename)
 {
 	bool result=false;
 	if (kernel)
@@ -694,7 +662,7 @@ bool CGUIKernel::load_kernel_init(CHAR* filename)
 	return result;
 }
 
-bool CGUIKernel::save_kernel_init(CHAR* filename)
+bool CGUIKernel::save_kernel_init(char* filename)
 {
 	bool result=false;
 
@@ -720,18 +688,16 @@ bool CGUIKernel::save_kernel_init(CHAR* filename)
 
 bool CGUIKernel::init_kernel_optimization()
 {
-	kernel->set_precompute_matrix(false, false);
-
 	CSVM* svm=(CSVM*) ui->ui_classifier->get_classifier();
 	if (svm)
 	{
 		if (kernel->has_property(KP_LINADD))
 		{
-			INT num_sv=svm->get_num_support_vectors();
-			INT* sv_idx=new INT[num_sv];
-			DREAL* sv_weight=new DREAL[num_sv];
+			int32_t num_sv=svm->get_num_support_vectors();
+			int32_t* sv_idx=new int32_t[num_sv];
+			float64_t* sv_weight=new float64_t[num_sv];
 			
-			for (INT i=0; i<num_sv; i++)
+			for (int32_t i=0; i<num_sv; i++)
 			{
 				sv_idx[i]=svm->get_support_vector(i);
 				sv_weight[i]=svm->get_alpha(i);
@@ -762,12 +728,11 @@ bool CGUIKernel::delete_kernel_optimization()
 }
 
 
-bool CGUIKernel::init_kernel(CHAR* target)
+bool CGUIKernel::init_kernel(char* target)
 {
 	if (!kernel)
 		SG_ERROR("No kernel available.\n");
 
-	kernel->set_precompute_matrix(false, false);
 	EFeatureClass k_fclass=kernel->get_feature_class();
 	EFeatureType k_ftype=kernel->get_feature_type();
 
@@ -824,7 +789,7 @@ bool CGUIKernel::init_kernel(CHAR* target)
 	return true;
 }
 
-bool CGUIKernel::save_kernel(CHAR* filename)
+bool CGUIKernel::save_kernel(char* filename)
 {
 	if (kernel && initialized)
 	{
@@ -842,7 +807,7 @@ bool CGUIKernel::save_kernel(CHAR* filename)
 	return false;
 }
 
-bool CGUIKernel::add_kernel(CKernel* kern, DREAL weight)
+bool CGUIKernel::add_kernel(CKernel* kern, float64_t weight)
 {
 	if (!kern)
 		SG_ERROR("Given kernel to add is invalid.\n");
@@ -894,7 +859,7 @@ bool CGUIKernel::clean_kernel()
 
 
 
-bool CGUIKernel::set_optimization_type(CHAR* opt_type)
+bool CGUIKernel::set_optimization_type(char* opt_type)
 {
 	EOptimizationType opt=SLOWBUTMEMEFFICIENT;
 	if (!kernel)
@@ -921,4 +886,15 @@ bool CGUIKernel::set_optimization_type(CHAR* opt_type)
 
 	return false;
 }
+
+bool CGUIKernel::precompute_subkernels()
+{
+	if (!kernel)
+		SG_ERROR("No kernel available.\n");
+
+	if (kernel->get_kernel_type()!=K_COMBINED)
+		SG_ERROR("Not a combined kernel.\n");
+
+	return ((CCombinedKernel*) kernel)->precompute_subkernels();
+}
 #endif
