@@ -30,26 +30,26 @@ class CGUIPluginEstimate : public CSGObject
 		~CGUIPluginEstimate();
 
 		/** create new estimator */
-		bool new_estimator(DREAL pos, DREAL neg);
+		bool new_estimator(float64_t pos, float64_t neg);
 		/** train estimator */
 		bool train();
-		bool marginalized_train(CHAR* param);
+		bool marginalized_train(char* param);
 		/** test estimator */
-		bool test(CHAR* filename_out, CHAR* filename_roc);
-		bool load(CHAR* param);
-		bool save(CHAR* param);
+		bool test(char* filename_out, char* filename_roc);
+		bool load(char* param);
+		bool save(char* param);
 
 		inline CPluginEstimate* get_estimator() { return estimator; }
 
 		CLabels* classify(CLabels* output=NULL);
-		DREAL classify_example(INT idx);
+		float64_t classify_example(int32_t idx);
 
 	protected:
 		CSGInterface* ui;
 
 		CPluginEstimate* estimator;
-		DREAL pos_pseudo;
-		DREAL neg_pseudo;
+		float64_t pos_pseudo;
+		float64_t neg_pseudo;
 };
 #endif //HAVE_SWIG
 #endif
