@@ -20,7 +20,7 @@
 #include <netinet/in.h>
 #endif
 
-const INT READLINE_BUFFER_SIZE = 10000;
+const int32_t READLINE_BUFFER_SIZE = 10000;
 extern CSGInterface* interface;
 extern CSGInterfaceMethod sg_methods[];
 
@@ -35,17 +35,17 @@ CCmdLineInterface::~CCmdLineInterface()
 	delete m_rhs;
 }
 
-void CCmdLineInterface::reset(const CHAR* line)
+void CCmdLineInterface::reset(const char* line)
 {
 	CSGInterface::reset();
 
 	if (!line)
 		return;
 
-	CHAR* element=NULL;
-	CHAR delim_equal[]="=";
-	CHAR delim_lhs[]="=, \t\n";
-	CHAR delim_rhs[]=" \t\n";
+	char* element=NULL;
+	char delim_equal[]="=";
+	char delim_lhs[]="=, \t\n";
+	char delim_rhs[]=" \t\n";
 
 	delete m_lhs;
 	m_lhs=NULL;
@@ -53,17 +53,17 @@ void CCmdLineInterface::reset(const CHAR* line)
 	m_rhs=NULL;
 
 	// split lhs from rhs
-	CHAR* equal_sign=strstr(line, delim_equal);
+	char* equal_sign=strstr(line, delim_equal);
 	if (equal_sign)
 	//if (strstr(line, delim_equal))
 	{
 #ifdef DEBUG_CMDLINEIF
 		SG_PRINT("has lhs!\n");
 #endif
-		element=strtok((CHAR*) line, delim_lhs);
+		element=strtok((char*) line, delim_lhs);
 		if (element)
 		{
-			m_lhs=new CDynamicArray<CHAR*>();
+			m_lhs=new CDynamicArray<char*>();
 			m_lhs->append_element(element);
 			m_nlhs++;
 			while ((element=strtok(NULL, delim_lhs)))
@@ -77,11 +77,11 @@ void CCmdLineInterface::reset(const CHAR* line)
 		}
 	}
 	else
-		element=strtok((CHAR*) line, delim_rhs);
+		element=strtok((char*) line, delim_rhs);
 
 	if (element)
 	{
-		m_rhs=new CDynamicArray<CHAR*>();
+		m_rhs=new CDynamicArray<char*>();
 		m_rhs->append_element(element);
 		m_nrhs++;
 		while ((element=strtok(NULL, delim_rhs)))
@@ -97,13 +97,13 @@ void CCmdLineInterface::reset(const CHAR* line)
 	SG_PRINT("nlhs=%d nrhs=%d\n", m_nlhs, m_nrhs);
 	if (m_lhs)
 	{
-		for (INT i=0; i<m_lhs->get_num_elements(); i++)
+		for (int32_t i=0; i<m_lhs->get_num_elements(); i++)
 			SG_PRINT("element lhs %i %s\n", i, m_lhs->get_element(i));
 	}
 
 	if (m_rhs)
 	{
-		for (INT i=0; i<m_rhs->get_num_elements(); i++)
+		for (int32_t i=0; i<m_rhs->get_num_elements(); i++)
 			SG_PRINT("element rhs %i %s\n", i, m_rhs->get_element(i));
 	}
 #endif
@@ -127,25 +127,25 @@ void CCmdLineInterface::reset(const CHAR* line)
  */
 IFType CCmdLineInterface::get_argument_type()
 {
-	const INT len=1024;
+	const int32_t len=1024;
 	IFType argtype=UNDEFINED;
-	const CHAR* filename=m_rhs->get_element(m_rhs_counter);
+	const char* filename=m_rhs->get_element(m_rhs_counter);
 
 	// read the first 1024 of the file and heuristically decide about its
 	// content
-	FILE* fh=fopen((CHAR*) filename, "r");
+	FILE* fh=fopen((char*) filename, "r");
 	if (!fh)
 		SG_ERROR("Could not find file %s.\n", filename);
 
-	CHAR* chunk=new CHAR[len+1];
-	memset(chunk, 0, sizeof(CHAR)*(len+1));
-	size_t nread=fread(chunk, sizeof(CHAR), len, fh);
+	char* chunk=new char[len+1];
+	memset(chunk, 0, sizeof(char)*(len+1));
+	size_t nread=fread(chunk, sizeof(char), len, fh);
 	fclose(fh);
 	if (nread<=0)
 		SG_ERROR("Could not read data from %s.\n");
 
-	CHAR* signature=new CHAR[len+1];
-	INT num=sscanf(chunk, "### SHOGUN V0 %s\n", signature);
+	char* signature=new char[len+1];
+	int32_t num=sscanf(chunk, "### SHOGUN V0 %s\n", signature);
 
 	// if file has valid shogun signature use it to determine file type
 	if (num==1)
@@ -194,28 +194,28 @@ IFType CCmdLineInterface::get_argument_type()
 }
 
 
-INT CCmdLineInterface::get_int()
+int32_t CCmdLineInterface::get_int()
 {
-	const CHAR* i=get_arg_increment();
+	const char* i=get_arg_increment();
 	if (!i)
 		SG_ERROR("Expected Scalar Integer as argument %d\n", m_rhs_counter);
 
-	INT value=-1;
-	INT num=sscanf(i, "%d", &value);
+	int32_t value=-1;
+	int32_t num=sscanf(i, "%d", &value);
 	if (num!=1)
 		SG_ERROR("Expected Scalar Integer as argument %d\n", m_rhs_counter);
 
 	return value;
 }
 
-DREAL CCmdLineInterface::get_real()
+float64_t CCmdLineInterface::get_real()
 {
-	const CHAR* r=get_arg_increment();
+	const char* r=get_arg_increment();
 	if (!r)
 		SG_ERROR("Expected Scalar Real as argument %d\n", m_rhs_counter);
 
-	DREAL value=-1;
-	INT num=sscanf(r, "%lf", &value);
+	float64_t value=-1;
+	int32_t num=sscanf(r, "%lf", &value);
 	if (num!=1)
 		SG_ERROR("Expected Scalar Real as argument %d\n", m_rhs_counter);
 
@@ -224,12 +224,12 @@ DREAL CCmdLineInterface::get_real()
 
 bool CCmdLineInterface::get_bool()
 {
-	const CHAR* b=get_arg_increment();
+	const char* b=get_arg_increment();
 	if (!b)
 		SG_ERROR("Expected Scalar Bool as argument %d\n", m_rhs_counter);
 
-	INT value=-1;
-	INT num=sscanf(b, "%i", &value);
+	int32_t value=-1;
+	int32_t num=sscanf(b, "%i", &value);
 	if (num!=1)
 		SG_ERROR("Expected Scalar Bool as argument %d\n", m_rhs_counter);
 
@@ -237,35 +237,35 @@ bool CCmdLineInterface::get_bool()
 }
 
 
-CHAR* CCmdLineInterface::get_string(INT& len)
+char* CCmdLineInterface::get_string(int32_t& len)
 {
-	const CHAR* s=get_arg_increment();
+	const char* s=get_arg_increment();
 	if (!s)
 		SG_ERROR("Expected 1 String as argument %d.\n", m_rhs_counter);
 
 	len=strlen(s);
 	ASSERT(len>0);
 
-	CHAR* result=new CHAR[len+1];
-	memcpy(result, s, len*sizeof(CHAR));
+	char* result=new char[len+1];
+	memcpy(result, s, len*sizeof(char));
 	result[len]='\0';
 
 	return result;
 }
 
-void CCmdLineInterface::get_byte_vector(BYTE*& vec, INT& len)
+void CCmdLineInterface::get_byte_vector(uint8_t*& vec, int32_t& len)
 {
 	vec=NULL;
 	len=0;
 }
 
-void CCmdLineInterface::get_char_vector(CHAR*& vec, INT& len)
+void CCmdLineInterface::get_char_vector(char*& vec, int32_t& len)
 {
 	vec=NULL;
 	len=0;
 }
 
-void CCmdLineInterface::get_int_vector(INT*& vec, INT& len)
+void CCmdLineInterface::get_int_vector(int32_t*& vec, int32_t& len)
 {
 	vec=NULL;
 	len=0;
@@ -278,35 +278,35 @@ void CCmdLineInterface::get_int_vector(INT*& vec, INT& len)
 		SG_ERROR("Expected Integer Vector as argument %d\n", m_rhs_counter);
 
 	len=LENGTH(rvec);
-	vec=new INT[len];
+	vec=new int32_t[len];
 	ASSERT(vec);
 
-	for (INT i=0; i<len; i++)
-		vec[i]= (INT) INTEGER(rvec)[i];
+	for (int32_t i=0; i<len; i++)
+		vec[i]= (int32_t) INTEGER(rvec)[i];
 		*/
 }
 
-void CCmdLineInterface::get_shortreal_vector(SHORTREAL*& vec, INT& len)
+void CCmdLineInterface::get_shortreal_vector(float32_t*& vec, int32_t& len)
 {
 	vec=NULL;
 	len=0;
 }
 
-void CCmdLineInterface::get_real_vector(DREAL*& vec, INT& len)
+void CCmdLineInterface::get_real_vector(float64_t*& vec, int32_t& len)
 {
 	vec=NULL;
 	len=0;
 
-	const CHAR* filename=get_arg_increment();
+	const char* filename=get_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to read REAL matrix.\n");
 
-	CFile f((CHAR*) filename, 'r', F_DREAL);
+	CFile f((char*) filename, 'r', F_DREAL);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to read REAL matrix.\n", filename);
 
-	INT num_feat=0;
-	INT num_vec=0;
+	int32_t num_feat=0;
+	int32_t num_vec=0;
 
 	if (!f.read_real_valued_dense(vec, num_feat, num_vec))
 		SG_ERROR("Could not read REAL data from %s.\n", filename);
@@ -328,54 +328,54 @@ void CCmdLineInterface::get_real_vector(DREAL*& vec, INT& len)
 
 }
 
-void CCmdLineInterface::get_short_vector(SHORT*& vec, INT& len)
+void CCmdLineInterface::get_short_vector(int16_t*& vec, int32_t& len)
 {
 	vec=NULL;
 	len=0;
 }
 
-void CCmdLineInterface::get_word_vector(WORD*& vec, INT& len)
+void CCmdLineInterface::get_word_vector(uint16_t*& vec, int32_t& len)
 {
 	vec=NULL;
 	len=0;
 }
 
 
-void CCmdLineInterface::get_byte_matrix(BYTE*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_byte_matrix(uint8_t*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
 	matrix=NULL;
 	num_feat=0;
 	num_vec=0;
 }
 
-void CCmdLineInterface::get_char_matrix(CHAR*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_char_matrix(char*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
 	matrix=NULL;
 	num_feat=0;
 	num_vec=0;
 }
 
-void CCmdLineInterface::get_int_matrix(INT*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_int_matrix(int32_t*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
 	matrix=NULL;
 	num_feat=0;
 	num_vec=0;
 }
 
-void CCmdLineInterface::get_shortreal_matrix(SHORTREAL*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_shortreal_matrix(float32_t*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
 	matrix=NULL;
 	num_feat=0;
 	num_vec=0;
 }
 
-void CCmdLineInterface::get_real_matrix(DREAL*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_real_matrix(float64_t*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
-	const CHAR* filename=get_arg_increment();
+	const char* filename=get_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to read REAL matrix.\n");
 
-	CFile f((CHAR*) filename, 'r', F_DREAL);
+	CFile f((char*) filename, 'r', F_DREAL);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to read REAL matrix.\n", filename);
 
@@ -385,55 +385,55 @@ void CCmdLineInterface::get_real_matrix(DREAL*& matrix, INT& num_feat, INT& num_
 	CMath::transpose_matrix(matrix, num_feat, num_vec);
 }
 
-void CCmdLineInterface::get_short_matrix(SHORT*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_short_matrix(int16_t*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
 	matrix=NULL;
 	num_feat=0;
 	num_vec=0;
 }
 
-void CCmdLineInterface::get_word_matrix(WORD*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_word_matrix(uint16_t*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
 	matrix=NULL;
 	num_feat=0;
 	num_vec=0;
 }
 
-void CCmdLineInterface::get_byte_ndarray(BYTE*& array, INT*& dims, INT& num_dims)
+void CCmdLineInterface::get_byte_ndarray(uint8_t*& array, int32_t*& dims, int32_t& num_dims)
 {
 }
 
-void CCmdLineInterface::get_char_ndarray(CHAR*& array, INT*& dims, INT& num_dims)
+void CCmdLineInterface::get_char_ndarray(char*& array, int32_t*& dims, int32_t& num_dims)
 {
 }
 
-void CCmdLineInterface::get_int_ndarray(INT*& array, INT*& dims, INT& num_dims)
+void CCmdLineInterface::get_int_ndarray(int32_t*& array, int32_t*& dims, int32_t& num_dims)
 {
 }
 
-void CCmdLineInterface::get_shortreal_ndarray(SHORTREAL*& array, INT*& dims, INT& num_dims)
+void CCmdLineInterface::get_shortreal_ndarray(float32_t*& array, int32_t*& dims, int32_t& num_dims)
 {
 }
 
-void CCmdLineInterface::get_real_ndarray(DREAL*& array, INT*& dims, INT& num_dims)
+void CCmdLineInterface::get_real_ndarray(float64_t*& array, int32_t*& dims, int32_t& num_dims)
 {
 }
 
-void CCmdLineInterface::get_short_ndarray(SHORT*& array, INT*& dims, INT& num_dims)
+void CCmdLineInterface::get_short_ndarray(int16_t*& array, int32_t*& dims, int32_t& num_dims)
 {
 }
 
-void CCmdLineInterface::get_word_ndarray(WORD*& array, INT*& dims, INT& num_dims)
+void CCmdLineInterface::get_word_ndarray(uint16_t*& array, int32_t*& dims, int32_t& num_dims)
 {
 }
 
-void CCmdLineInterface::get_real_sparsematrix(TSparse<DREAL>*& matrix, INT& num_feat, INT& num_vec)
+void CCmdLineInterface::get_real_sparsematrix(TSparse<float64_t>*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
-	const CHAR* filename=get_arg_increment();
+	const char* filename=get_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to read SPARSE REAL matrix.\n");
 
-	CFile f((CHAR*) filename, 'r', F_DREAL);
+	CFile f((char*) filename, 'r', F_DREAL);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to read SPARSE REAL matrix.\n", filename);
 
@@ -442,20 +442,20 @@ void CCmdLineInterface::get_real_sparsematrix(TSparse<DREAL>*& matrix, INT& num_
 }
 
 
-void CCmdLineInterface::get_byte_string_list(T_STRING<BYTE>*& strings, INT& num_str, INT& max_string_len)
+void CCmdLineInterface::get_byte_string_list(T_STRING<uint8_t>*& strings, int32_t& num_str, int32_t& max_string_len)
 {
 	strings=NULL;
 	num_str=0;
 	max_string_len=0;
 }
 
-void CCmdLineInterface::get_char_string_list(T_STRING<CHAR>*& strings, INT& num_str, INT& max_string_len)
+void CCmdLineInterface::get_char_string_list(T_STRING<char>*& strings, int32_t& num_str, int32_t& max_string_len)
 {
-	const CHAR* filename=get_arg_increment();
+	const char* filename=get_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to read CHAR string list.\n");
 
-	CFile f((CHAR*) filename, 'r', F_CHAR);
+	CFile f((char*) filename, 'r', F_CHAR);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to read CHAR string list.\n", filename);
 
@@ -463,26 +463,26 @@ void CCmdLineInterface::get_char_string_list(T_STRING<CHAR>*& strings, INT& num_
 		SG_ERROR("Could not read CHAR data from %s.\n", filename);
 
 /*
-	for (INT i=0; i<num_str; i++)
+	for (int32_t i=0; i<num_str; i++)
 		SG_PRINT("%s\n", strings[i].string);
 */
 }
 
-void CCmdLineInterface::get_int_string_list(T_STRING<INT>*& strings, INT& num_str, INT& max_string_len)
+void CCmdLineInterface::get_int_string_list(T_STRING<int32_t>*& strings, int32_t& num_str, int32_t& max_string_len)
 {
 	strings=NULL;
 	num_str=0;
 	max_string_len=0;
 }
 
-void CCmdLineInterface::get_short_string_list(T_STRING<SHORT>*& strings, INT& num_str, INT& max_string_len)
+void CCmdLineInterface::get_short_string_list(T_STRING<int16_t>*& strings, int32_t& num_str, int32_t& max_string_len)
 {
 	strings=NULL;
 	num_str=0;
 	max_string_len=0;
 }
 
-void CCmdLineInterface::get_word_string_list(T_STRING<WORD>*& strings, INT& num_str, INT& max_string_len)
+void CCmdLineInterface::get_word_string_list(T_STRING<uint16_t>*& strings, int32_t& num_str, int32_t& max_string_len)
 {
 	strings=NULL;
 	num_str=0;
@@ -490,7 +490,7 @@ void CCmdLineInterface::get_word_string_list(T_STRING<WORD>*& strings, INT& num_
 }
 
 /** set functions - to pass data from shogun to the target interface */
-bool CCmdLineInterface::create_return_values(INT num)
+bool CCmdLineInterface::create_return_values(int32_t num)
 {
 	if (num==m_nlhs)
 		return true;
@@ -506,12 +506,12 @@ void* CCmdLineInterface::get_return_values()
 
 /** set functions - to pass data from shogun to the target interface */
 
-void CCmdLineInterface::set_int(INT scalar)
+void CCmdLineInterface::set_int(int32_t scalar)
 {
 	//set_arg_increment(ScalarInteger(scalar));
 }
 
-void CCmdLineInterface::set_real(DREAL scalar)
+void CCmdLineInterface::set_real(float64_t scalar)
 {
 	//set_arg_increment(ScalarReal(scalar));
 }
@@ -522,33 +522,33 @@ void CCmdLineInterface::set_bool(bool scalar)
 }
 
 
-void CCmdLineInterface::set_char_vector(const CHAR* vec, INT len)
+void CCmdLineInterface::set_char_vector(const char* vec, int32_t len)
 {
 }
 
-void CCmdLineInterface::set_short_vector(const SHORT* vec, INT len)
+void CCmdLineInterface::set_short_vector(const int16_t* vec, int32_t len)
 {
 }
 
-void CCmdLineInterface::set_byte_vector(const BYTE* vec, INT len)
+void CCmdLineInterface::set_byte_vector(const uint8_t* vec, int32_t len)
 {
 }
 
-void CCmdLineInterface::set_int_vector(const INT* vec, INT len)
+void CCmdLineInterface::set_int_vector(const int32_t* vec, int32_t len)
 {
 }
 
-void CCmdLineInterface::set_shortreal_vector(const SHORTREAL* vec, INT len)
+void CCmdLineInterface::set_shortreal_vector(const float32_t* vec, int32_t len)
 {
 }
 
-void CCmdLineInterface::set_real_vector(const DREAL* vec, INT len)
+void CCmdLineInterface::set_real_vector(const float64_t* vec, int32_t len)
 {
-	const CHAR* filename=set_arg_increment();
+	const char* filename=set_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to write REAL vector.\n");
 
-	CFile f((CHAR*) filename, 'w', F_DREAL);
+	CFile f((char*) filename, 'w', F_DREAL);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to write REAL vector.\n", filename);
 
@@ -556,100 +556,100 @@ void CCmdLineInterface::set_real_vector(const DREAL* vec, INT len)
 		SG_ERROR("Could not write REAL data to %s.\n", filename);
 }
 
-void CCmdLineInterface::set_word_vector(const WORD* vec, INT len)
+void CCmdLineInterface::set_word_vector(const uint16_t* vec, int32_t len)
 {
 }
 
 /*
 #undef SET_VECTOR
 #define SET_VECTOR(function_name, r_type, r_cast, sg_type, if_type, error_string) \
-void CCmdLineInterface::function_name(const sg_type* vec, INT len)	\
+void CCmdLineInterface::function_name(const sg_type* vec, int32_t len)	\
 {																\
 	void* feat=NULL;												\
 	PROTECT( feat = allocVector(r_type, len) );					\
 																\
-	for (INT i=0; i<len; i++)									\
+	for (int32_t i=0; i<len; i++)									\
 		r_cast(feat)[i]=(if_type) vec[i];						\
 																\
 	UNPROTECT(1);												\
 	set_arg_increment(feat);									\
 }
 
-SET_VECTOR(set_byte_vector, INTSXP, INTEGER, BYTE, int, "Byte")
-SET_VECTOR(set_int_vector, INTSXP, INTEGER, INT, int, "Integer")
-SET_VECTOR(set_short_vector, INTSXP, INTEGER, SHORT, int, "Short")
-SET_VECTOR(set_shortreal_vector, XP, REAL, SHORTREAL, float, "Single Precision")
-SET_VECTOR(set_real_vector, XP, REAL, DREAL, double, "Double Precision")
-SET_VECTOR(set_word_vector, INTSXP, INTEGER, WORD, int, "Word")
+SET_VECTOR(set_byte_vector, INTSXP, INTEGER, uint8_t, int, "Byte")
+SET_VECTOR(set_int_vector, INTSXP, INTEGER, int32_t, int, "Integer")
+SET_VECTOR(set_short_vector, INTSXP, INTEGER, int16_t, int, "Short")
+SET_VECTOR(set_shortreal_vector, XP, REAL, float32_t, float, "Single Precision")
+SET_VECTOR(set_real_vector, XP, REAL, float64_t, double, "Double Precision")
+SET_VECTOR(set_word_vector, INTSXP, INTEGER, uint16_t, int, "Word")
 #undef SET_VECTOR
 */
 
 
-void CCmdLineInterface::set_char_matrix(const CHAR* matrix, INT num_feat, INT num_vec)
+void CCmdLineInterface::set_char_matrix(const char* matrix, int32_t num_feat, int32_t num_vec)
 {
 }
-void CCmdLineInterface::set_byte_matrix(const BYTE* matrix, INT num_feat, INT num_vec)
+void CCmdLineInterface::set_byte_matrix(const uint8_t* matrix, int32_t num_feat, int32_t num_vec)
 {
 }
-void CCmdLineInterface::set_int_matrix(const INT* matrix, INT num_feat, INT num_vec)
+void CCmdLineInterface::set_int_matrix(const int32_t* matrix, int32_t num_feat, int32_t num_vec)
 {
 }
-void CCmdLineInterface::set_short_matrix(const SHORT* matrix, INT num_feat, INT num_vec)
+void CCmdLineInterface::set_short_matrix(const int16_t* matrix, int32_t num_feat, int32_t num_vec)
 {
 }
-void CCmdLineInterface::set_shortreal_matrix(const SHORTREAL* matrix, INT num_feat, INT num_vec)
+void CCmdLineInterface::set_shortreal_matrix(const float32_t* matrix, int32_t num_feat, int32_t num_vec)
 {
 }
-void CCmdLineInterface::set_real_matrix(const DREAL* matrix, INT num_feat, INT num_vec)
+void CCmdLineInterface::set_real_matrix(const float64_t* matrix, int32_t num_feat, int32_t num_vec)
 {
-	const CHAR* filename=set_arg_increment();
+	const char* filename=set_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to write REAL matrix.\n");
 
-	CFile f((CHAR*) filename, 'w', F_DREAL);
+	CFile f((char*) filename, 'w', F_DREAL);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to write REAL matrix.\n", filename);
 
 	if (!f.write_real_valued_dense(matrix, num_feat, num_vec))
 		SG_ERROR("Could not write REAL data to %s.\n", filename);
 }
-void CCmdLineInterface::set_word_matrix(const WORD* matrix, INT num_feat, INT num_vec)
+void CCmdLineInterface::set_word_matrix(const uint16_t* matrix, int32_t num_feat, int32_t num_vec)
 {
 }
 
 /*
 #define SET_MATRIX(function_name, r_type, r_cast, sg_type, if_type, error_string) \
-void CCmdLineInterface::function_name(const sg_type* matrix, INT num_feat, INT num_vec) \
+void CCmdLineInterface::function_name(const sg_type* matrix, int32_t num_feat, int32_t num_vec) \
 {																			\
 	void* feat=NULL;															\
 	PROTECT( feat = allocMatrix(r_type, num_feat, num_vec) );				\
 																			\
-	for (INT i=0; i<num_vec; i++)											\
+	for (int32_t i=0; i<num_vec; i++)											\
 	{																		\
-		for (INT j=0; j<num_feat; j++)										\
+		for (int32_t j=0; j<num_feat; j++)										\
 			r_cast(feat)[i*num_feat+j]=(if_type) matrix[i*num_feat+j];		\
 	}																		\
 																			\
 	UNPROTECT(1);															\
 	set_arg_increment(feat);												\
 }
-SET_MATRIX(set_byte_matrix, INTSXP, INTEGER, BYTE, int, "Byte")
-SET_MATRIX(set_int_matrix, INTSXP, INTEGER, INT, int, "Integer")
-SET_MATRIX(set_short_matrix, INTSXP, INTEGER, SHORT, int, "Short")
-SET_MATRIX(set_shortreal_matrix, XP, REAL, SHORTREAL, float, "Single Precision")
-SET_MATRIX(set_real_matrix, XP, REAL, DREAL, double, "Double Precision")
-SET_MATRIX(set_word_matrix, INTSXP, INTEGER, WORD, int, "Word")
+SET_MATRIX(set_byte_matrix, INTSXP, INTEGER, uint8_t, int, "Byte")
+SET_MATRIX(set_int_matrix, INTSXP, INTEGER, int32_t, int, "Integer")
+SET_MATRIX(set_short_matrix, INTSXP, INTEGER, int16_t, int, "Short")
+SET_MATRIX(set_shortreal_matrix, XP, REAL, float32_t, float, "Single Precision")
+SET_MATRIX(set_real_matrix, XP, REAL, float64_t, double, "Double Precision")
+SET_MATRIX(set_word_matrix, INTSXP, INTEGER, uint16_t, int, "Word")
 #undef SET_MATRIX
 */
 
 
-void CCmdLineInterface::set_real_sparsematrix(const TSparse<DREAL>* matrix, INT num_feat, INT num_vec, LONG nnz)
+void CCmdLineInterface::set_real_sparsematrix(const TSparse<float64_t>* matrix, int32_t num_feat, int32_t num_vec, int64_t nnz)
 {
-	const CHAR* filename=set_arg_increment();
+	const char* filename=set_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to write SPARSE REAL matrix.\n");
 
-	CFile f((CHAR*) filename, 'w', F_DREAL);
+	CFile f((char*) filename, 'w', F_DREAL);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to write SPARSE REAL matrix.\n", filename);
 
@@ -657,17 +657,17 @@ void CCmdLineInterface::set_real_sparsematrix(const TSparse<DREAL>* matrix, INT
 		SG_ERROR("Could not write SPARSE REAL data to %s.\n", filename);
 }
 
-void CCmdLineInterface::set_byte_string_list(const T_STRING<BYTE>* strings, INT num_str)
+void CCmdLineInterface::set_byte_string_list(const T_STRING<uint8_t>* strings, int32_t num_str)
 {
 }
 
-void CCmdLineInterface::set_char_string_list(const T_STRING<CHAR>* strings, INT num_str)
+void CCmdLineInterface::set_char_string_list(const T_STRING<char>* strings, int32_t num_str)
 {
-	const CHAR* filename=set_arg_increment();
+	const char* filename=set_arg_increment();
 	if (!filename)
 		SG_ERROR("No filename given to write CHAR string list.\n");
 
-	CFile f((CHAR*) filename, 'w', F_CHAR);
+	CFile f((char*) filename, 'w', F_CHAR);
 	if (!f.is_ok())
 		SG_ERROR("Could not open file %s to write CHAR string list.\n", filename);
 
@@ -675,20 +675,20 @@ void CCmdLineInterface::set_char_string_list(const T_STRING<CHAR>* strings, INT
 		SG_ERROR("Could not write CHAR data to %s.\n", filename);
 }
 
-void CCmdLineInterface::set_int_string_list(const T_STRING<INT>* strings, INT num_str)
+void CCmdLineInterface::set_int_string_list(const T_STRING<int32_t>* strings, int32_t num_str)
 {
 }
 
-void CCmdLineInterface::set_short_string_list(const T_STRING<SHORT>* strings, INT num_str)
+void CCmdLineInterface::set_short_string_list(const T_STRING<int16_t>* strings, int32_t num_str)
 {
 }
 
-void CCmdLineInterface::set_word_string_list(const T_STRING<WORD>* strings, INT num_str)
+void CCmdLineInterface::set_word_string_list(const T_STRING<uint16_t>* strings, int32_t num_str)
 {
 }
 
 
-bool CCmdLineInterface::skip_line(const CHAR* line)
+bool CCmdLineInterface::skip_line(const char* line)
 {
 	if (!line)
 		return true;
@@ -702,7 +702,7 @@ bool CCmdLineInterface::skip_line(const CHAR* line)
 
 	//SG_PRINT("ascii(0) %d, %c\n", int(line[0]), line[0]);
 
-	CHAR* skipped=CIO::skip_blanks((CHAR*) line);
+	char* skipped=CIO::skip_blanks((char*) line);
 	if (skipped[0]==CMDLINE_COMMENT0 || skipped[0]==CMDLINE_COMMENT1)
 		return true;
 
@@ -716,7 +716,7 @@ void CCmdLineInterface::print_prompt()
 }
 
 
-CHAR* CCmdLineInterface::get_line(FILE* infile, bool interactive_mode)
+char* CCmdLineInterface::get_line(FILE* infile, bool interactive_mode)
 {
 	char* in=NULL;
 	memset(input, 0, sizeof(input));
@@ -755,7 +755,7 @@ CHAR* CCmdLineInterface::get_line(FILE* infile, bool interactive_mode)
 		return input;
 }
 
-bool CCmdLineInterface::parse_line(CHAR* line)
+bool CCmdLineInterface::parse_line(char* line)
 {
 	if (!line)
 		return false;
@@ -838,7 +838,7 @@ int main(int argc, char* argv[])
 		{
 			while (true)
 			{
-				CHAR* line=intf->get_line();
+				char* line=intf->get_line();
 
 				if (!line)
 					break;
@@ -887,7 +887,7 @@ int main(int argc, char* argv[])
 			int s2=accept(s, NULL, NULL);
 			SG_SINFO( "accepting connection\n");
 
-			CHAR input[READLINE_BUFFER_SIZE];
+			char input[READLINE_BUFFER_SIZE];
 			do
 			{
 				bzero(input, sizeof(input));
