@@ -89,6 +89,8 @@ class CSGInterface : public CSGObject
 		/** get labels */
 		bool cmd_get_labels();
 
+		/** set kernel normalization */
+		bool cmd_set_kernel_normalization();
 		/** set kernel */
 		bool cmd_set_kernel();
 		/** add kernel (to e.g. CombinedKernel) */
@@ -135,6 +137,10 @@ class CSGInterface : public CSGObject
 		bool cmd_use_diagonal_speedup();
 		/** set kernel optimization type */
 		bool cmd_set_kernel_optimization_type();
+		/** set Salzberg prior probs */
+		bool cmd_set_prior_probs();
+		/** set Salzberg prior probs from labels */
+		bool cmd_set_prior_probs_from_labels();
 
 
 
@@ -160,8 +166,12 @@ class CSGInterface : public CSGObject
 		bool cmd_new_classifier();
 		/** load SVM/classifier */
 		bool cmd_load_classifier();
+		/** save SVM/classifier */
+		bool cmd_save_classifier();
 		/** get SVM */
 		bool cmd_get_svm();
+		/** get number of SVMs in MultiClass */
+		bool cmd_get_num_svms();
 		/** set SVM */
 		bool cmd_set_svm();
 		/** classify */
@@ -198,8 +208,6 @@ class CSGInterface : public CSGObject
 		bool cmd_set_svm_mkl_parameters();
 		/** set max train time */
 		bool cmd_set_max_train_time();
-		/** set SVM precompute enabled */
-		bool cmd_set_svm_precompute_enabled();
 		/** set SVM MKL enabled */
 		bool cmd_set_svm_mkl_enabled();
 		/** set SVM shrinking enabled */
@@ -331,6 +339,10 @@ class CSGInterface : public CSGObject
 		 *  field of the struct
 		 */
 		bool cmd_get_plif_struct();
+		/**
+		 * precompute subkernels of a combined kernel
+		 */
+		bool cmd_precompute_subkernels();
 		/** 
 		 * -precompute content svms 
 		 *  and save the outputs 
@@ -344,6 +356,15 @@ class CSGInterface : public CSGObject
 		 *  type
 		 */
 		bool cmd_precompute_content_svms();
+		/**
+		 * -precompute tiling features 
+		 *  and save the outputs (# content types x 
+		 *  # features positons) in a member variable 
+		 *  of the DynProg Object
+		 * -the tiling intensities are transformed 
+		 *  and then stored as cumulative scores
+		 */
+		bool cmd_precompute_tiling_features();
 		/** 
  		 * -compute the matrix that links 
  		 *  the plif ids to the transitions
@@ -384,6 +405,8 @@ class CSGInterface : public CSGObject
 		bool cmd_set_output();
 		/** set threshold */
 		bool cmd_set_threshold();
+		/** initialize random number generator */
+		bool cmd_init_random();
 		/** set number of threads */
 		bool cmd_set_num_threads();
 		/** translate string */
@@ -416,99 +439,139 @@ class CSGInterface : public CSGObject
 		/// get type of current argument (does not increment argument counter)
 		virtual IFType get_argument_type()=0;
 
-		virtual INT get_int()=0;
-		virtual DREAL get_real()=0;
+		virtual int32_t get_int()=0;
+		virtual float64_t get_real()=0;
 		virtual bool get_bool()=0;
 
-		virtual CHAR* get_string(INT& len)=0;
+		virtual char* get_string(int32_t& len)=0;
 
-		virtual void get_bool_vector(bool*& vector, INT& len);
-		virtual void set_bool_vector(bool*& vector, INT& len);
-		virtual void get_byte_vector(BYTE*& vector, INT& len)=0;
-		virtual void get_char_vector(CHAR*& vector, INT& len)=0;
-		virtual void get_int_vector(INT*& vector, INT& len)=0;
-		virtual void get_real_vector(DREAL*& vector, INT& len)=0;
-		virtual void get_shortreal_vector(SHORTREAL*& vector, INT& len)=0;
-		virtual void get_short_vector(SHORT*& vector, INT& len)=0;
-		virtual void get_word_vector(WORD*& vector, INT& len)=0;
+		virtual void get_bool_vector(bool*& vector, int32_t& len);
+		virtual void set_bool_vector(bool*& vector, int32_t& len);
+		virtual void get_byte_vector(uint8_t*& vector, int32_t& len)=0;
+		virtual void get_char_vector(char*& vector, int32_t& len)=0;
+		virtual void get_int_vector(int32_t*& vector, int32_t& len)=0;
+		virtual void get_real_vector(float64_t*& vector, int32_t& len)=0;
+		virtual void get_shortreal_vector(float32_t*& vector, int32_t& len)=0;
+		virtual void get_short_vector(int16_t*& vector, int32_t& len)=0;
+		virtual void get_word_vector(uint16_t*& vector, int32_t& len)=0;
 
 
-		virtual void get_byte_matrix(BYTE*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_char_matrix(CHAR*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_int_matrix(INT*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_shortreal_matrix(SHORTREAL*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_real_matrix(DREAL*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_short_matrix(SHORT*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_word_matrix(WORD*& matrix, INT& num_feat, INT& num_vec)=0;
+		virtual void get_byte_matrix(
+			uint8_t*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_char_matrix(
+			char*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_int_matrix(
+			int32_t*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_shortreal_matrix(
+			float32_t*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_real_matrix(
+			float64_t*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_short_matrix(
+			int16_t*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_word_matrix(
+			uint16_t*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
 
-		virtual void get_byte_ndarray(BYTE*& array, INT*& dims, INT& num_dims)=0;
-		virtual void get_char_ndarray(CHAR*& array, INT*& dims, INT& num_dims)=0;
-		virtual void get_int_ndarray(INT*& array, INT*& dims, INT& num_dims)=0;
-		virtual void get_shortreal_ndarray(SHORTREAL*& array, INT*& dims, INT& num_dims)=0;
-		virtual void get_real_ndarray(DREAL*& array, INT*& dims, INT& num_dims)=0;
-		virtual void get_short_ndarray(SHORT*& array, INT*& dims, INT& num_dims)=0;
-		virtual void get_word_ndarray(WORD*& array, INT*& dims, INT& num_dims)=0;
+		virtual void get_byte_ndarray(
+			uint8_t*& array, int32_t*& dims, int32_t& num_dims)=0;
+		virtual void get_char_ndarray(
+			char*& array, int32_t*& dims, int32_t& num_dims)=0;
+		virtual void get_int_ndarray(
+			int32_t*& array, int32_t*& dims, int32_t& num_dims)=0;
+		virtual void get_shortreal_ndarray(
+			float32_t*& array, int32_t*& dims, int32_t& num_dims)=0;
+		virtual void get_real_ndarray(
+			float64_t*& array, int32_t*& dims, int32_t& num_dims)=0;
+		virtual void get_short_ndarray(
+			int16_t*& array, int32_t*& dims, int32_t& num_dims)=0;
+		virtual void get_word_ndarray(
+			uint16_t*& array, int32_t*& dims, int32_t& num_dims)=0;
 
 
-		virtual void get_real_sparsematrix(TSparse<DREAL>*& matrix, INT& num_feat, INT& num_vec)=0;
+		virtual void get_real_sparsematrix(
+			TSparse<float64_t>*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
 
-		/*  future versions might support types other than DREAL
+		/*  future versions might support types other than float64_t
 		
-		virtual void get_byte_sparsematrix(TSparse<BYTE>*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_char_sparsematrix(TSparse<CHAR>*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_int_sparsematrix(TSparse<INT>*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_shortreal_sparsematrix(TSparse<SHORTREAL>*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_short_sparsematrix(TSparse<SHORT>*& matrix, INT& num_feat, INT& num_vec)=0;
-		virtual void get_word_sparsematrix(TSparse<WORD>*& matrix, INT& num_feat, INT& num_vec)=0; */
+		virtual void get_byte_sparsematrix(TSparse<uint8_t>*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_char_sparsematrix(TSparse<char>*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_int_sparsematrix(TSparse<int32_t>*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_shortreal_sparsematrix(TSparse<float32_t>*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_short_sparsematrix(TSparse<int16_t>*& matrix, int32_t& num_feat, int32_t& num_vec)=0;
+		virtual void get_word_sparsematrix(TSparse<uint16_t>*& matrix, int32_t& num_feat, int32_t& num_vec)=0; */
 
-		virtual void get_byte_string_list(T_STRING<BYTE>*& strings, INT& num_str, INT& max_string_len)=0;
-		virtual void get_char_string_list(T_STRING<CHAR>*& strings, INT& num_str, INT& max_string_len)=0;
-		virtual void get_int_string_list(T_STRING<INT>*& strings, INT& num_str, INT& max_string_len)=0;
-		virtual void get_short_string_list(T_STRING<SHORT>*& strings, INT& num_str, INT& max_string_len)=0;
-		virtual void get_word_string_list(T_STRING<WORD>*& strings, INT& num_str, INT& max_string_len)=0;
+		virtual void get_byte_string_list(
+			T_STRING<uint8_t>*& strings, int32_t& num_str,
+			int32_t& max_string_len)=0;
+		virtual void get_char_string_list(
+			T_STRING<char>*& strings, int32_t& num_str,
+			int32_t& max_string_len)=0;
+		virtual void get_int_string_list(
+			T_STRING<int32_t>*& strings, int32_t& num_str,
+			int32_t& max_string_len)=0;
+		virtual void get_short_string_list(
+			T_STRING<int16_t>*& strings, int32_t& num_str,
+			int32_t& max_string_len)=0;
+		virtual void get_word_string_list(
+			T_STRING<uint16_t>*& strings, int32_t& num_str,
+			int32_t& max_string_len)=0;
 
 
 		/** set functions - to pass data from shogun to the target interface */
-		virtual bool create_return_values(INT num_val)=0;
+		virtual bool create_return_values(int32_t num_val)=0;
 
-		virtual void set_int(INT scalar)=0;
-		virtual void set_real(DREAL scalar)=0;
+		virtual void set_int(int32_t scalar)=0;
+		virtual void set_real(float64_t scalar)=0;
 		virtual void set_bool(bool scalar)=0;
 
-		virtual void set_byte_vector(const BYTE* vector, INT len)=0;
-		virtual void set_char_vector(const CHAR* vector, INT len)=0;
-		virtual void set_int_vector(const INT* vector, INT len)=0;
-		virtual void set_shortreal_vector(const SHORTREAL* vector, INT len)=0;
-		virtual void set_real_vector(const DREAL* vector, INT len)=0;
-		virtual void set_short_vector(const SHORT* vector, INT len)=0;
-		virtual void set_word_vector(const WORD* vector, INT len)=0;
+		virtual void set_byte_vector(const uint8_t* vector, int32_t len)=0;
+		virtual void set_char_vector(const char* vector, int32_t len)=0;
+		virtual void set_int_vector(const int32_t* vector, int32_t len)=0;
+		virtual void set_shortreal_vector(
+			const float32_t* vector, int32_t len)=0;
+		virtual void set_real_vector(const float64_t* vector, int32_t len)=0;
+		virtual void set_short_vector(const int16_t* vector, int32_t len)=0;
+		virtual void set_word_vector(const uint16_t* vector, int32_t len)=0;
 
 
-		virtual void set_byte_matrix(const BYTE* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_char_matrix(const CHAR* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_int_matrix(const INT* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_shortreal_matrix(const SHORTREAL* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_real_matrix(const DREAL* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_short_matrix(const SHORT* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_word_matrix(const WORD* matrix, INT num_feat, INT num_vec)=0;
+		virtual void set_byte_matrix(
+			const uint8_t* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_char_matrix(
+			const char* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_int_matrix(
+			const int32_t* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_shortreal_matrix(
+			const float32_t* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_real_matrix(
+			const float64_t* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_short_matrix(
+			const int16_t* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_word_matrix(
+			const uint16_t* matrix, int32_t num_feat, int32_t num_vec)=0;
 
-		virtual void set_real_sparsematrix(const TSparse<DREAL>* matrix, INT num_feat, INT num_vec, LONG nnz)=0;
+		virtual void set_real_sparsematrix(
+			const TSparse<float64_t>* matrix, int32_t num_feat,
+			int32_t num_vec, int64_t nnz)=0;
 
-		/*  future versions might support types other than DREAL
+		/*  future versions might support types other than float64_t
 		
-		virtual void set_byte_sparsematrix(const TSparse<BYTE>* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_char_sparsematrix(const TSparse<CHAR>* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_int_sparsematrix(const TSparse<INT>* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_shortreal_sparsematrix(const TSparse<SHORTREAL>* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_short_sparsematrix(const TSparse<SHORT>* matrix, INT num_feat, INT num_vec)=0;
-		virtual void set_word_sparsematrix(const TSparse<WORD>* matrix, INT num_feat, INT num_vec)=0; */
+		virtual void set_byte_sparsematrix(const TSparse<uint8_t>* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_char_sparsematrix(const TSparse<char>* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_int_sparsematrix(const TSparse<int32_t>* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_shortreal_sparsematrix(const TSparse<float32_t>* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_short_sparsematrix(const TSparse<int16_t>* matrix, int32_t num_feat, int32_t num_vec)=0;
+		virtual void set_word_sparsematrix(const TSparse<uint16_t>* matrix, int32_t num_feat, int32_t num_vec)=0; */
 
 
-		virtual void set_byte_string_list(const T_STRING<BYTE>* strings, INT num_str)=0;
-		virtual void set_char_string_list(const T_STRING<CHAR>* strings, INT num_str)=0;
-		virtual void set_int_string_list(const T_STRING<INT>* strings, INT num_str)=0;
-		virtual void set_short_string_list(const T_STRING<SHORT>* strings, INT num_str)=0;
-		virtual void set_word_string_list(const T_STRING<WORD>* strings, INT num_str)=0;
+		virtual void set_byte_string_list(
+			const T_STRING<uint8_t>* strings, int32_t num_str)=0;
+		virtual void set_char_string_list(
+			const T_STRING<char>* strings, int32_t num_str)=0;
+		virtual void set_int_string_list(
+			const T_STRING<int32_t>* strings, int32_t num_str)=0;
+		virtual void set_short_string_list(
+			const T_STRING<int16_t>* strings, int32_t num_str)=0;
+		virtual void set_word_string_list(
+			const T_STRING<uint16_t>* strings, int32_t num_str)=0;
 
 		/// general interface handler
 		bool handle();
@@ -534,10 +597,10 @@ class CSGInterface : public CSGObject
 		 *
 		 * @param str string to look in, not necessarily 0-terminated
 		 * @param cmd 0-terminated string const
-		 * @param len number of CHAR to compare, length of cmd if not given
+		 * @param len number of char to compare, length of cmd if not given
 		 *
 		 */
-		static bool strmatch(const CHAR* str, const CHAR* cmd, INT len=-1)
+		static bool strmatch(const char* str, const char* cmd, int32_t len=-1)
 		{
 			if (len==-1)
 			{
@@ -550,7 +613,7 @@ class CSGInterface : public CSGObject
 		}
 
 		/// get command name like 'get_svm', 'new_hmm', etc.
-		CHAR* get_command(INT &len)
+		char* get_command(int32_t &len)
 		{
 			ASSERT(m_rhs_counter==0);
 			if (m_nrhs<=0)
@@ -570,41 +633,43 @@ class CSGInterface : public CSGObject
 		CKernel* create_kernel();
 
 		/** legacy-related stuff - anybody got a better idea? */
-		CHAR* get_str_from_str_or_direct(INT& len);
-		INT get_int_from_int_or_str();
-		DREAL get_real_from_real_or_str();
+		char* get_str_from_str_or_direct(int32_t& len);
+		int32_t get_int_from_int_or_str();
+		float64_t get_real_from_real_or_str();
 		bool get_bool_from_bool_or_str();
-		void get_int_vector_from_int_vector_or_str(INT*& vector, INT& len);
-		void get_real_vector_from_real_vector_or_str(DREAL*& vector, INT& len);
-		INT get_vector_len_from_str(INT expected_len=0);
-		CHAR* get_str_from_str(INT& len);
-		INT get_num_args_in_str();
+		void get_int_vector_from_int_vector_or_str(
+			int32_t*& vector, int32_t& len);
+		void get_real_vector_from_real_vector_or_str(
+			float64_t*& vector, int32_t& len);
+		int32_t get_vector_len_from_str(int32_t expected_len=0);
+		char* get_str_from_str(int32_t& len);
+		int32_t get_num_args_in_str();
 
 		/// get line from user/stdin/file input
 		/// @return true at EOF
-		CHAR* get_line(FILE* infile=stdin, bool show_prompt=true);
+		char* get_line(FILE* infile=stdin, bool show_prompt=true);
 
 	protected:
-		INT m_lhs_counter;
-		INT m_rhs_counter;
-		INT m_nlhs;
-		INT m_nrhs;
+		int32_t m_lhs_counter;
+		int32_t m_rhs_counter;
+		int32_t m_nlhs;
+		int32_t m_nrhs;
 
 		// related to cmd_exec and cmd_echo
 		FILE* file_out;
-		CHAR input[10000];
+		char input[10000];
 		bool echo;
 
-		CHAR* m_legacy_strptr;
+		char* m_legacy_strptr;
 };
 
 typedef bool (CSGInterface::*CSGInterfacePtr)();
 
 typedef struct {
-	const CHAR* command;
+	const char* command;
 	CSGInterfacePtr method;
-	const CHAR* usage_prefix;
-	const CHAR* usage_suffix;
+	const char* usage_prefix;
+	const char* usage_suffix;
 } CSGInterfaceMethod;
 
 #endif // !HAVE_SWIG
