@@ -15,16 +15,18 @@
 #include "kernel/DistanceKernel.h"
 #include "features/RealFeatures.h"
 
-CDistanceKernel::CDistanceKernel(INT size, DREAL w ,CDistance* d)
+CDistanceKernel::CDistanceKernel(int32_t size, float64_t w, CDistance* d)
 : CKernel(size), distance(d), width(w)
 {
+	ASSERT(distance);
 	SG_REF(distance);
 }
 
 CDistanceKernel::CDistanceKernel(
-	CFeatures *l, CFeatures *r, DREAL w , CDistance* d)
+	CFeatures *l, CFeatures *r, float64_t w , CDistance* d)
 : CKernel(10), distance(d), width(w)
 {
+	ASSERT(distance);
 	SG_REF(distance);
 	init(l, r);
 }
@@ -39,12 +41,15 @@ CDistanceKernel::~CDistanceKernel()
 
 bool CDistanceKernel::init(CFeatures* l, CFeatures* r)
 {
-	return CKernel::init(l,r) && distance->init(l,r);
+	ASSERT(distance);
+	CKernel::init(l,r);
+	distance->init(l,r);
+	return init_normalizer();
 }
 
-DREAL CDistanceKernel::compute(INT idx_a, INT idx_b)
+float64_t CDistanceKernel::compute(int32_t idx_a, int32_t idx_b)
 {
-	DREAL result=distance->distance(idx_a, idx_b);
+	float64_t result=distance->distance(idx_a, idx_b);
 	return exp(-result/width);
 }
 
