@@ -14,13 +14,15 @@
 #include "features/RealFeatures.h"
 #include "lib/io.h"
 
-CGaussianShiftKernel::CGaussianShiftKernel(INT size, double w, int ms, int ss)
+CGaussianShiftKernel::CGaussianShiftKernel(
+	int32_t size, float64_t w, int32_t ms, int32_t ss)
 : CGaussianKernel(size, w), max_shift(ms), shift_step(ss)
 {
 }
 
 CGaussianShiftKernel::CGaussianShiftKernel(
-	CRealFeatures* l, CRealFeatures* r, double w, int ms, int ss, INT size)
+	CRealFeatures* l, CRealFeatures* r, float64_t w, int32_t ms, int32_t ss,
+	int32_t size)
 : CGaussianKernel(l, r, w, size), max_shift(ms), shift_step(ss)
 {
 	init(l,r);
@@ -30,30 +32,32 @@ CGaussianShiftKernel::~CGaussianShiftKernel()
 {
 }
 
-DREAL CGaussianShiftKernel::compute(INT idx_a, INT idx_b)
+float64_t CGaussianShiftKernel::compute(int32_t idx_a, int32_t idx_b)
 {
-	INT alen, blen;
+	int32_t alen, blen;
 	bool afree, bfree;
 
-	double* avec=((CRealFeatures*) lhs)->get_feature_vector(idx_a, alen, afree);
-	double* bvec=((CRealFeatures*) rhs)->get_feature_vector(idx_b, blen, bfree);
+	float64_t* avec=
+		((CRealFeatures*) lhs)->get_feature_vector(idx_a, alen, afree);
+	float64_t* bvec=
+		((CRealFeatures*) rhs)->get_feature_vector(idx_b, blen, bfree);
 	ASSERT(alen==blen);
 
-	DREAL result = 0.0 ;
-	DREAL sum=0.0 ;
-	for (INT i=0; i<alen; i++)
+	float64_t result = 0.0 ;
+	float64_t sum=0.0 ;
+	for (int32_t i=0; i<alen; i++)
 		sum+=(avec[i]-bvec[i])*(avec[i]-bvec[i]);
 	result += exp(-sum/width) ;
 
-	for (INT shift = shift_step, s=1; shift<max_shift; shift+=shift_step, s++)
+	for (int32_t shift = shift_step, s=1; shift<max_shift; shift+=shift_step, s++)
 	{
 		sum=0.0 ;
-		for (INT i=0; i<alen-shift; i++)
+		for (int32_t i=0; i<alen-shift; i++)
 			sum+=(avec[i+shift]-bvec[i])*(avec[i+shift]-bvec[i]);
 		result += exp(-sum/width)/(2*s) ;
 
 		sum=0.0 ;
-		for (INT i=0; i<alen-shift; i++)
+		for (int32_t i=0; i<alen-shift; i++)
 			sum+=(avec[i]-bvec[i+shift])*(avec[i]-bvec[i+shift]);
 		result += exp(-sum/width)/(2*s) ;
 	}
