@@ -14,29 +14,26 @@
 #include "lib/common.h"
 #include "kernel/StringKernel.h"
 
-/** kernel LinearString */
-class CLinearStringKernel: public CStringKernel<CHAR>
+/** Computes the standard linear kernel on dense char valued features
+ * \f[
+ * k({\bf x},{\bf x'})= \frac{1}{scale}{\bf x}\cdot {\bf x'}
+ * \f]
+ *
+ * Note: Basically the same as LinearByteKernel but on signed chars.
+ */
+class CLinearStringKernel: public CStringKernel<char>
 {
 	public:
 		/** constructor
-		 *
-		 * @param size cache size
-		 * @param do_rescale if rescaling shall be applied
-		 * @param scale scaling factor
 		 */
-		CLinearStringKernel(INT size,
-			bool do_rescale=true, DREAL scale=1.);
+		CLinearStringKernel();
 
 		/** constructor
 		 *
 		 * @param l features of left-hand side
 		 * @param r features of right-hand side
-		 * @param do_rescale if rescaling shall be applied
-		 * @param scale scaling factor
 		 */
-		CLinearStringKernel(
-			CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r,
-			bool do_rescale=true, DREAL scale=1.);
+		CLinearStringKernel(CStringFeatures<char>* l, CStringFeatures<char>* r);
 
 		virtual ~CLinearStringKernel();
 
@@ -78,7 +75,7 @@ class CLinearStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return name FixedDegree
 		 */
-		virtual const CHAR* get_name()
+		virtual const char* get_name()
 		{
 			return "Linear";
 		}
@@ -91,8 +88,8 @@ class CLinearStringKernel: public CStringKernel<CHAR>
 		 * @param alphas alphas
 		 * @return if optimization was successful
 		 */
-		virtual bool init_optimization(INT num_suppvec, INT* sv_idx,
-			DREAL* alphas);
+		virtual bool init_optimization(
+			int32_t num_suppvec, int32_t* sv_idx, float64_t* alphas);
 
 		/** delete optimization
 		 *
@@ -105,7 +102,7 @@ class CLinearStringKernel: public CStringKernel<CHAR>
 	 	* @param idx index to compute
 	 	* @return optimized value at given index
 	 	*/
-		virtual DREAL compute_optimized(INT idx);
+		virtual float64_t compute_optimized(int32_t idx);
 
 		/** clear normal */
 		virtual void clear_normal();
@@ -115,7 +112,7 @@ class CLinearStringKernel: public CStringKernel<CHAR>
 		 * @param idx where to add
 		 * @param weight what to add
 		 */
-		virtual void add_to_normal(INT idx, DREAL weight);
+		virtual void add_to_normal(int32_t idx, float64_t weight);
 
 	protected:
 		/** compute kernel function for features a and b
@@ -126,20 +123,11 @@ class CLinearStringKernel: public CStringKernel<CHAR>
 		 * @param idx_b index b
 		 * @return computed kernel function at indices a,b
 		 */
-		virtual DREAL compute(INT idx_a, INT idx_b);
-
-		/** initialize rescaling */
-		virtual void init_rescale();
+		virtual float64_t compute(int32_t idx_a, int32_t idx_b);
 
 	protected:
-		/** scaling factor */
-		double scale;
-		/** if rescaling shall be applied */
-		bool do_rescale;
-		/** if kernel is initialized */
-		bool initialized;
 		/** normal vector (used in case of optimized kernel) */
-		double* normal;
+		float64_t* normal;
 };
 
 #endif /* _LINEARSTRINGKERNEL_H___ */
