@@ -15,15 +15,16 @@
 #include "features/StringFeatures.h"
 
 CLocalityImprovedStringKernel::CLocalityImprovedStringKernel(
-	INT size, INT l, INT id, INT od)
-: CStringKernel<CHAR>(size), length(l), inner_degree(id), outer_degree(od)
+	int32_t size, int32_t l, int32_t id, int32_t od)
+: CStringKernel<char>(size), length(l), inner_degree(id), outer_degree(od)
 {
 	SG_INFO( "LIK with parms: l=%d, id=%d, od=%d created!\n", l, id, od);
 }
 
 CLocalityImprovedStringKernel::CLocalityImprovedStringKernel(
-	CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r, INT len, INT id, INT od)
-: CStringKernel<CHAR>(10), length(len), inner_degree(id), outer_degree(od)
+	CStringFeatures<char>* l, CStringFeatures<char>* r, int32_t len,
+	int32_t id, int32_t od)
+: CStringKernel<char>(10), length(len), inner_degree(id), outer_degree(od)
 {
 	SG_INFO( "LIK with parms: l=%d, id=%d, od=%d created!\n", len, id, od);
 
@@ -37,7 +38,8 @@ CLocalityImprovedStringKernel::~CLocalityImprovedStringKernel()
 
 bool CLocalityImprovedStringKernel::init(CFeatures* l, CFeatures* r)
 {
-	return CStringKernel<CHAR>::init(l,r);
+	CStringKernel<char>::init(l,r);
+	return init_normalizer();
 }
 
 bool CLocalityImprovedStringKernel::load_init(FILE* src)
@@ -50,31 +52,31 @@ bool CLocalityImprovedStringKernel::save_init(FILE* dest)
 	return false;
 }
 
-DREAL CLocalityImprovedStringKernel::compute(INT idx_a, INT idx_b)
+float64_t CLocalityImprovedStringKernel::compute(int32_t idx_a, int32_t idx_b)
 {
-	INT alen, blen;
+	int32_t alen, blen;
 
-	CHAR* avec = ((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx_a, alen);
-	CHAR* bvec = ((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx_b, blen);
+	char* avec = ((CStringFeatures<char>*) lhs)->get_feature_vector(idx_a, alen);
+	char* bvec = ((CStringFeatures<char>*) rhs)->get_feature_vector(idx_b, blen);
 	// can only deal with strings of same length
 	ASSERT(alen==blen && alen>0);
 
-	INT i,t;
-	DREAL* match=new DREAL[alen];
+	int32_t i,t;
+	float64_t* match=new float64_t[alen];
 
 	// initialize match table 1 -> match;  0 -> no match
 	for (i = 0; i<alen; i++)
 		match[i] = (avec[i] == bvec[i])? 1 : 0;
 
-	DREAL outer_sum = 0;
+	float64_t outer_sum = 0;
 
 	for (t = 0; t<alen-length; t++)
 	{
-		DREAL sum = 0;
+		float64_t sum = 0;
 		for (i = 0; i<length && t+i+length+1<alen; i++)
 			sum += (i+1)*match[t+i]+(length-i)*match[t+i+length+1];
 		//add middle element + normalize with sum_i=0^2l+1 i = (2l+1)(l+1)
-		DREAL inner_sum = (sum + (length+1)*match[t+length]) / ((2*length+1)*(length+1));
+		float64_t inner_sum = (sum + (length+1)*match[t+length]) / ((2*length+1)*(length+1));
 		inner_sum = pow(inner_sum, inner_degree + 1);
 		outer_sum += inner_sum;
 	}
