@@ -15,8 +15,17 @@
 #include "lib/common.h"
 #include "kernel/StringKernel.h"
 
-/** kernel LocalityImprovedString */
-class CLocalityImprovedStringKernel: public CStringKernel<CHAR>
+/** The LocalityImprovedString kernel is inspired by the polynomial kernel.
+ * Comparing neighboring characters it puts emphasize on local features.
+ *
+ * It can be defined as
+ * \f[
+ * K({\bf x},{\bf x'})=\left(\sum_{i=0}^{T-1}\left(\sum_{j=-l}^{+l}w_jI_{i+j}({\bf x},{\bf x'})\right)^{d_1}\right)^{d_2},
+ * \f]
+ * where
+ * \f$ I_i({\bf x},{\bf x'})=1\f$ if \f$x_i=x'_i\f$ and 0 otherwise.
+ */
+class CLocalityImprovedStringKernel: public CStringKernel<char>
 {
 	public:
 		/** constructor
@@ -26,8 +35,8 @@ class CLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 * @param inner_degree inner degree
 		 * @param outer_degree outer degree
 		 */
-		CLocalityImprovedStringKernel(INT size, INT length,
-			INT inner_degree, INT outer_degree);
+		CLocalityImprovedStringKernel(int32_t size, int32_t length,
+			int32_t inner_degree, int32_t outer_degree);
 
 		/** constructor
 		 *
@@ -38,8 +47,8 @@ class CLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 * @param outer_degree outer degree
 		 */
 		CLocalityImprovedStringKernel(
-			CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r,
-			INT length, INT inner_degree, INT outer_degree);
+			CStringFeatures<char>* l, CStringFeatures<char>* r,
+			int32_t length, int32_t inner_degree, int32_t outer_degree);
 
 		virtual ~CLocalityImprovedStringKernel();
 
@@ -75,7 +84,7 @@ class CLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return name LocalityImproved
 		 */
-		virtual const CHAR* get_name() { return "LocalityImproved"; }
+		virtual const char* get_name() { return "LocalityImproved"; }
 
 	protected:
 		/** compute kernel function for features a and b
@@ -86,15 +95,15 @@ class CLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 * @param idx_b index b
 		 * @return computed kernel function at indices a,b
 		 */
-		DREAL compute(INT idx_a, INT idx_b);
+		float64_t compute(int32_t idx_a, int32_t idx_b);
 
 	protected:
 		/** length */
-		INT length;
+		int32_t length;
 		/** inner degree */
-		INT inner_degree;
+		int32_t inner_degree;
 		/** outer degree */
-		INT outer_degree;
+		int32_t outer_degree;
 };
 
 #endif /* _LOCALITYIMPROVEDSTRINGKERNEL_H__ */
