@@ -14,8 +14,21 @@
 #include "lib/common.h"
 #include "kernel/StringKernel.h"
 
-/** kernel PolyMatchString */
-class CPolyMatchStringKernel: public CStringKernel<CHAR>
+/** The class PolyMatchStringKernel computes a variant of the polynomial kernel
+ * on strings of same length. It is computed as
+ * \f[
+ * k({\bf x},{\bf x'})= (\sum_{i=0}^L I(x_i=x'_i)+c)^d
+ * \f]
+ *
+ * where I is the indicator function which evaluates to 1 if its argument is
+ * true and to 0 otherwise.
+ *
+ * Note that additional normalisation is applied, i.e.
+ * \f[
+ *     k'({\bf x}, {\bf x'})=\frac{k({\bf x}, {\bf x'})}{\sqrt{k({\bf x}, {\bf x})k({\bf x'}, {\bf x'})}}
+ * \f]
+ */
+class CPolyMatchStringKernel: public CStringKernel<char>
 {
 	public:
 		/** constructor
@@ -23,10 +36,8 @@ class CPolyMatchStringKernel: public CStringKernel<CHAR>
 		 * @param size cache size
 		 * @param degree degree
 		 * @param inhomogene is inhomogeneous
-		 * @param use_normalization use normalization
 		 */
-		CPolyMatchStringKernel(INT size, INT degree, bool inhomogene,
-			bool use_normalization=true);
+		CPolyMatchStringKernel(int32_t size, int32_t degree, bool inhomogene);
 
 		/** constructor
 		 *
@@ -34,11 +45,10 @@ class CPolyMatchStringKernel: public CStringKernel<CHAR>
 		 * @param r features of right-hand side
 		 * @param degree degree
 		 * @param inhomogene is inhomogeneous
-		 * @param use_normalization use normalization
 		 */
 		CPolyMatchStringKernel(
-			CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r,
-			INT degree, bool inhomogene, bool use_normalization=true);
+			CStringFeatures<char>* l, CStringFeatures<char>* r,
+			int32_t degree, bool inhomogene);
 
 		virtual ~CPolyMatchStringKernel();
 
@@ -80,7 +90,7 @@ class CPolyMatchStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return name PolyMatchString
 		 */
-		virtual const CHAR* get_name()
+		virtual const char* get_name()
 		{
 			return "PolyMatchString";
 		}
@@ -94,22 +104,13 @@ class CPolyMatchStringKernel: public CStringKernel<CHAR>
 		 * @param idx_b index b
 		 * @return computed kernel function at indices a,b
 		 */
-		virtual DREAL compute(INT idx_a, INT idx_b);
+		virtual float64_t compute(int32_t idx_a, int32_t idx_b);
 
 	protected:
 		/** degree */
-		INT degree;
+		int32_t degree;
 		/** if kernel is inhomogeneous */
 		bool inhomogene;
-		/** if normalization is used */
-		bool use_normalization;
-
-		/** sqrt diagonal of left-hand side */
-		DREAL *sqrtdiag_lhs;
-		/** sqrt diagonal of right-hand side */
-		DREAL *sqrtdiag_rhs;
-		/** if kernel is initialized */
-		bool initialized;
 };
 
 #endif /* _POLYMATCHSTRINGKERNEL_H___ */
