@@ -15,16 +15,16 @@
 #include "features/StringFeatures.h"
 
 CSimpleLocalityImprovedStringKernel::CSimpleLocalityImprovedStringKernel(
-	INT size, INT l, INT id, INT od)
-: CStringKernel<CHAR>(size), length(l), inner_degree(id), outer_degree(od),
+	int32_t size, int32_t l, int32_t id, int32_t od)
+: CStringKernel<char>(size), length(l), inner_degree(id), outer_degree(od),
 	pyramid_weights(NULL)
 {
 }
 
 CSimpleLocalityImprovedStringKernel::CSimpleLocalityImprovedStringKernel(
-	CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r,
-	INT len, INT id, INT od)
-: CStringKernel<CHAR>(10), length(len), inner_degree(id), outer_degree(od),
+	CStringFeatures<char>* l, CStringFeatures<char>* r,
+	int32_t len, int32_t id, int32_t od)
+: CStringKernel<char>(10), length(len), inner_degree(id), outer_degree(od),
 	pyramid_weights(NULL)
 {
 	init(l, r);
@@ -37,25 +37,25 @@ CSimpleLocalityImprovedStringKernel::~CSimpleLocalityImprovedStringKernel()
 
 bool CSimpleLocalityImprovedStringKernel::init(CFeatures* l, CFeatures* r)
 {
-	bool result = CStringKernel<CHAR>::init(l,r);
+	bool result = CStringKernel<char>::init(l,r);
 
 	if (!result)
 		return false;
-	INT num_features = ((CStringFeatures<CHAR>*) l)->get_max_vector_length();
-	pyramid_weights = new DREAL[num_features];
+	int32_t num_features = ((CStringFeatures<char>*) l)->get_max_vector_length();
+	pyramid_weights = new float64_t[num_features];
 	ASSERT(pyramid_weights);
 	SG_INFO("initializing pyramid weights: size=%ld length=%i\n",
 		num_features, length);
 
-	const INT PYRAL = 2 * length - 1; // total window length
-	DREAL PYRAL_pot;
-	INT DEGREE1_1  = (inner_degree & 0x1)==0;
-	INT DEGREE1_1n = (inner_degree & ~0x1)!=0;
-	INT DEGREE1_2  = (inner_degree & 0x2)!=0;
-	INT DEGREE1_3  = (inner_degree & ~0x3)!=0;
-	INT DEGREE1_4  = (inner_degree & 0x4)!=0;
+	const int32_t PYRAL = 2 * length - 1; // total window length
+	float64_t PYRAL_pot;
+	int32_t DEGREE1_1  = (inner_degree & 0x1)==0;
+	int32_t DEGREE1_1n = (inner_degree & ~0x1)!=0;
+	int32_t DEGREE1_2  = (inner_degree & 0x2)!=0;
+	int32_t DEGREE1_3  = (inner_degree & ~0x3)!=0;
+	int32_t DEGREE1_4  = (inner_degree & 0x4)!=0;
 	{
-	DREAL PYRAL_ = PYRAL;
+	float64_t PYRAL_ = PYRAL;
 	PYRAL_pot = DEGREE1_1 ? 1.0 : PYRAL_;
 	if (DEGREE1_1n)
 	{
@@ -71,16 +71,17 @@ bool CSimpleLocalityImprovedStringKernel::init(CFeatures* l, CFeatures* r)
 	}
 	}
 
-	INT pyra_len  = num_features-PYRAL+1;
-	INT pyra_len2 = (int) pyra_len/2;
+	int32_t pyra_len  = num_features-PYRAL+1;
+	int32_t pyra_len2 = (int32_t) pyra_len/2;
 	{
-	INT j;
+	int32_t j;
 	for (j = 0; j < pyra_len; j++)
-		pyramid_weights[j] = 4*((DREAL)((j < pyra_len2)? j+1 : pyra_len-j))/((DREAL)pyra_len);
+		pyramid_weights[j] = 4*((float64_t)((j < pyra_len2)? j+1 : pyra_len-j))/((float64_t)pyra_len);
 	for (j = 0; j < pyra_len; j++)
 		pyramid_weights[j] /= PYRAL_pot;
 	}
-	return true;
+
+	return init_normalizer();
 }
 
 void CSimpleLocalityImprovedStringKernel::cleanup()
@@ -101,21 +102,21 @@ bool CSimpleLocalityImprovedStringKernel::save_init(FILE* dest)
 	return false;
 }
 
-DREAL CSimpleLocalityImprovedStringKernel::dot_pyr (const CHAR* const x1,
-	     const CHAR* const x2, const INT NOF_NTS, const INT NTWIDTH,
-	     const INT DEGREE1, const INT DEGREE2, DREAL *pyra)
+float64_t CSimpleLocalityImprovedStringKernel::dot_pyr (const char* const x1,
+	     const char* const x2, const int32_t NOF_NTS, const int32_t NTWIDTH,
+	     const int32_t DEGREE1, const int32_t DEGREE2, float64_t *pyra)
 {
-	const INT PYRAL = 2*NTWIDTH-1; // total window length
-	INT pyra_len, pyra_len2;
-	DREAL pot, PYRAL_pot;
-	DREAL sum;
-	INT DEGREE1_1 = (DEGREE1 & 0x1)==0;
-	INT DEGREE1_1n = (DEGREE1 & ~0x1)!=0;
-	INT DEGREE1_2 = (DEGREE1 & 0x2)!=0;
-	INT DEGREE1_3 = (DEGREE1 & ~0x3)!=0;
-	INT DEGREE1_4 = (DEGREE1 & 0x4)!=0;
+	const int32_t PYRAL = 2*NTWIDTH-1; // total window length
+	int32_t pyra_len, pyra_len2;
+	float64_t pot, PYRAL_pot;
+	float64_t sum;
+	int32_t DEGREE1_1 = (DEGREE1 & 0x1)==0;
+	int32_t DEGREE1_1n = (DEGREE1 & ~0x1)!=0;
+	int32_t DEGREE1_2 = (DEGREE1 & 0x2)!=0;
+	int32_t DEGREE1_3 = (DEGREE1 & ~0x3)!=0;
+	int32_t DEGREE1_4 = (DEGREE1 & 0x4)!=0;
 	{
-	DREAL PYRAL_ = PYRAL;
+	float64_t PYRAL_ = PYRAL;
 	PYRAL_pot = DEGREE1_1 ? 1.0 : PYRAL_;
 	if (DEGREE1_1n)
 	{
@@ -133,18 +134,18 @@ DREAL CSimpleLocalityImprovedStringKernel::dot_pyr (const CHAR* const x1,
 	ASSERT((DEGREE2 & ~0x7) == 0);
 
 	pyra_len = NOF_NTS-PYRAL+1;
-	pyra_len2 = (int) pyra_len/2;
+	pyra_len2 = (int32_t) pyra_len/2;
 	{
-	INT j;
+	int32_t j;
 	for (j = 0; j < pyra_len; j++)
-		pyra[j] = 4*((DREAL)((j < pyra_len2) ? j+1 : pyra_len-j))/((DREAL)pyra_len);
+		pyra[j] = 4*((float64_t)((j < pyra_len2) ? j+1 : pyra_len-j))/((float64_t)pyra_len);
 	for (j = 0; j < pyra_len; j++)
 		pyra[j] /= PYRAL_pot;
 	}
 
-	register INT conv;
-	register INT i;
-	register INT j;
+	register int32_t conv;
+	register int32_t i;
+	register int32_t j;
 
 	sum = 0.0;
 	conv = 0;
@@ -153,12 +154,12 @@ DREAL CSimpleLocalityImprovedStringKernel::dot_pyr (const CHAR* const x1,
 
 	for (i = 0; i < NOF_NTS-PYRAL+1; i++)
 	{
-		register DREAL pot2;
+		register float64_t pot2;
 		if (i>0)
 			conv += ((x1[i+PYRAL-1] == x2[i+PYRAL-1]) ? 1 : 0 ) - 
 				((x1[i-1] == x2[i-1]) ? 1 : 0);
-		{ /* potencing of conv -- double is faster*/
-		register DREAL conv2 = conv;
+		{ /* potencing of conv -- float64_t is faster*/
+		register float64_t conv2 = conv;
 		pot2 = (DEGREE1_1) ? 1.0 : conv2;
 			if (DEGREE1_1n)
 			{
@@ -188,19 +189,20 @@ DREAL CSimpleLocalityImprovedStringKernel::dot_pyr (const CHAR* const x1,
 	return pot;
 }
 
-DREAL CSimpleLocalityImprovedStringKernel::compute(INT idx_a, INT idx_b)
+float64_t CSimpleLocalityImprovedStringKernel::compute(
+	int32_t idx_a, int32_t idx_b)
 {
-	INT alen, blen;
+	int32_t alen, blen;
 
-	CHAR* avec = ((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx_a, alen);
-	CHAR* bvec = ((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx_b, blen);
+	char* avec = ((CStringFeatures<char>*) lhs)->get_feature_vector(idx_a, alen);
+	char* bvec = ((CStringFeatures<char>*) rhs)->get_feature_vector(idx_b, blen);
 
 	// can only deal with strings of same length
 	ASSERT(alen==blen);
 
-	DREAL dpt;
+	float64_t dpt;
 
 	dpt = dot_pyr(avec, bvec, alen, length, inner_degree, outer_degree, pyramid_weights);
-	dpt = dpt / pow((double)alen, (double)outer_degree);
-	return (DREAL) dpt;
+	dpt = dpt / pow((float64_t)alen, (float64_t)outer_degree);
+	return (float64_t) dpt;
 }
