@@ -14,8 +14,17 @@
 #include "lib/common.h"
 #include "kernel/StringKernel.h"
 
-/** kernel SimpleLocalityImprovedString */
-class CSimpleLocalityImprovedStringKernel: public CStringKernel<CHAR>
+/** SimpleLocalityImprovedString kernel, is a ``simplified'' and better
+ * performing version of the Locality improved kernel.
+ *
+ * It can be defined as
+ * \f[
+ * K({\bf x},{\bf x'})=\left(\sum_{i=0}^{T-1}\left(\sum_{j=-l}^{+l}w_jI_{i+j}({\bf x},{\bf x'})\right)^{d_1}\right)^{d_2},
+ * \f]
+ * where
+ * \f$ I_i({\bf x},{\bf x'})=1\f$ if \f$x_i=x'_i\f$ and 0 otherwise.
+ */
+class CSimpleLocalityImprovedStringKernel: public CStringKernel<char>
 {
 	public:
 		/** constructor
@@ -25,8 +34,8 @@ class CSimpleLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 * @param inner_degree inner degree
 		 * @param outer_degree outer degree
 		 */
-		CSimpleLocalityImprovedStringKernel(int size, INT length,
-			INT inner_degree, INT outer_degree);
+		CSimpleLocalityImprovedStringKernel(int32_t size, int32_t length,
+			int32_t inner_degree, int32_t outer_degree);
 
 		/** constructor
 		 *
@@ -37,8 +46,8 @@ class CSimpleLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 * @param outer_degree outer degree
 		 */
 		CSimpleLocalityImprovedStringKernel(
-			CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r,
-			INT length, INT inner_degree, INT outer_degree);
+			CStringFeatures<char>* l, CStringFeatures<char>* r,
+			int32_t length, int32_t inner_degree, int32_t outer_degree);
 
 		virtual ~CSimpleLocalityImprovedStringKernel();
 
@@ -80,7 +89,7 @@ class CSimpleLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return name SimpleLocalityImproved
 		 */
-		virtual const CHAR *get_name()
+		virtual const char *get_name()
 		{
 			return "SimpleLocalityImproved";
 		}
@@ -97,9 +106,9 @@ class CSimpleLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 * @param pyra pyramid
 		 * @return dot product of pyramid (?)
 		 */
-		DREAL dot_pyr (const CHAR* const x1, const CHAR* const x2,
-				const INT NOF_NTS, const INT NTWIDTH,
-				const INT DEGREE1, const INT DEGREE2, DREAL *pyra);
+		float64_t dot_pyr (const char* const x1, const char* const x2,
+				const int32_t NOF_NTS, const int32_t NTWIDTH,
+				const int32_t DEGREE1, const int32_t DEGREE2, float64_t *pyra);
 
 	protected:
 		/** compute kernel function for features a and b
@@ -110,17 +119,17 @@ class CSimpleLocalityImprovedStringKernel: public CStringKernel<CHAR>
 		 * @param idx_b index b
 		 * @return computed kernel function at indices a,b
 		 */
-		DREAL compute(INT idx_a, INT idx_b);
+		float64_t compute(int32_t idx_a, int32_t idx_b);
 
 	protected:
 		/** length */
-		INT length;
+		int32_t length;
 		/** inner degree */
-		INT inner_degree;
+		int32_t inner_degree;
 		/** outer degree */
-		INT outer_degree;
+		int32_t outer_degree;
 		/** pyramid weights */
-		DREAL *pyramid_weights;
+		float64_t *pyramid_weights;
 };
 
 #endif /* _SIMPLELOCALITYIMPROVEDSTRINGKERNEL_H___ */
