@@ -18,7 +18,32 @@
 
 class CCommWordStringKernel;
 
-/** kernel WeightedCommWordString */
+/** The WeightedCommWordString kernel may be used to compute the weighted
+ * spectrum kernel (i.e. a spectrum kernel for 1 to K-mers, where each k-mer
+ * length is weighted by some coefficient \f$\beta_k\f$)
+ * from strings that have been mapped into unsigned 16bit integers. These 16bit
+ * integers correspond to k-mers. To applicable in this kernel they need to be
+ * sorted (e.g. via the SortWordString pre-processor).
+ *
+ * It basically uses the algorithm in the unix "comm" command (hence the name)
+ * to compute:
+ *
+ * \f[
+ * k({\bf x},({\bf x'})= \sum_{k=1}^K\beta_k\Phi_k({\bf x})\cdot \Phi_k({\bf x'})
+ * \f]
+ *
+ * where \f$\Phi_k\f$ maps a sequence \f${\bf x}\f$ that consists of letters in
+ * \f$\Sigma\f$ to a feature vector of size \f$|\Sigma|^k\f$. In this feature
+ * vector each entry denotes how often the k-mer appears in that \f${\bf x}\f$.
+ *
+ * Note that this representation is especially tuned to small alphabets
+ * (like the 2-bit alphabet DNA), for which it enables spectrum kernels
+ * of order 8.
+ *
+ * For this kernel the linadd speedups are quite efficiently implemented using
+ * direct maps.
+ *
+ */
 class CWeightedCommWordStringKernel: public CCommWordStringKernel
 {
 	public:
@@ -26,24 +51,19 @@ class CWeightedCommWordStringKernel: public CCommWordStringKernel
 		 *
 		 * @param size cache size
 		 * @param use_sign if sign shall be used
-		 * @param normalization_ type of normalization
 		 */
-		CWeightedCommWordStringKernel(INT size, bool use_sign,
-			ENormalizationType normalization_=FULL_NORMALIZATION);
+		CWeightedCommWordStringKernel(int32_t size, bool use_sign);
 
 		/** constructor
 		 *
 		 * @param l features of left-hand side
 		 * @param r features of right-hand side
 		 * @param use_sign if sign shall be used
-		 * @param normalization_ type of normalization
 		 * @param size cache size
 		 */
 		CWeightedCommWordStringKernel(
-			CStringFeatures<WORD>* l, CStringFeatures<WORD>* r,
-			bool use_sign=false,
-			ENormalizationType normalization_=FULL_NORMALIZATION,
-			INT size=10);
+			CStringFeatures<uint16_t>* l, CStringFeatures<uint16_t>* r,
+			bool use_sign=false, int32_t size=10);
 
 		virtual ~CWeightedCommWordStringKernel();
 
@@ -63,14 +83,14 @@ class CWeightedCommWordStringKernel: public CCommWordStringKernel
 	 	* @param idx index to compute
 	 	* @return optimized value at given index
 	 	*/
-		virtual DREAL compute_optimized(INT idx);
+		virtual float64_t compute_optimized(int32_t idx);
 
 		/** add to normal
 		 *
 		 * @param idx where to add
 		 * @param weight what to add
 		 */
-		virtual void add_to_normal(INT idx, DREAL weight);
+		virtual void add_to_normal(int32_t idx, float64_t weight);
 
 		/** merge normal */
 		void merge_normal();
@@ -87,7 +107,7 @@ class CWeightedCommWordStringKernel: public CCommWordStringKernel
 		 * @param d degree (must match number of weights)
 		 * @return if setting was successful
 		 */
-		bool set_weights(DREAL* w, INT d);
+		bool set_weights(float64_t* w, int32_t d);
 
 		/** return what type of kernel we are
 		 *
@@ -99,7 +119,7 @@ class CWeightedCommWordStringKernel: public CCommWordStringKernel
 		 *
 		 * @return name WeightedCommWordString
 		 */
-		virtual const CHAR* get_name() { return "WeightedCommWordString"; }
+		virtual const char* get_name() { return "WeightedCommWordString"; }
 
 		/** return feature type the kernel can deal with
 		 *
@@ -119,9 +139,10 @@ class CWeightedCommWordStringKernel: public CCommWordStringKernel
 		 * @param do_init if initialization shall be performed
 		 * @return computed score
 		 */
-		virtual DREAL* compute_scoring(INT max_degree, INT& num_feat,
-			INT& num_sym, DREAL* target, INT num_suppvec, INT* IDX,
-			DREAL* alphas, bool do_init=true);
+		virtual float64_t* compute_scoring(
+			int32_t max_degree, int32_t& num_feat, int32_t& num_sym,
+			float64_t* target, int32_t num_suppvec, int32_t* IDX,
+			float64_t* alphas, bool do_init=true);
 
 	protected:
 		/** helper for compute
@@ -130,13 +151,14 @@ class CWeightedCommWordStringKernel: public CCommWordStringKernel
 		 * @param idx_b index b
 		 * @param do_sort if sorting shall be performed
 		 */
-		virtual DREAL compute_helper(INT idx_a, INT idx_b, bool do_sort);
+		virtual float64_t compute_helper(
+			int32_t idx_a, int32_t idx_b, bool do_sort);
 
 		/** degree */
-		INT degree;
+		int32_t degree;
 
 		/** weights for each of the subkernels of degree 1...d */
-		DREAL* weights;
+		float64_t* weights;
 };
 
 #endif /* _WEIGHTEDCOMMWORDSTRINGKERNEL_H__ */
