@@ -14,12 +14,33 @@
 
 #include "lib/common.h"
 #include "kernel/StringKernel.h"
+#include "kernel/WeightedDegreeStringKernel.h"
 #include "lib/Trie.h"
 
 class CSVM ;
 
-/** kernel WeightedDegreePositionString */
-class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
+/** The WeightedDegreePositionString kernel (Weighted Degree kernel with shifts)
+ *  of order d compares two sequences \f${\bf x}\f$ and \f${\bf x'}\f$ of length
+ *  L by summing all contributions of k-mer matches of lengths
+ *  \f$k\in\{1,\dots,d\}\f$, weighted by coefficients \f$\beta_k\f$ allowing for
+ *  a positional tolerance of up to shift s.
+ *
+ *  It is formally defined as
+ * \f{eqnarray*}
+ * &&\!\!\!\!\!\!\!k({\bf x}_i,{\bf x}_j)=\sum_{k=1}^d\beta_k\sum_{l=1}^{\!\!\!\!L-k+1\!\!\!\!}\gamma_l\sum_{\begin{array}{c}s=0\\
+ *   \!\!\!\!s+l\leq L\!\!\!\!\end{array}}^{S(l)}
+ *   \delta_s\;\mu_{k,l,s,{{\bf x}_i},{{\bf x}_j}},\\
+ *   &&\!\!\!\!\!\!\!\!\!\! {\footnotesize \mu_{k,l,s,{{\bf x}_i},{{\bf x}_j}}\!\!\! =\!\!
+ *   I({\bf u}_{k,l+s}({\bf x}_i)\! =\!{\bf u}_{k,l}({\bf x}_j))\! +\!I({\bf u}_{k,l}({\bf x}_i)\!
+ *   =\!{\bf u}_{k,l+s}({\bf x}_j))},\nonumber
+ *   \f}
+ *   where \f$\beta_j\f$ are the weighting coefficients of the j-mers,
+ *   \f$\gamma_l\f$ is a weighting over the
+ *   position in the sequence, \f$\delta_s=1/(2(s+1))\f$ is the weight assigned
+ *   to shifts (in either direction) of extent s, and S(l) determines
+ *   the shift range at position l.
+ */
+class CWeightedDegreePositionStringKernel: public CStringKernel<char>
 {
 	public:
 		/** constructor
@@ -27,11 +48,11 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param size cache size
 		 * @param degree degree
 		 * @param max_mismatch maximum mismatch
-		 * @param use_norm if normalization shall be used
 		 * @param mkl_stepsize MKL stepsize
 		 */
-		CWeightedDegreePositionStringKernel(INT size, INT degree,
-			INT max_mismatch=0, bool use_norm=true, INT mkl_stepsize=1);
+		CWeightedDegreePositionStringKernel(
+			int32_t size, int32_t degree,
+			int32_t max_mismatch=0, int32_t mkl_stepsize=1);
 
 		/** constructor
 		 *
@@ -41,12 +62,12 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param max_mismatch maximum mismatch
 		 * @param shift position shifts
 		 * @param shift_len number of shifts
-		 * @param use_norm if normalization shall be used
 		 * @param mkl_stepsize MKL stepsize
 		 */
-		CWeightedDegreePositionStringKernel(INT size, DREAL* weights,
-			INT degree, INT max_mismatch, INT* shift, INT shift_len,
-			bool use_norm=true, INT mkl_stepsize=1);
+		CWeightedDegreePositionStringKernel(
+			int32_t size, float64_t* weights, int32_t degree,
+			int32_t max_mismatch, int32_t* shift, int32_t shift_len,
+			int32_t mkl_stepsize=1);
 
 		/** constructor
 		 *
@@ -55,8 +76,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param degree degree
 		 */
 		CWeightedDegreePositionStringKernel(
-			CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r,
-			INT degree);
+			CStringFeatures<char>* l, CStringFeatures<char>* r, int32_t degree);
 
 		virtual ~CWeightedDegreePositionStringKernel();
 
@@ -95,7 +115,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return name WeightedDegreePos
 		 */
-		virtual const CHAR* get_name() { return "WeightedDegreePos" ; } ;
+		virtual const char* get_name() { return "WeightedDegreePos"; }
 
 		/** initialize optimization
 		 *
@@ -104,7 +124,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param alphas alphas
 		 * @return if initializing was successful
 		 */
-		inline virtual bool init_optimization(INT p_count, INT *IDX, DREAL * alphas)
+		inline virtual bool init_optimization(
+			int32_t p_count, int32_t *IDX, float64_t * alphas)
 		{ 
 			return init_optimization(p_count, IDX, alphas, -1);
 		}
@@ -120,8 +141,9 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param upto_tree up to this tree
 		 * @return if initializing was successful
 		 */
-		virtual bool init_optimization(INT count, INT *IDX, DREAL * alphas,
-			INT tree_num, INT upto_tree=-1);
+		virtual bool init_optimization(
+			int32_t count, int32_t *IDX, float64_t * alphas, int32_t tree_num,
+			int32_t upto_tree=-1);
 
 		/** delete optimization
 		 *
@@ -134,7 +156,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 	 	* @param idx index to compute
 	 	* @return optimized value at given index
 	 	*/
-		inline virtual DREAL compute_optimized(INT idx)
+		inline virtual float64_t compute_optimized(int32_t idx)
 		{ 
 			ASSERT(get_is_initialized());
 			ASSERT(alphabet);
@@ -158,8 +180,10 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param alphas alphas
 		 * @param factor factor
 		 */
-		virtual void compute_batch(INT num_vec, INT* vec_idx, DREAL* target,
-			INT num_suppvec, INT* IDX, DREAL* alphas, DREAL factor=1.0);
+		virtual void compute_batch(
+			int32_t num_vec, int32_t* vec_idx, float64_t* target,
+			int32_t num_suppvec, int32_t* IDX, float64_t* alphas,
+			float64_t factor=1.0);
 
 		/** clear normal
 		 * subkernel functionality
@@ -190,7 +214,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param idx where to add
 		 * @param weight what to add
 		 */
-		inline virtual void add_to_normal(INT idx, DREAL weight)
+		inline virtual void add_to_normal(int32_t idx, float64_t weight)
 		{
 			add_example_to_tree(idx, weight);
 			set_is_initialized(true);
@@ -200,13 +224,13 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return number of subkernels
 		 */
-		inline virtual INT get_num_subkernels()
+		inline virtual int32_t get_num_subkernels()
 		{
 			if (position_weights!=NULL)
-				return (INT) ceil(1.0*seq_length/mkl_stepsize) ;
+				return (int32_t) ceil(1.0*seq_length/mkl_stepsize) ;
 			if (length==0)
-				return (INT) ceil(1.0*get_degree()/mkl_stepsize);
-			return (INT) ceil(1.0*get_degree()*length/mkl_stepsize) ;
+				return (int32_t) ceil(1.0*get_degree()/mkl_stepsize);
+			return (int32_t) ceil(1.0*get_degree()*length/mkl_stepsize) ;
 		}
 
 		/** compute by subkernel
@@ -214,7 +238,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param idx index
 		 * @param subkernel_contrib subkernel contribution
 		 */
-		inline void compute_by_subkernel(INT idx, DREAL * subkernel_contrib)
+		inline void compute_by_subkernel(
+			int32_t idx, float64_t * subkernel_contrib)
 		{ 
 			if (get_is_initialized())
 			{
@@ -230,18 +255,18 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param num_weights number of weights will be stored here
 		 * @return subkernel weights
 		 */
-		inline const DREAL* get_subkernel_weights(INT& num_weights)
+		inline const float64_t* get_subkernel_weights(int32_t& num_weights)
 		{
 			num_weights = get_num_subkernels() ;
 
 			delete[] weights_buffer ;
-			weights_buffer = new DREAL[num_weights] ;
+			weights_buffer = new float64_t[num_weights] ;
 
 			if (position_weights!=NULL)
-				for (INT i=0; i<num_weights; i++)
+				for (int32_t i=0; i<num_weights; i++)
 					weights_buffer[i] = position_weights[i*mkl_stepsize] ;
 			else
-				for (INT i=0; i<num_weights; i++)
+				for (int32_t i=0; i<num_weights; i++)
 					weights_buffer[i] = weights[i*mkl_stepsize] ;
 
 			return weights_buffer ;
@@ -252,30 +277,31 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param weights2 weights
 		 * @param num_weights2 number of weights
 		 */
-		inline void set_subkernel_weights(DREAL* weights2, INT num_weights2)
+		inline void set_subkernel_weights(
+			float64_t* weights2, int32_t num_weights2)
 		{
-			INT num_weights = get_num_subkernels() ;
+			int32_t num_weights = get_num_subkernels() ;
 			if (num_weights!=num_weights2)
 				SG_ERROR( "number of weights do not match\n") ;
 
 			if (position_weights!=NULL)
-				for (INT i=0; i<num_weights; i++)
-					for (INT j=0; j<mkl_stepsize; j++)
+				for (int32_t i=0; i<num_weights; i++)
+					for (int32_t j=0; j<mkl_stepsize; j++)
 					{
 						if (i*mkl_stepsize+j<seq_length)
 							position_weights[i*mkl_stepsize+j] = weights2[i] ;
 					}
 			else if (length==0)
 			{
-				for (INT i=0; i<num_weights; i++)
-					for (INT j=0; j<mkl_stepsize; j++)
+				for (int32_t i=0; i<num_weights; i++)
+					for (int32_t j=0; j<mkl_stepsize; j++)
 						if (i*mkl_stepsize+j<get_degree())
 							weights[i*mkl_stepsize+j] = weights2[i] ;
 			}
 			else
 			{
-				for (INT i=0; i<num_weights; i++)
-					for (INT j=0; j<mkl_stepsize; j++)
+				for (int32_t i=0; i<num_weights; i++)
+					for (int32_t j=0; j<mkl_stepsize; j++)
 						if (i*mkl_stepsize+j<get_degree()*length)
 							weights[i*mkl_stepsize+j] = weights2[i] ;
 			}
@@ -287,7 +313,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param len len
 		 * @return computed abs weights
 		 */
-		DREAL *compute_abs_weights(INT & len);
+		float64_t* compute_abs_weights(int32_t & len);
 
 		/** check if tree is initialized
 		 *
@@ -299,26 +325,20 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return maximum mismatch
 		 */
-		inline INT get_max_mismatch() { return max_mismatch; }
+		inline int32_t get_max_mismatch() { return max_mismatch; }
 
 		/** get degree
 		 *
 		 * @return the degree
 		 */
-		inline INT get_degree() { return degree; }
-
-		/** get normalization constant
-		 *
-		 * @return normalization constant
-		 */
-		inline DREAL get_normalization_const() { return normalization_const; }
+		inline int32_t get_degree() { return degree; }
 
 		/** get degree weights
 		 *
 		 * @param d degree weights will be stored here
 		 * @param len number of degree weights will be stored here
 		 */
-		inline DREAL *get_degree_weights(INT& d, INT& len)
+		inline float64_t *get_degree_weights(int32_t& d, int32_t& len)
 		{
 			d=degree;
 			len=length;
@@ -330,7 +350,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param num_weights number of weights will be stored here
 		 * @return weights
 		 */
-		inline DREAL *get_weights(INT& num_weights)
+		inline float64_t *get_weights(int32_t& num_weights)
 		{
 			if (position_weights!=NULL)
 			{
@@ -349,7 +369,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param len number of position weights will be stored here
 		 * @return position weights
 		 */
-		inline DREAL *get_position_weights(INT& len)
+		inline float64_t *get_position_weights(int32_t& len)
 		{
 			len=seq_length;
 			return position_weights;
@@ -360,7 +380,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param shifts new shifts
 		 * @param len number of shifts
 		 */
-		bool set_shifts(INT* shifts, INT len);
+		bool set_shifts(int32_t* shifts, int32_t len);
 
 		/** set weights
 		 *
@@ -368,7 +388,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param d degree
 		 * @param len number of weights
 		 */
-		virtual bool set_weights(DREAL* weights, INT d, INT len=0);
+		virtual bool set_weights(float64_t* weights, int32_t d, int32_t len=0);
 
 		/** set wd weights
 		 *
@@ -382,7 +402,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param len number of position weights
 		 * @return if setting was successful
 		 */
-		virtual bool set_position_weights(DREAL* position_weights, INT len=0);
+		virtual bool set_position_weights(
+			float64_t* position_weights, int32_t len=0);
 
 		/** set position weights for left-hand side
 		 *
@@ -391,7 +412,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param num num
 		 * @return if setting was successful
 		 */
-		bool set_position_weights_lhs(DREAL* pws, INT len, INT num);
+		bool set_position_weights_lhs(float64_t* pws, int32_t len, int32_t num);
 
 		/** set position weights for right-hand side
 		 *
@@ -400,7 +421,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param num num
 		 * @return if setting was successful
 		 */
-		bool set_position_weights_rhs(DREAL* pws, INT len, INT num);
+		bool set_position_weights_rhs(float64_t* pws, int32_t len, int32_t num);
 
 		/** initialize block weights
 		 *
@@ -466,39 +487,48 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return if deleting was successful
 		 */
-		bool delete_position_weights() { delete[] position_weights ; position_weights=NULL ; return true ; } ;
+		bool delete_position_weights()
+		{
+			delete[] position_weights;
+			position_weights=NULL;
+			return true;
+		}
 
 		/** delete position weights left-hand side
 		 *
 		 * @return if deleting was successful
 		 */
-		bool delete_position_weights_lhs() { delete[] position_weights_lhs ; position_weights_lhs=NULL ; return true ; } ;
+		bool delete_position_weights_lhs()
+		{
+			delete[] position_weights_lhs;
+			position_weights_lhs=NULL;
+			return true;
+		}
 
 		/** delete position weights right-hand side
 		 *
 		 * @return if deleting was successful
 		 */
-		bool delete_position_weights_rhs() { delete[] position_weights_rhs ; position_weights_rhs=NULL ; return true ; } ;
-
-		/** check if normalization is used
-		 *
-		 * @return if normalization is used
-		 */
-		inline bool get_use_normalization() { return use_normalization; }
+		bool delete_position_weights_rhs()
+		{
+			delete[] position_weights_rhs;
+			position_weights_rhs=NULL;
+			return true;
+		}
 
 		/** compute by tree
 		 *
 		 * @param idx index
 		 * @return computed value
 		 */
-		virtual DREAL compute_by_tree(INT idx);
+		virtual float64_t compute_by_tree(int32_t idx);
 
 		/** compute by tree
 		 *
 		 * @param idx index
 		 * @param LevelContrib level contribution
 		 */
-		virtual void compute_by_tree(INT idx, DREAL* LevelContrib);
+		virtual void compute_by_tree(int32_t idx, float64_t* LevelContrib);
 
 		/** compute positional scoring function, which assigns a
 		 * weight per position, per symbol in the sequence
@@ -512,8 +542,10 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param weights weights
 		 * @return computed scores
 		 */
-		DREAL* compute_scoring(INT max_degree, INT& num_feat, INT& num_sym,
-			DREAL* target, INT num_suppvec, INT* IDX, DREAL* weights);
+		float64_t* compute_scoring(
+			int32_t max_degree, int32_t& num_feat, int32_t& num_sym,
+			float64_t* target, int32_t num_suppvec, int32_t* IDX,
+			float64_t* weights);
 
 		/** compute consensus string
 		 *
@@ -523,8 +555,9 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param alphas alphas
 		 * @return consensus string
 		 */
-		CHAR* compute_consensus(INT &num_feat, INT num_suppvec, INT* IDX,
-			DREAL* alphas);
+		char* compute_consensus(
+			int32_t &num_feat, int32_t num_suppvec, int32_t* IDX,
+			float64_t* alphas);
 
 		/** extract w
 		 *
@@ -537,8 +570,10 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param alphas alphas
 		 * @return w
 		 */
-		DREAL* extract_w( INT max_degree, INT& num_feat, INT& num_sym,
-			DREAL* w_result, INT num_suppvec, INT* IDX, DREAL* alphas);
+		float64_t* extract_w(
+			int32_t max_degree, int32_t& num_feat, int32_t& num_sym,
+			float64_t* w_result, int32_t num_suppvec, int32_t* IDX,
+			float64_t* alphas);
 
 		/** compute POIM
 		 *
@@ -552,8 +587,10 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param distrib distribution
 		 * @return computed POIMs
 		 */
-		DREAL* compute_POIM( INT max_degree, INT& num_feat, INT& num_sym,
-			DREAL* poim_result, INT num_suppvec, INT* IDX, DREAL* alphas, DREAL* distrib );
+		float64_t* compute_POIM(
+			int32_t max_degree, int32_t& num_feat, int32_t& num_sym,
+			float64_t* poim_result, int32_t num_suppvec, int32_t* IDX,
+			float64_t* alphas, float64_t* distrib);
 
 		/** prepare POIM2
 		 *
@@ -561,7 +598,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param num_sym number of symbols
 		 * @param distrib distribution
 		 */
-		void prepare_POIM2(DREAL* distrib, INT num_sym, INT num_feat);		
+		void prepare_POIM2(
+			float64_t* distrib, int32_t num_sym, int32_t num_feat);
 
 		/** compute POIM2
 		 *
@@ -569,14 +607,14 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param svm SVM
 		 */
 
-		void compute_POIM2(INT max_degree, CSVM* svm);
+		void compute_POIM2(int32_t max_degree, CSVM* svm);
 
 		/** get POIM2
 		 *
 		 * @param poim POIMs (returned)
 		 * @param result_len (returned)
 		 */
-		void get_POIM2(DREAL** poim, INT* result_len);
+		void get_POIM2(float64_t** poim, int32_t* result_len);
 
 		/// cleanup POIM2
 		void cleanup_POIM2();
@@ -590,7 +628,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param idx index
 		 * @param weight weight
 		 */
-		virtual void add_example_to_tree(INT idx, DREAL weight);
+		virtual void add_example_to_tree(
+			int32_t idx, float64_t weight);
 
 		/** add example to single tree
 		 *
@@ -598,7 +637,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param weight weight
 		 * @param tree_num which tree
 		 */
-		void add_example_to_single_tree(INT idx, DREAL weight, INT tree_num);
+		void add_example_to_single_tree(
+			int32_t idx, float64_t weight, int32_t tree_num);
 
 		/** compute kernel function for features a and b
 		 * idx_{a,b} denote the index of the feature vectors
@@ -608,7 +648,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param idx_b index b
 		 * @return computed kernel function at indices a,b
 		 */
-		virtual DREAL compute(INT idx_a, INT idx_b);
+		virtual float64_t compute(int32_t idx_a, int32_t idx_b);
 
 		/** compute with mismatch
 		 *
@@ -618,8 +658,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_with_mismatch(CHAR* avec, INT alen,
-			CHAR* bvec, INT blen);
+		float64_t compute_with_mismatch(
+			char* avec, int32_t alen, char* bvec, int32_t blen);
 
 		/** compute without mismatch
 		 *
@@ -629,8 +669,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_without_mismatch(CHAR* avec, INT alen,
-			CHAR* bvec, INT blen);
+		float64_t compute_without_mismatch(
+			char* avec, int32_t alen, char* bvec, int32_t blen);
 
 		/** compute without mismatch matrix
 		 *
@@ -640,8 +680,8 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_without_mismatch_matrix(CHAR* avec, INT alen,
-			CHAR* bvec, INT blen);
+		float64_t compute_without_mismatch_matrix(
+			char* avec, int32_t alen, char* bvec, int32_t blen);
 
 		/** compute without mismatch position weights
 		 *
@@ -653,68 +693,61 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_without_mismatch_position_weights(
-			CHAR* avec, DREAL *posweights_lhs, INT alen,
-			CHAR* bvec, DREAL *posweights_rhs, INT blen);
+		float64_t compute_without_mismatch_position_weights(
+			char* avec, float64_t *posweights_lhs, int32_t alen,
+			char* bvec, float64_t *posweights_rhs, int32_t blen);
 
 		/** remove lhs from kernel */
 		virtual void remove_lhs();
 
 	protected:
 		/** weights */
-		DREAL* weights;
+		float64_t* weights;
 		/** position weights */
-		DREAL* position_weights;
+		float64_t* position_weights;
 		/** position weights left-hand side */
-		DREAL* position_weights_lhs;
+		float64_t* position_weights_lhs;
 		/** position weights right-hand side */
-		DREAL* position_weights_rhs;
+		float64_t* position_weights_rhs;
 		/** position mask */
 		bool* position_mask;
 
 		/** weights buffer */
-		DREAL* weights_buffer;
+		float64_t* weights_buffer;
 		/** MKL stepsize */
-		INT mkl_stepsize;
+		int32_t mkl_stepsize;
 
 		/** degree */
-		INT degree;
+		int32_t degree;
 		/** length */
-		INT length;
+		int32_t length;
 
 		/** maximum mismatch */
-		INT max_mismatch;
+		int32_t max_mismatch;
 		/** length of sequence */
-		INT seq_length;
+		int32_t seq_length;
 
 		/** shifts */
-		INT *shift;
+		int32_t *shift;
 		/** length of shifts */
-		INT shift_len;
+		int32_t shift_len;
 		/** maximum shift */
-		INT max_shift;
+		int32_t max_shift;
 
-		/** if kernel is initialized */
-		bool initialized;
-		/** if normalization is used */
-		bool use_normalization;
 		/** if block computation is used */
 		bool block_computation;
 
-		/** normalization constant */
-		DREAL normalization_const;
-
 		/** number of external block weights */
-		INT num_block_weights_external;
+		int32_t num_block_weights_external;
 		/** external block weights */
-		DREAL* block_weights_external;
+		float64_t* block_weights_external;
 
 		/** (internal) block weights */
-		DREAL* block_weights;
+		float64_t* block_weights;
 		/** WeightedDegree kernel type */
 		EWDKernType type;
 		/** which degree */
-		INT which_degree;
+		int32_t which_degree;
 
 		/** tries */
 		CTrie<DNATrie> tries;
@@ -727,16 +760,16 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<CHAR>
 		bool use_poim_tries;
 
 		/** temporary memory for the interface to the poim functions */ 
-		DREAL* m_poim_distrib;
+		float64_t* m_poim_distrib;
 		/** temporary memory for the interface to the poim functions */ 
-		DREAL* m_poim;
+		float64_t* m_poim;
 
 		/** number of symbols */
-		INT m_poim_num_sym;
+		int32_t m_poim_num_sym;
 		/** length of string (==num_feat) */
-		INT m_poim_num_feat;
+		int32_t m_poim_num_feat;
 		/** total size of poim array */
-		INT m_poim_result_len;
+		int32_t m_poim_result_len;
 
 		/** alphabet of features */
 		CAlphabet* alphabet;
