@@ -16,6 +16,7 @@
 #include "base/Parallel.h"
 
 #include "kernel/WeightedDegreeStringKernel.h"
+#include "kernel/FirstElementKernelNormalizer.h"
 #include "features/Features.h"
 #include "features/StringFeatures.h"
 
@@ -25,28 +26,27 @@
 
 struct S_THREAD_PARAM
 {
-	INT* vec;
-	DREAL* result;
-	DREAL* weights;
+	int32_t* vec;
+	float64_t* result;
+	float64_t* weights;
 	CWeightedDegreeStringKernel* kernel;
 	CTrie<DNATrie>* tries;
-	DREAL factor;
-	INT j;
-	INT start;
-	INT end;
-	INT length;
-	INT* vec_idx;
+	float64_t factor;
+	int32_t j;
+	int32_t start;
+	int32_t end;
+	int32_t length;
+	int32_t* vec_idx;
 };
 
 CWeightedDegreeStringKernel::CWeightedDegreeStringKernel (
-	INT degree_, EWDKernType type_)
-: CStringKernel<CHAR>(10),weights(NULL),position_weights(NULL),
+	int32_t degree_, EWDKernType type_)
+: CStringKernel<char>(10),weights(NULL),position_weights(NULL),
 	weights_buffer(NULL), mkl_stepsize(1),degree(degree_), length(0),
-	max_mismatch(0), seq_length(0), initialized(false),
-	use_normalization(true),block_computation(true),
-	normalization_const(1.0), num_block_weights_external(0),
-	block_weights_external(NULL), block_weights(NULL), type(type_),
-	which_degree(-1), tries(NULL), tree_initialized(false), alphabet(NULL)
+	max_mismatch(0), seq_length(0), block_computation(true),
+	num_block_weights_external(0), block_weights_external(NULL),
+	block_weights(NULL), type(type_), which_degree(-1), tries(NULL),
+	tree_initialized(false), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	lhs=NULL;
@@ -54,40 +54,42 @@ CWeightedDegreeStringKernel::CWeightedDegreeStringKernel (
 
 	if (type!=E_EXTERNAL)
 		set_wd_weights_by_type(type);
+
+	set_normalizer(new CFirstElementKernelNormalizer());
 }
 
 CWeightedDegreeStringKernel::CWeightedDegreeStringKernel (
-	DREAL *weights_, INT degree_)
-: CStringKernel<CHAR>(10), weights(NULL), position_weights(NULL),
+	float64_t *weights_, int32_t degree_)
+: CStringKernel<char>(10), weights(NULL), position_weights(NULL),
 	weights_buffer(NULL), mkl_stepsize(1), degree(degree_), length(0),
-	max_mismatch(0), seq_length(0), initialized(false),
-	use_normalization(true), block_computation(true),
-	normalization_const(1.0), num_block_weights_external(0),
-	block_weights_external(NULL), block_weights(NULL), type(E_EXTERNAL),
-	which_degree(-1), tries(NULL), tree_initialized(false), alphabet(NULL)
+	max_mismatch(0), seq_length(0), block_computation(true),
+	num_block_weights_external(0), block_weights_external(NULL),
+	block_weights(NULL), type(E_EXTERNAL), which_degree(-1), tries(NULL),
+	tree_initialized(false), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	lhs=NULL;
 	rhs=NULL;
 
-	weights=new DREAL[degree*(1+max_mismatch)];
-	for (INT i=0; i<degree*(1+max_mismatch); i++)
+	weights=new float64_t[degree*(1+max_mismatch)];
+	for (int32_t i=0; i<degree*(1+max_mismatch); i++)
 		weights[i]=weights_[i];
+	set_normalizer(new CFirstElementKernelNormalizer());
 }
 
 CWeightedDegreeStringKernel::CWeightedDegreeStringKernel(
-	CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r, INT degree_)
-: CStringKernel<CHAR>(10), weights(NULL), position_weights(NULL),
+	CStringFeatures<char>* l, CStringFeatures<char>* r, int32_t degree_)
+: CStringKernel<char>(10), weights(NULL), position_weights(NULL),
 	weights_buffer(NULL), mkl_stepsize(1), degree(degree_), length(0),
-	max_mismatch(0), seq_length(0), initialized(false),
-	use_normalization(true), block_computation(true),
-	normalization_const(1.0), num_block_weights_external(0),
-	block_weights_external(NULL), block_weights(NULL), type(E_WD),
-	which_degree(-1), tries(NULL), tree_initialized(false), alphabet(NULL)
+	max_mismatch(0), seq_length(0), block_computation(true),
+	num_block_weights_external(0), block_weights_external(NULL),
+	block_weights(NULL), type(E_WD), which_degree(-1), tries(NULL),
+	tree_initialized(false), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 
 	set_wd_weights_by_type(type);
+	set_normalizer(new CFirstElementKernelNormalizer());
 	init(l, r);
 }
 
@@ -113,7 +115,6 @@ void CWeightedDegreeStringKernel::remove_lhs()
 { 
 	SG_DEBUG( "deleting CWeightedDegreeStringKernel optimization\n");
 	delete_optimization();
-	initialized = false;
 
 	if (tries!=NULL)
 		tries->destroy();
@@ -123,7 +124,7 @@ void CWeightedDegreeStringKernel::remove_lhs()
 
 void CWeightedDegreeStringKernel::create_empty_tries()
 {
-	seq_length=((CStringFeatures<CHAR>*) lhs)->get_max_vector_length();
+	seq_length=((CStringFeatures<char>*) lhs)->get_max_vector_length();
 
 	if (tries!=NULL)
 	{
@@ -134,19 +135,18 @@ void CWeightedDegreeStringKernel::create_empty_tries()
   
 bool CWeightedDegreeStringKernel::init(CFeatures* l, CFeatures* r)
 {
-	INT lhs_changed=(lhs!=l);
-	INT rhs_changed=(rhs!=r);
+	int32_t lhs_changed=(lhs!=l);
+	int32_t rhs_changed=(rhs!=r);
 
-	bool result=CStringKernel<CHAR>::init(l,r);
-	initialized=false;
+	CStringKernel<char>::init(l,r);
 
 	SG_DEBUG("lhs_changed: %i\n", lhs_changed);
 	SG_DEBUG("rhs_changed: %i\n", rhs_changed);
 
-	CStringFeatures<CHAR>* sf_l=(CStringFeatures<CHAR>*) l;
-	CStringFeatures<CHAR>* sf_r=(CStringFeatures<CHAR>*) r;
+	CStringFeatures<char>* sf_l=(CStringFeatures<char>*) l;
+	CStringFeatures<char>* sf_r=(CStringFeatures<char>*) r;
 
-	INT len=sf_l->get_max_vector_length();
+	int32_t len=sf_l->get_max_vector_length();
 	if (lhs_changed && !sf_l->have_same_length(len))
 		SG_ERROR("All strings in WD kernel must have same length (lhs wrong)!\n");
 
@@ -157,7 +157,7 @@ bool CWeightedDegreeStringKernel::init(CFeatures* l, CFeatures* r)
 	alphabet=new CAlphabet(sf_l->get_alphabet());
 	CAlphabet* ralphabet=(sf_r->get_alphabet());
 	if (!((alphabet->get_alphabet()==DNA) || (alphabet->get_alphabet()==RNA)))
-		properties &= ((ULONG) (-1)) ^ (KP_LINADD | KP_BATCHEVALUATION);
+		properties &= ((uint64_t) (-1)) ^ (KP_LINADD | KP_BATCHEVALUATION);
 
 	ASSERT(ralphabet->get_alphabet()==alphabet->get_alphabet());
 	SG_UNREF(ralphabet);
@@ -171,13 +171,7 @@ bool CWeightedDegreeStringKernel::init(CFeatures* l, CFeatures* r)
 
 	init_block_weights();
 
-	if (use_normalization)
-		normalization_const=block_weights[seq_length-1];
-	else
-		normalization_const=1.0;
-	
-	initialized=true;
-	return result;
+	return init_normalizer();
 }
 
 void CWeightedDegreeStringKernel::cleanup()
@@ -196,7 +190,6 @@ void CWeightedDegreeStringKernel::cleanup()
 	}
 
 	seq_length=0;
-	initialized=false;
 	tree_initialized = false;
 
 	delete alphabet;
@@ -216,7 +209,7 @@ bool CWeightedDegreeStringKernel::save_init(FILE* dest)
 }
   
 
-bool CWeightedDegreeStringKernel::init_optimization(INT count, INT* IDX, DREAL* alphas, INT tree_num)
+bool CWeightedDegreeStringKernel::init_optimization(int32_t count, int32_t* IDX, float64_t* alphas, int32_t tree_num)
 {
 	if (tree_num<0)
 		SG_DEBUG( "deleting CWeightedDegreeStringKernel optimization\n");
@@ -226,8 +219,7 @@ bool CWeightedDegreeStringKernel::init_optimization(INT count, INT* IDX, DREAL*
 	if (tree_num<0)
 		SG_DEBUG( "initializing CWeightedDegreeStringKernel optimization\n") ;
 
-	int i=0;
-	for (i=0; i<count; i++)
+	for (int32_t i=0; i<count; i++)
 	{
 		if (tree_num<0)
 		{
@@ -273,16 +265,17 @@ bool CWeightedDegreeStringKernel::delete_optimization()
 }
 
 
-DREAL CWeightedDegreeStringKernel::compute_with_mismatch(CHAR* avec, INT alen, CHAR* bvec, INT blen)
+float64_t CWeightedDegreeStringKernel::compute_with_mismatch(
+	char* avec, int32_t alen, char* bvec, int32_t blen)
 {
-	DREAL sum = 0.0 ;
-	
-	for (INT i=0; i<alen; i++)
+	float64_t sum = 0.0;
+
+	for (int32_t i=0; i<alen; i++)
 	{
-		DREAL sumi = 0.0 ;
-		INT mismatches=0 ;
-		
-		for (INT j=0; (i+j<alen) && (j<degree); j++)
+		float64_t sumi = 0.0;
+		int32_t mismatches=0;
+
+		for (int32_t j=0; (i+j<alen) && (j<degree); j++)
 		{
 			if (avec[i+j]!=bvec[i+j])
 			{
@@ -300,14 +293,15 @@ DREAL CWeightedDegreeStringKernel::compute_with_mismatch(CHAR* avec, INT alen, C
 	return sum ;
 }
 
-DREAL CWeightedDegreeStringKernel::compute_using_block(CHAR* avec, INT alen, CHAR* bvec, INT blen)
+float64_t CWeightedDegreeStringKernel::compute_using_block(
+	char* avec, int32_t alen, char* bvec, int32_t blen)
 {
 	ASSERT(alen==blen);
 
-	DREAL sum=0;
-	INT match_len=-1;
+	float64_t sum=0;
+	int32_t match_len=-1;
 
-	for (INT i=0; i<alen; i++)
+	for (int32_t i=0; i<alen; i++)
 	{
 		if (avec[i]==bvec[i])
 			match_len++;
@@ -325,15 +319,16 @@ DREAL CWeightedDegreeStringKernel::compute_using_block(CHAR* avec, INT alen, CHA
 	return sum;
 }
 
-DREAL CWeightedDegreeStringKernel::compute_without_mismatch(CHAR* avec, INT alen, CHAR* bvec, INT blen)
+float64_t CWeightedDegreeStringKernel::compute_without_mismatch(
+	char* avec, int32_t alen, char* bvec, int32_t blen)
 {
-	DREAL sum = 0.0 ;
-	
-	for (INT i=0; i<alen; i++)
+	float64_t sum = 0.0;
+
+	for (int32_t i=0; i<alen; i++)
 	{
-		DREAL sumi = 0.0 ;
-		
-		for (INT j=0; (i+j<alen) && (j<degree); j++)
+		float64_t sumi = 0.0;
+
+		for (int32_t j=0; (i+j<alen) && (j<degree); j++)
 		{
 			if (avec[i+j]!=bvec[i+j])
 				break ;
@@ -347,14 +342,15 @@ DREAL CWeightedDegreeStringKernel::compute_without_mismatch(CHAR* avec, INT alen
 	return sum ;
 }
 
-DREAL CWeightedDegreeStringKernel::compute_without_mismatch_matrix(CHAR* avec, INT alen, CHAR* bvec, INT blen)
+float64_t CWeightedDegreeStringKernel::compute_without_mismatch_matrix(
+	char* avec, int32_t alen, char* bvec, int32_t blen)
 {
-	DREAL sum = 0.0 ;
+	float64_t sum = 0.0;
 
-	for (INT i=0; i<alen; i++)
+	for (int32_t i=0; i<alen; i++)
 	{
-		DREAL sumi=0.0 ;
-		for (INT j=0; (i+j<alen) && (j<degree); j++)
+		float64_t sumi=0.0;
+		for (int32_t j=0; (i+j<alen) && (j<degree); j++)
 		{
 			if (avec[i+j]!=bvec[i+j])
 				break;
@@ -370,12 +366,12 @@ DREAL CWeightedDegreeStringKernel::compute_without_mismatch_matrix(CHAR* avec, I
 }
 
 
-DREAL CWeightedDegreeStringKernel::compute(INT idx_a, INT idx_b)
+float64_t CWeightedDegreeStringKernel::compute(int32_t idx_a, int32_t idx_b)
 {
-	INT alen, blen;
-	CHAR* avec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx_a, alen);
-	CHAR* bvec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx_b, blen);
-	DREAL result=0;
+	int32_t alen, blen;
+	char* avec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx_a, alen);
+	char* bvec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx_b, blen);
+	float64_t result=0;
 
 	if (max_mismatch==0 && length==0 && block_computation)
 		result=compute_using_block(avec, alen, bvec, blen);
@@ -389,115 +385,118 @@ DREAL CWeightedDegreeStringKernel::compute(INT idx_a, INT idx_b)
 			result=compute_without_mismatch_matrix(avec, alen, bvec, blen);
 	}
 
-	return result/normalization_const;
+	return result;
 }
 
 
-void CWeightedDegreeStringKernel::add_example_to_tree(INT idx, DREAL alpha)
+void CWeightedDegreeStringKernel::add_example_to_tree(
+	int32_t idx, float64_t alpha)
 {
 	ASSERT(alphabet);
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len=0;
-	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
+	int32_t len=0;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
 	ASSERT(max_mismatch==0);
-	INT *vec=new INT[len];
+	int32_t *vec=new int32_t[len];
 
-	for (INT i=0; i<len; i++)
+	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 	
 	if (length == 0 || max_mismatch > 0)
 	{
-		for (INT i=0; i<len; i++)
+		for (int32_t i=0; i<len; i++)
 		{
-			DREAL alpha_pw=alpha;
+			float64_t alpha_pw=alpha;
 			/*if (position_weights!=NULL)
 			  alpha_pw *= position_weights[i] ;*/
 			if (alpha_pw==0.0)
 				continue;
 			ASSERT(tries);
-			tries->add_to_trie(i, 0, vec, alpha_pw, weights, (length!=0));
+			tries->add_to_trie(i, 0, vec, normalizer->normalize_lhs(alpha_pw, idx), weights, (length!=0));
 		}
 	}
 	else
 	{
-		for (INT i=0; i<len; i++)
+		for (int32_t i=0; i<len; i++)
 		{
-			DREAL alpha_pw=alpha;
+			float64_t alpha_pw=alpha;
 			/*if (position_weights!=NULL) 
 			  alpha_pw = alpha*position_weights[i] ;*/
 			if (alpha_pw==0.0)
 				continue ;
 			ASSERT(tries);
-			tries->add_to_trie(i, 0, vec, alpha_pw, weights, (length!=0));
+			tries->add_to_trie(i, 0, vec, normalizer->normalize_lhs(alpha_pw, idx), weights, (length!=0));
 		}
 	}
 	delete[] vec ;
 	tree_initialized=true ;
 }
 
-void CWeightedDegreeStringKernel::add_example_to_single_tree(INT idx, DREAL alpha, INT tree_num) 
+void CWeightedDegreeStringKernel::add_example_to_single_tree(
+	int32_t idx, float64_t alpha, int32_t tree_num) 
 {
 	ASSERT(alphabet);
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len ;
-	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
+	int32_t len ;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
 	ASSERT(max_mismatch==0);
-	INT *vec = new INT[len] ;
+	int32_t *vec = new int32_t[len] ;
 
-	for (INT i=tree_num; i<tree_num+degree && i<len; i++)
+	for (int32_t i=tree_num; i<tree_num+degree && i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 	
 	ASSERT(tries);
 	if (alpha!=0.0)
-		tries->add_to_trie(tree_num, 0, vec, alpha, weights, (length!=0));
+		tries->add_to_trie(tree_num, 0, vec, normalizer->normalize_lhs(alpha, idx), weights, (length!=0));
 
 	delete[] vec ;
 	tree_initialized=true ;
 }
 
-void CWeightedDegreeStringKernel::add_example_to_tree_mismatch(INT idx, DREAL alpha)
+void CWeightedDegreeStringKernel::add_example_to_tree_mismatch(int32_t idx, float64_t alpha)
 {
 	ASSERT(tries);
 	ASSERT(alphabet);
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len ;
-	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
+	int32_t len ;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
 	
-	INT *vec = new INT[len] ;
+	int32_t *vec = new int32_t[len] ;
 	
-	for (INT i=0; i<len; i++)
+	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 	
-	for (INT i=0; i<len; i++)
+	for (int32_t i=0; i<len; i++)
 	{
 		if (alpha!=0.0)
-			tries->add_example_to_tree_mismatch_recursion(NO_CHILD, i, alpha, &vec[i], len-i, 0, 0, max_mismatch, weights);
+			tries->add_example_to_tree_mismatch_recursion(NO_CHILD, i, normalizer->normalize_lhs(alpha, idx), &vec[i], len-i, 0, 0, max_mismatch, weights);
 	}
 	
 	delete[] vec ;
 	tree_initialized=true ;
 }
 
-void CWeightedDegreeStringKernel::add_example_to_single_tree_mismatch(INT idx, DREAL alpha, INT tree_num)
+void CWeightedDegreeStringKernel::add_example_to_single_tree_mismatch(
+	int32_t idx, float64_t alpha, int32_t tree_num)
 {
 	ASSERT(tries);
 	ASSERT(alphabet);
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len=0;
-	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
-	INT *vec=new INT[len];
+	int32_t len=0;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
+	int32_t *vec=new int32_t[len];
 
-	for (INT i=tree_num; i<len && i<tree_num+degree; i++)
+	for (int32_t i=tree_num; i<len && i<tree_num+degree; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
 	if (alpha!=0.0)
 	{
 		tries->add_example_to_tree_mismatch_recursion(
-			NO_CHILD, tree_num, alpha, &vec[tree_num], len-tree_num,
+			NO_CHILD, tree_num, normalizer->normalize_lhs(alpha, idx), &vec[tree_num], len-tree_num,
 			0, 0, max_mismatch, weights);
 	}
 
@@ -506,49 +505,54 @@ void CWeightedDegreeStringKernel::add_example_to_single_tree_mismatch(INT idx, D
 }
 
 
-DREAL CWeightedDegreeStringKernel::compute_by_tree(INT idx)
+float64_t CWeightedDegreeStringKernel::compute_by_tree(int32_t idx)
 {
 	ASSERT(alphabet);
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len=0;
-	CHAR* char_vec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx, len);
+	int32_t len=0;
+	char* char_vec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx, len);
 	ASSERT(char_vec && len>0);
-	INT *vec=new INT[len];
+	int32_t *vec=new int32_t[len];
 
-	for (INT i=0; i<len; i++)
+	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
-	DREAL sum=0;
+	float64_t sum=0;
 	ASSERT(tries);
-	for (INT i=0; i<len; i++)
+	for (int32_t i=0; i<len; i++)
 		sum+=tries->compute_by_tree_helper(vec, len, i, i, i, weights, (length!=0));
 
 	delete[] vec;
-	return sum/normalization_const;
+	return normalizer->normalize_rhs(sum, idx);
 }
 
-void CWeightedDegreeStringKernel::compute_by_tree(INT idx, DREAL* LevelContrib)
+void CWeightedDegreeStringKernel::compute_by_tree(
+	int32_t idx, float64_t* LevelContrib)
 {
 	ASSERT(alphabet);
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len ;
-	CHAR* char_vec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx, len);
+	int32_t len ;
+	char* char_vec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx, len);
 
-	INT *vec = new INT[len] ;
+	int32_t *vec = new int32_t[len] ;
 
-	for (INT i=0; i<len; i++)
+	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
 	ASSERT(tries);
-	for (INT i=0; i<len; i++)
-	  tries->compute_by_tree_helper(vec, len, i, i, i, LevelContrib, 1.0/normalization_const, mkl_stepsize, weights, (length!=0));
+	for (int32_t i=0; i<len; i++)
+	{
+		tries->compute_by_tree_helper(vec, len, i, i, i, LevelContrib,
+				normalizer->normalize_rhs(1.0, idx),
+				mkl_stepsize, weights, (length!=0));
+	}
 
 	delete[] vec ;
 }
 
-DREAL *CWeightedDegreeStringKernel::compute_abs_weights(int &len)
+float64_t *CWeightedDegreeStringKernel::compute_abs_weights(int32_t &len)
 {
 	ASSERT(tries);
 	return tries->compute_abs_weights(len);
@@ -560,11 +564,11 @@ bool CWeightedDegreeStringKernel::set_wd_weights_by_type(EWDKernType p_type)
 	ASSERT(p_type==E_WD); /// if we know a better weighting later on do a switch
 
 	delete[] weights;
-	weights=new DREAL[degree];
+	weights=new float64_t[degree];
 	if (weights)
 	{
-		INT i;
-		DREAL sum=0;
+		int32_t i;
+		float64_t sum=0;
 		for (i=0; i<degree; i++)
 		{
 			weights[i]=degree-i;
@@ -575,11 +579,11 @@ bool CWeightedDegreeStringKernel::set_wd_weights_by_type(EWDKernType p_type)
 
 		for (i=0; i<degree; i++)
 		{
-			for (INT j=1; j<=max_mismatch; j++)
+			for (int32_t j=1; j<=max_mismatch; j++)
 			{
 				if (j<i+1)
 				{
-					INT nk=CMath::nchoosek(i+1, j);
+					int32_t nk=CMath::nchoosek(i+1, j);
 					weights[i+j*degree]=weights[i]/(nk*pow(3,j));
 				}
 				else
@@ -604,7 +608,8 @@ bool CWeightedDegreeStringKernel::set_wd_weights_by_type(EWDKernType p_type)
 		return false;
 }
 
-bool CWeightedDegreeStringKernel::set_weights(DREAL* ws, INT d, INT len)
+bool CWeightedDegreeStringKernel::set_weights(
+	float64_t* ws, int32_t d, int32_t len)
 {
 	SG_DEBUG("degree = %i  d=%i\n", degree, d);
 	degree=d;
@@ -613,12 +618,12 @@ bool CWeightedDegreeStringKernel::set_weights(DREAL* ws, INT d, INT len)
 	length=len;
 	
 	if (length==0) length=1;
-	INT num_weights=degree*(length+max_mismatch);
+	int32_t num_weights=degree*(length+max_mismatch);
 	delete[] weights;
-	weights=new DREAL[num_weights];
+	weights=new float64_t[num_weights];
 	if (weights)
 	{
-		for (INT i=0; i<num_weights; i++) {
+		for (int32_t i=0; i<num_weights; i++) {
 			if (ws[i]) // len(ws) might be != num_weights?
 				weights[i]=ws[i];
 		}
@@ -628,7 +633,8 @@ bool CWeightedDegreeStringKernel::set_weights(DREAL* ws, INT d, INT len)
 		return false;
 }
 
-bool CWeightedDegreeStringKernel::set_position_weights(DREAL* pws, INT len)
+bool CWeightedDegreeStringKernel::set_position_weights(
+	float64_t* pws, int32_t len)
 {
 	if (len==0)
 	{
@@ -642,13 +648,13 @@ bool CWeightedDegreeStringKernel::set_position_weights(DREAL* pws, INT len)
 		SG_ERROR("seq_length = %i, position_weights_length=%i\n", seq_length, len);
 
 	delete[] position_weights;
-	position_weights=new DREAL[len];
+	position_weights=new float64_t[len];
 	ASSERT(tries);
 	tries->set_position_weights(position_weights);
 	
 	if (position_weights)
 	{
-		for (int i=0; i<len; i++)
+		for (int32_t i=0; i<len; i++)
 			position_weights[i]=pws[i];
 		return true;
 	}
@@ -659,16 +665,18 @@ bool CWeightedDegreeStringKernel::set_position_weights(DREAL* pws, INT len)
 bool CWeightedDegreeStringKernel::init_block_weights_from_wd()
 {
 	delete[] block_weights;
-	block_weights=new DREAL[CMath::max(seq_length,degree)];
+	block_weights=new float64_t[CMath::max(seq_length,degree)];
 
 	if (block_weights)
 	{
-		double deg=degree;
-		INT k;
-		for (k=0; k<degree ; k++)
-			block_weights[k]=(-pow(k,3) + (3*deg-3)*pow(k,2) + (9*deg-2)*k + 6*deg) / (3*deg*(deg+1));
-		for (k=degree; k<seq_length ; k++)
-			block_weights[k]=(-deg+3*k+4)/3;
+		int32_t k;
+		float64_t d=degree; // use float to evade rounding errors below
+
+		for (k=0; k<degree; k++)
+			block_weights[k]=
+				(-pow(k, 3)+(3*d-3)*pow(k, 2)+(9*d-2)*k+6*d)/(3*d*(d+1));
+		for (k=degree; k<seq_length; k++)
+			block_weights[k]=(-d+3*k+4)/3;
 	}
 
 	return (block_weights!=NULL);
@@ -678,11 +686,11 @@ bool CWeightedDegreeStringKernel::init_block_weights_from_wd_external()
 {
 	ASSERT(weights);
 	delete[] block_weights;
-	block_weights=new DREAL[CMath::max(seq_length,degree)];
+	block_weights=new float64_t[CMath::max(seq_length,degree)];
 
 	if (block_weights)
 	{
-		INT i=0;
+		int32_t i=0;
 		block_weights[0]=weights[0];
 		for (i=1; i<CMath::max(seq_length,degree); i++)
 			block_weights[i]=0;
@@ -691,8 +699,8 @@ bool CWeightedDegreeStringKernel::init_block_weights_from_wd_external()
 		{
 			block_weights[i]=block_weights[i-1];
 
-			DREAL contrib=0;
-			for (INT j=0; j<CMath::min(degree,i+1); j++)
+			float64_t contrib=0;
+			for (int32_t j=0; j<CMath::min(degree,i+1); j++)
 				contrib+=weights[j];
 
 			block_weights[i]+=contrib;
@@ -705,11 +713,11 @@ bool CWeightedDegreeStringKernel::init_block_weights_from_wd_external()
 bool CWeightedDegreeStringKernel::init_block_weights_const()
 {
 	delete[] block_weights;
-	block_weights=new DREAL[seq_length];
+	block_weights=new float64_t[seq_length];
 
 	if (block_weights)
 	{
-		for (int i=1; i<seq_length+1 ; i++)
+		for (int32_t i=1; i<seq_length+1 ; i++)
 			block_weights[i-1]=1.0/seq_length;
 	}
 
@@ -719,11 +727,11 @@ bool CWeightedDegreeStringKernel::init_block_weights_const()
 bool CWeightedDegreeStringKernel::init_block_weights_linear()
 {
 	delete[] block_weights;
-	block_weights=new DREAL[seq_length];
+	block_weights=new float64_t[seq_length];
 
 	if (block_weights)
 	{
-		for (int i=1; i<seq_length+1 ; i++)
+		for (int32_t i=1; i<seq_length+1 ; i++)
 			block_weights[i-1]=degree*i;
 	}
 
@@ -733,14 +741,14 @@ bool CWeightedDegreeStringKernel::init_block_weights_linear()
 bool CWeightedDegreeStringKernel::init_block_weights_sqpoly()
 {
 	delete[] block_weights;
-	block_weights=new DREAL[seq_length];
+	block_weights=new float64_t[seq_length];
 
 	if (block_weights)
 	{
-		for (int i=1; i<degree+1 ; i++)
-			block_weights[i-1]=((double) i)*i;
+		for (int32_t i=1; i<degree+1 ; i++)
+			block_weights[i-1]=((float64_t) i)*i;
 
-		for (int i=degree+1; i<seq_length+1 ; i++)
+		for (int32_t i=degree+1; i<seq_length+1 ; i++)
 			block_weights[i-1]=i;
 	}
 
@@ -750,14 +758,14 @@ bool CWeightedDegreeStringKernel::init_block_weights_sqpoly()
 bool CWeightedDegreeStringKernel::init_block_weights_cubicpoly()
 {
 	delete[] block_weights;
-	block_weights=new DREAL[seq_length];
+	block_weights=new float64_t[seq_length];
 
 	if (block_weights)
 	{
-		for (int i=1; i<degree+1 ; i++)
-			block_weights[i-1]=((double) i)*i*i;
+		for (int32_t i=1; i<degree+1 ; i++)
+			block_weights[i-1]=((float64_t) i)*i*i;
 
-		for (int i=degree+1; i<seq_length+1 ; i++)
+		for (int32_t i=degree+1; i<seq_length+1 ; i++)
 			block_weights[i-1]=i;
 	}
 
@@ -767,14 +775,14 @@ bool CWeightedDegreeStringKernel::init_block_weights_cubicpoly()
 bool CWeightedDegreeStringKernel::init_block_weights_exp()
 {
 	delete[] block_weights;
-	block_weights=new DREAL[seq_length];
+	block_weights=new float64_t[seq_length];
 
 	if (block_weights)
 	{
-		for (int i=1; i<degree+1 ; i++)
-			block_weights[i-1]=exp(((double) i/10.0));
+		for (int32_t i=1; i<degree+1 ; i++)
+			block_weights[i-1]=exp(((float64_t) i/10.0));
 
-		for (int i=degree+1; i<seq_length+1 ; i++)
+		for (int32_t i=degree+1; i<seq_length+1 ; i++)
 			block_weights[i-1]=i;
 	}
 
@@ -784,14 +792,14 @@ bool CWeightedDegreeStringKernel::init_block_weights_exp()
 bool CWeightedDegreeStringKernel::init_block_weights_log()
 {
 	delete[] block_weights;
-	block_weights=new DREAL[seq_length];
+	block_weights=new float64_t[seq_length];
 
 	if (block_weights)
 	{
-		for (int i=1; i<degree+1 ; i++)
+		for (int32_t i=1; i<degree+1 ; i++)
 			block_weights[i-1]=pow(log(i),2);
 
-		for (int i=degree+1; i<seq_length+1 ; i++)
+		for (int32_t i=degree+1; i<seq_length+1 ; i++)
 			block_weights[i-1]=i-degree+1+pow(log(degree+1),2);
 	}
 
@@ -803,11 +811,11 @@ bool CWeightedDegreeStringKernel::init_block_weights_external()
 	if (block_weights_external && (seq_length == num_block_weights_external) )
 	{
 		delete[] block_weights;
-		block_weights=new DREAL[seq_length];
+		block_weights=new float64_t[seq_length];
 
 		if (block_weights)
 		{
-			for (int i=0; i<seq_length; i++)
+			for (int32_t i=0; i<seq_length; i++)
 				block_weights[i]=block_weights_external[i];
 		}
 	}
@@ -848,31 +856,30 @@ bool CWeightedDegreeStringKernel::init_block_weights()
 void* CWeightedDegreeStringKernel::compute_batch_helper(void* p)
 {
 	S_THREAD_PARAM* params = (S_THREAD_PARAM*) p;
-	INT j=params->j;
+	int32_t j=params->j;
 	CWeightedDegreeStringKernel* wd=params->kernel;
 	CTrie<DNATrie>* tries=params->tries;
-	DREAL* weights=params->weights;
-	INT length=params->length;
-	INT* vec=params->vec;
-	DREAL* result=params->result;
-	DREAL factor=params->factor;
-	INT* vec_idx=params->vec_idx;
+	float64_t* weights=params->weights;
+	int32_t length=params->length;
+	int32_t* vec=params->vec;
+	float64_t* result=params->result;
+	float64_t factor=params->factor;
+	int32_t* vec_idx=params->vec_idx;
 
-	CStringFeatures<CHAR>* rhs_feat=((CStringFeatures<CHAR>*) wd->get_rhs());
+	CStringFeatures<char>* rhs_feat=((CStringFeatures<char>*) wd->get_rhs());
 	CAlphabet* alpha=wd->alphabet;
 
-	for (INT i=params->start; i<params->end; i++)
+	for (int32_t i=params->start; i<params->end; i++)
 	{
-		INT len=0;
-		CHAR* char_vec=rhs_feat->get_feature_vector(vec_idx[i], len);
-		for (INT k=j; k<CMath::min(len,j+wd->get_degree()); k++)
+		int32_t len=0;
+		char* char_vec=rhs_feat->get_feature_vector(vec_idx[i], len);
+		for (int32_t k=j; k<CMath::min(len,j+wd->get_degree()); k++)
 			vec[k]=alpha->remap_to_bin(char_vec[k]);
 
 		ASSERT(tries);
 
 		result[i]+=factor*
-			tries->compute_by_tree_helper(vec, len, j, j, j, weights, (length!=0))/
-			wd->get_normalization_const();
+			wd->normalizer->normalize_rhs(tries->compute_by_tree_helper(vec, len, j, j, j, weights, (length!=0)), vec_idx[i]);
 	}
 
 	SG_UNREF(rhs_feat);
@@ -880,7 +887,9 @@ void* CWeightedDegreeStringKernel::compute_batch_helper(void* p)
 	return NULL;
 }
 
-void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL* result, INT num_suppvec, INT* IDX, DREAL* alphas, DREAL factor)
+void CWeightedDegreeStringKernel::compute_batch(
+	int32_t num_vec, int32_t* vec_idx, float64_t* result, int32_t num_suppvec,
+	int32_t* IDX, float64_t* alphas, float64_t factor)
 {
 	ASSERT(tries);
 	ASSERT(alphabet);
@@ -892,19 +901,19 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 	ASSERT(result);
 	create_empty_tries();
 
-	INT num_feat=((CStringFeatures<CHAR>*) rhs)->get_max_vector_length();
+	int32_t num_feat=((CStringFeatures<char>*) rhs)->get_max_vector_length();
 	ASSERT(num_feat>0);
-	INT num_threads=parallel.get_num_threads();
+	int32_t num_threads=parallel.get_num_threads();
 	ASSERT(num_threads>0);
-	INT* vec=new INT[num_threads*num_feat];
+	int32_t* vec=new int32_t[num_threads*num_feat];
 
 	if (num_threads < 2)
 	{
 #ifdef CYGWIN
-		for (INT j=0; j<num_feat; j++)
+		for (int32_t j=0; j<num_feat; j++)
 #else
-        CSignal::clear() ;
-		for (INT j=0; j<num_feat && !CSignal::cancel_computations(); j++)
+        CSignal::clear_cancel();
+		for (int32_t j=0; j<num_feat && !CSignal::cancel_computations(); j++)
 #endif
 		{
 			init_optimization(num_suppvec, IDX, alphas, j);
@@ -928,14 +937,14 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 #ifndef WIN32
 	else
 	{
-        CSignal::clear() ;
-		for (INT j=0; j<num_feat && !CSignal::cancel_computations(); j++)
+        CSignal::clear_cancel();
+		for (int32_t j=0; j<num_feat && !CSignal::cancel_computations(); j++)
 		{
 			init_optimization(num_suppvec, IDX, alphas, j);
 			pthread_t threads[num_threads-1];
 			S_THREAD_PARAM params[num_threads];
-			INT step= num_vec/num_threads;
-			INT t;
+			int32_t step= num_vec/num_threads;
+			int32_t t;
 
 			for (t=0; t<num_threads-1; t++)
 			{
@@ -979,7 +988,7 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 	create_empty_tries();
 }
 
-bool CWeightedDegreeStringKernel::set_max_mismatch(INT max)
+bool CWeightedDegreeStringKernel::set_max_mismatch(int32_t max)
 {
 	if (type==E_EXTERNAL && max!=0) {
 		return false;
@@ -992,4 +1001,3 @@ bool CWeightedDegreeStringKernel::set_max_mismatch(INT max)
 	else
 		return true;
 }
-
