@@ -17,8 +17,35 @@
 #include "kernel/StringKernel.h"
 #include "features/StringFeatures.h"
 
-/** kernel WeightedDegreeString */
-class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
+
+enum EWDKernType
+{
+	E_WD=0,
+	E_EXTERNAL=1,
+
+	E_BLOCK_CONST=2,
+	E_BLOCK_LINEAR=3,
+	E_BLOCK_SQPOLY=4,
+	E_BLOCK_CUBICPOLY=5,
+	E_BLOCK_EXP=6,
+	E_BLOCK_LOG=7,
+	E_BLOCK_EXTERNAL=8
+};
+
+
+/** The Weighted Degree kernel of order d compares two sequences \f${\bf x}\f$ and
+ *  \f${\bf x'}\f$ of length L by summing all contributions of k-mer matches of
+ *  lengths \f$k\in\{1,\dots,d\}\f$, weighted by coefficients \f$\beta_k\f$. It
+ *  is defined as
+ *  \f[
+ *      k({\bf x},{\bf x'})=\sum_{k=1}^d\beta_k\sum_{l=1}^{L-k+1}I({\bf u}_{k,l}({\bf x})={\bf u}_{k,l}({\bf x'})).
+ *  \f]
+ *      Here, \f${\bf u}_{k,l}({\bf x})\f$ is the string of length k starting at position
+ *      l of the sequence \f${\bf x}\f$ and \f$I(\cdot)\f$ is the indicator function
+ *      which evaluates to 1 when its argument is true and to 0
+ *      otherwise.
+ */
+class CWeightedDegreeStringKernel: public CStringKernel<char>
 {
 	public:
 		/** constructor
@@ -26,14 +53,14 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param degree degree
 		 * @param type weighted degree kernel type
 		 */
-		CWeightedDegreeStringKernel(INT degree, EWDKernType type=E_WD);
+		CWeightedDegreeStringKernel(int32_t degree, EWDKernType type=E_WD);
 
 		/** constructor
 		 *
 		 * @param weights kernel's weights
 		 * @param degree degree
 		 */
-		CWeightedDegreeStringKernel(DREAL* weights, INT degree);
+		CWeightedDegreeStringKernel(float64_t* weights, int32_t degree);
 
 		/** constructor
 		 *
@@ -42,7 +69,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param degree degree
 		 */
 		CWeightedDegreeStringKernel(
-			CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r, INT degree);
+			CStringFeatures<char>* l, CStringFeatures<char>* r, int32_t degree);
 
 		virtual ~CWeightedDegreeStringKernel();
 
@@ -81,7 +108,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return name WeightedDegree
 		 */
-		virtual const CHAR* get_name() { return "WeightedDegree" ; } ;
+		virtual const char* get_name() { return "WeightedDegree"; } ;
 
 		/** initialize optimization
 		 *
@@ -90,7 +117,8 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param alphas alphas
 		 * @return if initializing was successful
 		 */
-		inline virtual bool init_optimization(INT count, INT *IDX, DREAL* alphas)
+		inline virtual bool init_optimization(
+			int32_t count, int32_t *IDX, float64_t* alphas)
 		{
 			return init_optimization(count, IDX, alphas, -1);
 		}
@@ -105,8 +133,8 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param tree_num which tree
 		 * @return if initializing was successful
 		 */
-		virtual bool init_optimization(INT count, INT *IDX, DREAL* alphas,
-			INT tree_num);
+		virtual bool init_optimization(
+			int32_t count, int32_t *IDX, float64_t* alphas, int32_t tree_num);
 
 		/** delete optimization
 		 *
@@ -119,7 +147,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 	 	* @param idx index to compute
 	 	* @return optimized value at given index
 	 	*/
-		virtual DREAL compute_optimized(INT idx)
+		virtual float64_t compute_optimized(int32_t idx)
 		{ 
 			if (get_is_initialized())
 				return compute_by_tree(idx);
@@ -144,8 +172,10 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param alphas alphas
 		 * @param factor factor
 		 */
-		virtual void compute_batch(INT num_vec, INT* vec_idx, DREAL* target,
-			INT num_suppvec, INT* IDX, DREAL* alphas, DREAL factor=1.0);
+		virtual void compute_batch(
+			int32_t num_vec, int32_t* vec_idx, float64_t* target,
+			int32_t num_suppvec, int32_t* IDX, float64_t* alphas,
+			float64_t factor=1.0);
 
 		/** clear normal
 		 * subkernel functionality
@@ -164,7 +194,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param idx where to add
 		 * @param weight what to add
 		 */
-		inline virtual void add_to_normal(INT idx, DREAL weight)
+		inline virtual void add_to_normal(int32_t idx, float64_t weight)
 		{
 			if (max_mismatch==0)
 				add_example_to_tree(idx, weight);
@@ -178,13 +208,13 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return number of subkernels
 		 */
-		inline virtual INT get_num_subkernels()
+		inline virtual int32_t get_num_subkernels()
 		{
 			if (position_weights!=NULL)
-				return (INT) ceil(1.0*seq_length/mkl_stepsize) ;
+				return (int32_t) ceil(1.0*seq_length/mkl_stepsize) ;
 			if (length==0)
-				return (INT) ceil(1.0*get_degree()/mkl_stepsize);
-			return (INT) ceil(1.0*get_degree()*length/mkl_stepsize) ;
+				return (int32_t) ceil(1.0*get_degree()/mkl_stepsize);
+			return (int32_t) ceil(1.0*get_degree()*length/mkl_stepsize) ;
 		}
 
 		/** compute by subkernel
@@ -192,7 +222,8 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param idx index
 		 * @param subkernel_contrib subkernel contribution
 		 */
-		inline void compute_by_subkernel(INT idx, DREAL * subkernel_contrib)
+		inline void compute_by_subkernel(
+			int32_t idx, float64_t * subkernel_contrib)
 		{ 
 			if (get_is_initialized())
 			{
@@ -208,18 +239,18 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param num_weights number of weights will be stored here
 		 * @return subkernel weights
 		 */
-		inline const DREAL* get_subkernel_weights(INT& num_weights)
+		inline const float64_t* get_subkernel_weights(int32_t& num_weights)
 		{
 			num_weights = get_num_subkernels();
 
 			delete[] weights_buffer ;
-			weights_buffer = new DREAL[num_weights];
+			weights_buffer = new float64_t[num_weights];
 
 			if (position_weights!=NULL)
-				for (INT i=0; i<num_weights; i++)
+				for (int32_t i=0; i<num_weights; i++)
 					weights_buffer[i] = position_weights[i*mkl_stepsize];
 			else
-				for (INT i=0; i<num_weights; i++)
+				for (int32_t i=0; i<num_weights; i++)
 					weights_buffer[i] = weights[i*mkl_stepsize];
 
 			return weights_buffer;
@@ -230,17 +261,18 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param weights2 weights
 		 * @param num_weights2 number of weights
 		 */
-		inline void set_subkernel_weights(DREAL* weights2, INT num_weights2)
+		inline void set_subkernel_weights(
+			float64_t* weights2, int32_t num_weights2)
 		{
-			INT num_weights = get_num_subkernels();
+			int32_t num_weights = get_num_subkernels();
 			if (num_weights!=num_weights2)
 				SG_ERROR( "number of weights do not match\n");
 
 			if (position_weights!=NULL)
 			{
-				for (INT i=0; i<num_weights; i++)
+				for (int32_t i=0; i<num_weights; i++)
 				{
-					for (INT j=0; j<mkl_stepsize; j++)
+					for (int32_t j=0; j<mkl_stepsize; j++)
 					{
 						if (i*mkl_stepsize+j<seq_length)
 							position_weights[i*mkl_stepsize+j] = weights2[i];
@@ -249,9 +281,9 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 			}
 			else if (length==0)
 			{
-				for (INT i=0; i<num_weights; i++)
+				for (int32_t i=0; i<num_weights; i++)
 				{
-					for (INT j=0; j<mkl_stepsize; j++)
+					for (int32_t j=0; j<mkl_stepsize; j++)
 					{
 						if (i*mkl_stepsize+j<get_degree())
 							weights[i*mkl_stepsize+j] = weights2[i];
@@ -260,9 +292,9 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 			}
 			else
 			{
-				for (INT i=0; i<num_weights; i++)
+				for (int32_t i=0; i<num_weights; i++)
 				{
-					for (INT j=0; j<mkl_stepsize; j++)
+					for (int32_t j=0; j<mkl_stepsize; j++)
 					{
 						if (i*mkl_stepsize+j<get_degree()*length)
 							weights[i*mkl_stepsize+j] = weights2[i];
@@ -277,7 +309,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param len len
 		 * @return computed abs weights
 		 */
-		DREAL *compute_abs_weights(INT & len);
+		float64_t *compute_abs_weights(int32_t & len);
 
 		/** compute by tree
 		 *
@@ -285,7 +317,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param LevelContrib level contribution
 		 * @return computed value
 		 */
-		void compute_by_tree(INT idx, DREAL *LevelContrib);
+		void compute_by_tree(int32_t idx, float64_t *LevelContrib);
 
 		/** check if tree is initialized
 		 *
@@ -293,18 +325,12 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 */
 		bool is_tree_initialized() { return tree_initialized; }
 
-		/** get normalization constant
-		 *
-		 * @return normalization constant
-		 */
-		inline DREAL get_normalization_const() { return normalization_const; }
-
 		/** get degree weights
 		 *
 		 * @param d degree weights will be stored here
 		 * @param len number of degree weights will be stored here
 		 */
-		inline DREAL *get_degree_weights(INT& d, INT& len)
+		inline float64_t *get_degree_weights(int32_t& d, int32_t& len)
 		{
 			d=degree;
 			len=length;
@@ -316,7 +342,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param num_weights number of weights will be stored here
 		 * @return weights
 		 */
-		inline DREAL *get_weights(INT& num_weights)
+		inline float64_t *get_weights(int32_t& num_weights)
 		{
 			if (position_weights!=NULL)
 			{
@@ -335,7 +361,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param len number of position weights will be stored here
 		 * @return position weights
 		 */
-		inline DREAL *get_position_weights(INT& len)
+		inline float64_t *get_position_weights(int32_t& len)
 		{
 			len=seq_length;
 			return position_weights;
@@ -354,7 +380,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param d degree
 		 * @return if setting was successful
 		 */
-		void set_wd_weights(DREAL* p_weights, INT d)
+		void set_wd_weights(float64_t* p_weights, int32_t d)
 		{
 			set_weights(p_weights,d,0);
 		}
@@ -365,7 +391,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param d degree
 		 * @param len number of weights
 		 */
-		bool set_weights(DREAL* weights, INT d, INT len);
+		bool set_weights(float64_t* weights, int32_t d, int32_t len);
 
 		/** set position weights
 		 *
@@ -373,7 +399,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param len number of position weights
 		 * @return if setting was successful
 		 */
-		bool set_position_weights(DREAL* position_weights, INT len=0);
+		bool set_position_weights(float64_t* position_weights, int32_t len=0);
 
 		/** initialize block weights
 		 *
@@ -439,53 +465,49 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 *
 		 * @return if deleting was successful
 		 */
-		bool delete_position_weights() { delete[] position_weights; position_weights=NULL; return true; }
+		bool delete_position_weights()
+		{
+			delete[] position_weights;
+			position_weights=NULL;
+			return true;
+		}
 
 		/** set maximum mismatch
 		 *
 		 * @param max new maximum mismatch
 		 * @return if setting was succesful
 		 */
-		bool set_max_mismatch(INT max);
+		bool set_max_mismatch(int32_t max);
 
 		/** get maximum mismatch
 		 *
 		 * @return maximum mismatch
 		 */
-		inline INT get_max_mismatch() { return max_mismatch; }
+		inline int32_t get_max_mismatch() { return max_mismatch; }
 
 		/** set degree
 		 *
 		 * @param deg new degree
 		 * @return if setting was successful
 		 */
-		inline bool set_degree(INT deg) { degree=deg; return true; }
+		inline bool set_degree(int32_t deg) { degree=deg; return true; }
 
 		/** get degree
 		 *
 		 * @return degree
 		 */
-		inline INT get_degree() { return degree; }
-
-		/** set if normalization shall be used
-		 *
-		 * @param opt if normalization shall be used
-		 * @return if setting was successful
-		 */
-		inline bool set_use_normalization(bool opt) { use_normalization=opt; return true; }
-
-		/** check if normalization is used
-		 *
-		 * @return if normalization is used
-		 */
-		inline bool get_use_normalization() { return use_normalization; }
+		inline int32_t get_degree() { return degree; }
 
 		/** set if block computation shall be performed
 		 *
 		 * @param block if block computation shall be performed
 		 * @return if setting was successful
 		 */
-		inline bool set_use_block_computation(bool block) { block_computation=block; return true; }
+		inline bool set_use_block_computation(bool block)
+		{
+			block_computation=block;
+			return true;
+		}
 
 		/** check if block computation is performed
 		 *
@@ -498,26 +520,34 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param step new step size
 		 * @return if setting was successful
 		 */
-		inline bool set_mkl_stepsize(INT step) { mkl_stepsize=step; return true; }
+		inline bool set_mkl_stepsize(int32_t step)
+		{
+			mkl_stepsize=step;
+			return true;
+		}
 
 		/** get MKL step size
 		 *
 		 * @return MKL step size
 		 */
-		inline INT get_mkl_stepsize() { return mkl_stepsize; }
+		inline int32_t get_mkl_stepsize() { return mkl_stepsize; }
 
 		/** set which degree
 		 *
 		 * @param which which degree
 		 * @return if setting was successful
 		 */
-		inline bool set_which_degree(INT which) { which_degree=which; return true; }
+		inline bool set_which_degree(int32_t which)
+		{
+			which_degree=which;
+			return true;
+		}
 
 		/** get which degree
 		 *
 		 * @return which degree
 		 */
-		inline INT get_which_degree() { return which_degree; }
+		inline int32_t get_which_degree() { return which_degree; }
 
 	protected:
 		/** create emtpy tries */
@@ -528,7 +558,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param idx index
 		 * @param weight weight
 		 */
-		void add_example_to_tree(INT idx, DREAL weight);
+		void add_example_to_tree(int32_t idx, float64_t weight);
 
 		/** add example to single tree
 		 *
@@ -536,14 +566,15 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param weight weight
 		 * @param tree_num which tree
 		 */
-		void add_example_to_single_tree(INT idx, DREAL weight, INT tree_num);
+		void add_example_to_single_tree(
+			int32_t idx, float64_t weight, int32_t tree_num);
 
 		/** add example to tree mismatch
 		 *
 		 * @param idx index
 		 * @param weight weight
 		 */
-		void add_example_to_tree_mismatch(INT idx, DREAL weight);
+		void add_example_to_tree_mismatch(int32_t idx, float64_t weight);
 
 		/** add example to single tree mismatch
 		 *
@@ -551,27 +582,15 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param weight weight
 		 * @param tree_num which tree
 		 */
-		void add_example_to_single_tree_mismatch(INT idx, DREAL weight, INT tree_num);
-
-		/** add example to tree mismatch recursion
-		 *
-		 * @param tree tree
-		 * @param alpha alpha
-		 * @param vec vector
-		 * @param len_rem length of rem
-		 * @param depth_rec depth rec
-		 * @param mismatch_rec mismatch rec
-		 */
-		void add_example_to_tree_mismatch_recursion(DNATrie *tree,
-			DREAL alpha, INT *vec, INT len_rem,
-			INT depth_rec, INT mismatch_rec);
+		void add_example_to_single_tree_mismatch(
+			int32_t idx, float64_t weight, int32_t tree_num);
 
 		/** compute by tree
 		 *
 		 * @param idx index
 		 * @return computed value
 		 */
-		DREAL compute_by_tree(INT idx);
+		float64_t compute_by_tree(int32_t idx);
 
 		/** compute kernel function for features a and b
 		 * idx_{a,b} denote the index of the feature vectors
@@ -581,7 +600,7 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param idx_b index b
 		 * @return computed kernel function at indices a,b
 		 */
-		DREAL compute(INT idx_a, INT idx_b);
+		float64_t compute(int32_t idx_a, int32_t idx_b);
 
 		/** compute with mismatch
 		 *
@@ -591,8 +610,8 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_with_mismatch(CHAR* avec, INT alen,
-			CHAR* bvec, INT blen) ;
+		float64_t compute_with_mismatch(
+			char* avec, int32_t alen, char* bvec, int32_t blen);
 
 		/** compute without mismatch
 		 *
@@ -602,8 +621,8 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_without_mismatch(CHAR* avec, INT alen,
-			CHAR* bvec, INT blen);
+		float64_t compute_without_mismatch(
+			char* avec, int32_t alen, char* bvec, int32_t blen);
 
 		/** compute without mismatch matrix
 		 *
@@ -613,8 +632,8 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_without_mismatch_matrix(CHAR* avec, INT alen,
-			CHAR* bvec, INT blen);
+		float64_t compute_without_mismatch_matrix(
+			char* avec, int32_t alen, char* bvec, int32_t blen);
 
 		/** compute using block
 		 *
@@ -624,8 +643,8 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		 * @param blen length of vector b
 		 * @return computed value
 		 */
-		DREAL compute_using_block(CHAR* avec, INT alen,
-			CHAR* bvec, INT blen);
+		float64_t compute_using_block(char* avec, int32_t alen,
+			char* bvec, int32_t blen);
 
 		/** remove lhs from kernel */
 		virtual void remove_lhs();
@@ -634,44 +653,40 @@ class CWeightedDegreeStringKernel: public CStringKernel<CHAR>
 		/** degree*length weights
 		 *length must match seq_length if != 0
 		 */
-		DREAL* weights;
+		float64_t* weights;
 		/** position weights */
-		DREAL* position_weights;
+		float64_t* position_weights;
 		/** weights buffer */
-		DREAL* weights_buffer;
+		float64_t* weights_buffer;
 		/** MKL step size */
-		INT mkl_stepsize;
+		int32_t mkl_stepsize;
 		/** degree */
-		INT degree;
+		int32_t degree;
 		/** length */
-		INT length;
+		int32_t length;
 
 		/** maximum mismatch */
-		INT max_mismatch;
+		int32_t max_mismatch;
 		/** sequence length */
-		INT seq_length;
+		int32_t seq_length;
 
 		/** if kernel is initialized */
 		bool initialized;
-		/** if normalization is used */
-		bool use_normalization;
+
 		/** if block computation is used */
 		bool block_computation;
 
-		/** normalization constant */
-		DREAL normalization_const;
-
 		/** number of external block weights */
-		INT num_block_weights_external;
+		int32_t num_block_weights_external;
 		/** external block weights */
-		DREAL* block_weights_external;
+		float64_t* block_weights_external;
 
 		/** (internal) block weights */
-		DREAL* block_weights;
+		float64_t* block_weights;
 		/** WeightedDegree kernel type */
 		EWDKernType type;
 		/** which degree */
-		INT which_degree;
+		int32_t which_degree;
 
 		/** tries */
 		CTrie<DNATrie>* tries;
