@@ -15,20 +15,19 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-#include "lib/common.h"
-#include "lib/DynamicArray.h"
 #include "base/SGObject.h"
+#include "lib/DynamicArray.h"
+#include "features/Features.h"
 
 template <class ST> struct T_STRING;
 template <class ST> struct TSparse;
 
-/// A file consists of a header
-/// then an alternation of a type header and data
-///
-/// or just raw data (simplefile=true)
-///
-/// the more complex stuff is currently not implemented
-///
+/** A File access class. A file consists of a fourcc header then an alternation
+ * of a type header and data or just raw data (simplefile=true). However this
+ * implementation is not complete - the more complex stuff is currently not
+ * implemented.
+ */
+
 class CFile : public CSGObject
 {
 public:
@@ -49,7 +48,7 @@ public:
 	 *               will be checked to contain the specified fourcc
 	 *               (e.g. 'RFEA')
 	 */
-	CFile(CHAR* fname, CHAR rw, EFeatureType type, CHAR fourcc[4]=NULL);
+	CFile(char* fname, char rw, EFeatureType type, char fourcc[4]=NULL);
 
 	~CFile();
 
@@ -58,14 +57,14 @@ public:
 	 * @param type feature type
 	 * @return -1
 	 */
-	INT parse_first_header(EFeatureType &type);
+	int32_t parse_first_header(EFeatureType &type);
 
 	/** parse next header - defunct!
 	 *
 	 * @param type feature type
 	 * @return -1
 	 */
-	INT parse_next_header(EFeatureType &type);
+	int32_t parse_next_header(EFeatureType &type);
 
 	// set target to NULL to get it automagically allocated
 	// set num to 0 if whole file is to be read
@@ -75,7 +74,7 @@ public:
 	 * @param num number of data elements
 	 * @return loaded data
 	 */
-	INT*   load_int_data(INT* target, LONG& num);
+	int32_t*   load_int_data(int32_t* target, int64_t& num);
 
 	/** load real data
 	 *
@@ -83,7 +82,7 @@ public:
 	 * @param num number of data elements
 	 * @return loaded data
 	 */
-	DREAL*  load_real_data(DREAL* target, LONG& num);
+	float64_t*  load_real_data(float64_t* target, int64_t& num);
 
 	/** load shortreal data
 	 *
@@ -91,7 +90,7 @@ public:
 	 * @param num number of data elements
 	 * @return loaded data
 	 */
-	SHORTREAL*  load_shortreal_data(SHORTREAL* target, LONG& num);
+	float32_t*  load_shortreal_data(float32_t* target, int64_t& num);
 
 	/** load char data
 	 *
@@ -99,7 +98,7 @@ public:
 	 * @param num number of data elements
 	 * @return loaded data
 	 */
-	CHAR*  load_char_data(CHAR* target, LONG& num);
+	char*  load_char_data(char* target, int64_t& num);
 
 	/** load byte data
 	 *
@@ -107,7 +106,7 @@ public:
 	 * @param num number of data elements
 	 * @return loaded data
 	 */
-	BYTE*  load_byte_data(BYTE* target, LONG& num);
+	uint8_t*  load_byte_data(uint8_t* target, int64_t& num);
 
 	/** load word data
 	 *
@@ -115,7 +114,7 @@ public:
 	 * @param num number of data elements
 	 * @return loaded data
 	 */
-	WORD*  load_word_data(WORD* target, LONG& num);
+	uint16_t*  load_word_data(uint16_t* target, int64_t& num);
 
 	/** load short data
 	 *
@@ -123,7 +122,7 @@ public:
 	 * @param num number of data elements
 	 * @return loaded data
 	 */
-	SHORT* load_short_data(SHORT* target, LONG& num);
+	int16_t* load_short_data(int16_t* target, int64_t& num);
 
 	/** save integer data
 	 *
@@ -131,7 +130,7 @@ public:
 	 * @param num number of data elements
 	 * @return whether operation was successful
 	 */
-	bool save_int_data(INT* src, LONG num);
+	bool save_int_data(int32_t* src, int64_t num);
 
 	/** save real data
 	 *
@@ -139,7 +138,7 @@ public:
 	 * @param num number of data elements
 	 * @return whether operation was successful
 	 */
-	bool save_real_data(DREAL* src, LONG num);
+	bool save_real_data(float64_t* src, int64_t num);
 
 	/** save shortreal data
 	 *
@@ -147,7 +146,7 @@ public:
 	 * @param num number of data elements
 	 * @return whether operation was successful
 	 */
-	bool save_shortreal_data(SHORTREAL* src, LONG num);
+	bool save_shortreal_data(float32_t* src, int64_t num);
 
 	/** save char data
 	 *
@@ -155,7 +154,7 @@ public:
 	 * @param num number of data elements
 	 * @return whether operation was successful
 	 */
-	bool save_char_data(CHAR* src, LONG num);
+	bool save_char_data(char* src, int64_t num);
 
 	/** save byte data
 	 *
@@ -163,7 +162,7 @@ public:
 	 * @param num number of data elements
 	 * @return whether operation was successful
 	 */
-	bool save_byte_data(BYTE* src, LONG num);
+	bool save_byte_data(uint8_t* src, int64_t num);
 
 	/** save word data
 	 *
@@ -171,7 +170,7 @@ public:
 	 * @param num number of data elements
 	 * @return whether operation was successful
 	 */
-	bool save_word_data(WORD* src, LONG num);
+	bool save_word_data(uint16_t* src, int64_t num);
 
 	/** save short data
 	 *
@@ -179,7 +178,7 @@ public:
 	 * @param num number of data elements
 	 * @return whether operation was successful
 	 */
-	bool save_short_data(SHORT* src, LONG num);
+	bool save_short_data(int16_t* src, int64_t num);
 
 	/** check if status is ok
 	 *
@@ -203,7 +202,8 @@ public:
 	 * @param num_vec number of vectors in matrix
 	 * @return if reading was successful
 	 */
-	bool read_real_valued_sparse(TSparse<DREAL>*& matrix, INT& num_feat, INT& num_vec);
+	bool read_real_valued_sparse(
+		TSparse<float64_t>*& matrix, int32_t& num_feat, int32_t& num_vec);
 
 	/** write sparse real valued features in svm light format
 	 *
@@ -212,7 +212,8 @@ public:
 	 * @param num_vec number of vectros in matrix
 	 * @return if writing was successful
 	 */
-	bool write_real_valued_sparse(const TSparse<DREAL>* matrix, INT num_feat, INT num_vec);
+	bool write_real_valued_sparse(
+		const TSparse<float64_t>* matrix, int32_t num_feat, int32_t num_vec);
 
 	/** read dense real valued features, simple ascii format
 	 * e.g. 1.0 1.1 0.2 
@@ -225,7 +226,8 @@ public:
 	 * @param num_vec number of vectors in matrix
 	 * @return if reading was successful
 	 */
-	bool read_real_valued_dense(DREAL*& matrix, INT& num_feat, INT& num_vec);
+	bool read_real_valued_dense(
+		float64_t*& matrix, int32_t& num_feat, int32_t& num_vec);
 
 	/** write dense real valued features, simple ascii format
 	 *
@@ -234,7 +236,8 @@ public:
 	 * @param num_vec number of vectros in matrix
 	 * @return if writing was successful
 	 */
-	bool write_real_valued_dense(const DREAL* matrix, INT num_feat, INT num_vec);
+	bool write_real_valued_dense(
+		const float64_t* matrix, int32_t num_feat, int32_t num_vec);
 
 	/** read char string features, simple ascii format
 	 * e.g. foo bar
@@ -247,7 +250,7 @@ public:
 	 * @param max_string_len length of longest string
 	 * @return if reading was successful
 	 */
-	bool read_char_valued_strings(T_STRING<CHAR>*& strings, INT& num_str, INT& max_string_len);
+	bool read_char_valued_strings(T_STRING<char>*& strings, int32_t& num_str, int32_t& max_string_len);
 
 	/** write char string features, simple ascii format
 	 *
@@ -255,7 +258,7 @@ public:
 	 * @param num_str number of strings
 	 * @return if writing was successful
 	 */
-	bool write_char_valued_strings(const T_STRING<CHAR>* strings, INT num_str);
+	bool write_char_valued_strings(const T_STRING<char>* strings, int32_t num_str);
 
 protected:
 	/** read header
@@ -271,7 +274,7 @@ protected:
 
 private:
 	/** helper function to read_*valued_* */
-	template <class T> void append_item(CDynamicArray<T>* items, CHAR* ptr_data, CHAR* ptr_item);
+	template <class T> void append_item(CDynamicArray<T>* items, char* ptr_data, char* ptr_item);
 
 protected:
 	/** file object */
@@ -279,14 +282,14 @@ protected:
 	/** status */
 	bool status;
 	/** task */
-	CHAR task;
+	char task;
 	/** name of the handled file */
-	CHAR* filename;
+	char* filename;
 	/** expected feature type */
 	EFeatureType expected_type;
 	/** number of headers */
-	INT num_header;
+	int32_t num_header;
 	/** fourcc */
-	CHAR fourcc[4];
+	char fourcc[4];
 };
 #endif
