@@ -56,6 +56,9 @@
 extern "C" int	finite(double);
 #endif
 
+/* Size of RNG seed */
+#define RNG_SEED_SIZE 256
+
 /* Maximum stack size */
 #define RADIX_STACK_SIZE	    512
 
@@ -71,34 +74,27 @@ template <class T> struct radix_stack_t
 	/** Number of grams in pile */
 	size_t sn;
 	/** Byte in current focus */
-	unsigned short si;
+	uint16_t si;
 };
 
-/** pair */
-struct pair
-{
-	/** index 1 */
-	int idx1;
-	/** index 2 */
-	int idx2;
-};
+///** pair */
 
 /** thread qsort */
 struct thread_qsort
 {
 	/** output */
-	DREAL* output;
+	float64_t* output;
 	/** index */
-	INT* index;
+	int32_t* index;
 	/** size */
-	INT size;
+	int32_t size;
 
 	/** qsort threads */
-	INT* qsort_threads;
+	int32_t* qsort_threads;
 	/** sort limit */
-	INT sort_limit;
+	int32_t sort_limit;
 	/** number of threads */
-	INT num_threads;
+	int32_t num_threads;
 };
 
 /** Mathematical Functions.
@@ -167,19 +163,19 @@ class CMath : public CSGObject
 		//@{
 
 		/// crc32
-		static UINT crc32(BYTE *data, INT len);
+		static uint32_t crc32(uint8_t *data, int32_t len);
 
-		static inline DREAL round(DREAL d)
+		static inline float64_t round(float64_t d)
 		{
 			return ::floor(d+0.5);
 		}
 
-		static inline DREAL floor(DREAL d)
+		static inline float64_t floor(float64_t d)
 		{
 			return ::floor(d);
 		}
 
-		static inline DREAL ceil(DREAL d)
+		static inline float64_t ceil(float64_t d)
 		{
 			return ::ceil(d);
 		}
@@ -210,19 +206,19 @@ class CMath : public CSGObject
 			}
 
 		/// x^0.5
-		static inline SHORTREAL sqrt(SHORTREAL x)
+		static inline float32_t sqrt(float32_t x)
 		{
 			return ::sqrtf(x);
 		}
 
 		/// x^0.5
-		static inline DREAL sqrt(DREAL x)
+		static inline float64_t sqrt(float64_t x)
 		{
 			return ::sqrt(x);
 		}
 
 		/// x^0.5
-		static inline LONGREAL sqrt(LONGREAL x)
+		static inline float128_t sqrt(float128_t x)
 		{
 			//fall back to double precision sqrt if sqrtl is not
 			//available
@@ -235,90 +231,105 @@ class CMath : public CSGObject
 
 
 		/// x^n
-		static inline long double powl(long double x, long double n)
+		static inline float128_t powl(float128_t x, float128_t n)
 		{
 			//fall back to double precision pow if powl is not
 			//available
 #ifdef HAVE_POWL
-			return ::powl(x, n);
+			return ::powl((long double) x, (long double) n);
 #else
-			return ::pow(x, n);
+			return ::pow((double) x, (double) n);
 #endif
 		}
 
-		static inline INT pow(INT x, INT n)
+		static inline int32_t pow(int32_t x, int32_t n)
 		{
 			ASSERT(n>=0);
-			INT result=1;
+			int32_t result=1;
 			while (n--)
 				result*=x;
 
 			return result;
 		}
 
-		static inline DREAL pow(DREAL x, DREAL n)
+		static inline float64_t pow(float64_t x, int32_t n)
 		{
-			return ::pow(x, n);
+			ASSERT(n>=0);
+			float64_t result=1;
+			while (n--)
+				result*=x;
+
+			return result;
 		}
 
-		static inline DREAL log10(DREAL v)
+		static inline float64_t pow(float64_t x, float64_t n)
+		{
+			return ::pow((double) x, (double) n);
+		}
+
+		static inline float64_t log10(float64_t v)
 		{
 			return ::log(v)/::log(10.0);
 		}
 
 #ifdef HAVE_LOG2
-		static inline DREAL log2(DREAL v)
+		static inline float64_t log2(float64_t v)
 		{
 			return ::log2(v);
 		}
 #else
-		static inline DREAL log2(DREAL v)
+		static inline float64_t log2(float64_t v)
 		{
 			return ::log(v)/::log(2.0);
 		}
 #endif //HAVE_LOG2
 
-		static inline DREAL log(DREAL v)
+		static inline float64_t log(float64_t v)
 		{
 			return ::log(v);
 		}
 
-		template <class T> static void transpose_matrix(T*& matrix, INT& num_feat, INT& num_vec)
+		template <class T>
+		static void transpose_matrix(
+			T*& matrix, int32_t& num_feat, int32_t& num_vec)
 		{
 			T* transposed=new T[num_vec*num_feat];
-			for (INT i=0; i<num_vec; i++)
+			for (int32_t i=0; i<num_vec; i++)
 			{
-				for (INT j=0; j<num_feat; j++)
-					transposed[j+i*num_feat]=matrix[i+j*num_vec];
+				for (int32_t j=0; j<num_feat; j++)
+					transposed[i+j*num_vec]=matrix[i*num_feat+j];
 			}
 
 			delete[] matrix;
 			matrix=transposed;
 
-			INT tmp=num_feat;
-			num_feat=num_vec;
-			num_vec=tmp;
+			CMath::swap(num_feat, num_vec);
 		}
 
 #ifdef HAVE_LAPACK
 		/// return the pseudo inverse for matrix
 		/// when matrix has shape (rows, cols) the pseudo inverse has (cols, rows)
-		static DREAL* pinv(DREAL* matrix, INT rows, INT cols, DREAL* target=NULL);
+		static float64_t* pinv(
+			float64_t* matrix, int32_t rows, int32_t cols,
+			float64_t* target=NULL);
 
 
 		//C := alpha*op( A )*op( B ) + beta*C
 		//op( X ) = X   or   op( X ) = X',
-		static inline void dgemm(double alpha, const double* A, int rows, int cols, CBLAS_TRANSPOSE transposeA,
-				double *B, int cols_B, CBLAS_TRANSPOSE transposeB,
-				double beta, double *C)
+		static inline void dgemm(
+			double alpha, const double* A, int rows, int cols,
+			CBLAS_TRANSPOSE transposeA, double *B, int cols_B,
+			CBLAS_TRANSPOSE transposeB, double beta, double *C)
 		{
 			cblas_dgemm(CblasColMajor, transposeA, transposeB, rows, cols, cols_B,
 					alpha, A, cols, B, cols_B, beta, C, cols);
 		}
 
 		//y := alpha*A*x + beta*y,   or   y := alpha*A'*x + beta*y,
-		static inline void dgemv(double alpha, const double *A, int rows, int cols, const CBLAS_TRANSPOSE transposeA,
-				const double* X, double beta, double* Y)
+		static inline void dgemv(
+			double alpha, const double *A, int rows, int cols,
+			const CBLAS_TRANSPOSE transposeA, const double* X, double beta,
+			double* Y)
 		{
 			cblas_dgemv(CblasColMajor, transposeA,
 					rows, cols, alpha, A, cols,
@@ -326,32 +337,32 @@ class CMath : public CSGObject
 		}
 #endif
 
-		static inline LONG factorial(INT n)
+		static inline int64_t factorial(int32_t n)
 		{
-			LONG res=1 ;
+			int64_t res=1;
 			for (int i=2; i<=n; i++)
 				res*=i ;
 			return res ;
 		}
 
-		static void init_random(UINT initseed=0)
+		static void init_random(uint32_t initseed=0)
 		{
 			if (initseed==0)
 			{
 				struct timeval tv;
 				gettimeofday(&tv, NULL);
-				seed=(UINT) (4223517*getpid()*tv.tv_sec*tv.tv_usec);
+				seed=(uint32_t) (4223517*getpid()*tv.tv_sec*tv.tv_usec);
 			}
 			else
 				seed=initseed;
 #if !defined(CYGWIN) && !defined(__INTERIX)
-			//seed=42;
-			//SG_SPRINT("initializing random number generator with %d\n", seed);
-			initstate(seed, CMath::rand_state, sizeof(CMath::rand_state));
+			//seed=42
+			//SG_SPRINT("initializing random number generator with %d (seed size %d)\n", seed, RNG_SEED_SIZE);
+			initstate(seed, CMath::rand_state, RNG_SEED_SIZE);
 #endif
 		}
 
-		static inline LONG random()
+		static inline int64_t random()
 		{
 #if defined(CYGWIN) || defined(__INTERIX)
 			return rand();
@@ -360,16 +371,16 @@ class CMath : public CSGObject
 #endif
 		}
 
-		static inline INT random(INT min_value, INT max_value)
+		static inline int32_t random(int32_t min_value, int32_t max_value)
 		{
-			INT ret = min_value + (INT) ((max_value-min_value+1) * (random() / (RAND_MAX+1.0)));
+			int32_t ret = min_value + (int32_t) ((max_value-min_value+1) * (random() / (RAND_MAX+1.0)));
 			ASSERT(ret>=min_value && ret<=max_value);
 			return ret ;
 		}
 
-		static inline SHORTREAL random(SHORTREAL min_value, SHORTREAL max_value)
+		static inline float32_t random(float32_t min_value, float32_t max_value)
 		{
-			SHORTREAL ret = min_value + ((max_value-min_value) * (random() / (1.0*RAND_MAX)));
+			float32_t ret = min_value + ((max_value-min_value) * (random() / (1.0*RAND_MAX)));
 
 			if (ret<min_value || ret>max_value)
 				SG_SPRINT("min_value:%10.10f value: %10.10f max_value:%10.10f", min_value, ret, max_value);
@@ -377,9 +388,9 @@ class CMath : public CSGObject
 			return ret;
 		}
 
-		static inline DREAL random(DREAL min_value, DREAL max_value)
+		static inline float64_t random(float64_t min_value, float64_t max_value)
 		{
-			DREAL ret = min_value + ((max_value-min_value) * (random() / (1.0*RAND_MAX)));
+			float64_t ret = min_value + ((max_value-min_value) * (random() / (1.0*RAND_MAX)));
 
 			if (ret<min_value || ret>max_value)
 				SG_SPRINT("min_value:%10.10f value: %10.10f max_value:%10.10f", min_value, ret, max_value);
@@ -388,115 +399,173 @@ class CMath : public CSGObject
 		}
 
 		template <class T>
-			static void fill_vector(T* vec, INT len, T value)
+			static void fill_vector(T* vec, int32_t len, T value)
 			{
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					vec[i]=value;
 			}
 		template <class T>
-			static void range_fill_vector(T* vec, INT len, T start=0)
+			static void range_fill_vector(T* vec, int32_t len, T start=0)
 			{
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					vec[i]=i+start;
 			}
 
 		template <class T>
-			static void random_vector(T* vec, INT len, T min_value, T max_value)
+			static void random_vector(T* vec, int32_t len, T min_value, T max_value)
 			{
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					vec[i]=CMath::random(min_value, max_value);
 			}
 
-		static inline INT* randperm(INT n)
+		static inline int32_t* randperm(int32_t n)
 		{
-			INT* perm = new INT[n];
+			int32_t* perm = new int32_t[n];
 
 			if (!perm)
 				return NULL;
-			for (INT i = 0; i < n; i++)
+			for (int32_t i = 0; i < n; i++)
 				perm[i] = i;
-			for (INT i = 0; i < n; i++)
+			for (int32_t i = 0; i < n; i++)
 				swap(perm[random(0, n - 1)], perm[i]);
 			return perm;
 		}
 
-		static inline LONG nchoosek(INT n, INT k)
+		static inline int64_t nchoosek(int32_t n, int32_t k)
 		{
-			long res=1 ;
+			int64_t res=1;
 
-			for (INT i=n-k+1; i<=n; i++)
-				res*=i ;
+			for (int32_t i=n-k+1; i<=n; i++)
+				res*=i;
 
-			return res/factorial(k) ;
+			return res/factorial(k);
 		}
 
 		/// x=x+alpha*y
 		template <class T>
 			static inline void vec1_plus_scalar_times_vec2(T* vec1,
-					T scalar, const T* vec2, INT n)
+					T scalar, const T* vec2, int32_t n)
 			{
-				for (INT i=0; i<n; i++)
+				for (int32_t i=0; i<n; i++)
 					vec1[i]+=scalar*vec2[i];
 			}
 
 		/// compute dot product between v1 and v2 (blas optimized)
-		static inline DREAL dot(const DREAL* v1, const DREAL* v2, INT n)
+		static inline float64_t dot(const float64_t* v1, const float64_t* v2, int32_t n)
 		{
-			DREAL r=0;
+			float64_t r=0;
 #ifdef HAVE_LAPACK
-			INT skip=1;
+			int32_t skip=1;
 			r = cblas_ddot(n, v1, skip, v2, skip);
 #else
-			for (INT i=0; i<n; i++)
+			for (int32_t i=0; i<n; i++)
 				r+=v1[i]*v2[i];
 #endif
 			return r;
 		}
 
 		/// compute dot product between v1 and v2 (blas optimized)
-		static inline SHORTREAL dot(const SHORTREAL* v1, const SHORTREAL* v2, INT n)
+		static inline float32_t dot(
+			const float32_t* v1, const float32_t* v2, int32_t n)
 		{
-			DREAL r=0;
+			float64_t r=0;
 #ifdef HAVE_LAPACK
-			INT skip=1;
+			int32_t skip=1;
 			r = cblas_sdot(n, v1, skip, v2, skip);
 #else
-			for (INT i=0; i<n; i++)
+			for (int32_t i=0; i<n; i++)
 				r+=v1[i]*v2[i];
 #endif
 			return r;
 		}
 
+		/// compute dot product between v1 and v2 (for 64bit ints)
+		static inline float64_t dot(
+			const int64_t* v1, const int64_t* v2, int32_t n)
+		{
+			float64_t r=0;
+			for (int32_t i=0; i<n; i++)
+				r+=((float64_t) v1[i])*v2[i];
+
+			return r;
+		}
+
+		/// compute dot product between v1 and v2 (for 16bit unsigned ints)
+		static inline float64_t dot(
+			const uint16_t* v1, const uint16_t* v2, int32_t n)
+		{
+			float64_t r=0;
+			for (int32_t i=0; i<n; i++)
+				r+=((float64_t) v1[i])*v2[i];
+
+			return r;
+		}
+
+		/// compute dot product between v1 and v2 (for 8bit (un)signed ints)
+		static inline float64_t dot(
+			const char* v1, const char* v2, int32_t n)
+		{
+			float64_t r=0;
+			for (int32_t i=0; i<n; i++)
+				r+=((float64_t) v1[i])*v2[i];
+
+			return r;
+		}
+
+		/// compute dot product between v1 and v2 (for 8bit (un)signed ints)
+		static inline float64_t dot(
+			const uint8_t* v1, const uint8_t* v2, int32_t n)
+		{
+			float64_t r=0;
+			for (int32_t i=0; i<n; i++)
+				r+=((float64_t) v1[i])*v2[i];
+
+			return r;
+		}
+
+		/// compute dot product between v1 and v2
+		static inline float64_t dot(
+			const float64_t* v1, const char* v2, int32_t n)
+		{
+			float64_t r=0;
+			for (int32_t i=0; i<n; i++)
+				r+=((float64_t) v1[i])*v2[i];
+
+			return r;
+		}
+
 		/// target=alpha*vec1 + beta*vec2
 		template <class T>
-			static inline void add(T* target, T alpha, const T* v1, T beta, const T* v2, INT len)
+			static inline void add(
+				T* target, T alpha, const T* v1, T beta, const T* v2,
+				int32_t len)
 			{
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					target[i]=alpha*v1[i]+beta*v2[i];
 			}
 
 		/// add scalar to vector inplace
 		template <class T>
-			static inline void add_scalar(T alpha, T* vec, INT len)
+			static inline void add_scalar(T alpha, T* vec, int32_t len)
 			{
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					vec[i]+=alpha;
 			}
 
 		/// scale vector inplace
 		template <class T>
-			static inline void scale_vector(T alpha, T* vec, INT len)
+			static inline void scale_vector(T alpha, T* vec, int32_t len)
 			{
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					vec[i]*=alpha;
 			}
 
 		/// return sum(vec)
 		template <class T>
-			static inline T sum(T* vec, INT len)
+			static inline T sum(T* vec, int32_t len)
 			{
 				T result=0;
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					result+=vec[i];
 
 				return result;
@@ -504,12 +573,12 @@ class CMath : public CSGObject
 
 		/// return max(vec)
 		template <class T>
-			static inline T max(T* vec, INT len)
+			static inline T max(T* vec, int32_t len)
 			{
 				ASSERT(len>0);
 				T maxv=vec[0];
 
-				for (INT i=1; i<len; i++)
+				for (int32_t i=1; i<len; i++)
 					maxv=CMath::max(vec[i], maxv);
 
 				return maxv;
@@ -517,30 +586,31 @@ class CMath : public CSGObject
 
 		/// return sum(abs(vec))
 		template <class T>
-			static inline T sum_abs(T* vec, INT len)
+			static inline T sum_abs(T* vec, int32_t len)
 			{
 				T result=0;
-				for (INT i=0; i<len; i++)
+				for (int32_t i=0; i<len; i++)
 					result+=CMath::abs(vec[i]);
 
 				return result;
 			}
 
-		static inline DREAL mean(DREAL* vec, INT len)
+		static inline float64_t mean(float64_t* vec, int32_t len)
 		{
 			ASSERT(vec);
 			ASSERT(len>0);
 
-			DREAL mean=0;
-			for (INT i=0; i<len; i++)
+			float64_t mean=0;
+			for (int32_t i=0; i<len; i++)
 				mean+=vec[i];
 			return mean/len;
 		}
 
-		static inline DREAL trace(DREAL* mat, INT cols, INT rows)
+		static inline float64_t trace(
+			float64_t* mat, int32_t cols, int32_t rows)
 		{
-			DREAL trace=0;
-			for (INT i=0; i<rows; i++)
+			float64_t trace=0;
+			for (int32_t i=0; i<rows; i++)
 				trace+=mat[i*cols+i];
 			return trace;
 		}
@@ -548,8 +618,8 @@ class CMath : public CSGObject
 		/** performs a bubblesort on a given matrix a.
 		 * it is sorted in ascending order from top to bottom
 		 * and left to right */
-		static void sort(INT *a, INT cols, INT sort_col=0) ;
-		static void sort(DREAL *a, INT*idx, INT N) ;
+		static void sort(int32_t *a, int32_t cols, int32_t sort_col=0);
+		static void sort(float64_t *a, int32_t*idx, int32_t N);
 
 		/*
 		 * Inline function to extract the byte at position p (from left)
@@ -565,17 +635,17 @@ class CMath : public CSGObject
 			}
 
 		template <class T>
-			static inline BYTE byte(T word, unsigned short p)
+			static inline uint8_t byte(T word, uint16_t p)
 			{
 				return (word >> (sizeof(T)-p-1) * 8) & 0xff;
 			}
 
 		template <class T>
-			static void radix_sort_helper(T* array, size_t size, unsigned short i)
+			static void radix_sort_helper(T* array, size_t size, uint16_t i)
 			{
 				static size_t count[256], nc, cmin;
 				T *ak;
-				BYTE c=0;
+				uint8_t c=0;
 				radix_stack_t<T> s[RADIX_STACK_SIZE], *sp, *olds, *bigs;
 				T *an, *aj, *pile[256];
 				size_t *cp, cmax;
@@ -633,7 +703,7 @@ class CMath : public CSGObject
 							}
 
 							if (i < sizeof(T)-1)
-								radix_push(ak, *cp, (unsigned short) (i + 1));
+								radix_push(ak, *cp, (uint16_t) (i + 1));
 						}
 						pile[cp - count] = ak += *cp;
 						nc--;
@@ -671,11 +741,11 @@ class CMath : public CSGObject
 		/** performs insertion sort of an array output of length size
 		 * it is sorted from in ascending (for type T) */
 		template <class T>
-			static void insertion_sort(T* output, INT size)
+			static void insertion_sort(T* output, int32_t size)
 			{
-				for (INT i=0; i<size-1; i++)
+				for (int32_t i=0; i<size-1; i++)
 				{
-					INT j=i-1;
+					int32_t j=i-1;
 					T value=output[i];
 					while (j >= 0 && output[j] > value)
 					{
@@ -690,9 +760,9 @@ class CMath : public CSGObject
 		/** performs a quicksort on an array output of length size
 		 * it is sorted from in ascending (for type T) */
 		//template <class T>
-		//static void qsort(T* output, INT size) ;
+		//static void qsort(T* output, int32_t size) ;
 		template <class T>
-			static void qsort(T* output, INT size)
+			static void qsort(T* output, int32_t size)
 			{
 				if (size==2)
 				{
@@ -702,8 +772,8 @@ class CMath : public CSGObject
 				}
 				T split=output[random(0,size-1)];
 
-				INT left=0;
-				INT right=size-1;
+				int32_t left=0;
+				int32_t right=size-1;
 
 				while (left<=right)
 				{
@@ -728,10 +798,12 @@ class CMath : public CSGObject
 			}
 
 		/// display vector (useful for debugging)
-		template <class T> static void display_vector(T* vector, INT n, const char* name="vector");
+		template <class T> static void display_vector(
+			T* vector, int32_t n, const char* name="vector");
 
 		/// display matrix (useful for debugging)
-		template <class T> static void display_matrix(T* matrix, INT rows, INT cols, const char* name="matrix");
+		template <class T> static void display_matrix(
+			T* matrix, int32_t rows, int32_t cols, const char* name="matrix");
 
 		/** performs a quicksort on an array output of length size
 		 * it is sorted in ascending order 
@@ -739,7 +811,7 @@ class CMath : public CSGObject
 		 * matlab alike [sorted,index]=sort(output) 
 		 */
 		template <class T1,class T2>
-			static void qsort_index(T1* output, T2* index, UINT size);
+			static void qsort_index(T1* output, T2* index, uint32_t size);
 
 		template <class T1,class T2>
 			static void* parallel_qsort_index(void* p);
@@ -750,25 +822,27 @@ class CMath : public CSGObject
 		 * matlab alike [sorted,index]=sort(output) 
 		 */
 		template <class T1,class T2>
-			static void qsort_backward_index(T1* output, T2* index, INT size);
+			static void qsort_backward_index(
+				T1* output, T2* index, int32_t size);
 
 		/* finds the smallest element in output and puts that element as the 
 		   first element  */
 		template <class T>
-			static void min(DREAL* output, T* index, INT size);
+			static void min(float64_t* output, T* index, int32_t size);
 
 		/* finds the n smallest elements in output and puts these elements as the 
 		   first n elements  */
 		template <class T>
-			static void nmin(DREAL* output, T* index, INT size, INT n);
+			static void nmin(
+				float64_t* output, T* index, int32_t size, int32_t n);
 
-		/* performs a inplace unique of a vector of type T using quicksort 
+		/* performs a inplace unique of a vector of type T using quicksort
 		 * returns the new number of elements */
 		template <class T>
-			static INT unique(T* output, INT size) 
+			static int32_t unique(T* output, int32_t size)
 			{
 				qsort(output, size);
-				INT i,j=0 ;
+				int32_t i,j=0 ;
 				for (i=0; i<size; i++)
 					if (i==0 || output[i]!=output[i-1])
 						output[j++]=output[i];
@@ -778,17 +852,17 @@ class CMath : public CSGObject
 		/* finds an element in a sorted array via binary search
 		 * returns -1 if not found */
 		template <class T>
-			static INT binary_search_helper(T* output, INT size, T elem)
+			static int32_t binary_search_helper(T* output, int32_t size, T elem)
 			{
-				INT start=0;
-				INT end=size-1;
+				int32_t start=0;
+				int32_t end=size-1;
 
 				if (size<1)
 					return -1;
 
 				while (start<end)
 				{
-					INT middle=(start+end)/2; 
+					int32_t middle=(start+end)/2; 
 
 					if (output[middle]>elem)
 						end=middle-1;
@@ -807,9 +881,9 @@ class CMath : public CSGObject
 		/* finds an element in a sorted array via binary search
 		 *     * returns -1 if not found */
 		template <class T>
-			static inline INT binary_search(T* output, INT size, T elem)
+			static inline int32_t binary_search(T* output, int32_t size, T elem)
 			{
-				INT ind = binary_search_helper(output, size, elem);
+				int32_t ind = binary_search_helper(output, size, elem);
 				if (ind >= 0 && output[ind] == elem)
 					return ind;
 				return -1;
@@ -820,9 +894,10 @@ class CMath : public CSGObject
 		 * is returned
 		 * note: a successor is not mandatory */
 		template <class T>
-			static INT binary_search_max_lower_equal(T* output, INT size, T elem)
+			static int32_t binary_search_max_lower_equal(
+				T* output, int32_t size, T elem)
 			{
-				INT ind = binary_search_helper(output, size, elem);
+				int32_t ind = binary_search_helper(output, size, elem);
 
 				if (output[ind]<=elem)
 					return ind;
@@ -833,28 +908,33 @@ class CMath : public CSGObject
 
 		/// align two sequences seq1 & seq2 of length l1 and l2 using gapCost
 		/// return alignment cost
-		static DREAL Align(CHAR * seq1, CHAR* seq2, INT l1, INT l2, DREAL gapCost);
+		static float64_t Align(
+			char * seq1, char* seq2, int32_t l1, int32_t l2, float64_t gapCost);
 
 		/** calculates ROC into (fp,tp)
 		 * from output and label of length size 
 		 * returns index with smallest error=fp+fn
 		 */
-		static INT calcroc(DREAL* fp, DREAL* tp, DREAL* output, INT* label, INT& size, INT& possize, INT& negsize, DREAL& tresh, FILE* rocfile);
+		static int32_t calcroc(
+			float64_t* fp, float64_t* tp, float64_t* output, int32_t* label,
+			int32_t& size, int32_t& possize, int32_t& negsize,
+			float64_t& tresh, FILE* rocfile);
 		//@}
 
 		/// returns the mutual information of p which is given in logspace
 		/// where p,q are given in logspace
-		static double mutual_info(DREAL* p1, DREAL* p2, INT len);
+		static float64_t mutual_info(float64_t* p1, float64_t* p2, int32_t len);
 
 		/// returns the relative entropy H(P||Q), 
 		/// where p,q are given in logspace
-		static double relative_entropy(DREAL* p, DREAL* q, INT len);
+		static float64_t relative_entropy(
+			float64_t* p, float64_t* q, int32_t len);
 
 		/// returns entropy of p which is given in logspace
-		static double entropy(DREAL* p, INT len);
+		static float64_t entropy(float64_t* p, int32_t len);
 
 		/// returns number generator seed
-		inline static UINT get_seed()
+		inline static uint32_t get_seed()
 		{
 			return CMath::seed;
 		}
@@ -871,9 +951,9 @@ class CMath : public CSGObject
 		 * and b min(p,q).
 		 */
 #ifdef USE_LOGCACHE
-		static inline DREAL logarithmic_sum(DREAL p, DREAL q)
+		static inline float64_t logarithmic_sum(float64_t p, float64_t q)
 		{
-			DREAL diff;
+			float64_t diff;
 
 			if (!finite(p))
 				return q;
@@ -891,15 +971,16 @@ class CMath : public CSGObject
 						///init log table of form log(1+exp(x))
 						static void init_log_table();
 
-						/// determine INT x for that log(1+exp(-x)) == 0
-						static INT determine_logrange();
+						/// determine int32_t x for that log(1+exp(-x)) == 0
+						static int32_t determine_logrange();
 
 						/// determine accuracy, such that the thing fits into MAX_LOG_TABLE_SIZE, needs logrange as argument
-						static INT determine_logaccuracy(INT range);
+						static int32_t determine_logaccuracy(int32_t range);
 #else
-						static inline DREAL logarithmic_sum(DREAL p, DREAL q)
+						static inline float64_t logarithmic_sum(
+							float64_t p, float64_t q)
 						{
-						DREAL diff;
+						float64_t diff;
 
 						if (!finite(p))
 							return q;
@@ -917,7 +998,8 @@ class CMath : public CSGObject
 				 * each of the elements to some variable. Instead array neighbours are summed up until one element remains.
 				 * Whilst the number of additions remains the same, the error is only in the order of log(N) instead N.
 				 */
-				static inline DREAL logarithmic_sum_array(DREAL *p, INT len)
+				static inline float64_t logarithmic_sum_array(
+					float64_t *p, int32_t len)
 				{
 					if (len<=2)
 					{
@@ -929,9 +1011,9 @@ class CMath : public CSGObject
 					}
 					else
 					{
-						register DREAL *pp=p ;
+						register float64_t *pp=p ;
 						if (len%2==1) pp++ ;
-						for (register INT j=0; j < len>>1; j++)
+						for (register int32_t j=0; j < len>>1; j++)
 							pp[j]=logarithmic_sum(pp[j<<1], pp[1+(j<<1)]) ;
 					}
 					return logarithmic_sum_array(p,len%2+len>>1) ;
@@ -942,28 +1024,28 @@ class CMath : public CSGObject
 				/**@name constants*/
 				//@{
 				/// infinity
-				static const DREAL INFTY;
-				static const DREAL ALMOST_INFTY;
+				static const float64_t INFTY;
+				static const float64_t ALMOST_INFTY;
 
 				/// almost neg (log) infinity
-				static const DREAL ALMOST_NEG_INFTY;
+				static const float64_t ALMOST_NEG_INFTY;
 
 				/// range for logtable: log(1+exp(x))  -LOGRANGE <= x <= 0
-				static INT LOGRANGE;
+				static int32_t LOGRANGE;
 
 				/// random generator seed
-				static UINT seed;
+				static uint32_t seed;
 
 #ifdef USE_LOGCACHE	
 
 				/// number of steps per integer
-				static INT LOGACCURACY;
+				static int32_t LOGACCURACY;
 				//@}
 	protected:
 				///table with log-values
-				static DREAL* logtable;	
+				static float64_t* logtable;	
 #endif
-				static CHAR* rand_state;
+				static char* rand_state;
 };
 
 	template <class T1,class T2>
@@ -972,10 +1054,10 @@ void* CMath::parallel_qsort_index(void* p)
 	struct thread_qsort* ps=(thread_qsort*) p;
 	T1* output=ps->output;
 	T2* index=ps->index;
-	INT size=ps->size;
-	INT* qsort_threads=ps->qsort_threads;
-	INT sort_limit=ps->sort_limit;
-	INT num_threads=ps->num_threads;
+	int32_t size=ps->size;
+	int32_t* qsort_threads=ps->qsort_threads;
+	int32_t sort_limit=ps->sort_limit;
+	int32_t num_threads=ps->num_threads;
 
 	if (size==2)
 	{
@@ -986,11 +1068,11 @@ void* CMath::parallel_qsort_index(void* p)
 		}
 		return NULL;
 	}
-	/*double split=output[(((uint64_t) size)*rand())/(((uint64_t)RAND_MAX)+1)];*/
-	double split=output[size/2];
+	/*float64_t split=output[(((uint64_t) size)*rand())/(((uint64_t)RAND_MAX)+1)];*/
+	float64_t split=output[size/2];
 
-	INT left=0;
-	INT right=size-1;
+	int32_t left=0;
+	int32_t right=size-1;
 
 	while (left<=right)
 	{
@@ -1067,7 +1149,7 @@ void* CMath::parallel_qsort_index(void* p)
 
 //implementations of template functions
 	template <class T1,class T2>
-void CMath::qsort_index(T1* output, T2* index, UINT size)
+void CMath::qsort_index(T1* output, T2* index, uint32_t size)
 {
 	if (size==2)
 	{
@@ -1080,8 +1162,8 @@ void CMath::qsort_index(T1* output, T2* index, UINT size)
 	}
 	T1 split=output[random(0,size-1)];
 
-	INT left=0;
-	INT right=size-1;
+	int32_t left=0;
+	int32_t right=size-1;
 
 	while (left<=right)
 	{
@@ -1107,7 +1189,7 @@ void CMath::qsort_index(T1* output, T2* index, UINT size)
 }
 
 	template <class T1,class T2>
-void CMath::qsort_backward_index(T1* output, T2* index, INT size)
+void CMath::qsort_backward_index(T1* output, T2* index, int32_t size)
 {
 	if (size==2)
 	{
@@ -1121,8 +1203,8 @@ void CMath::qsort_backward_index(T1* output, T2* index, INT size)
 
 	T1 split=output[random(0,size-1)];
 
-	INT left=0;
-	INT right=size-1;
+	int32_t left=0;
+	int32_t right=size-1;
 
 	while (left<=right)
 	{
@@ -1148,10 +1230,10 @@ void CMath::qsort_backward_index(T1* output, T2* index, INT size)
 }
 
 	template <class T> 
-void CMath::nmin(DREAL* output, T* index, INT size, INT n)
+void CMath::nmin(float64_t* output, T* index, int32_t size, int32_t n)
 {
 	if (6*n*size<13*size*CMath::log(size))
-		for (INT i=0; i<n; i++)
+		for (int32_t i=0; i<n; i++)
 			min(&output[i], &index[i], size-i) ;
 	else
 		qsort_index(output, index, size) ;
@@ -1159,13 +1241,13 @@ void CMath::nmin(DREAL* output, T* index, INT size, INT n)
 
 /* move the smallest entry in the array to the beginning */
 	template <class T>
-void CMath::min(DREAL* output, T* index, INT size)
+void CMath::min(float64_t* output, T* index, int32_t size)
 {
 	if (size<=1)
 		return;
-	DREAL min_elem=output[0];
-	INT min_index=0;
-	for (INT i=1; i<size; i++)
+	float64_t min_elem=output[0];
+	int32_t min_index=0;
+	for (int32_t i=1; i<size; i++)
 	{
 		if (output[i]<min_elem)
 		{
