@@ -1,10 +1,15 @@
 #include "lib/ShogunException.h"
+#include "lib/Signal.h"
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 
 ShogunException::ShogunException(const char* str)
 {
+#ifndef WIN32
+	CSignal::unset_handler();
+#endif
    val = new char[4096];
    if (val)
        strncpy(val,str,4096);
