@@ -14,7 +14,7 @@
 #include "features/RealFeatures.h"
 
 CNormDerivativeLem3::CNormDerivativeLem3()
-: CSimplePreProc<DREAL>("NormDerivativeLem3", "NDL3")
+: CSimplePreProc<float64_t>("NormDerivativeLem3", "NDL3")
 {
 }
 
@@ -51,14 +51,15 @@ bool CNormDerivativeLem3::save(FILE* f)
 /// apply preproc on feature matrix
 /// result in feature matrix
 /// return pointer to feature_matrix, i.e. f->get_feature_matrix();
-DREAL* CNormDerivativeLem3::apply_to_feature_matrix(CFeatures* f)
+float64_t* CNormDerivativeLem3::apply_to_feature_matrix(CFeatures* f)
 {
 	return NULL;
 }
 
 /// apply preproc on single feature vector
 /// result in feature matrix
-DREAL* CNormDerivativeLem3::apply_to_feature_vector(DREAL* f, INT len)
+float64_t* CNormDerivativeLem3::apply_to_feature_vector(
+	float64_t* f, int32_t len)
 {
 	return NULL;
 }
@@ -66,9 +67,9 @@ DREAL* CNormDerivativeLem3::apply_to_feature_vector(DREAL* f, INT len)
 //#warning TODO implement jahau 
 //#ifdef JaaHau
 // //this is the normalization used in jaahau
-//    INT o_p=1;
-//    double sum_p=0;
-//    double sum_q=0;
+//    int32_t o_p=1;
+//    float64_t sum_p=0;
+//    float64_t sum_q=0;
 //    //first do positive model
 //    for (i=0; i<pos->get_N(); i++)
 //    {
@@ -77,7 +78,7 @@ DREAL* CNormDerivativeLem3::apply_to_feature_vector(DREAL* f, INT len)
 //	featurevector[p]=exp(pos->model_derivative_q(i, x)-posx);
 //	sum_q=exp(pos->get_q(i))*featurevector[p++];
 //
-//	double sum_a=0;
+//	float64_t sum_a=0;
 //	for (j=0; j<pos->get_N(); j++)
 //	{
 //	    featurevector[p]=exp(pos->model_derivative_a(i, j, x)-posx);
@@ -87,7 +88,7 @@ DREAL* CNormDerivativeLem3::apply_to_feature_vector(DREAL* f, INT len)
 //	for (j=0; j<pos->get_N(); j++)
 //	    featurevector[p++]-=sum_a;
 //
-//	double sum_b=0;
+//	float64_t sum_b=0;
 //	for (j=0; j<pos->get_M(); j++)
 //	{
 //	    featurevector[p]=exp(pos->model_derivative_b(i, j, x)-posx);
@@ -114,7 +115,7 @@ DREAL* CNormDerivativeLem3::apply_to_feature_vector(DREAL* f, INT len)
 //	featurevector[p]=-exp(neg->model_derivative_q(i, x)-negx);
 //	sum_q=exp(neg->get_q(i))*featurevector[p++];
 //
-//	double sum_a=0;
+//	float64_t sum_a=0;
 //	for (j=0; j<neg->get_N(); j++)
 //	{
 //	    featurevector[p]=-exp(neg->model_derivative_a(i, j, x)-negx);
@@ -124,7 +125,7 @@ DREAL* CNormDerivativeLem3::apply_to_feature_vector(DREAL* f, INT len)
 //	for (j=0; j<neg->get_N(); j++)
 //	    featurevector[p++]-=sum_a;
 //
-//	double sum_b=0;
+//	float64_t sum_b=0;
 //	for (j=0; j<neg->get_M(); j++)
 //	{
 //	    featurevector[p]=-exp(neg->model_derivative_b(i, j, x)-negx);
