@@ -17,8 +17,16 @@
 
 #include <stdio.h>
 
-/** Preprocessor NormOne */
-class CNormOne : public CSimplePreProc<DREAL>
+/** Preprocessor NormOne, normalizes vectors to have norm 1, i.e.
+ *
+ * \f[
+ * {\bf x} \leftarrow \frac{{\bf x}}{||{\bf x}||}
+ * \f]
+ *
+ * It therefore does not need any initialization. It is most useful to get data
+ * onto a ball of radius one.
+ */
+class CNormOne : public CSimplePreProc<float64_t>
 {
 	public:
 		/** default constructor */
@@ -41,10 +49,10 @@ class CNormOne : public CSimplePreProc<DREAL>
 		/// apply preproc on feature matrix
 		/// result in feature matrix
 		/// return pointer to feature_matrix, i.e. f->get_feature_matrix();
-		virtual DREAL* apply_to_feature_matrix(CFeatures* f);
+		virtual float64_t* apply_to_feature_matrix(CFeatures* f);
 
 		/// apply preproc on single feature vector
 		/// result in feature matrix
-		virtual DREAL* apply_to_feature_vector(DREAL* f, INT &len);
+		virtual float64_t* apply_to_feature_vector(float64_t* f, int32_t &len);
 };
 #endif
