@@ -25,8 +25,8 @@
 #include "features/RealFeatures.h"
 #include "lib/io.h"
 
-CPCACut::CPCACut(INT do_whitening_, double thresh_)
-: CSimplePreProc<DREAL>("PCACut", "PCAC"), T(NULL), num_dim(0), mean(NULL),
+CPCACut::CPCACut(int32_t do_whitening_, float64_t thresh_)
+: CSimplePreProc<float64_t>("PCACut", "PCAC"), T(NULL), num_dim(0), mean(NULL),
 	initialized(false), do_whitening(do_whitening_), thresh(thresh_)
 {
 }
@@ -46,32 +46,29 @@ bool CPCACut::init(CFeatures* f)
 		ASSERT(f->get_feature_type()==F_DREAL);
 
 		SG_INFO("calling CPCACut::init\n") ;
-		INT num_vectors=((CRealFeatures*)f)->get_num_vectors() ;
-		INT num_features=((CRealFeatures*)f)->get_num_features() ;
+		int32_t num_vectors=((CRealFeatures*)f)->get_num_vectors() ;
+		int32_t num_features=((CRealFeatures*)f)->get_num_features() ;
 		SG_INFO("num_examples: %ld num_features: %ld \n", num_vectors, num_features);
 		delete[] mean ;
-		mean=new double[num_features+1] ;
+		mean=new float64_t[num_features+1] ;
 
-		INT i,j;
+		int32_t i,j;
 
 		/// compute mean
 
 		// clear
 		for (j=0; j<num_features; j++)
-		{
 			mean[j]=0 ; 
-		}
 
 		// sum 
 		for (i=0; i<num_vectors; i++)
 		{
-			INT len;
+			int32_t len;
 			bool free;
-			DREAL* vec=((CRealFeatures*) f)->get_feature_vector(i, len, free);
+			float64_t* vec=((CRealFeatures*) f)->get_feature_vector(i, len, free);
 			for (j=0; j<num_features; j++)
-			{
 				mean[j]+= vec[j];
-			}
+
 			((CRealFeatures*) f)->free_feature_vector(vec, i, free);
 		}
 
@@ -81,7 +78,7 @@ bool CPCACut::init(CFeatures* f)
 
 		SG_DONE();
 		SG_DEBUG("Computing covariance matrix... of size %.2f M\n", num_features*num_features/1024.0/1024.0);
-		double *cov=new double[num_features*num_features];
+		float64_t *cov=new float64_t[num_features*num_features];
 
 		for (j=0; j<num_features*num_features; j++)
 			cov[j]=0.0 ;
@@ -91,20 +88,22 @@ bool CPCACut::init(CFeatures* f)
 			if (!(i % (num_vectors/10+1)))
 				SG_PROGRESS(i, 0, num_vectors);
 
-			INT len;
+			int32_t len;
 			bool free;
 
-			DREAL* vec=((CRealFeatures*) f)->get_feature_vector(i, len, free) ;
+			float64_t* vec=((CRealFeatures*) f)->get_feature_vector(i, len, free) ;
 
-			for (INT jj=0; jj<num_features; jj++)
+			for (int32_t jj=0; jj<num_features; jj++)
 				vec[jj]-=mean[jj] ;
 
 			/// A = 1.0*xy^T+A blas
-			cblas_dger(CblasColMajor, num_features,num_features, 1.0, vec, 1, 
-					vec, 1, cov, (int)num_features) ;
+			int nf = (int) num_features; /* calling external lib */
+			double* vec_double = (double*) vec; /* calling external lib */
+			cblas_dger(CblasColMajor, nf, nf, 1.0, vec_double, 1, vec_double,
+				1, (double*) cov, nf);
 
-			//for (INT k=0; k<num_features; k++)
-			//	for (INT l=0; l<num_features; l++)
+			//for (int32_t k=0; k<num_features; k++)
+			//	for (int32_t l=0; l<num_features; l++)
 			//          cov[k*num_features+l]+=feature[l]*feature[k] ;
 
 			((CRealFeatures*) f)->free_feature_vector(vec, i, free) ;
@@ -121,25 +120,18 @@ bool CPCACut::init(CFeatures* f)
 		SG_INFO("Computing Eigenvalues ... ") ;
 		char V='V';
 		char U='U';
-		//#ifdef DARWIN
-		//		__CLPK_integer ord= (int) num_features;
-		//		__CLPK_integer lda= (int) num_features;
-		//		__CLPK_integer info;
-		//		__CLPK_integer lwork=3*num_features ;
-		//		__CLPK_doublereal* work=new __CLPK_doublereal[lwork] ;
-		//		__CLPK_doublereal* eigenvalues=new __CLPK_doublereal[num_features] ;
-		//#else
-		int info;
-		int ord= (int) num_features;
-		int lda= (int) num_features;
-		double* eigenvalues=new double[num_features] ;
-		//#endif
+		int32_t info;
+		int32_t ord= num_features;
+		int32_t lda= num_features;
+		float64_t* eigenvalues=new float64_t[num_features] ;
 
 		for (i=0; i<num_features; i++)
 			eigenvalues[i]=0;
 
 		// lapack sym matrix eigenvalues+vectors
-		wrap_dsyev(V, U, ord, cov, lda, eigenvalues, &info);
+		wrap_dsyev(V, U, (int) ord, (double*) cov, (int) lda,
+			(double*) eigenvalues, (int*) &info);
+
 
 		num_dim=0;
 		for (i=0; i<num_features; i++)
@@ -152,17 +144,17 @@ bool CPCACut::init(CFeatures* f)
 		SG_INFO("Done\nReducing from %i to %i features..", num_features, num_dim) ;
 
 		delete[] T;
-		T=new DREAL[num_dim*num_features];
+		T=new float64_t[num_dim*num_features];
 		num_old_dim=num_features;
 
 		if (do_whitening)
 		{
-			INT offs=0 ;
+			int32_t offs=0 ;
 			for (i=0; i<num_features; i++)
 			{
-				if (eigenvalues[i]>1e-6)
+				if (eigenvalues[i]>thresh)
 				{
-					for (INT jj=0; jj<num_features; jj++)
+					for (int32_t jj=0; jj<num_features; jj++)
 						T[offs+jj*num_dim]=cov[num_features*i+jj]/sqrt(eigenvalues[i]) ;
 					offs++ ;
 				} ;
@@ -189,31 +181,32 @@ void CPCACut::cleanup()
 /// apply preproc on feature matrix
 /// result in feature matrix
 /// return pointer to feature_matrix, i.e. f->get_feature_matrix();
-DREAL* CPCACut::apply_to_feature_matrix(CFeatures* f)
+float64_t* CPCACut::apply_to_feature_matrix(CFeatures* f)
 {
-	INT num_vectors=0;
-	INT num_features=0;
+	int32_t num_vectors=0;
+	int32_t num_features=0;
 
-	DREAL* m=((CRealFeatures*) f)->get_feature_matrix(num_features, num_vectors);
+	float64_t* m=((CRealFeatures*) f)->get_feature_matrix(num_features, num_vectors);
 	SG_INFO("get Feature matrix: %ix%i\n", num_vectors, num_features) ;
 
 	if (m)
 	{
 		SG_INFO("Preprocessing feature matrix\n");
-		DREAL* res= new DREAL[num_dim];
-		double* sub_mean= new double[num_features];
+		float64_t* res= new float64_t[num_dim];
+		float64_t* sub_mean= new float64_t[num_features];
 
-		for (INT vec=0; vec<num_vectors; vec++)
+		for (int32_t vec=0; vec<num_vectors; vec++)
 		{
-			INT i;
+			int32_t i;
 
 			for (i=0; i<num_features; i++)
 				sub_mean[i]=m[num_features*vec+i]-mean[i] ;
 
-			cblas_dgemv(CblasColMajor, CblasNoTrans, num_dim, num_features, 1.0,
-					T, num_dim, sub_mean, 1, 0, res, 1); 
+			int nd = (int) num_dim; /* calling external lib */
+			cblas_dgemv(CblasColMajor, CblasNoTrans, nd, (int) num_features,
+				1.0, T, nd, (double*) sub_mean, 1, 0, (double*) res, 1);
 
-			DREAL* m_transformed=&m[num_dim*vec];
+			float64_t* m_transformed=&m[num_dim*vec];
 			for (i=0; i<num_dim; i++)
 				m_transformed[i]=m[i];
 		}
@@ -230,20 +223,16 @@ DREAL* CPCACut::apply_to_feature_matrix(CFeatures* f)
 
 /// apply preproc on single feature vector
 /// result in feature matrix
-DREAL* CPCACut::apply_to_feature_vector(DREAL* f, INT &len)
+float64_t* CPCACut::apply_to_feature_vector(float64_t* f, int32_t &len)
 {
-	DREAL *ret=new DREAL[num_dim];
-	DREAL *sub_mean=new DREAL[len];
-	for (INT i=0; i<len; i++)
+	float64_t *ret=new float64_t[num_dim];
+	float64_t *sub_mean=new float64_t[len];
+	for (int32_t i=0; i<len; i++)
 		sub_mean[i]=f[i]-mean[i];
 
-	cblas_dgemv(CblasColMajor, CblasNoTrans, num_dim, len, 1.0 , T, num_dim, sub_mean, 1, 0, ret, 1) ;
-	//void cblas_dgemv(const enum CBLAS_ORDER order,
-	//                 const enum CBLAS_TRANSPOSE TransA, const int M, const int N,
-	//                 const double alpha, const double *A, const int lda,
-	//                 const double *X, const int incX, const double beta,
-	//                 double *Y, const int incY);
-	//
+	int nd = (int) num_dim;  /* calling external lib */
+	cblas_dgemv(CblasColMajor, CblasNoTrans, nd, (int) len, 1.0, (double*) T,
+		nd, (double*) sub_mean, 1, 0, (double*) ret, 1);
 
 	delete[] sub_mean ;
 	len=num_dim ;
@@ -258,11 +247,11 @@ bool CPCACut::load_init_data(FILE* src)
 	ASSERT(fread(&num_old_dim, sizeof(int), 1, src)==1);
 	delete[] mean;
 	delete[] T;
-	mean=new double[num_dim];
-	T=new double[num_dim*num_old_dim];
+	mean=new float64_t[num_dim];
+	T=new float64_t[num_dim*num_old_dim];
 	ASSERT (mean!=NULL && T!=NULL);
-	ASSERT(fread(mean, sizeof(double), num_old_dim, src)==(UINT) num_old_dim);
-	ASSERT(fread(T, sizeof(double), num_dim*num_old_dim, src)==(UINT) num_old_dim*num_dim);
+	ASSERT(fread(mean, sizeof(float64_t), num_old_dim, src)==(uint32_t) num_old_dim);
+	ASSERT(fread(T, sizeof(float64_t), num_dim*num_old_dim, src)==(uint32_t) num_old_dim*num_dim);
 	return true;
 }
 
@@ -271,8 +260,8 @@ bool CPCACut::save_init_data(FILE* dst)
 {
 	ASSERT(fwrite(&num_dim, sizeof(int), 1, dst)==1);
 	ASSERT(fwrite(&num_old_dim, sizeof(int), 1, dst)==1);
-	ASSERT(fwrite(mean, sizeof(double), num_old_dim, dst)==(UINT) num_old_dim);
-	ASSERT(fwrite(T, sizeof(double), num_dim*num_old_dim, dst)==(UINT) num_old_dim*num_dim);
+	ASSERT(fwrite(mean, sizeof(float64_t), num_old_dim, dst)==(uint32_t) num_old_dim);
+	ASSERT(fwrite(T, sizeof(float64_t), num_dim*num_old_dim, dst)==(uint32_t) num_old_dim*num_dim);
 	return true;
 }
 #endif // HAVE_LAPACK
