@@ -23,8 +23,17 @@
 #include "lib/common.h"
 
 
-/** Preprocessor PCACut */
-class CPCACut : public CSimplePreProc<DREAL>
+/** Preprocessor PCACut performs principial component analysis on the input
+ * vectors and keeps only the n eigenvectors with eigenvalues above a certain
+ * threshold. On preprocessing the stored covariance matrix is used to project
+ * vectors into eigenspace only returning vectors of reduced dimension n.
+ * Optional whitening is performed.
+ *
+ * This is only useful if the dimensionality of the data is rather low, as the
+ * covariance matrix is of size num_feat*num_feat. Note that vectors don't have
+ * to have zero mean as it is substracted.
+ */
+class CPCACut : public CSimplePreProc<float64_t>
 {
 	public:
 		/** constructor
@@ -32,7 +41,7 @@ class CPCACut : public CSimplePreProc<DREAL>
 		 * @param do_whitening do whitening
 		 * @param thresh threshold
 		 */
-		CPCACut(INT do_whitening=0, double thresh=1e-6);
+		CPCACut(int32_t do_whitening=0, float64_t thresh=1e-6);
 		virtual ~CPCACut();
 
 		/// initialize preprocessor from features
@@ -47,29 +56,29 @@ class CPCACut : public CSimplePreProc<DREAL>
 		/// apply preproc on feature matrix
 		/// result in feature matrix
 		/// return pointer to feature_matrix, i.e. f->get_feature_matrix();
-		virtual DREAL* apply_to_feature_matrix(CFeatures* f);
+		virtual float64_t* apply_to_feature_matrix(CFeatures* f);
 
 		/// apply preproc on single feature vector
 		/// result in feature matrix
-		virtual DREAL* apply_to_feature_vector(DREAL* f, INT &len);
+		virtual float64_t* apply_to_feature_vector(float64_t* f, int32_t &len);
 
 	protected:
 		/** T */
 		double* T ;
 		/** num dim */
-		INT num_dim;
+		int32_t num_dim;
 		/** num old dim */
-		INT num_old_dim;
+		int32_t num_old_dim;
 		/** mean */
-		double *mean ;
+		float64_t *mean ;
 
 		/// true when already initialized
 		bool initialized;
 
 		/** do whitening */
-		INT do_whitening;
+		int32_t do_whitening;
 		/** thresh */
-		double thresh;
+		float64_t thresh;
 };
 #endif
 #endif
