@@ -14,12 +14,37 @@
 
 #include "lib/common.h"
 #include "base/SGObject.h"
-#include "features/Features.h"
 
-class CPreProc;
+
+enum EPreProcType
+{
+	P_UNKNOWN=0,
+	P_NORMONE=10,
+	P_LOGPLUSONE=20,
+	P_SORTWORDSTRING=30,
+	P_SORTULONGSTRING=40,
+	P_SORTWORD=50,
+	P_PRUNEVARSUBMEAN=60
+};
+
+
+#include "features/Features.h"
 class CFeatures;
 
-/** class PreProc */
+
+/** Class PreProc defines a preprocessor interface. Preprocessors are
+ * transformation functions that don't change the domain of the input features.
+ * These functions can be applied in-place if the input features fit in memory
+ * or can be applied on-the-fly when (depending on features) a feature caching
+ * strategy is applied. However, if the individual features are in
+ * \f$\bf{R}\f$ they have to stay in \f$\bf{R}\f$ although the
+ * dimensionality of the feature vectors is allowed change.
+ *
+ * As preprocessors might need a certain initialization they may expect that
+ * the init() function is called before anything else. The actual preprocessing
+ * is feature type dependent and thus coordinated in the sub-classes, cf. e.g.
+ * CSimplePreProc .
+ */
 class CPreProc : public CSGObject
 {
 public:
@@ -28,7 +53,7 @@ public:
 	 * @param name preprocessor's name
 	 * @param id preprocessor's id
 	 */
-	CPreProc(const CHAR* name, const CHAR* id);
+	CPreProc(const char* name, const char* id);
 	virtual ~CPreProc();
 
 	/// initialize preprocessor from features
@@ -53,15 +78,15 @@ public:
 	virtual EFeatureClass get_feature_class()=0;
 
 	/// return the name of the preprocessor
-	const CHAR * get_name() const { return preproc_name; }
+	const char * get_name() const { return preproc_name; }
 	
 	/// return a FOUR letter id of the preprocessor
-	const CHAR * get_id() const { return preproc_id; }
+	const char * get_id() const { return preproc_id; }
 
  protected:
  	/** preprocessor name */
-	const CHAR* preproc_name;
+	const char* preproc_name;
 	/** preprocessor id */
-	const CHAR* preproc_id;
+	const char* preproc_id;
 };
 #endif
