@@ -17,7 +17,7 @@
 #include "lib/Mathematics.h"
 
 CPruneVarSubMean::CPruneVarSubMean(bool divide)
-: CSimplePreProc<DREAL>("PruneVarSubMean","PVSM"), idx(NULL), mean(NULL),
+: CSimplePreProc<float64_t>("PruneVarSubMean","PVSM"), idx(NULL), mean(NULL),
 	std(NULL), num_idx(0), divide_by_std(divide), initialized(false)
 {
 }
@@ -36,8 +36,8 @@ bool CPruneVarSubMean::init(CFeatures* p_f)
 		ASSERT(p_f->get_feature_type()==F_DREAL);
 
 		CRealFeatures *f=(CRealFeatures*) p_f;
-		INT num_examples=f->get_num_vectors();
-		INT num_features=((CRealFeatures*)f)->get_num_features();
+		int32_t num_examples=f->get_num_vectors();
+		int32_t num_features=((CRealFeatures*)f)->get_num_features();
 
 		delete[] mean;
 		delete[] idx;
@@ -46,9 +46,9 @@ bool CPruneVarSubMean::init(CFeatures* p_f)
 		idx=NULL;
 		std=NULL;
 
-		mean=new double[num_features];
-		double* var=new double[num_features];
-		INT i,j;
+		mean=new float64_t[num_features];
+		float64_t* var=new float64_t[num_features];
+		int32_t i,j;
 
 		for (i=0; i<num_features; i++)
 		{
@@ -59,8 +59,8 @@ bool CPruneVarSubMean::init(CFeatures* p_f)
 		// compute mean
 		for (i=0; i<num_examples; i++)
 		{
-			INT len ; bool free ;
-			DREAL* feature=f->get_feature_vector(i, len, free) ;
+			int32_t len ; bool free ;
+			float64_t* feature=f->get_feature_vector(i, len, free) ;
 
 			for (j=0; j<len; j++)
 				mean[j]+=feature[j];
@@ -74,8 +74,8 @@ bool CPruneVarSubMean::init(CFeatures* p_f)
 		// compute var
 		for (i=0; i<num_examples; i++)
 		{
-			INT len ; bool free ;
-			DREAL* feature=f->get_feature_vector(i, len, free) ;
+			int32_t len ; bool free ;
+			float64_t* feature=f->get_feature_vector(i, len, free) ;
 
 			for (j=0; j<num_features; j++)
 				var[j]+=(mean[j]-feature[j])*(mean[j]-feature[j]) ;
@@ -83,8 +83,8 @@ bool CPruneVarSubMean::init(CFeatures* p_f)
 			f->free_feature_vector(feature, i, free) ;
 		}
 
-		INT num_ok=0;
-		INT* idx_ok=new int[num_features];
+		int32_t num_ok=0;
+		int32_t* idx_ok=new int[num_features];
 
 		for (j=0; j<num_features; j++)
 		{
@@ -101,8 +101,8 @@ bool CPruneVarSubMean::init(CFeatures* p_f)
 
 		delete[] idx ;
 		idx=new int[num_ok];
-		DREAL* new_mean=new DREAL[num_ok];
-		std=new DREAL[num_ok];
+		float64_t* new_mean=new float64_t[num_ok];
+		std=new float64_t[num_ok];
 
 		for (j=0; j<num_ok; j++)
 		{
@@ -137,29 +137,29 @@ void CPruneVarSubMean::cleanup()
 /// apply preproc on feature matrix
 /// result in feature matrix
 /// return pointer to feature_matrix, i.e. f->get_feature_matrix();
-DREAL* CPruneVarSubMean::apply_to_feature_matrix(CFeatures* f)
+float64_t* CPruneVarSubMean::apply_to_feature_matrix(CFeatures* f)
 {
 	ASSERT(initialized);
 
-	INT num_vectors=0;
-	INT num_features=0;
-	DREAL* m=((CRealFeatures*) f)->get_feature_matrix(num_features, num_vectors);
+	int32_t num_vectors=0;
+	int32_t num_features=0;
+	float64_t* m=((CRealFeatures*) f)->get_feature_matrix(num_features, num_vectors);
 
 	SG_INFO( "get Feature matrix: %ix%i\n", num_vectors, num_features);
 	SG_INFO( "Preprocessing feature matrix\n");
-	for (INT vec=0; vec<num_vectors; vec++)
+	for (int32_t vec=0; vec<num_vectors; vec++)
 	{
-		DREAL* v_src=&m[num_features*vec];
-		DREAL* v_dst=&m[num_idx*vec];
+		float64_t* v_src=&m[num_features*vec];
+		float64_t* v_dst=&m[num_idx*vec];
 
 		if (divide_by_std)
 		{
-			for (INT feat=0; feat<num_idx; feat++)
+			for (int32_t feat=0; feat<num_idx; feat++)
 				v_dst[feat]=(v_src[idx[feat]]-mean[feat])/std[feat];
 		}
 		else
 		{
-			for (INT feat=0; feat<num_idx; feat++)
+			for (int32_t feat=0; feat<num_idx; feat++)
 				v_dst[feat]=(v_src[idx[feat]]-mean[feat]);
 		}
 	}
@@ -173,30 +173,30 @@ DREAL* CPruneVarSubMean::apply_to_feature_matrix(CFeatures* f)
 
 /// apply preproc on single feature vector
 /// result in feature matrix
-DREAL* CPruneVarSubMean::apply_to_feature_vector(DREAL* f, INT &len)
+float64_t* CPruneVarSubMean::apply_to_feature_vector(float64_t* f, int32_t &len)
 {
-	DREAL* ret=NULL;
+	float64_t* ret=NULL;
 
 	if (initialized)
 	{
-		ret=new DREAL[num_idx] ;
+		ret=new float64_t[num_idx] ;
 
 		if (divide_by_std)
 		{
-			for (INT i=0; i<num_idx; i++)
+			for (int32_t i=0; i<num_idx; i++)
 				ret[i]=(f[idx[i]]-mean[i])/std[i];
 		}
 		else
 		{
-			for (INT i=0; i<num_idx; i++)
+			for (int32_t i=0; i<num_idx; i++)
 				ret[i]=(f[idx[i]]-mean[i]);
 		}
 		len=num_idx ;
 	}
 	else
 	{
-		ret=new DREAL[len] ;
-		for (INT i=0; i<len; i++)
+		ret=new float64_t[len] ;
+		for (int32_t i=0; i<len; i++)
 			ret[i]=f[i];
 	}
 
@@ -207,21 +207,21 @@ DREAL* CPruneVarSubMean::apply_to_feature_vector(DREAL* f, INT &len)
 bool CPruneVarSubMean::load_init_data(FILE* src)
 {
 	bool result=false;
-	INT divide=0;
+	int32_t divide=0;
 
-	ASSERT(fread(&divide, sizeof(int), 1, src)==1);
-	ASSERT(fread(&num_idx, sizeof(int), 1, src)==1);
+	ASSERT(fread(&divide, sizeof(int32_t), 1, src)==1);
+	ASSERT(fread(&num_idx, sizeof(int32_t), 1, src)==1);
 	SG_INFO( "divide:%d num_idx:%d\n", divide, num_idx);
 	delete[] mean;
 	delete[] idx;
 	delete[] std;
-	idx=new int[num_idx];
-	mean=new DREAL[num_idx];
-	std=new DREAL[num_idx];
+	idx=new int32_t[num_idx];
+	mean=new float64_t[num_idx];
+	std=new float64_t[num_idx];
 	ASSERT (mean!=NULL && idx!=NULL && std!=NULL);
-	ASSERT(fread(idx, sizeof(int), num_idx, src)==(UINT) num_idx);
-	ASSERT(fread(mean, sizeof(DREAL), num_idx, src)==(UINT) num_idx);
-	ASSERT(fread(std, sizeof(DREAL), num_idx, src)==(UINT) num_idx);
+	ASSERT(fread(idx, sizeof(int32_t), num_idx, src)==(size_t) num_idx);
+	ASSERT(fread(mean, sizeof(float64_t), num_idx, src)==(size_t) num_idx);
+	ASSERT(fread(std, sizeof(float64_t), num_idx, src)==(size_t) num_idx);
 
 	result=true;
 	divide_by_std=(divide==1);
