@@ -17,8 +17,13 @@
 
 #include <stdio.h>
 
-/** Preprocessor SortUlongString */
-class CSortUlongString : public CStringPreProc<ULONG>
+/** Preprocessor SortUlongString, sorts the indivual strings in ascending order.
+ * This is useful in conjunction with the CCommUlongStringKernel and will result
+ * in the spectrum kernel. For this to work the strings have to be mapped into
+ * a binary higher order representation first (cf. obtain_from_*() functions in
+ * CStringFeatures)
+ */
+class CSortUlongString : public CStringPreProc<uint64_t>
 {
 public:
 	/** default constructor */
@@ -46,6 +51,6 @@ public:
 
 	/// apply preproc on single feature vector
 	/// result in feature matrix
-	virtual ULONG* apply_to_string(ULONG* f, INT &len);
+	virtual uint64_t* apply_to_string(uint64_t* f, int32_t &len);
 };
 #endif
