@@ -20,7 +20,14 @@
 template <class ST> class TSparse;
 template <class ST> class CSparseFeatures;
 
-/** template class SparsePreProc */
+/** Template class SparsePreProc, base class for preprocessors (cf. CPreProc)
+ * that apply to CSparseFeatures
+ *
+ * Two new functions apply_to_sparse_feature_vector() and
+ * apply_to_sparse_feature_matrix() are defined in this interface that need to
+ * be implemented in each particular preprocessor operating on CSparseFeatures.
+ *
+ * */
 template <class ST> class CSparsePreProc : public CPreProc
 {
 public:
@@ -29,7 +36,7 @@ public:
 	 * @param name sparse preprocessor's name
 	 * @param id sparse preprocessor's id
 	 */
-	CSparsePreProc(const CHAR *name, const CHAR* id) : CPreProc(name,id)
+	CSparsePreProc(const char *name, const char* id) : CPreProc(name,id)
 	{
 	}
 
@@ -40,7 +47,7 @@ public:
 
 	/// apply preproc on single feature vector
 	/// result in feature matrix
-	virtual TSparse<ST>* apply_to_sparse_feature_vector(TSparse<ST>* f, INT &len)=0;
+	virtual TSparse<ST>* apply_to_sparse_feature_vector(TSparse<ST>* f, int32_t &len)=0;
 
   /// return that we are simple minded features (just fixed size matrices)
   inline virtual EFeatureClass get_feature_class() { return C_SPARSE; }
